/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.feesapi.bjgl.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.feesapi.bjgl.entity.YhZfb;
import com.tofly.feesapi.bjgl.service.YhZfbService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 总分表关系
 *
 * @author admin
 * @date Wed Feb 28 00:00:00 CST 2024
 */
@RestController
@AllArgsConstructor
@RequestMapping("api/bjgl/yhzfb")
@Api(tags="水表档案-总分表关系接口")
public class YhZfbController {

    private final  YhZfbService yhZfbService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param yhZfb 总分表关系
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getYhZfbPage(Page page, @ModelAttribute YhZfb yhZfb) {
      return ResultRespone.success(yhZfbService.page(page, Wrappers.query(yhZfb)));
    }


    /**
     * 通过id查询总分表关系
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("id" ) String id) {
      return ResultRespone.success(yhZfbService.getById(id));
    }

    /**
     * 新增总分表关系
     * @param yhZfb 总分表关系
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增总分表关系" )
    @ApiOperation(value = "新增总分表关系")
    @PostMapping
    public ResultRespone save(@RequestBody YhZfb yhZfb) {
      return ResultRespone.success(yhZfbService.save(yhZfb));
    }

    /**
     * 修改总分表关系
     * @param yhZfb 总分表关系
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改总分表关系" )
    @ApiOperation(value = "修改总分表关系")
    @PutMapping
    public ResultRespone updateById(@RequestBody YhZfb yhZfb) {
      return ResultRespone.success(yhZfbService.updateById(yhZfb));
    }

    /**
     * 通过id删除总分表关系
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除总分表关系" )
    @ApiOperation(value = "通过ID删除总分表关系")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable String id) {
      return ResultRespone.success(yhZfbService.removeById(id));
    }

    /**
     * 通过id批量删除总分表关系
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除总分表关系" )
    @ApiOperation(value = "通过ID批量删除总分表关系")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(yhZfbService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
