/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.feesapi.bjgl.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.feesapi.bjgl.entity.Yhbgl;
import com.tofly.feesapi.bjgl.service.YhbglService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 校表管理
 *
 * @author sxb
 * @date Wed Aug 09 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("api/bjgl/yhbgl")
@Api(tags="水表档案-校表管理接口")
public class YhbglController {

    private final  YhbglService yhbglService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param yhbgl 校表管理
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getYhbglPage(Page page, @ModelAttribute Yhbgl yhbgl) {
      return ResultRespone.success(yhbglService.page(page, Wrappers.query(yhbgl)));
    }


    /**
     * 通过id查询校表管理
     * @param lsbh id
     * @return ResultRespone
     */
    @GetMapping("/{lsbh}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "lsbh", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("lsbh" ) String lsbh) {
      return ResultRespone.success(yhbglService.getById(lsbh));
    }

    /**
     * 新增校表管理
     * @param yhbgl 校表管理
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增校表管理" )
    @ApiOperation(value = "新增校表管理")
    @PostMapping
    public ResultRespone save(@RequestBody Yhbgl yhbgl) {
      return ResultRespone.success(yhbglService.save(yhbgl));
    }

    /**
     * 修改校表管理
     * @param yhbgl 校表管理
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改校表管理" )
    @ApiOperation(value = "修改校表管理")
    @PutMapping
    public ResultRespone updateById(@RequestBody Yhbgl yhbgl) {
      return ResultRespone.success(yhbglService.updateById(yhbgl));
    }

    /**
     * 通过id删除校表管理
     * @param lsbh id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除校表管理" )
    @ApiOperation(value = "通过ID删除校表管理")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "lsbh", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{lsbh}" )
    public ResultRespone removeById(@PathVariable String lsbh) {
      return ResultRespone.success(yhbglService.removeById(lsbh));
    }

    /**
     * 通过id批量删除校表管理
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除校表管理" )
    @ApiOperation(value = "通过ID批量删除校表管理")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(yhbglService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
