package com.tofly.feesapi.bmgl.controller;

import com.tofly.common.core.entity.ResultRespone;
import com.tofly.feesapi.bmgl.controller.common.bmscyz;
import com.tofly.feesapi.bmgl.entity.BmYslx;
import com.tofly.feesapi.bmgl.entity.dto.BmYslxFilter;
import com.tofly.feesapi.bmgl.entity.vo.BmYslxSelect;
import com.tofly.feesapi.bmgl.mapper.BmYslxMapper;
import com.tofly.feesapi.bmgl.service.BmYslxService;
import com.tofly.feesapi.common.entity.dto.PageDto;
import com.tofly.feesapi.common.entity.vo.PageResult;
import com.tofly.feesapi.common.model.CollectionData;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 用水类型
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/bmgl/bmyslx")
@Api(tags = "用水类型")
public class BmYslxController {

    private final BmYslxService bmYslxService;
    private final BmYslxMapper bmYslxMapper;
    private final bmscyz bmscyz;

    @GetMapping("/list")
    @ApiOperation(value = "收费渠道列表list")
    public ResultRespone getyslxList() {
        List<Map<String, Object>> list = bmYslxMapper.selectListMap();
        return ResultRespone.success(new CollectionData(list, list.size()));
    }

    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone<PageResult<BmYslxSelect>> getPage(PageDto pageDto, @ModelAttribute BmYslxFilter filter) {
        return ResultRespone.success(bmYslxService.getPage(pageDto, filter));
    }


    /**
     * 通过id查询用水类型
     */
    @GetMapping("/{code}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "编码", required = true, dataType = "BigDecimal")
    })
    public ResultRespone<BmYslx> getById(@PathVariable("code") String code) {
        return ResultRespone.success(bmYslxService.getById(code));
    }

    /**
     * 新增用水类型
     *
     * @param bmYslx 用水类型
     * @return ResultRespone
     */
    @ApiOperation(value = "新增用水类型")
    @PostMapping
    public ResultRespone<Boolean> save(@RequestBody BmYslx bmYslx) {
        BigDecimal num1 = new BigDecimal(1);
        bmYslx.setCode(bmYslxMapper.selectMaxId().add(num1).toString());
        return ResultRespone.success(bmYslxService.save(bmYslx));
    }

    /**
     * 修改用水类型
     *
     * @param bmYslx 用水类型
     * @return ResultRespone
     */
    @ApiOperation(value = "修改用水类型")
    @PutMapping
    public ResultRespone<Boolean> updateById(@RequestBody BmYslx bmYslx) {
        return ResultRespone.success(bmYslxService.updateById(bmYslx));
    }

    /**
     * 通过id删除用水类型
     *
     * @param code id
     * @return ResultRespone
     */
    @ApiOperation(value = "通过ID删除用水类型")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "ID", required = true, dataType = "BigDecimal")
    })
    @DeleteMapping("/{code}")
    public ResultRespone<Boolean> removeById(@PathVariable String code) {
        if (bmscyz.IsczSJsy("yslx", code)) {
            return ResultRespone.failed("该用水类型已使用，不可删除");
        }
        return ResultRespone.success(bmYslxService.removeById(code));
    }

    /**
     * 通过id批量删除用水类型
     *
     * @param ids id
     * @return ResultRespone
     */
    @ApiOperation(value = "通过ID批量删除用水类型")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone<Boolean> removeByIds(String ids) {
        if (bmscyz.IsczSJsy("yslx", ids)) {
            return ResultRespone.failed("该用水类型已使用，不可删除");
        }
        return ResultRespone.success(bmYslxService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
