package com.tofly.feesapi.cbgl.controller;

import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.feesapi.cbgl.entity.dto.YyCblrFilter;
import com.tofly.feesapi.cbgl.entity.dto.YyCblrUpdate;
import com.tofly.feesapi.cbgl.entity.dto.YyCbycmxFilter;
import com.tofly.feesapi.cbgl.entity.vo.*;
import com.tofly.feesapi.cbgl.service.YyCblrService;
import com.tofly.feesapi.common.Excel.ExcleFileUtil;
import com.tofly.feesapi.common.entity.dto.PageDto;
import com.tofly.feesapi.common.entity.vo.PageResult;
import com.tofly.feesapi.common.model.CollectionData;
import com.tofly.feesapi.security.model.CzyUserDetails;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;
import java.util.Properties;

/**
 * 抄表录入
 */
@RestController
@AllArgsConstructor
@RequestMapping("api/cbgl/yycblr")
@Api(tags = "抄表录入")
public class YyCblrController {

    private final YyCblrService yyCblrService;

    @ApiOperationSupport(order = 1)
    @GetMapping("/page")
    @ApiOperation(value = "查询抄表录入（分页）", tags = {"抄表计划", "抄表照片管理", "抄表算费"})
    public ResultRespone<PageResult<YyCblrSelect>> getPage(@ModelAttribute PageDto pageDto, @ModelAttribute YyCblrFilter filter) {
        return ResultRespone.success(yyCblrService.getPage(pageDto, filter));
    }

    @ApiOperationSupport(order = 2)
    @GetMapping("/{yhbh}_{cbkh}")
    @ApiOperation(value = "查看抄表录入", tags = {"抄表照片管理"})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "yhbh", value = "用户编号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "cbkh", value = "抄表卡号", required = true, dataType = "String")
    })
    public ResultRespone<YyCblrView> getById(@PathVariable("yhbh") String yhbh, @PathVariable("cbkh") String cbkh) {
        return ResultRespone.success(yyCblrService.getViewById(yhbh, cbkh));
    }

    @ApiOperationSupport(order = 3)
    @ApiOperation(value = "生成抄表录入记录")
    @PostMapping
    public ResultRespone<Boolean> sccblr() {
        yyCblrService.sccblr();
        return ResultRespone.success(true);
    }

    @ApiOperationSupport(order = 4)
    @ApiOperation(value = "修改抄表录入(抄表)")
    @PutMapping
    public ResultRespone<Boolean> update(@RequestBody YyCblrUpdate update,
                                         @ApiIgnore @AuthenticationPrincipal CzyUserDetails czyUserDetails) {
        return ResultRespone.success(yyCblrService.update(update, czyUserDetails));
    }

    @ApiOperationSupport(order = 5)
    @GetMapping("/page2")
    @ApiOperation(value = "查询抄表录入（含历史记录）（分页）")
    public ResultRespone<PageResult<YyCblrSelect>> getPage2(@ModelAttribute PageDto pageDto, @ModelAttribute YyCblrFilter filter) {
        return ResultRespone.success(yyCblrService.getPage2(pageDto, filter));
    }

    @ApiOperationSupport(order = 15)
    @GetMapping("/export")
    @ApiOperation(value = "导出抄表录入(抄表计划)", tags = "抄表计划")
    public void export(HttpServletResponse response, @ModelAttribute YyCblrFilter filter) throws IOException {
        List<YyCblrSelect> list = yyCblrService.getList(filter);
        if (!list.isEmpty()) {
            ExcleFileUtil.exportExcel(response, list, YyCblrSelect.class, "抄表计划.xlsx");
        }
    }

    @ApiOperationSupport(order = 16)
    @GetMapping("/cbsltj")
    @ApiOperation(value = "抄表水量统计", tags = {"抄表水量统计"})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ssgs", value = "所属公司编码", required = true, dataType = "String", example = "1"),
            @ApiImplicitParam(name = "cwrq", value = "财务日期", required = true, dataType = "String", example = "202401"),
            @ApiImplicitParam(name = "tjlb", value = "统计类别,按抄表人员统计:cbry,按用水性质统计:sjbm", required = true, dataType = "String", allowableValues = "cbry,sjbm")
    })
    public ResultRespone<CollectionData<CbSltjSelect>> cbsltj(@RequestParam("ssgs") String ssgs,
                                                    @RequestParam("cwrq") String cwrq,
                                                    @RequestParam("tjlb") String tjlb) {
        List<CbSltjSelect> list = yyCblrService.cbsltj(ssgs, cwrq, tjlb);
        return ResultRespone.success(new CollectionData<>(list, list.size()));
    }

    @ApiOperationSupport(order = 17)
    @GetMapping("/cbsltj/export")
    @ApiOperation(value = "导出抄表水量统计", tags = {"抄表水量统计"})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ssgs", value = "所属公司编码", required = true, dataType = "String", example = "1"),
            @ApiImplicitParam(name = "cwrq", value = "财务日期", required = true, dataType = "String", example = "202401"),
            @ApiImplicitParam(name = "tjlb", value = "统计类别,按抄表人员统计:cbry,按用水性质统计:sjbm", required = true, dataType = "String", allowableValues = "cbry,sjbm")
    })
    public void cbsltjExport(HttpServletResponse response,
                             @RequestParam("ssgs") String ssgs,
                             @RequestParam("cwrq") String cwrq,
                             @RequestParam("tjlb") String tjlb) throws IOException {
        List<CbSltjSelect> list = yyCblrService.cbsltj(ssgs, cwrq, tjlb);
        Properties properties = new Properties();
        if (tjlb.equals("cbry")) {
            properties.put("tjlb", "抄表人员");
        } else if (tjlb.equals("sjbm")) {
            properties.put("tjlb", "用水性质");
        }
        if (!list.isEmpty()) {
            ExcleFileUtil.exportExcel(response, list, CbSltjSelect.class, "抄表水量统计.xlsx", properties);
        }
    }

    @ApiOperationSupport(order = 18)
    @GetMapping("/cbltj")
    @ApiOperation(value = "抄表率统计", tags = {"抄表率统计"})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ssgs", value = "所属公司编码", required = true, dataType = "String", example = "1"),
            @ApiImplicitParam(name = "cwrq", value = "财务日期", required = true, dataType = "String", example = "202401"),
            @ApiImplicitParam(name = "tjlb", value = "统计类别,按抄表人员统计:cbry,按用水性质统计:sjbm", required = true, dataType = "String", allowableValues = "cbry,sscbb")
    })
    public ResultRespone<CollectionData<CbltjSelect>> cbltj(@RequestParam("ssgs") String ssgs,
                                                                 @RequestParam("cwrq") String cwrq,
                                                                 @RequestParam("tjlb") String tjlb) {
        List<CbltjSelect> cbltj = yyCblrService.cbltj(ssgs, cwrq, tjlb);
        return ResultRespone.success(new CollectionData<>(cbltj, cbltj.size()));
    }

    @ApiOperationSupport(order = 19)
    @GetMapping("/cbltj/export")
    @ApiOperation(value = "导出抄表率统计", tags = {"抄表率统计"})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ssgs", value = "所属公司编码", required = true, dataType = "String", example = "1"),
            @ApiImplicitParam(name = "cwrq", value = "财务日期", required = true, dataType = "String", example = "202401"),
            @ApiImplicitParam(name = "tjlb", value = "统计类别,按抄表人员统计:cbry,按抄表本(册)统计:cbb", required = true, dataType = "String", allowableValues = "cbry,sscbb")
    })
    public void cbltjExport(HttpServletResponse response,
                            @RequestParam("ssgs") String ssgs,
                            @RequestParam("cwrq") String cwrq,
                            @RequestParam("tjlb") String tjlb) throws IOException {
        List<CbltjSelect> list = yyCblrService.cbltj(ssgs, cwrq, tjlb);
        Properties properties = new Properties();
        if (tjlb.equals("cbry")) {
            properties.put("tjlb", "抄表人员");
        } else if (tjlb.equals("sscbb")) {
            properties.put("tjlb", "抄表册");
        }
        if (!list.isEmpty()) {
            ExcleFileUtil.exportExcel(response, list, CbltjSelect.class, "抄表率统计.xlsx", properties);
        }
    }

    @ApiOperationSupport(order = 20)
    @GetMapping("/cbycmx")
    @ApiOperation(value = "查询抄表异常明细", tags = {"抄表异常明细"})
    public ResultRespone<PageResult<YyCbycmxSelect>> cbycmx(@ModelAttribute PageDto pageDto, @ModelAttribute YyCbycmxFilter filter) {
        return ResultRespone.success(yyCblrService.cbycmx(pageDto, filter));
    }

    @ApiOperationSupport(order = 21)
    @GetMapping("/cbycmx/export")
    @ApiOperation(value = "导出抄表异常明细", tags = {"抄表异常明细"})
    public void cbycmxExport(HttpServletResponse response, @ModelAttribute YyCbycmxFilter filter) throws IOException {
        List<YyCbycmxSelect> list = yyCblrService.cbycmx(filter);
        if (!list.isEmpty()) {
            ExcleFileUtil.exportExcel(response, list, YyCbycmxSelect.class, "抄表异常明细.xlsx");
        }
    }

}
