package com.tofly.feesapi.common.Excel;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.metadata.Head;
import com.alibaba.excel.write.builder.ExcelWriterSheetBuilder;
import com.alibaba.excel.write.handler.CellWriteHandler;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.holder.WriteSheetHolder;
import com.alibaba.excel.write.metadata.holder.WriteTableHolder;
import org.apache.poi.ss.usermodel.Row;
import org.springframework.util.PropertyPlaceholderHelper;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.*;

public class ExcleFileUtil {
    public static InputStream getResourcesFileInputStream(String fileName) {
        return Thread.currentThread().getContextClassLoader().getResourceAsStream("" + fileName);
    }


    public static String getPath() {
        return ExcleFileUtil.class.getResource("/").getPath();
    }


    public static File createNewFile(String pathName) {
        File file = new File(getPath() + pathName);
        if (file.exists()) {
            file.delete();
        } else {
            if (!file.getParentFile().exists()) {
                file.getParentFile().mkdirs();
            }
        }
        return file;
    }

    public static File readFile(String pathName) {
        return new File(getPath() + pathName);
    }

    public static File readUserHomeFile(String pathName) {
        return new File(System.getProperty("user.home") + File.separator + pathName);
    }


    /**
     * Excel  导出公共方法 List<HashMap<String, Object>> dataList
     *
     * @param dataList
     * @param filePath
     * @param sheetName
     */
    public static void exportExcel(List<HashMap<String, Object>> dataList, String filePath, String sheetName) {
        ExcelWriter excelWriter = null;
        try {
            excelWriter = EasyExcel.write(filePath).build();
            WriteSheet writeSheet = EasyExcel.writerSheet(sheetName).build();
            excelWriter.write(dataList, writeSheet);
        } finally {
            if (excelWriter != null) {
                excelWriter.finish();
            }
        }
    }


    /**
     * Excel  导出公共方法
     *
     * @param response
     * @param dataList
     * @param clazz
     * @param sheetName
     * @param fileName
     * @throws IOException
     */
    public static void exportExcel(HttpServletResponse response, List<Object> dataList, Class<?> clazz, String sheetName, String fileName) throws IOException {
        // 设置响应头
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
        // 使用 Easy Excel 写入数据到 Excel 文件
        ExcelWriter writer = EasyExcel.write(response.getOutputStream(), clazz).build();
        WriteSheet sheet = EasyExcel.writerSheet(sheetName).build();
        writer.write(dataList, sheet);
        writer.finish();
    }

    /**
     * exportExcel 导出数据 list
     *
     * @param response
     * @param list
     * @param fileName
     * @param sheetName
     * @param clazz
     * @throws IOException
     */
    public static void exportExcel(HttpServletResponse response, List<?> list, String fileName, String sheetName, Class<?> clazz) throws IOException {
        // 创建 ExcelWriter 对象
        // 根据用户传入字段 假设我们要忽略 date
        ExcelWriter writer = EasyExcel.write(response.getOutputStream(), clazz).build();
        WriteSheet sheet = EasyExcel.writerSheet(sheetName).build();    // 写入数据
        writer.write(list, sheet);
        // 设置文件名称和响应头
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        response.setHeader("Content-disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
        //关闭流
        writer.finish();
    }

    /**
     * exportExcel 导出数据 list 可排除不展示数据
     *
     * @param response
     * @param list
     * @param fileName
     * @param sheetName
     * @param clazz
     * @param excludeColumnFieldNames
     * @throws IOException
     */
    public static void exportExcel(HttpServletResponse response, List<?> list, String fileName, String sheetName, Class<?> clazz, Set<String> excludeColumnFieldNames) throws IOException {
        // 创建 ExcelWriter 对象
        // 根据用户传入字段 假设我们要忽略 date
        ExcelWriter writer = EasyExcel.write(response.getOutputStream(), clazz).excludeColumnFieldNames(excludeColumnFieldNames).build();
        WriteSheet sheet = EasyExcel.writerSheet(sheetName).build();    // 写入数据
        writer.write(list, sheet);
        // 设置文件名称和响应头
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        response.setHeader("Content-disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
        //关闭流
        writer.finish();
    }


    /**
     * 根据导出数据分多个sheet
     *
     * @param response
     * @param dataList
     * @param clazz
     * @param sheetSize
     * @param fileName
     * @throws IOException
     */
    public static void exportExcel(HttpServletResponse response, List<?> dataList, Class<?> clazz, int sheetSize, String fileName) throws IOException {
        // 创建 ExcelWriter 对象
        ExcelWriter writer = EasyExcel.write(response.getOutputStream(), clazz).build();
        // 数据总行数
        int dataSize = dataList.size();
        // 计算 sheet 数量
        int sheetCount = dataSize / sheetSize + (dataSize % sheetSize == 0 ? 0 : 1);
        // 分批写入数据
        for (int sheetNo = 1; sheetNo <= sheetCount; sheetNo++) {
            // 创建当前 sheet 对应的 WriteSheet 对象
            WriteSheet sheet = EasyExcel.writerSheet(sheetNo, "Sheet" + sheetNo).build();
            // 计算当前 sheet 中的数据范围
            int fromIndex = (sheetNo - 1) * sheetSize;
            int toIndex = Math.min(sheetNo * sheetSize, dataSize);
            // 写入数据
            List<?> sheetDataList = dataList.subList(fromIndex, toIndex);
            writer.write(sheetDataList, sheet);
        }

        // 设置文件名称和响应头
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        response.setHeader("Content-disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
        // 关闭 ExcelWriter 对象
        writer.finish();
    }

    /**
     * @param response
     * @param dataList
     * @param clazz
     * @param fileName
     * @throws IOException
     */
    public static void exportExcel(HttpServletResponse response, List<?> dataList, Class<?> clazz, String fileName) throws IOException {
        exportExcel(response, dataList, clazz, fileName, null);
    }

    /**
     * @param response
     * @param dataList
     * @param clazz
     * @param fileName
     * @param properties
     * @throws IOException
     */
    public static void exportExcel(HttpServletResponse response, List<?> dataList, Class<?> clazz, String fileName, Properties properties) throws IOException {
        // 创建 ExcelWriter 对象
        ExcelWriter writer = EasyExcel.write(response.getOutputStream(), clazz).build();
        // 每个 sheet 的行数
        int sheetSize = 65000;
        // 数据总行数
        int dataSize = dataList.size();
        // 计算 sheet 数量
        int sheetCount = dataSize / sheetSize + (dataSize % sheetSize == 0 ? 0 : 1);
        CellWriteHandler cellWriteHandler = properties == null ? null : new CellWriteHandler() {
            private final PropertyPlaceholderHelper placeholderHelper = new PropertyPlaceholderHelper("${", "}");

            @Override
            public void beforeCellCreate(WriteSheetHolder writeSheetHolder, WriteTableHolder writeTableHolder, Row row, Head head, Integer columnIndex, Integer relativeRowIndex, Boolean isHead) {
                if (isHead) {
                    List<String> headNameList = head.getHeadNameList();
                    if (headNameList != null) {
                        headNameList.forEach(e -> {
                            int i = headNameList.indexOf(e);
                            String newHead = placeholderHelper.replacePlaceholders(e, properties);
                            headNameList.set(i, newHead);
                        });
                    }
                }
            }
        };

        // 分批写入数据
        for (int sheetNo = 1; sheetNo <= sheetCount; sheetNo++) {
            ExcelWriterSheetBuilder excelWriterSheetBuilder = EasyExcel.writerSheet(sheetNo, "Sheet" + sheetNo);
            if (properties != null)
                excelWriterSheetBuilder.registerWriteHandler(cellWriteHandler);

            // 创建当前 sheet 对应的 WriteSheet 对象
            WriteSheet sheet = excelWriterSheetBuilder.build();
            // 计算当前 sheet 中的数据范围
            int fromIndex = (sheetNo - 1) * sheetSize;
            int toIndex = Math.min(sheetNo * sheetSize, dataSize);
            // 写入数据
            List<?> sheetDataList = dataList.subList(fromIndex, toIndex);
            writer.write(sheetDataList, sheet);
        }

        // 设置文件名称和响应头
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        response.setHeader("Content-disposition", "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
        // 关闭 ExcelWriter 对象
        writer.finish();
    }

    /**
     * 去读 excle文件数据，返回list,根据实体类
     *
     * @param filePath
     * @param clazz
     * @param <T>
     * @return
     */
    public static <T> List<T> readExcel(String filePath, Class<T> clazz) {
        String fullPath = ExcleFileUtil.getPath() + filePath;
        ;
        List<T> list = EasyExcel.read(fullPath).head(clazz).sheet().doReadSync();
        return list;
    }


    public static List<Map<Integer, String>> readExcel(String filePath) {
        return EasyExcel.read(filePath).sheet().doReadSync();
    }
}
