package com.tofly.feesapi.common.dbhelper;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.annotation.TableName;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.feesapi.common.enums.LogOperateEnum;
import com.tofly.feesapi.common.enums.SysEnum;
import com.tofly.feesapi.common.model.CollectionData;
import com.tofly.feesapi.common.util.DateUtils;
import io.swagger.annotations.ApiModelProperty;
import org.apache.commons.lang.StringUtils;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * 公共日志处理类
 *
 * @author shudong
 */
public class LogHelper {

    //region 对外提供的方法

    //region 以SQL语句返回的方法

    /**
     * 获取修改类型的日志SQL语句【针对传入的2个对象比较，返回的SQL可能为空使用前进行判断】
     *
     * @param tOld     原对象，不能为null
     * @param tNew     新对象，不能为null
     * @param strCause 变更原因，不能为空否则直接返回null
     * @param <T>      实体类
     * @return SQL语句
     */
    public static <T> String getModifySQL(T tOld, T tNew, String strCause) {
        String strResult = "";
        try {
            if (tOld == null || tNew == null || StringUtils.isBlank(strCause)) {
                return null;
            }
            StringBuilder sbResult = new StringBuilder();
            JSONArray jsonArray = new JSONArray();
            Field[] fields = tOld.getClass().getDeclaredFields();
            Field[] newFields = tNew.getClass().getDeclaredFields();
            for (Field oldField : fields) {
                //设置允许通过反射访问私有变量
                oldField.setAccessible(true);
                //获取字段的值
                Object oldValue = oldField.get(tOld);
                if (oldValue != null && oldField.getName().equals("serialVersionUID")) {
                    continue;
                }
                for (Field newField : newFields) {
                    //设置允许通过反射访问私有变量
                    newField.setAccessible(true);
                    //判断属性名一样时，字段值是否相同
                    if (oldField.getName().equals(newField.getName())) {
                        Object newValue = newField.get(tNew);
                        if (newValue != null && !oldValue.equals(newValue)) {
                            //不相同证明有更改
                            setSaveInfo(tNew, oldValue, sbResult, jsonArray, newField, LogOperateEnum.Modify);
                        }
                        break;
                    }
                }
            }

            //修改详情
            String strNotes = sbResult.toString();
            //判断对象是否有修改，如果不为空则返回存储的SQL
            if (!StringUtils.isBlank(strNotes)) {
                //去掉最后的分号
                strNotes = strNotes.substring(0, strNotes.length() - 1);
                //数据库表名
                String strDbTableName = ((TableName) getTableNameAnnotation(tOld)).value();
                //返回存储的日志SQL
                strResult = saveLogSQL(strCause, strNotes, jsonArray.toJSONString(), LogOperateEnum.Modify, strDbTableName);
            }
        } catch (Exception ex) {
            strResult = "";
        }
        return strResult;
    }

    /**
     * 获取修改类型的日志SQL语句【针对单个对象加where条件入的方式，返回的SQL可能为空使用前进行判断】
     *
     * @param tNew     修改的实体对象
     * @param strWhere 查询当前修改的对象的唯一sql条件；条件必须符合查询出来的数据是唯一，否则直接返回null
     * @param strCause 变更原因，不能为空否则直接返回null
     * @param <T>      实体类
     * @return SQL语句
     */
    public static <T> String getModifySQL(T tNew, String strWhere, String strCause) {
        String strResult = "";
        try {
            if (tNew == null || StringUtils.isBlank(strWhere) || StringUtils.isBlank(strCause)) {
                return null;
            }
            //数据库表名
            String strDbTableName = ((TableName) getTableNameAnnotation(tNew)).value();
            //获取当前对象的数据库存储数据
            HashMap hashMap = getModel(strDbTableName, strWhere);
            if (hashMap == null) {
                return null;
            }
            StringBuilder sbResult = new StringBuilder();
            JSONArray jsonArray = new JSONArray();
            Field[] newFields = tNew.getClass().getDeclaredFields();
            for (Field newField : newFields) {
                //设置允许通过反射访问私有变量
                newField.setAccessible(true);

                //判断值是否有修改
                Object newValue = newField.get(tNew);
                if (newValue != null && newField.getName().equals("serialVersionUID")) {
                    continue;
                }
                if (newValue != null) {
                    Object oldValue = hashMap.get(newField.getName());
                    setSaveInfo(tNew, oldValue, sbResult, jsonArray, newField, LogOperateEnum.Modify);
                }
            }

            //修改详情
            String strNotes = sbResult.toString();
            //判断对象是否有修改，如果不为空则返回存储的SQL
            if (!StringUtils.isBlank(strNotes)) {
                //去掉最后的分号
                strNotes = strNotes.substring(0, strNotes.length() - 1);

                //返回存储的日志SQL
                strResult = saveLogSQL(strCause, strNotes, jsonArray.toJSONString(), LogOperateEnum.Modify, strDbTableName);
            }
        } catch (Exception ex) {
            strResult = "";
        }
        return strResult;
    }

    /**
     * 获取添加类型的日志SQL语句【返回的SQL可能为空使用前进行判断】
     *
     * @param tNew     添加的实体对象
     * @param strCause 添加原因，不能为空否则直接返回null
     * @param <T>      实体类
     * @return SQL语句
     */
    public static <T> String getAddSQL(T tNew, String strCause) {
        return getAddOrDeleteSQL(tNew, strCause, LogOperateEnum.Add);
    }

    /**
     * 获取删除类型的日志SQL语句【返回的SQL可能为空使用前进行判断】
     *
     * @param tNew     删除的实体对象
     * @param strCause 删除原因，不能为空否则直接返回null
     * @param <T>      实体类
     * @return SQL语句
     */
    public static <T> String getDeleteSQL(T tNew, String strCause) {
        return getAddOrDeleteSQL(tNew, strCause, LogOperateEnum.Delete);
    }

    /**
     * 获取自定义日志的SQL语句
     *
     * @param t        实体类名，使用【类名.class】调用，如：BwMeterReadPlan.class
     * @param strCause 处理原因，不能为空否则直接返回null
     * @param strNotes 日志描述内容
     * @return SQL语句
     */
    public static String getCustomSQL(Class t, String strCause, String strNotes) {
        //数据库表名
        String strDbTableName = ((TableName) getTableNameAnnotation(t)).value();

        //详情添加到json对象中
        JSONArray jsonArray = new JSONArray();
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("text", "备注信息");
        jsonObject.put("value", strNotes);
        jsonArray.add(jsonObject);

        //返回存储的日志SQL
        return saveLogSQL(strCause, strNotes, jsonArray.toJSONString(), LogOperateEnum.Custom, strDbTableName);
    }

    //endregion

    //region 直接调用的方法

    /**
     * 执行修改记录日志的方法
     *
     * @param tNew     修改的实体对象
     * @param strWhere 查询当前修改的对象的唯一sql条件；条件必须符合查询出来的数据是唯一，否则直接返回null
     * @param strCause 变更原因，不能为空否则直接返回null
     * @param <T>      实体类
     * @return true或false true=成功，false=失败
     * @throws Exception
     */
    public static <T> boolean excuteModify(T tNew, String strWhere, String strCause) throws Exception {
        String strLog = getModifySQL(tNew, strWhere, strCause);
        if (!StringUtils.isBlank(strLog)) {
            return DbHelper.getDbHelper().executeCmd(strLog) > 0;
        } else {
            return false;
        }
    }

    /**
     * 执行修改记录日志的方法【针对传入的2个对象比较】
     *
     * @param tOld     原对象，不能为null
     * @param tNew     新对象，不能为null
     * @param strCause 变更原因，不能为空否则执行无效
     * @param <T>      实体类
     * @return true或false true=成功，false=失败
     * @throws Exception
     */
    public static <T> boolean excuteModify(T tOld, T tNew, String strCause) throws Exception {
        String strLog = getModifySQL(tOld, tNew, strCause);
        if (!StringUtils.isBlank(strLog)) {
            return DbHelper.getDbHelper().executeCmd(strLog) > 0;
        } else {
            return false;
        }
    }

    /**
     * 执行添加记录日志的方法
     *
     * @param tNew     添加的实体对象
     * @param strCause 添加原因，不能为空否则执行无效
     * @param <T>      实体类
     * @return true或false true=成功，false=失败
     * @throws Exception
     */

    public static <T> boolean excuteAdd(T tNew, String strCause) throws Exception {
        String strLog = getAddOrDeleteSQL(tNew, strCause, LogOperateEnum.Add);
        if (!StringUtils.isBlank(strLog)) {
            return DbHelper.getDbHelper().executeCmd(strLog) > 0;
        } else {
            return false;
        }
    }

    /**
     * 执行删除记录日志的方法
     *
     * @param tNew     删除的实体对象
     * @param strCause 删除原因，不能为空否则直接返回null
     * @param <T>      实体类
     * @return true或false true=成功，false=失败
     * @throws Exception
     */
    public static <T> boolean excuteDelete(T tNew, String strCause) throws Exception {
        String strLog = getAddOrDeleteSQL(tNew, strCause, LogOperateEnum.Delete);
        if (!StringUtils.isBlank(strLog)) {
            return DbHelper.getDbHelper().executeCmd(strLog) > 0;
        } else {
            return false;
        }
    }

    /**
     * 执行自定义记录日志的方法
     *
     * @param t        实体类名，使用【类名.class】调用，如：BwMeterReadPlan.class
     * @param strCause 日志原因
     * @param strNotes 日志描述
     * @return true或false true=成功，false=失败
     * @throws Exception
     */
    public static boolean excuteCustom(Class t, String strCause, String strNotes) throws Exception {
        String strLog = getCustomSQL(t, strCause, strNotes);
        if (!StringUtils.isBlank(strLog)) {
            return DbHelper.getDbHelper().executeCmd(strLog) > 0;
        } else {
            return false;
        }
    }

    //endregion

    //endregion

    //region 内部私有方法

    /**
     * 获取添加或删除类型的日志SQL语句【返回的SQL可能为空使用前进行判断】
     *
     * @param tNew           添加的实体对象
     * @param strCause       备注原因，不能为空否则直接返回null
     * @param logOperateEnum 操作类型枚举
     * @param <T>            实体类
     * @return SQL语句
     */
    private static <T> String getAddOrDeleteSQL(T tNew, String strCause, LogOperateEnum logOperateEnum) {
        String strResult = "";
        try {
            if (tNew == null || StringUtils.isBlank(strCause)) {
                return null;
            }
            StringBuilder sbResult = new StringBuilder();
            JSONArray jsonArray = new JSONArray();
            Field[] newFields = tNew.getClass().getDeclaredFields();
            for (Field newField : newFields) {
                //设置允许通过反射访问私有变量
                newField.setAccessible(true);

                //判断值是否有修改
                Object newValue = newField.get(tNew);
                if (newValue != null && newField.getName().equals("serialVersionUID")) {
                    continue;
                }
                if (newValue != null) {
                    setSaveInfo(tNew, "", sbResult, jsonArray, newField, logOperateEnum);
                }
            }

            //添加详情
            String strNotes = sbResult.toString();
            //判断对象是否添加，如果不为空则返回存储的SQL
            if (!StringUtils.isBlank(strNotes)) {
                //去掉最后的分号
                strNotes = strNotes.substring(0, strNotes.length() - 1);
                //数据库表名
                String strDbTableName = ((TableName) getTableNameAnnotation(tNew)).value();
                //返回存储的日志SQL
                strResult = saveLogSQL(strCause, strNotes, jsonArray.toJSONString(), logOperateEnum, strDbTableName);
            }
        } catch (Exception ex) {
            strResult = "";
        }
        return strResult;
    }

    /**
     * 设置修改信息
     *
     * @param tNew      实体对象
     * @param oldValue  修改前的值
     * @param sbResult  存储的StringBuilder对象
     * @param jsonArray 存储的JsonArray对象
     * @param newField  实体类Field对象
     * @param <T>       泛型实体
     * @throws IllegalAccessException
     */
    private static <T> void setSaveInfo(T tNew, Object oldValue, StringBuilder sbResult, JSONArray jsonArray, Field newField, LogOperateEnum logOperateEnum) throws IllegalAccessException {
        Object newValue = newField.get(tNew);
        String fieldText = getApiModelPropertyValue(tNew, newField.getName());
        String fieldValue = newField.getName();
        if (oldValue == null) {
            oldValue = "";
        } else {
            if (oldValue instanceof Date) {
                oldValue = DateUtils.formatDate((Date) oldValue);
            }
        }
        if (newValue == null) {
            newValue = "";
        } else {
            if (newValue instanceof Date) {
                newValue = DateUtils.formatDate((Date) newValue);
            }
        }

        switch (logOperateEnum) {
            case Modify:
                sbResult.append(MessageFormat.format("「{0}（{1}）」由{2}修改为{3}；", fieldText, fieldValue, oldValue, newValue));
                break;
            case Add:
                sbResult.append(MessageFormat.format("「{0}（{1}）」添加为{2}；", fieldText, fieldValue, newValue));
                break;
            case Delete:
                sbResult.append(MessageFormat.format("「{0}（{1}）」删除前的值为{2}；", fieldText, fieldValue, oldValue));
                break;
        }

        //修改详情添加到json对象中
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("fieldText", fieldText);
        jsonObject.put("fieldValue", fieldValue);
        jsonObject.put("oldValue", oldValue);
        jsonObject.put("newValue", newValue);
        jsonArray.add(jsonObject);
    }

    /**
     * 获取指定泛型实体类的ApiModelProperty值
     *
     * @param t         泛型实体类
     * @param fieldName 字段名
     * @param <T>       泛型
     * @return 实体类的ApiModelProperty值
     */
    private static <T> String getApiModelPropertyValue(T t, String fieldName) {
        String strValue = "";
        Field[] fields = t.getClass().getDeclaredFields();
        for (Field field : fields) {
            for (Annotation annotation : field.getAnnotations()) {
                if (annotation instanceof ApiModelProperty) {
                    if (field.getName().equals(fieldName)) {
                        strValue = ((ApiModelProperty) annotation).value();
                        break;
                    }
                }
            }
        }
        return strValue;
    }

    /**
     * 获取泛型实体类对应的TableName注解对象
     *
     * @param t   泛型实体类
     * @param <T> 泛型
     * @return 实体类对应的TableName注解对象
     */
    private static <T> Annotation getTableNameAnnotation(T t) {
        Annotation[] annotations = t.getClass().getDeclaredAnnotations();
        for (Annotation annotation : annotations) {
            if (annotation instanceof TableName) {
                return annotation;
            }
        }
        return null;
    }

    /**
     * 获取泛型实体类对应的TableName注解对象
     *
     * @param t 泛型实体类
     * @return 实体类对应的TableName注解对象
     */
    private static Annotation getTableNameAnnotation(Class t) {
        Annotation[] annotations = t.getDeclaredAnnotations();
        for (Annotation annotation : annotations) {
            if (annotation instanceof TableName) {
                return annotation;
            }
        }
        return null;
    }

    /**
     * 保存日志SQL
     *
     * @param strCause       变更原因
     * @param strNotes       修改详情
     * @param strJsonNotes   修改详情的json字符
     * @param logOperateEnum 枚举修改类型
     * @param strTableName   数据库表名
     * @return 日志存储SQL
     */
    private static String saveLogSQL(String strCause, String strNotes, String strJsonNotes, LogOperateEnum logOperateEnum, String strTableName) {
        String userName = "test";//SecurityUtils.getUserName();
        String realName = "测试";//SecurityUtils.getUser().getRealName();

        StringBuilder sbSQL = new StringBuilder();
        sbSQL.append("insert into tf_ywys_sys_commlog(id,btype,btable,changecause,notes,json_notes,optype,usercode,username,optime)");
        sbSQL.append(MessageFormat.format("select sys_guid(),name,btable,''{0}'',''{1}'',''{2}'',''{3}'',''{4}'',''{5}'',sysdate from tf_ywys_bm_logtype where btable=''{6}''", strCause, strNotes, strJsonNotes, logOperateEnum.toString(), userName, realName, strTableName));
        return sbSQL.toString();
    }

    /**
     * 通过数据库表名，where唯一条件查询数据库实体model，如果where查询出多条则返回null
     *
     * @param strTableName 数据库表名
     * @param strWhere     查询的where条件
     * @return HashMap对象
     * @throws Exception
     */
    private static HashMap getModel(String strTableName, String strWhere) throws Exception {
        String strSql = MessageFormat.format("select * from {0} where {1}", strTableName, strWhere);
        DbHelper dbHelper = DbHelper.getDbHelper();
        Object object = dbHelper.getList(strSql, SysEnum.MyBatis.getName());
        ArrayList arrayList = (ArrayList) ((CollectionData) object).getRecords();
        if (arrayList.size() != 1) {
            //查询的数据有多条，where条件不对，必须为一条数据
            return null;
        }
        return (HashMap) getConvertList(arrayList).get(0);
    }

    /**
     * 获取转换后的驼峰命名ArrayList集合
     *
     * @param listData
     * @return
     */
    public static ArrayList getConvertList(ArrayList listData) {
        if (listData != null && listData.size() > 0) {
            Object obj = (Object) listData.get(0);
            if (!(obj instanceof HashMap)) {
                return listData;
            }
            List<HashMap> mapList = new ArrayList<>();
            for (int i = 0, len = listData.size(); i < len; i++) {
                HashMap<String, Object> resultHashMap = new HashMap<>();
                HashMap<String, Object> hasMap = (HashMap) listData.get(i);
                for (Object key : hasMap.keySet()) {
                    if (key instanceof ArrayList) {
                        mapList = getConvertList((ArrayList) key);
                    } else if (key instanceof String) {
                        String[] arrSrc = key.toString().split("_");
                        String strNewKey = "";
                        String strTemp = "";
                        for (int k = 0; k < arrSrc.length; k++) {
                            if (arrSrc[k].equals(arrSrc[k].toUpperCase())) {
                                strTemp = arrSrc[k].toLowerCase();
                            } else {
                                strTemp = arrSrc[k];
                            }
                            if (k > 0) {
                                strTemp = strTemp.substring(0, 1).toUpperCase() + strTemp.substring(1);
                            }
                            strNewKey += strTemp;
                        }
                        resultHashMap.put(strNewKey, hasMap.get(key));
                    }
                }
                mapList.add(resultHashMap);
            }

            return (ArrayList) mapList;
        }
        return listData;

    }
    //endregion
}
