package com.tofly.feesapi.common.dbhelper;

import cn.hutool.core.util.ReflectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.tofly.feesapi.common.enums.TableIdEnum;
import com.tofly.feesapi.common.util.DateUtils;
import com.tofly.feesapi.common.util.SpringContextHolder;
import com.tofly.feesapi.common.util.ToolsUtils;
import com.tofly.feesapi.common.util.XmlUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.data.redis.core.RedisTemplate;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.text.MessageFormat;
import java.util.Date;
import java.util.HashMap;

import static com.tofly.feesapi.common.ControllerConstants.Mapper_Redis_Prix;

/**
 * @description:
 * @Title: ProduceSql
 * @Package com.tofly.fees.common.dbhelper
 * @Author
 * @Date 2023-06-09 13:41
 */
public class ProduceSql {
    //String insertSql = getInsertSql("user", User.class, user);

    private static RedisTemplate redisTemplate = SpringContextHolder.getBean("redisTemplate");

    /**
     * 生成插入语句
     *
     * @param tablename 表明
     * @param t         有数据的实体
     * @param <T>       数据实体类型 如 User
     */
    public static <T> String getInsertSql(String tablename, T t) throws IllegalArgumentException {
        //insert into table_name (column_name1,column_name2, ...) values (value1,value2, ...)

        boolean flag = false;
        String sql = "";
        Field[] fields = ReflectUtil.getFieldsDirectly(t.getClass(), false);
        StringBuffer topHalf = new StringBuffer("insert into " + tablename + " (");
        StringBuffer afterAalf = new StringBuffer("values (");
        for (Field field : fields) {
            //过滤不需要字段
            if ("ID".equals(field.getName()) || "id".equals(field.getName())) {
                continue;
            }

            if ("serialVersionUID".equals(field.getName()) || "serialVersionUID".equals(field.getName())) {
                continue;//更新的时候无需set id=xxx
            }

            if (ReflectUtil.getFieldValue(t, field.getName()) == null) {
                continue;
            }

            if (ReflectUtil.getFieldValue(t, field.getName()).equals(field.getName())) {
                continue;
            }


            topHalf.append(field.getName() + ",");
            if (ReflectUtil.getFieldValue(t, field.getName()) instanceof String) {
                afterAalf.append("'" + ReflectUtil.getFieldValue(t, field.getName()) + "',");
                flag = true;
            } else {
                afterAalf.append(ReflectUtil.getFieldValue(t, field.getName()) + ",");
                flag = true;
            }
        }
        if (!flag) {
            throw new IllegalArgumentException(t.getClass() + "NullException.\nThere is no attribute that is not empty.You must provide an object with at least one attribute.");
        }
        topHalf = new StringBuffer(StrUtil.removeSuffix(topHalf.toString(), ","));
        afterAalf = new StringBuffer(StrUtil.removeSuffix(afterAalf.toString(), ","));
        topHalf.append(") ");
        afterAalf.append(") ");
        sql = topHalf.toString() + afterAalf.toString();
        return sql;
    }

    /**
     * 生成更新语句
     * 必须含有id
     * 数据实体中 null 与 空字段不参与更新
     *
     * @param tablename 数据库中的表明
     * @param t         有数据的实体
     * @param <T>       数据实体类型,如 User
     */
    public static <T> String getUpdateSql(String tablename, T t) throws IllegalArgumentException {
        //UPDATE table_name SET column_name1 = value1, column_name2 = value2, ... where ID=xxx
        //or
        //UPDATE table_name SET column_name1 = value1, column_name2 = value2, ... where id=xxx
        boolean flag = false;
        String sql = "";
        String id = ""; //保存id列名：ID or id
        Field[] fields = ReflectUtil.getFieldsDirectly(t.getClass(), false);
        sql = "update " + tablename + " set ";
        for (Field field : fields) {
            StringBuffer tmp = new StringBuffer();
            if ("ID".equals(field.getName()) || "id".equals(field.getName())) {
                id = field.getName();
                continue;//更新的时候无需set id=xxx
            }

            if ("serialVersionUID".equals(field.getName()) || "serialVersionUID".equals(field.getName())) {
                id = field.getName();
                continue;//更新的时候无需set id=xxx
            }


            if (ReflectUtil.getFieldValue(t, field.getName()) == null) {
                continue;
            }

            if (ReflectUtil.getFieldValue(t, field.getName()).equals(field.getName())) {
                continue;
            }

            if (ReflectUtil.getFieldValue(t, field.getName()) != null && (String) ReflectUtil.getFieldValue(t, field.getName()) != "") {
                tmp.append(field.getName() + "=");
                if (ReflectUtil.getFieldValue(t, field.getName()) instanceof String) {
                    tmp.append("'" + ReflectUtil.getFieldValue(t, field.getName()) + "',");
                    flag = true;
                } else {
                    tmp.append(ReflectUtil.getFieldValue(t, field.getName()) + ",");
                    flag = true;
                }
                sql += tmp;
            }
        }
        if (!flag) {
            throw new IllegalArgumentException(t.getClass() + "NullException.\nThere is no attribute that is not empty except for ID.You must provide an object with at least one attribute exclude ID.");
        }
        sql = StrUtil.removeSuffix(sql, ",") + " where " + id + "='" + ReflectUtil.getFieldValue(t, id) + "'";
        return sql;
    }

    /**
     * 生成删除语句
     * 根据 user 中第一个不为空的字段删除,应该尽量使用 id,提供至少一个非空属性
     *
     * @param tablename 表明
     * @param t         有数据的实体
     * @param <T>       数据实体类型 如 User
     */
    public static <T> String getDeleteSql(String tablename, T t) throws IllegalArgumentException {
        //delete from table_name where column_name = value
        return getSelectOrDeleteSql(tablename, t, "delete");
    }

    /**
     * 生成查询语句
     * 根据 user 中第一个不为空的字段查询
     *
     * @param tablename 表名
     * @param t         有数据的实体
     * @param <T>       数据实体类型 如 User
     */
    public static <T> String getSelectSql(String tablename, T t) throws IllegalArgumentException {
        //delete from table_name where column_name = value
        return getSelectOrDeleteSql(tablename, t, "select *");
    }

    /**
     * 根据 operation 生成一个如：operation from table_name where column_name = value 的sql语句
     *
     * @param tablename
     * @param t
     * @param operation "select *"  or "delete"
     * @param <T>
     * @return
     * @throws IllegalArgumentException
     */
    private static <T> String getSelectOrDeleteSql(String tablename, T t, String operation) throws IllegalArgumentException {
        //operation from table_name where column_name = value
        boolean flag = false;
        String sql = "";
        Field[] fields = ReflectUtil.getFieldsDirectly(t.getClass(), false);
        StringBuffer topHalf = new StringBuffer(operation + " from " + tablename + " where ");
        for (Field field : fields) {
            if ("ID".equals(field.getName()) || "id".equals(field.getName())) {
                if (ReflectUtil.getFieldValue(t, field.getName()) != null && (int) ReflectUtil.getFieldValue(t, field.getName()) != 0) {
                    //id 不为空
                    topHalf.append(field.getName() + " = " + ReflectUtil.getFieldValue(t, field.getName()));
                    flag = true;
                    break;
                }
            } else {
                if (ReflectUtil.getFieldValue(t, field.getName()) != null && (String) ReflectUtil.getFieldValue(t, field.getName()) != "") {
                    topHalf.append(field.getName() + " = '" + ReflectUtil.getFieldValue(t, field.getName()) + "'");
                    flag = true;
                    break;
                }
            }
        }
        if (!flag) {
            throw new IllegalArgumentException(t.getClass() + "NullException.\nThere is no attribute that is not empty.You must provide an object with at least one attribute.");
        }
        sql = topHalf.toString();
        return sql;
    }

    /**
     * 根据泛型实体类自动生成需要赋值的 Insert SQL 添加语句
     * 外部调用必须定义后赋值才会处理，如果设置为null会忽略
     *
     * @param t           泛型实体类
     * @param tableIdEnum 数据表对应的主键ID枚举
     * @param <T>         泛型
     * @return 返回Insert SQL添加语句
     */
    public static <T> String getInsertSql(T t, TableIdEnum tableIdEnum) {
        //如果实体对象为null直接返回空字符
        if (t == null) {
            return "";
        }

        //获取Redis缓存的HashMap
        HashMap<String, String> hmRedisMap = getRedisHashMap(t);
        if (hmRedisMap == null) {
            return "";
        }

        //获取主键ID
        String strTableId = getTableIdValue(t);

        //判断TableId是否为空
        Boolean bIsNull = StringUtils.isBlank(strTableId);

        //数据库表名
        String strDbTableName = ((TableName) getTableNameAnnotation(t)).value();

        //组装insert语句
        StringBuilder sbField = new StringBuilder("insert into " + strDbTableName + " (");
        StringBuilder sbValue = new StringBuilder("values (");
        for (String field : hmRedisMap.keySet()) {
            Object fieldValue = ReflectUtil.getFieldValue(t, field);
            //判断是否为主键列
            if (!bIsNull) {
                String strPkName = hmRedisMap.get(field);
                if (strPkName.equals(strTableId)) {
                    setTableIdValue(tableIdEnum, strDbTableName, sbField, sbValue, fieldValue, strPkName);
                    continue;
                }
            }
            if (fieldValue == null) {
                continue;//未对字段赋值直接终止
            }
            sbField.append(hmRedisMap.get(field) + ",");
            if (fieldValue instanceof String) {
                sbValue.append("'" + fieldValue + "',");
            } else if (fieldValue instanceof Date) {
                sbValue.append("to_date('" + DateUtils.formatDate((Date) fieldValue) + "','yyyy-mm-dd hh24:mi:ss'),");
            } else {
                sbValue.append(fieldValue + ",");
            }
        }
        String strSql = MessageFormat.format("{0}) {1}) "
                , sbField.substring(0, sbField.length() - 1)
                , sbValue.substring(0, sbValue.length() - 1));
        return strSql;
    }

    /**
     * 根据泛型实体类自动生成需要赋值的 Update SQL 添加语句
     * 外部调用必须定义后赋值才会处理，如果设置为null会忽略
     *
     * @param t          泛型实体类
     * @param p_whereSql where 语句,【谨慎全部更新，如果确实需要可传入字符“1=1”】
     * @param <T>        泛型
     * @return 返回Update SQL更新语句
     */
    public static <T> String getUpdateSql(T t, String p_whereSql) {
        //如果实体对象为null直接返回空字符
        if (t == null) {
            return "";
        }

        //获取Redis缓存的HashMap
        HashMap<String, String> hmRedisMap = getRedisHashMap(t);
        if (hmRedisMap == null) {
            return "";
        }

        //数据库表名
        String strDbTableName = ((TableName) getTableNameAnnotation(t)).value();

        //组装update语句
        StringBuilder stringBuilder = new StringBuilder("update " + strDbTableName + " set ");
        for (String field : hmRedisMap.keySet()) {
            Object fieldValue = ReflectUtil.getFieldValue(t, field);
            if (fieldValue == null) {
                continue;//未对字段赋值直接终止
            }
            if (fieldValue instanceof Date) {
                String strDateValue = "to_date('" + DateUtils.formatDate((Date) fieldValue) + "','yyyy-mm-dd hh24:mi:ss')";
                stringBuilder.append(hmRedisMap.get(field) + "=" + strDateValue + ",");
            } else {
                stringBuilder.append(hmRedisMap.get(field) + "='" + fieldValue + "',");
            }
        }
        String strSql = MessageFormat.format("{0} where {1}"
                , stringBuilder.substring(0, stringBuilder.length() - 1)
                , p_whereSql);
        return strSql;
    }

    /**
     * 根据泛型实体类自动生成需要赋值的 Delete SQL 添加语句
     * 外部调用必须定义后赋值才会处理，如果设置为null会忽略
     *
     * @param p_tableName 数据库表名
     * @param p_whereSql  where 语句，【谨慎全部删除，如果确实需要可传入字符“1=1”】
     * @return 返回Delete SQL删除语句
     */
    public static <T> String getDeleteSql(String p_tableName, String p_whereSql) {
        //如果表名为空则返回空字符
        if (StringUtils.isBlank(p_tableName)) {
            return "";
        }

        //组装delete语句
        return MessageFormat.format("delete from {0} where {1}", p_tableName, p_whereSql);
    }

    /**
     * 设置Insert SQL中对应的 TableId值
     *
     * @param tableIdEnum    枚举
     * @param strDbTableName 数据库表名
     * @param sbField        insert 语句中的 insert 字符
     * @param sbValue        insert 语句中的 values 字符
     * @param fieldValue     主键id字段值
     * @param strPkName      主键数据库列名
     */
    private static void setTableIdValue(TableIdEnum tableIdEnum, String strDbTableName
            , StringBuilder sbField, StringBuilder sbValue
            , Object fieldValue, String strPkName) {
        boolean isAddPK = true;
        switch (tableIdEnum) {
            case GUID:
                sbValue.append("'" + ToolsUtils.getGuid() + "',");
                break;
            case MaxID:
                sbValue.append(MessageFormat.format("(select max(cast({0} as int))+1 from {1}),", strPkName, strDbTableName));
                break;
            case Other:
                //当做普通字段赋值
                if (fieldValue instanceof String) {
                    sbValue.append("'" + fieldValue + "',");
                } else {
                    sbValue.append(fieldValue + ",");
                }
                break;
            case AutoIncrement:
                //自增，不添加到sql语句中
            default:
                isAddPK = false;
                break;
        }
        if (isAddPK) {
            sbField.append(strPkName + ",");
        }
    }

    /**
     * 获取指定泛型实体类的TableId值
     *
     * @param t   泛型实体类
     * @param <T> 泛型
     * @return 实体类的TableId值
     */
    private static <T> String getTableIdValue(T t) {
        String strTableId = "";
        Field[] fields = t.getClass().getDeclaredFields();
        for (Field field : fields) {
            for (Annotation annotation : field.getAnnotations()) {
                if (annotation instanceof TableId) {
                    strTableId = ((TableId) annotation).value();
                }
            }
        }
        return strTableId;
    }

    /**
     * 获取泛型实体类对应的TableName注解对象
     *
     * @param t   泛型实体类
     * @param <T> 泛型
     * @return 实体类对应的TableName注解对象
     */
    private static <T> Annotation getTableNameAnnotation(T t) {
        Annotation[] annotations = t.getClass().getDeclaredAnnotations();
        for (Annotation annotation : annotations) {
            if (annotation instanceof TableName) {
                return annotation;
            }
        }
        return null;
    }

    /**
     * 获取泛型实体类对应的Mapper文件路径
     *
     * @param t   泛型实体类
     * @param <T> 泛型
     * @return Mapper文件路径
     */
    private static <T> String getMapperPath(T t) {
        String strProjectName = "fees";
        String strNamespace = t.getClass().getName();
        strNamespace = strNamespace.substring(strNamespace.indexOf(strProjectName) + strProjectName.length() + 1);
        strNamespace = strNamespace.substring(0, strNamespace.indexOf("."));
        String strClassName = t.getClass().getSimpleName();
        return MessageFormat.format("{0}/{1}mapper.xml", strNamespace, strClassName);
    }

    /**
     * 通过泛型实体类获取Mapper文件中的数据库对应关系，存储至redis中
     *
     * @param t   泛型实体类
     * @param <T> 泛型
     * @return Mapper文件路径
     */
    private static <T> HashMap<String, String> getRedisHashMap(T t) {
        String strMapperPath = getMapperPath(t);
        String strMapperId = ((TableName) getTableNameAnnotation(t)).resultMap();

        String strCacheKey = Mapper_Redis_Prix + strMapperId;
        HashMap<String, String> es = (HashMap<String, String>) redisTemplate.opsForValue().get(strCacheKey);
        if (es == null) {
            es = XmlUtils.getHashMapperColumnByElement(strMapperPath, strMapperId);
            redisTemplate.opsForValue().set(strCacheKey, es);
        }
        return es;
    }
}
