package com.tofly.feesapi.common.mapper;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.feesapi.common.enums.UserRoleEnum;
import com.tofly.feesapi.common.model.SystemUserInfo;
import org.apache.ibatis.annotations.Param;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 通用 Mapper 接口
 * </p>
 *
 * @author decai
 * @since 2023-05-26
 */
public interface CommonMapper {
    Page<Map<String, Object>> queryForPage(Page<Map<String, Object>> page, @Param("sql") String sql);

    List<HashMap<String, Object>> query(@Param("sql") String sql);

    /**
     * 通过角色ID查询用户信息（登录账号、真实姓名）
     *
     * @param userRoleEnum 角色枚举
     * @return
     */
    List<HashMap<String, Object>> getUserListByRoleId(UserRoleEnum userRoleEnum);

    /**
     * @MethodName procedureCmd
     * @Description 执行存储过程
     * @Date 10:10 2023-07-06
     */
    Map<String, Object> procedureCmd(@Param("sql") String procurdeName, @Param("map") Map<String, Object> objectHashMap);


    /**
     * 通过配置表ID查询配置的值
     *
     * @param pzid 配置ID
     * @return 配置值
     */
    String getConfigValueByPZID(String pzid);

    /**
     * 通过当前公司编码查询当前的财务日期
     *
     * @param companyBranch 公司编码
     * @return 财务日期
     */
    String getBillMonthByCompanyBranch(String companyBranch);

    /**
     * 通过用户ID查询用户信息
     *
     * @param userId 用户ID，登录ID
     * @return
     */
    SystemUserInfo getUserInfoByUserId(String userId);
}
