package com.tofly.feesapi.config;

import com.tofly.feesapi.security.AccessTokenAuthenticationProcessingFilter;
import com.tofly.feesapi.security.SkipPathRequestMatcher;
import com.tofly.feesapi.security.extractor.TokenExtractor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.builders.WebSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.web.authentication.AuthenticationFailureHandler;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;

import java.util.Arrays;

@EnableWebSecurity
public class SecurityConfig extends WebSecurityConfigurerAdapter {


    @Override
    protected void configure(HttpSecurity http) throws Exception {
        http
                .addFilterBefore(buildAccessTokenAuthenticationProcessingFilter(), UsernamePasswordAuthenticationFilter.class)
                .headers(s -> s.httpStrictTransportSecurity().disable())
                .csrf(s -> s.disable())
                .securityContext(s -> s.disable())
                .requestCache(s -> s.disable())
                .httpBasic(s -> s.disable())
                .formLogin(s -> s.disable())
                .logout(s -> s.disable())
                .sessionManagement(s -> s.disable())
                .authorizeRequests()
//				.antMatchers("/css/**", "/js/**", "/fonts/**", "/images/**").permitAll()
//				.antMatchers("/admin/**").hasRole("ADMIN")
//				.antMatchers("/welcome").hasRole("ADMIN")
//				.anyRequest().authenticated()
                //TODO 功能完成后统一配置
                .anyRequest().permitAll();


    }

    @Autowired
    private AuthenticationManager authenticationManager;

    @Autowired
    @Qualifier("defaultAuthenticationFailureHandler")
    private AuthenticationFailureHandler failureHandler;

    @Autowired
    @Qualifier("headerTokenExtractor")
    private TokenExtractor headerTokenExtractor;

    @Autowired
    @Qualifier("queryTokenExtractor")
    private TokenExtractor queryTokenExtractor;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    protected AccessTokenAuthenticationProcessingFilter buildAccessTokenAuthenticationProcessingFilter() throws Exception {
        final String TOKEN_REFRESH_ENTRY_POINT = "/api/auth/token";
        SkipPathRequestMatcher matcher = new SkipPathRequestMatcher(Arrays.asList(TOKEN_REFRESH_ENTRY_POINT), Arrays.asList("/api/**","/yysspjxx/**"));
        AccessTokenAuthenticationProcessingFilter filter
                = new AccessTokenAuthenticationProcessingFilter(failureHandler, headerTokenExtractor, redisTemplate, matcher);
        filter.setAuthenticationManager(this.authenticationManager);
        return filter;
    }

    @Override
    public void configure(WebSecurity web) throws Exception {
        //忽略
        //web.ignoring().antMatchers(NON_AUTH_ENTRY_POINTS);
    }

    @Bean
    @Override
    public AuthenticationManager authenticationManagerBean() throws Exception {
        return super.authenticationManagerBean();
    }

//    @Bean
//    CzyUserDetailsService customUserDetailsService(PasswordEncoder passwordEncoder) {
//        return new CzyUserDetailsService(passwordEncoder);
//    }
//
//    @Bean
//    PasswordEncoder passwordEncoder() {
//        return new BCryptPasswordEncoder(BCryptPasswordEncoder.BCryptVersion.$2Y, 6);
//    }
}