package com.tofly.feesapi.file.fileconfiguration;

import com.tofly.feesapi.common.exception.FeesErrorCode;
import com.tofly.feesapi.common.exception.FeesException;
import com.tofly.feesapi.file.entity.dto.FileUploadBase64Dto;
import com.tofly.feesapi.file.entity.dto.FileUploadResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.apache.commons.net.ftp.FTPReply;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotNull;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Base64;
import java.util.Objects;


@Service
@Slf4j
public class FTPFileStorageService implements FileStorageService {

    @Autowired
    private FTPFileStorageProperties fileStorageProperties;
    private final FTPClient ftpClient = new FTPClient();

//    @Autowired
//    public FTPFileStorageService(FTPFileStorageProperties fileStorageProperties)
//            throws IOException {
//        this.fileStorageProperties = fileStorageProperties;
//        // 创建客户端对象
//        this.ftpClient.setControlEncoding("UTF-8");
//        this.ftpClient.connect(fileStorageProperties.getServer(), fileStorageProperties.getPort());
//        this.ftpClient.login(fileStorageProperties.getUserName(), fileStorageProperties.getPassword());
//        if (!FTPReply.isPositiveCompletion(this.ftpClient.getReplyCode())) {
//            throw new FeesException("FTP连接失败", FeesErrorCode.BUSINESS_ERROR);
//        }
//    }

    public void FTPFileLogin() {
        try {
            fileStorageProperties.setRandomFileName(true);
            // 创建客户端对象
            this.ftpClient.setControlEncoding("UTF-8");
            this.ftpClient.connect(fileStorageProperties.getServer(), fileStorageProperties.getPort());
            this.ftpClient.login(fileStorageProperties.getUserName(), fileStorageProperties.getPassword());
            if (!FTPReply.isPositiveCompletion(this.ftpClient.getReplyCode())) {
                throw new FeesException("FTP连接失败", FeesErrorCode.BUSINESS_ERROR);
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    // 储存文件
    public String storeFile(MultipartFile file, String Path) {
        // Normalize file name
        String fileName = StringUtils.cleanPath(Objects.requireNonNull(file.getOriginalFilename()));
        // random fileName if needed
        // 储存
        try {
            return storeFileByInputStream(Path, fileName, file.getContentType(), file.getInputStream());
        } catch (Exception ex) {
            ex.printStackTrace();
            //throw new FileStorageException(ErrorMessage.STORE_FILE_FAILED);
            throw new FeesException("当前新增对象中,文件上传失败,请重试", FeesErrorCode.BUSINESS_ERROR);
        }
    }

    public String storeFileBase64(String path, String originalFilename, String contentType, InputStream inputStream) {
        // Normalize file name
        String fileName = StringUtils.cleanPath(Objects.requireNonNull(originalFilename));
        // random fileName if needed
        // 储存
        try {
            return storeFileByInputStream(path, fileName, contentType, inputStream);
        } catch (Exception ex) {
            ex.printStackTrace();
            //throw new FileStorageException(ErrorMessage.STORE_FILE_FAILED);
            throw new FeesException("当前新增对象中,文件上传失败,请重试", FeesErrorCode.BUSINESS_ERROR);
        }
    }

    @Override
    public FileUploadResponse upload(MultipartFile file, String Path) {
        FTPFileLogin();
        String fileName = this.storeFile(file, Path);
        String staticPath = getPath(fileName);
        String realDownloadPath = null;
        try {
            // 获取request
            HttpServletRequest request =
                    ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
            // 生成下载接口的地址
            realDownloadPath =
                    request.getRequestURL().toString().replace("upload", "downloadFile/" + fileName);
        } catch (Exception e) {
            e.printStackTrace();
            log.info("Could not upload file " + fileName + ". Please try again!");
        }
        return new FileUploadResponse(
                fileName, realDownloadPath, file.getContentType(), file.getSize(), staticPath);
    }

    public FileUploadResponse uploadFileBase64(FileUploadBase64Dto file, String Path) {
        String fileBase64 = file.getBase64String();
        if (fileBase64.contains(",")) {

            fileBase64 = fileBase64.split(",")[1];
        }
        byte[] bytes = Base64.getDecoder().decode(fileBase64);
        InputStream inputStream = new ByteArrayInputStream(bytes);
        long fileSize = bytes.length;
        String fileName = this.storeFileBase64(Path, file.getOriginalFilename(), file.getContentType(), inputStream);
        String staticPath = getPath(fileName);
        String realDownloadPath = null;
        try {
            // 获取request
            HttpServletRequest request =
                    ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
            // 生成下载接口的地址
            realDownloadPath =
                    request.getRequestURL().toString().replace("upload", "downloadFile/" + fileName);
        } catch (Exception e) {
            e.printStackTrace();
            log.info("Could not upload file " + fileName + ". Please try again!");
        }
        return new FileUploadResponse(
                fileName, realDownloadPath, file.getContentType(), fileSize, staticPath);
    }


    @Override
    public ResponseEntity<?> download(
            String fileName, HttpServletRequest request, HttpServletResponse response) {
        FTPFileLogin();
        //将客户端设置为被动模式
        this.ftpClient.enterLocalPassiveMode();

        try (
                // 获取文件流
                InputStream inputStream = this.ftpClient.retrieveFileStream(fileName);
                // 获取输出流
                OutputStream outputStream = response.getOutputStream()
        ) {
            // 清空
            response.reset();
            // 写入流
            byte[] buffer = inputStream.readAllBytes();
            outputStream.write(buffer);
            outputStream.flush();
            // 配置header
            response.addHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;filename=" + fileName);
            response.addHeader(HttpHeaders.CONTENT_LENGTH, String.valueOf(buffer.length));
            return ResponseEntity.status(HttpStatus.OK).body("download...");
        } catch (Exception e) {
            // 找不到文件时会抛出异常
            String err = fileName + "file not found";
            log.info(err);
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body(err);
        }
    }

    @Override
    public byte[] getByteDataforFile(String fileName) {
        try {
            FTPFileLogin();
            // 获取文件流
            InputStream inputStream = this.ftpClient.retrieveFileStream(fileName);
            // 写入流
            byte[] buffer = inputStream.readAllBytes();

            return buffer;
        } catch (Exception e) {
            // 找不到文件时会抛出异常
            String err = fileName + " file not found";
            log.info(err);
            log.info(e.getMessage());
            return null;
        }
    }

    public InputStream getByteDataforFileInputStream(String fileName) {
        try {
            FTPFileLogin();
            // 获取文件流
            InputStream inputStream = this.ftpClient.retrieveFileStream(fileName);
            // 写入流
            return inputStream;
        } catch (Exception e) {
            // 找不到文件时会抛出异常
            String err = fileName + " file not found";
            log.info(err);
            log.info(e.getMessage());
            return null;
        }
    }

    @Override
    public Boolean retrieveFile(String remotePath, OutputStream outputStream) {
        try {
            FTPFileLogin();
            InputStream inputStream = this.ftpClient.retrieveFileStream(remotePath);
            if (IOUtils.copy(inputStream, outputStream) > -1) {
                return true;
            }
            return false;
        } catch (Exception e) {
            log.info(e.getMessage());
            return false;
        }
    }

    @Override
    public String uploadFile(String Path, String fileName, String contentType, InputStream inputStream)
            throws IOException {
        try {
            return getPath(storeFileByInputStream(Path, fileName, contentType, inputStream));
        } catch (Exception e) {
            throw new IOException(e.getMessage());
        }
    }

    /**
     * 删除文件
     *
     * @param PathfileName 文件地址
     * @throws IOException
     */
    @Override
    public void removeFile(String PathfileName) throws IOException {
        try {
            this.ftpClient.deleteFile(PathfileName);
        } catch (Exception e) {
            throw new IOException(e.getMessage());
        }
    }

    /**
     * @param fileName 文件名
     * @return 文件访问地址
     */
    @NotNull
    private String getPath(String fileName) {
        return fileStorageProperties
                .getServer()
                .concat(":")
                .concat(String.valueOf(fileStorageProperties.getPort()))
                .concat("/")
                .concat(fileName);
    }

    private String storeFileByInputStream(String Path,
                                          String fileName, String contentType, InputStream inputStream) throws IOException {
        if (fileStorageProperties.isRandomFileName()) {
            if (!fileName.contains(".")) {
                fileName = RandomStringUtils.randomAlphabetic(15);
            } else {
                fileName =
                        RandomStringUtils.randomAlphabetic(15) + fileName.substring(fileName.lastIndexOf("."));
            }
        }
        // Check if the file's name contains invalid characters
        if (fileName.contains("..")) {
            //throw new FileStorageException(ErrorMessage.STORE_FILE_FAILED);
        }
        Path += "/" + LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMM"));
        //切换到上传目录
        if (!this.ftpClient.changeWorkingDirectory(Path)) {
            //如果目录不存在创建目录
            String[] dirs = Path.split("/");
            String tempPath = "";
            for (String dir : dirs) {
                if (null == dir || "".equals(dir))
                    continue;
                tempPath += "/" + dir;
                if (!this.ftpClient.changeWorkingDirectory(tempPath)) {
                    if (!this.ftpClient.makeDirectory(tempPath)) {
                        throw new FeesException("FTP创建文件夹失败", FeesErrorCode.BUSINESS_ERROR);
                    } else {
                        this.ftpClient.changeWorkingDirectory(tempPath);
                    }
                }
            }
        }
        //fileName = Path + "/" + fileName;
        //将客户端设置为被动模式
        this.ftpClient.enterLocalPassiveMode();
        //设置上传文件的类型为二进制类型
        this.ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
        // 上传文件到FTP服务器
        this.ftpClient.storeFile(fileName, inputStream);
        //inputStream.close();
        // 关闭FTP连接
        this.ftpClient.logout();
        this.ftpClient.disconnect();
        return Path + "/" + fileName;
    }

    @Override
    public FTPFile[] results(String filePathBean) throws IOException {
        return this.ftpClient.listFiles(filePathBean);
    }
}
