package com.tofly.feesapi.file.fileconfiguration;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.plugins.pagination.PageDTO;
import com.tofly.feesapi.common.exception.FeesErrorCode;
import com.tofly.feesapi.common.exception.FeesException;
import com.tofly.feesapi.file.entity.dto.FileStorageQueryDto;
import com.tofly.feesapi.file.entity.dto.FileUploadBase64Dto;
import com.tofly.feesapi.file.entity.dto.FileUploadResponse;
import com.tofly.feesapi.file.entity.vo.FileStorageItemVo;
import com.tofly.feesapi.file.entity.vo.FileStorageStatVo;
import io.minio.*;
import io.minio.errors.*;
import io.minio.messages.Item;
import lombok.extern.slf4j.Slf4j;
import okhttp3.OkHttpClient;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnExpression;
import org.springframework.core.io.FileSystemResource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.multipart.MultipartFile;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotNull;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardCopyOption;
import java.security.InvalidKeyException;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.cert.X509Certificate;
import java.text.DecimalFormat;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;


@Service
@Slf4j
@ConditionalOnExpression("'${file.storage.type}'=='minio'")
public class MinioFileStorageService /*implements FileStorageService*/ {

    private final MinioFileStorageProperties fileStorageProperties;
    private final MinioClient minioClient;

    @Autowired
    public MinioFileStorageService(MinioFileStorageProperties fileStorageProperties)
            throws KeyManagementException {
        this.fileStorageProperties = fileStorageProperties;
        // 创建客户端对象
        this.minioClient =
                MinioClient.builder()
                        .endpoint(fileStorageProperties.getMinioUrl())
                        .credentials(fileStorageProperties.getMinioName(), fileStorageProperties.getMinioPass())
                        .httpClient(getUnsafeOkHttpClient())
                        .build();
        // 创建储存桶
        checkBucket();
    }

    public static OkHttpClient getUnsafeOkHttpClient() throws KeyManagementException {
        try {
            final TrustManager[] trustAllCerts =
                    new TrustManager[]{
                            new X509TrustManager() {
                                //@Override
                                public void checkClientTrusted(X509Certificate[] x509Certificates, String s) {
                                }

                                //@Override
                                public void checkServerTrusted(X509Certificate[] x509Certificates, String s) {
                                }

                                //@Override
                                public X509Certificate[] getAcceptedIssuers() {
                                    return new X509Certificate[]{};
                                }
                            }
                    };

            final SSLContext sslContext = SSLContext.getInstance("SSL");
            sslContext.init(null, trustAllCerts, new SecureRandom());
            final SSLSocketFactory sslSocketFactory = sslContext.getSocketFactory();
            OkHttpClient.Builder builder = new OkHttpClient.Builder();
            builder.sslSocketFactory(sslSocketFactory, (X509TrustManager) trustAllCerts[0]);

            builder.hostnameVerifier((s, sslSession) -> true);
            return builder.build();

        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        }
        return null;
    }

    // 储存文件
    public String storeFile(MultipartFile file, String Path) {
        // Normalize file name
        String fileName = StringUtils.cleanPath(Objects.requireNonNull(file.getOriginalFilename()));
        // random fileName if needed
        // 储存
        try {
            return storeFileByInputStream(Path, fileName, file.getContentType(), file.getInputStream());
        } catch (Exception ex) {
            ex.printStackTrace();
            //throw new FileStorageException(ErrorMessage.STORE_FILE_FAILED);
            throw new FeesException("当前新增对象中,文件上传失败,请重试", FeesErrorCode.BUSINESS_ERROR);
        }
    }

    public String storeFileBase64(String path,String originalFilename, String contentType,InputStream inputStream) {
        // Normalize file name
        String fileName = StringUtils.cleanPath(Objects.requireNonNull(originalFilename));
        // random fileName if needed
        // 储存
        try {
            return storeFileByInputStream(path, fileName, contentType, inputStream);
        } catch (Exception ex) {
            ex.printStackTrace();
            //throw new FileStorageException(ErrorMessage.STORE_FILE_FAILED);
            throw new FeesException("当前新增对象中,文件上传失败,请重试", FeesErrorCode.BUSINESS_ERROR);
        }
    }

    //@Override
    public FileUploadResponse upload(MultipartFile file, String Path) {
        String fileName = this.storeFile(file, Path);
        String staticPath = getPath(fileName);
        String realDownloadPath = null;
        try {
            // 获取request
            HttpServletRequest request =
                    ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
            // 生成下载接口的地址
            realDownloadPath =
                    request.getRequestURL().toString().replace("upload", "downloadFile/" + fileName);
        } catch (Exception e) {
            e.printStackTrace();
            log.info("Could not upload file " + fileName + ". Please try again!");
        }
        return new FileUploadResponse(
                fileName, realDownloadPath, file.getContentType(), file.getSize(), staticPath);
    }

    public FileUploadResponse uploadFileBase64(FileUploadBase64Dto file, String Path) {
        String fileBase64=file.getBase64String();
       if(fileBase64.contains(",")){

           fileBase64=fileBase64.split(",")[1];
       }
        byte [] bytes= Base64.getDecoder().decode(fileBase64);
        InputStream inputStream=new ByteArrayInputStream(bytes);
        long fileSize = bytes.length;
        String fileName = this.storeFileBase64(Path,file.getOriginalFilename() ,file.getContentType(),inputStream);
        String staticPath = getPath(fileName);
        String realDownloadPath = null;
        try {
            // 获取request
            HttpServletRequest request =
                    ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
            // 生成下载接口的地址
            realDownloadPath =
                    request.getRequestURL().toString().replace("upload", "downloadFile/" + fileName);
        } catch (Exception e) {
            e.printStackTrace();
            log.info("Could not upload file " + fileName + ". Please try again!");
        }
        return new FileUploadResponse(
                fileName, realDownloadPath, file.getContentType(), fileSize, staticPath);
    }


    //@Override
    public ResponseEntity<?> download(
            String fileName, HttpServletRequest request, HttpServletResponse response) {
        // get object given the bucket and object name
        try (
                // 获取文件流
                InputStream inputStream =
                        minioClient.getObject(
                                GetObjectArgs.builder()
                                        .bucket(fileStorageProperties.getBucketName())
                                        .object(fileName)
                                        .build());
                // 获取输出流
                OutputStream outputStream = response.getOutputStream()) {
            // 清空
            response.reset();
            // 写入流
            byte[] buffer = inputStream.readAllBytes();
            outputStream.write(buffer);
            outputStream.flush();
            // 配置header
            response.addHeader(HttpHeaders.CONTENT_DISPOSITION, "attachment;filename=" + fileName);
            response.addHeader(HttpHeaders.CONTENT_LENGTH, String.valueOf(buffer.length));
            return ResponseEntity.status(HttpStatus.OK).body("download...");
        } catch (Exception e) {
            // 找不到文件时会抛出异常
            String err = fileName + "file not found";
            log.info(err);
            return ResponseEntity.status(HttpStatus.NOT_FOUND).body(err);
        }
    }

    //@Override
    public byte[] getByteDataforFile(String fileName) {
        try {
            // 获取文件流
            InputStream inputStream =
                    minioClient.getObject(
                            GetObjectArgs.builder()
                                    .bucket(fileStorageProperties.getBucketName())
                                    .object(fileName)
                                    .build());
            // 写入流
            byte[] buffer = inputStream.readAllBytes();

            return buffer;
        } catch (Exception e) {
            // 找不到文件时会抛出异常
            String err = fileName + " file not found";
            log.info(err);
            log.info(e.getMessage());
            return null;
        }
    }

    public InputStream getByteDataforFileInputStream(String fileName) {
        try {
            // 获取文件流
            InputStream inputStream =
                    minioClient.getObject(
                            GetObjectArgs.builder()
                                    .bucket(fileStorageProperties.getBucketName())
                                    .object(fileName)
                                    .build());
            // 写入流
          return inputStream;
        } catch (Exception e) {
            // 找不到文件时会抛出异常
            String err = fileName + " file not found";
            log.info(err);
            log.info(e.getMessage());
            return null;
        }
    }

    //@Override
    public Boolean retrieveFile(String remotePath, OutputStream outputStream) {
        try {
            InputStream inputStream =
                    minioClient.getObject(
                            GetObjectArgs.builder()
                                    .bucket(fileStorageProperties.getBucketName())
                                    .object(remotePath)
                                    .build());

            if (IOUtils.copy(inputStream, outputStream) > -1) {
                return true;
            }
            return false;
        } catch (Exception e) {
            log.info(e.getMessage());
            return false;
        }
    }

    //@Override
    public String uploadFile(String Path, String fileName, String contentType, InputStream inputStream)
            throws IOException {
        try {
            return getPath(storeFileByInputStream(Path, fileName, contentType, inputStream));
        } catch (Exception e) {
            throw new IOException(e.getMessage());
        }
    }

    //@Override
    public void removeFile(String PathfileName) throws IOException {
        try {
            storeFileByemove(PathfileName);
        } catch (Exception e) {
            throw new IOException(e.getMessage());
        }
    }


    public void storeFileByemove(String PathfileName)
            throws ServerException, InsufficientDataException, ErrorResponseException, IOException,
            NoSuchAlgorithmException, InvalidKeyException, InvalidResponseException, XmlParserException, InternalException {
        RemoveObjectArgs args = RemoveObjectArgs.builder()
                .bucket(fileStorageProperties.getBucketName())
                .object(PathfileName)
                .build();
        minioClient.removeObject(args);
    }


    // 检查储存桶情况
    private void checkBucket() {
        try {
            // 桶是否存在
            boolean exists =
                    minioClient.bucketExists(
                            BucketExistsArgs.builder().bucket(fileStorageProperties.getBucketName()).build());
            if (!exists) {
                // 不存在创建
                minioClient.makeBucket(
                        MakeBucketArgs.builder().bucket(fileStorageProperties.getBucketName()).build());
                // 设置储存桶策略
                String str =
                        "{\"Statement\": [{\"Action\": [\"s3:GetBucketLocation\",\"s3:ListBucket\"],\"Effect\": \"Allow\",\"Principal\": \"*\",\"Resource\": \"arn:aws:s3:::%\"},{\"Action\": \"s3:GetObject\",\"Effect\": \"Allow\",\"Principal\": \"*\",\"Resource\": \"arn:aws:s3:::%/*\"}],\"Version\": \"2012-10-17\"}";
                String replace = str.replace("%", fileStorageProperties.getBucketName());
                minioClient.setBucketPolicy(
                        SetBucketPolicyArgs.builder()
                                .bucket(fileStorageProperties.getBucketName())
                                .config(replace)
                                .build());
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    /**
     * @param fileName 文件名
     * @return 文件访问地址
     */
    @NotNull
    private String getPath(String fileName) {
        return fileStorageProperties
                .getMinioUrl()
                .concat("/")
                .concat(fileStorageProperties.getBucketName())
                .concat("/")
                .concat(fileName);
    }

    private String storeFileByInputStream(String Path,
                                          String fileName, String contentType, InputStream inputStream)
            throws ServerException, InsufficientDataException, ErrorResponseException, IOException,
            NoSuchAlgorithmException, InvalidKeyException, InvalidResponseException,
            XmlParserException, InternalException {
        if (fileStorageProperties.isRandomFileName()) {
            if (!fileName.contains(".")) {
                fileName = RandomStringUtils.randomAlphabetic(15);
            } else {
                fileName =
                        RandomStringUtils.randomAlphabetic(15) + fileName.substring(fileName.lastIndexOf("."));
            }
        }
        // Check if the file's name contains invalid characters
        if (fileName.contains("..")) {
            //throw new FileStorageException(ErrorMessage.STORE_FILE_FAILED);
        }
        fileName = Path + fileName;
        checkBucket();
        PutObjectArgs build =
                PutObjectArgs.builder()
                        .bucket(fileStorageProperties.getBucketName())
                        .object(fileName)
                        .contentType(contentType)
                        .stream(inputStream, inputStream.available(), -1)
                        .build();
        minioClient.putObject(build);
        return fileName;
    }

    //@Override
    public Iterable<Result<Item>> results(String filePathBean) {
        return minioClient.listObjects(
                io.minio.ListObjectsArgs.builder()
                        .bucket(fileStorageProperties.getBucketName())
                        .prefix(filePathBean)
                        .recursive(true)
                        .build()
        );
    }

    //@Override
    public Page<FileStorageItemVo> page(Page page, FileStorageQueryDto queryDto) {
        List<FileStorageItemVo> list = getFileStorageList();


        Stream<FileStorageItemVo> stream = list.stream();

        String keywords = queryDto.getKeywords();
        if (org.apache.commons.lang3.StringUtils.isNotBlank(keywords)) {
            stream = stream.filter(e -> e.getFileName().contains(keywords));
        }

        final LocalDateTime startTime = queryDto.getStartTime();
        LocalDateTime endTime = queryDto.getEndTime();
        if (endTime != null) {
            endTime = endTime.withNano(999_999_999);
        }
        if (startTime != null && endTime != null) {
            final LocalDateTime finalEndTime = endTime;
            stream = stream.filter(e -> (e.getLastModified().isEqual(startTime) || e.getLastModified().isAfter(startTime))
                    && (e.getLastModified().isEqual(finalEndTime) || e.getLastModified().isBefore(finalEndTime)));
        } else if (startTime != null) {
            stream = stream.filter(e -> (e.getLastModified().isEqual(startTime) || e.getLastModified().isAfter(startTime)));
        } else if (endTime != null) {
            final LocalDateTime finalEndTime = endTime;
            stream = stream.filter(e -> (e.getLastModified().isEqual(finalEndTime) || e.getLastModified().isBefore(finalEndTime)));
        }

        long size = page.getSize();
        long current = page.getCurrent();
        List<FileStorageItemVo> filtered = stream.collect(Collectors.toList());
        Page<FileStorageItemVo> page1 = new PageDTO<>();
        page1.setRecords(filtered.stream().skip((current - 1) * size)
                .limit(size)
                .sorted(Comparator.comparing(FileStorageItemVo::getLastModified).reversed())
                .collect(Collectors.toList()));

        int total = filtered.size();
        long pages = 0;
        if (size > 0) {
            if (total % size == 0) {
                pages = total / size;
            } else {
                pages = total / size + 1;
            }
        }
        page1.setCurrent(current);
        page1.setPages(pages);
        page1.setSize(size);
        page1.setTotal(total);
        return page1;
    }

    private List<FileStorageItemVo> getFileStorageList() {
        Iterable<Result<Item>> results = minioClient.listObjects(
                ListObjectsArgs.builder()
                        .bucket(fileStorageProperties.getBucketName())
                        .recursive(true)
                        .build());

        List<FileStorageItemVo> list = new ArrayList<>();

        results.forEach(e -> {
            try {
                Item item = e.get();
                if (!item.isDir()) {
                    list.add(new FileStorageItemVo(item.objectName(), getFileName(item.objectName()), formatSpaceSize(item.size()), item.size(), item.lastModified().toLocalDateTime()));
                }
            } catch (Exception ex) {
                throw new RuntimeException(ex);
            }
        });
        return list;
    }

    //@Override
    public ResponseEntity<?> download(String id) {
        try {
            GetObjectResponse getObjectResponse =
                    minioClient.getObject(
                            GetObjectArgs.builder()
                                    .bucket(fileStorageProperties.getBucketName())
                                    .object(id)
                                    .build());
            Path path = Files.createTempFile("minio_", ".tmp");
            Files.delete(path);
            Files.copy(getObjectResponse, path, StandardCopyOption.REPLACE_EXISTING);
            return ResponseEntity.ok()
                    .header(HttpHeaders.CONTENT_DISPOSITION, "attachment;filename=" + getFileName(getObjectResponse.object()))
                    .contentType(MediaType.APPLICATION_OCTET_STREAM).body(new FileSystemResource(path.toFile()));
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    //@Override
    public FileStorageStatVo stat() {
        List<FileStorageItemVo> fileStorageList = getFileStorageList();
        int total = fileStorageList.size();
        Long spaceSize = fileStorageList.stream().map(e -> e.getSize()).reduce(0L, (result, item) -> result + item);
        return new FileStorageStatVo(total, formatSpaceSize(spaceSize));
    }

    private String formatSpaceSize(long fileLength) {
        String fileSizeString = null;
        DecimalFormat df = new DecimalFormat("#.00");
        if (fileLength < 1024) {
            fileSizeString = df.format((double) fileLength) + "B";
        } else if (fileLength < 1048576) {
            fileSizeString = df.format((double) fileLength / 1024) + "KB";
        } else if (fileLength < 1073741824) {
            fileSizeString = df.format((double) fileLength / 1048576) + "MB";
        } else {
            fileSizeString = df.format((double) fileLength / 1073741824) + "GB";
        }
        return fileSizeString;
    }

    private String getFileName(String objectName) {
        int i = objectName.lastIndexOf("/");
        if (i >= 0) {
            int beginIndex = i + 1;
            if (beginIndex < objectName.length() - 1) {
                return objectName.substring(beginIndex);
            } else {
                return null;
            }
        }
        return objectName;
    }
}
