package com.tofly.feesapi.security;

import com.tofly.feesapi.security.model.CzyUserDetails;
import com.tofly.feesapi.security.model.token.RawAccessToken;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.authentication.AuthenticationProvider;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.oauth2.common.DefaultOAuth2AccessToken;
import org.springframework.stereotype.Component;

import java.util.LinkedHashSet;
import java.util.Map;

/**
 * Access Token身份认证提供者
 */
@Component
@RequiredArgsConstructor
@Slf4j
public class AccessAuthenticationProvider implements AuthenticationProvider {

    private final RedisTemplate<String, Object> redisTemplate;

    public final static String REDIS_KEY_ACCESS_TOKEN_PREFIX = "tofly_oauth:access";


    /**
     * 进行身份鉴定
     *
     * @param authentication 证明
     * @return 证明
     * @throws AuthenticationException 身份验证异常
     */
    @Override
    public Authentication authenticate(Authentication authentication) throws AuthenticationException {
        RawAccessToken rawAccessToken = (RawAccessToken) authentication.getCredentials();
        CzyUserDetails securityUser = parseAccessToken(rawAccessToken);
        return new AccessAuthenticationToken(securityUser);
    }

    private CzyUserDetails parseAccessToken(RawAccessToken rawAccessToken) {
        String accessToken = rawAccessToken.getToken();
        String key = String.format("%s:%s", REDIS_KEY_ACCESS_TOKEN_PREFIX, accessToken);
        DefaultOAuth2AccessToken defaultOAuth2AccessToken = (DefaultOAuth2AccessToken) redisTemplate.opsForValue().get(key);
        Map<String, Object> additionalInformation = defaultOAuth2AccessToken.getAdditionalInformation();
        long id = (Long) additionalInformation.get("id");
        String username = (String) additionalInformation.get("username");
        String realName = (String) additionalInformation.get("realName");
        LinkedHashSet<GrantedAuthority> authorities = new LinkedHashSet<>();
        CzyUserDetails czyUserDetails = new CzyUserDetails(id, username, realName, "******", authorities, true, true, true, true);
        return czyUserDetails;
    }

    @Override
    public boolean supports(Class<?> authentication) {
        return (AccessAuthenticationToken.class.isAssignableFrom(authentication));
    }
}
