package com.tofly.feesapi.security;

import com.tofly.feesapi.security.extractor.TokenExtractor;
import com.tofly.feesapi.security.model.token.RawAccessToken;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.oauth2.common.DefaultOAuth2AccessToken;
import org.springframework.security.web.authentication.AbstractAuthenticationProcessingFilter;
import org.springframework.security.web.authentication.AuthenticationFailureHandler;
import org.springframework.security.web.util.matcher.RequestMatcher;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

import static com.tofly.feesapi.security.AccessAuthenticationProvider.REDIS_KEY_ACCESS_TOKEN_PREFIX;

public class AccessTokenAuthenticationProcessingFilter extends AbstractAuthenticationProcessingFilter {
    private final AuthenticationFailureHandler failureHandler;
    private final TokenExtractor tokenExtractor;

    private final RedisTemplate<String, Object> redisTemplate;


    @Autowired
    public AccessTokenAuthenticationProcessingFilter(AuthenticationFailureHandler failureHandler,
                                                     TokenExtractor tokenExtractor,
                                                     RedisTemplate<String, Object> redisTemplate,
                                                     RequestMatcher matcher) {
        super(matcher);
        this.failureHandler = failureHandler;
        this.tokenExtractor = tokenExtractor;
        this.redisTemplate = redisTemplate;
    }

    @Override
    protected boolean requiresAuthentication(HttpServletRequest request, HttpServletResponse response) {
        String accessToken = tokenExtractor.extract(request);
        if (accessToken == null) return false;
        String key = String.format("%s:%s",REDIS_KEY_ACCESS_TOKEN_PREFIX, accessToken);
        DefaultOAuth2AccessToken defaultOAuth2AccessToken = (DefaultOAuth2AccessToken) redisTemplate.opsForValue().get(key);
        if (defaultOAuth2AccessToken == null) return false;
        return super.requiresAuthentication(request, response);
    }

    @Override
    public Authentication attemptAuthentication(HttpServletRequest request, HttpServletResponse response)
            throws AuthenticationException {
        RawAccessToken token = new RawAccessToken(tokenExtractor.extract(request));
        return getAuthenticationManager().authenticate(new AccessAuthenticationToken(token));
    }

    @Override
    protected void successfulAuthentication(HttpServletRequest request, HttpServletResponse response, FilterChain chain,
                                            Authentication authResult) throws IOException, ServletException {
        SecurityContext context = SecurityContextHolder.createEmptyContext();
        context.setAuthentication(authResult);
        SecurityContextHolder.setContext(context);
        chain.doFilter(request, response);
    }

    @Override
    protected void unsuccessfulAuthentication(HttpServletRequest request, HttpServletResponse response,
                                              AuthenticationException failed) throws IOException, ServletException {
        SecurityContextHolder.clearContext();
        failureHandler.onAuthenticationFailure(request, response, failed);
    }
}
