package com.tofly.feesapi.sfgl.controller;

import com.alibaba.fastjson.JSONObject;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.feesapi.bjgl.entity.YhCbkxx;
import com.tofly.feesapi.bjgl.service.YhCbkxxService;
import com.tofly.feesapi.common.Excel.ExcleFileUtil;
import com.tofly.feesapi.common.constant.PjztEnum;
import com.tofly.feesapi.common.entity.dto.PageDto;
import com.tofly.feesapi.common.entity.vo.PageResult;
import com.tofly.feesapi.common.enums.BcFylxEnum;
import com.tofly.feesapi.common.exception.FeesErrorCode;
import com.tofly.feesapi.common.exception.FeesException;
import com.tofly.feesapi.common.model.CollectionData;
import com.tofly.feesapi.security.model.CzyUserDetails;
import com.tofly.feesapi.sfgl.entity.YyQfmx;
import com.tofly.feesapi.sfgl.entity.dto.*;
import com.tofly.feesapi.sfgl.entity.vo.SfJsJg;
import com.tofly.feesapi.sfgl.entity.vo.YyQfmxMonthlyStat;
import com.tofly.feesapi.sfgl.entity.vo.YyQfmxSelect;
import com.tofly.feesapi.sfgl.entity.vo.YyStat;
import com.tofly.feesapi.sfgl.service.YyQfmxService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

@RestController
@AllArgsConstructor
@RequestMapping("api/sfgl/yyqfmx")
@Api(tags = "欠费明细")
public class YyQfmxController {

    private final YyQfmxService yyQfmxService;

    private final YhCbkxxService yhCbkxxService;

    @ApiOperationSupport(order = 1)
    @GetMapping("/page")
    @ApiOperation(value = "查询欠费明细（分页）", tags = {"欠费明细变更", "水费补差", "营业厅收费", "综合查询"})
    public ResultRespone<PageResult<YyQfmxSelect>> getPage(@ModelAttribute PageDto pageDto,
                                                           @ModelAttribute YyQfmxFilter filter) {
        return ResultRespone.success(yyQfmxService.getPage(pageDto, filter, "0", PjztEnum.正常.getValue()));
    }

    @ApiOperationSupport(order = 2)
    @GetMapping("/page2")
    @ApiOperation(value = "查询用户抄表信息（分页）", tags = {"综合查询"})
    public ResultRespone<PageResult<YyQfmxSelect>> getPage2(@ModelAttribute PageDto pageDto,
                                                            @ModelAttribute YyQfmxFilter filter) {
        return ResultRespone.success(yyQfmxService.getPage(pageDto, filter, null, null));
    }

    @ApiOperationSupport(order = 3)
    @GetMapping
    @ApiOperation(value = "查询欠费(应收)明细", tags = {"应收明细月报表"})
    public ResultRespone<CollectionData<YyQfmxSelect>> export(@ModelAttribute YyQfmxFilter filter) {
        List<YyQfmxSelect> list = yyQfmxService.getList(filter, "0", PjztEnum.正常.getValue());
        return ResultRespone.success(new CollectionData<>(list, list.size()));
    }

    @ApiOperationSupport(order = 4)
    @GetMapping("/export")
    @ApiOperation(value = "导出欠费(应收)明细", tags = {"应收明细月报表"})
    public void export(HttpServletResponse response, @ModelAttribute YyQfmxFilter filter) throws IOException {
        List<YyQfmxSelect> list = yyQfmxService.getList(filter, "0", PjztEnum.正常.getValue());
        if (!list.isEmpty()) {
            ExcleFileUtil.exportExcel(response, list, YyQfmxSelect.class, "应收明细月报表.xlsx");
        }
    }

    @ApiOperationSupport(order = 5)
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询欠费明细", tags = "欠费明细变更")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "欠费ID", required = true, dataType = "String")
    })
    public ResultRespone<YyQfmx> getById(@PathVariable("id") String id) {
        return ResultRespone.success(yyQfmxService.getById(id));
    }

    @ApiOperationSupport(order = 8)
    @ApiOperation(value = "撤销算费", tags = "欠费明细变更")
    @PostMapping("/{id}/cxsf")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "欠费ID", required = true, dataType = "String")
    })
    public ResultRespone<Boolean> cxsf(@PathVariable("id") String id,
                                       @ApiIgnore @AuthenticationPrincipal CzyUserDetails czyUserDetails) {

        boolean result = yyQfmxService.cxSf(id, czyUserDetails);
        return result ? ResultRespone.success(true, "撤销算费成功") : ResultRespone.failed(false, "撤销算费失败");
    }

    @ApiOperationSupport(order = 9)
    @ApiOperation(value = "修改欠费明细", tags = "欠费明细变更")
    @PutMapping
    public ResultRespone updateById(@RequestBody YyQfmxBg yyQfmxBg,
                                    @ApiIgnore @AuthenticationPrincipal CzyUserDetails czyUserDetails) {
        if (StringUtils.isEmpty(yyQfmxBg.getCbkid()))
            throw new FeesException("抄表卡ID为空", FeesErrorCode.BUSINESS_ERROR);
        YhCbkxx cbkxx = yhCbkxxService.getById(yyQfmxBg.getCbkid());
        if (StringUtils.isEmpty(cbkxx))
            throw new FeesException("水表档案信息不存在", FeesErrorCode.BUSINESS_ERROR);
        Sfcs sfcs = new Sfcs();
        sfcs.setCbkid(yyQfmxBg.getCbkid());
        sfcs.setFylx(BcFylxEnum.BcSf.getValue().toString());
        sfcs.setSjbm(yyQfmxBg.getSjbm());
        sfcs.setSl(yyQfmxBg.getSl());
        sfcs.setSjbm1(yyQfmxBg.getSjbm1());
        sfcs.setSl1(yyQfmxBg.getSl1());
        sfcs.setSjbm2(yyQfmxBg.getSjbm2());
        sfcs.setSl2(yyQfmxBg.getSl2());
        sfcs.setLjfbz(cbkxx.getLjfbz());
        sfcs.setWsfbz(cbkxx.getWsfbz());
        if (!StringUtils.isEmpty(sfcs.getSjbm1()) && !StringUtils.isEmpty(sfcs.getSl1())) {
            sfcs.setYbdjbz("1");
        }
        sfcs.setJtzqljl(yyQfmxBg.getJtzqljlJsq());
        SfJsJg sfJsJg = yyQfmxService.sfCs(sfcs, czyUserDetails);

        Set<String> stringSet = new HashSet<>();
        stringSet.add("jsrq");
        stringSet.add("lrrq");
        stringSet.add("cbrq");
        stringSet.add("bz");
        if (!areFieldsEqual(sfJsJg, yyQfmxBg, stringSet)) {
            throw new FeesException("修改欠费明细失败，验证信息不匹配！", FeesErrorCode.BUSINESS_ERROR);
        }
        YyQfmx yyQfmx = JSONObject.parseObject(JSONObject.toJSONString(yyQfmxBg), YyQfmx.class);
        return ResultRespone.success(yyQfmxService.updateById(yyQfmx));
    }


    @ApiOperationSupport(order = 9)
    @ApiOperation(value = "阶梯水费计算", tags = "欠费明细变更")
    @PostMapping("jtsfjs")
    public ResultRespone jtSfjs(@RequestBody Jtsfjs jtsfjs,
                                @ApiIgnore @AuthenticationPrincipal CzyUserDetails czyUserDetails) {
        SfJsJg sfJsJg = yyQfmxService.jtSfjs(jtsfjs, czyUserDetails);
        return ResultRespone.success(sfJsJg);
    }

    public static boolean areFieldsEqual(Object obj1, Object obj2, Set<String> excludeFields) {
        if (obj1 == null || obj2 == null) {
            throw new FeesException("参数对象不能为空！", FeesErrorCode.BUSINESS_ERROR);
        }

        Class<?> clazz1 = obj1.getClass();
        Class<?> clazz2 = obj2.getClass();

        Field[] fields1 = clazz1.getDeclaredFields();
        Field[] fields2 = clazz2.getDeclaredFields();

        Set<String> commonFieldNames = new HashSet<>();
        for (Field field1 : fields1) {
            for (Field field2 : fields2) {
                if (field1.getName().equals(field2.getName()) && field1.getClass().equals(field2.getClass())) {
                    commonFieldNames.add(field1.getName());
                    break;
                }
            }
        }
        for (String fieldName : commonFieldNames) {
            if (excludeFields != null && excludeFields.contains(fieldName)) {
                continue;
            }
            try {
                Field field1 = clazz1.getDeclaredField(fieldName);
                Field field2 = clazz2.getDeclaredField(fieldName);

                field1.setAccessible(true);
                field2.setAccessible(true);

                Object value1 = field1.get(obj1);
                Object value2 = field2.get(obj2);

                if (StringUtils.isEmpty(value1) && StringUtils.isEmpty(value2)) {
                    continue;
                }
                if ((value1 == null && value2 != null && value2.equals(BigDecimal.ZERO))
                        || (value1 != null && value1.equals(BigDecimal.ZERO) && value2 == null)) {
                    continue;
                }
                if (value1.getClass() == BigDecimal.class) {
                    if (!(((BigDecimal) value1).compareTo((BigDecimal) value2) == 0)) {
                        System.out.println("验证算费参数失败字段：" + fieldName);
                        return false;
                    }
                } else {
                    if (!value1.equals(value2)) {
                        System.out.println("验证算费参数失败字段：" + fieldName);
                        return false;
                    }
                }
            } catch (NoSuchFieldException | IllegalAccessException e) {
                e.printStackTrace();
                return false;
            }
        }
        return true;
    }


//    @ApiOperationSupport(order = 10)
//    @ApiOperation(value = "生成欠费明细（按抄表本（册）算费）", tags = {"抄表算费"})
//    @PostMapping("ccbsf/{cbbbm}")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "cbbbm", value = "抄表本（册）编码,多个逗号分隔", required = true, dataType = "String")
//    })
//    public ResultRespone<Boolean> sfByCbb(@PathVariable String cbbbm) {
//        yyQfmxService.sfByCbb(cbbbm);
//        return ResultRespone.success(true);
//    }
//
//    @ApiOperationSupport(order = 11)
//    @ApiOperation(value = "生成欠费明细（按用户算费）", tags = {"抄表算费"})
//    @PostMapping("yhsf/{cbkid}")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "cbkid", value = "抄表卡号,多个逗号分隔", required = true, dataType = "String")
//    })
//    public ResultRespone<Boolean> sfByYh(@PathVariable String cbkid) {
//        yyQfmxService.sfByYh(cbkid);
//        return ResultRespone.success(true);
//    }

    @ApiOperationSupport(order = 14)
    @ApiOperation(value = "生成欠费明细（按抄表本（册）算费）", tags = {"抄表算费"})
    @PostMapping("ccbsf")
    public ResultRespone<Boolean> cbbSf(@RequestBody CbbSf cbbSf,
                                        @ApiIgnore @AuthenticationPrincipal CzyUserDetails czyUserDetails) {
        boolean result = yyQfmxService.cbbSf(cbbSf, czyUserDetails);
        return result ? ResultRespone.success(true) : ResultRespone.failed(false);
    }

    @ApiOperationSupport(order = 15)
    @ApiOperation(value = "生成欠费明细（按用户算费）", tags = {"抄表算费"})
    @PostMapping("yhsf")
    public ResultRespone<Boolean> yhSf(@RequestBody YhSf yhSf,
                                       @ApiIgnore @AuthenticationPrincipal CzyUserDetails czyUserDetails) {
        boolean result = yyQfmxService.yhSf(yhSf, czyUserDetails);
        return result ? ResultRespone.success(true, "算费成功") : ResultRespone.failed(false, "算费失败");
    }


    @ApiOperationSupport(order = 17)
    @ApiOperation(value = "水费测算", tags = {"水费补差"})
    @PostMapping("sfjs")
    public ResultRespone<SfJsJg> sfJs(@RequestBody Sfcs sfcs,
                                      @ApiIgnore @AuthenticationPrincipal CzyUserDetails czyUserDetails) {
        if (StringUtils.isEmpty(sfcs.getCbkid())) {
            throw new FeesException("抄表卡ID不能为空", FeesErrorCode.BUSINESS_ERROR);
        }
        if (StringUtils.isEmpty(sfcs.getYsyf())) {
            throw new FeesException("费用月份不能为空", FeesErrorCode.BUSINESS_ERROR);
        }
        if (StringUtils.isEmpty(sfcs.getFylx())) {
            throw new FeesException("费用类型不能为空", FeesErrorCode.BUSINESS_ERROR);
        }
        if (StringUtils.isEmpty(sfcs.getSjbm())) {
            throw new FeesException("水价1不能为空", FeesErrorCode.BUSINESS_ERROR);
        }
        if (sfcs.getYbdjbz().equals("1")) {
            if (StringUtils.isEmpty(sfcs.getSjbm1())) {
                throw new FeesException("水价2不能为空", FeesErrorCode.BUSINESS_ERROR);
            }
        }
        SfJsJg sfJsJg = yyQfmxService.sfCs(sfcs, czyUserDetails);
        return ResultRespone.success(sfJsJg);
    }

    @ApiOperationSupport(order = 18)
    @ApiOperation(value = "水费补差", tags = {"水费补差"})
    @PostMapping("sfbc")
    public ResultRespone<Boolean> sfBc(@RequestBody SfBc sfBc,
                                       @ApiIgnore @AuthenticationPrincipal CzyUserDetails czyUserDetails) {
        boolean result = yyQfmxService.sfBc(sfBc, czyUserDetails);
        return result ? ResultRespone.success(true) : ResultRespone.failed(false);
    }

    @ApiOperationSupport(order = 20)
    @GetMapping("/monthlyStat")
    @ApiOperation(value = "应收汇总月报表", tags = {"应收汇总月报表"})
    public ResultRespone<CollectionData<YyQfmxMonthlyStat>> monthlyStat(@ModelAttribute YyQfmxStatFilter filter) {
        List<YyQfmxMonthlyStat> list = yyQfmxService.monthlyStat(filter);
        return ResultRespone.success(new CollectionData<>(list, list.size()));
    }


    @ApiOperationSupport(order = 21)
    @GetMapping("/stat")
    @ApiOperation(value = "欠费信息统计", tags = {"综合查询"})
    public ResultRespone<YyStat> stat(@ModelAttribute YyQfmxStatFilter filter) {
        YyStat stat = yyQfmxService.stat(filter, "0");
        return ResultRespone.success(stat);
    }

    @ApiOperationSupport(order = 21)
    @GetMapping("/stat2")
    @ApiOperation(value = "抄表信息统计", tags = {"综合查询"})
    public ResultRespone<YyStat> stat2(@ModelAttribute YyQfmxStatFilter filter) {
        YyStat stat = yyQfmxService.stat(filter, null);
        return ResultRespone.success(stat);
    }

    @ApiOperationSupport(order = 22)
    @PostMapping("/pjyl")
    @ApiOperation(value = "票据预览", tags = {"营业厅收费"})
    public ResultRespone pjyl(@RequestBody pjYl pjyl,
                              @ApiIgnore @AuthenticationPrincipal CzyUserDetails czyUserDetails) {
        return ResultRespone.success(yyQfmxService.pjyl(pjyl, czyUserDetails));
    }

}
