package com.tofly.feesapi.sfgl.controller;

import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.feesapi.common.Excel.ExcleFileUtil;
import com.tofly.feesapi.common.dbhelper.DbHelper;
import com.tofly.feesapi.common.entity.dto.PageDto;
import com.tofly.feesapi.common.entity.vo.PageResult;
import com.tofly.feesapi.common.model.CollectionData;
import com.tofly.feesapi.common.model.ProcParamModel;
import com.tofly.feesapi.security.model.CzyUserDetails;
import com.tofly.feesapi.sfgl.entity.YySspjxx;
import com.tofly.feesapi.sfgl.entity.dto.*;
import com.tofly.feesapi.sfgl.entity.vo.*;
import com.tofly.feesapi.sfgl.service.YySspjxxService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang.StringUtils;
import org.apache.ibatis.mapping.ParameterMode;
import org.springframework.security.core.annotation.AuthenticationPrincipal;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.sql.JDBCType;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@RestController
@AllArgsConstructor
@RequestMapping("/yysspjxx")
@Api(tags = "收费明细")
public class YySspjxxController {

    private final YySspjxxService yySspjxxService;

    private final static String SFFL_DESC = "收费分类1预存抵扣;2欠费收取(含预存抵扣);3预付费收取;4余额转存;7预存转入（含预付费收取、余额转存转入）;8预存转出（含预存抵扣、余额转存转出）;";

    @ApiOperationSupport(order = 2)
    @ApiOperation(value = "收费(新增收费明细)", tags = {"营业厅收费"})
    @PostMapping
    public ResultRespone save(@RequestBody YySspjxxInsert insert,
                              @ApiIgnore @AuthenticationPrincipal CzyUserDetails czyUserDetails) throws Exception {
        return ResultRespone.success(yySspjxxService.save(insert, czyUserDetails));
    }

    @ApiOperationSupport(order = 4)
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询收费明细")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "实时ID", required = true, dataType = "String")
    })
    public ResultRespone<YySspjxx> getById(@PathVariable("id") String id) {
        return ResultRespone.success(yySspjxxService.getById(id));
    }

    @ApiOperationSupport(order = 5)
    @GetMapping(value = "/page")
    @ApiOperation(value = "查询收费明细(分页)", tags = {"营业厅收费", "综合查询"})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sffl", value = SFFL_DESC, dataType = "String", defaultValue = "2"),
    })
    public ResultRespone<PageResult<YySspjxxSelect>> getPage(PageDto pageDto,
                                                             @ModelAttribute YySspjxxFilter filter,
                                                             @RequestParam(value = "sffl", required = false, defaultValue = "2") String sffl,
                                                             @RequestParam Map<String, String> params) {
        if (params.containsKey("chbz")) {//参数适配
            String sflx = params.get("sflx");
            String chbz = params.get("chbz");
            return getPageWithCh(pageDto, filter, sflx, StringUtils.isNotBlank(chbz) ? chbz : null);//收费票据管理
        }
        return ResultRespone.success(yySspjxxService.getPage(pageDto, filter, sffl, "0"));
    }

    @ApiOperationSupport(order = 6)
    @GetMapping
    @ApiOperation(value = "查询收费明细", tags = {"收费明细报表", "第三方明细报表"})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sffl", value = SFFL_DESC, dataType = "String"),
    })
    public ResultRespone<CollectionData<YySspjxxSelect>> getList(@ModelAttribute YySspjxxFilter filter,
                                                                 @RequestParam(value = "sffl", required = false) String sffl) {
        List<YySspjxxSelect> list = yySspjxxService.getList(filter, sffl, "0");
        return ResultRespone.success(new CollectionData<>(list, list.size()));
    }

    @ApiOperationSupport(order = 7)
    @GetMapping("/export")
    @ApiOperation(value = "导出收费明细", tags = "收费明细报表")
    public void export(HttpServletResponse response,
                       @ModelAttribute YySspjxxFilter filter,
                       @RequestParam("sffl") String sffl,
                       @RequestParam("chbz") String chbz) throws IOException {
        List<YySspjxxSelect> list = yySspjxxService.getList(filter, sffl, chbz);
        if (!list.isEmpty()) {
            ExcleFileUtil.exportExcel(response, list, YySspjxxSelect.class, "收费明细报表.xlsx");
        }
    }

    @ApiOperationSupport(order = 8)
    @GetMapping(value = "/page2")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sffl", value = SFFL_DESC, dataType = "String"),
            @ApiImplicitParam(name = "chbz", value = "票据状态（冲红标志）：0未冲红(正常)，1已冲红，2红票（被冲红）", dataType = "String")
    })
    @ApiOperation(value = "收费票据管理", tags = {"收费票据管理"})
    public ResultRespone<PageResult<YySspjxxSelect>> getPageWithCh(PageDto pageDto,
                                                                   @ModelAttribute YySspjxxFilter filter,
                                                                   @RequestParam(value = "sffl", required = false) String sffl,
                                                                   @RequestParam(value = "chbz", required = false) String chbz) {
        return ResultRespone.success(yySspjxxService.getPage(pageDto, filter, sffl, chbz));
    }

    @ApiOperationSupport(order = 10)
    @GetMapping("/dailyStat2")
    @ApiOperation(value = "按日汇总报表", tags = {"按日汇总报表", "第三方按日统计报表"})
    public ResultRespone<CollectionData<YySspjxxDailyStat2>> dailyStat2(@ModelAttribute YySspjxxStatFilter filter) {
        List<YySspjxxDailyStat2> list = yySspjxxService.dailyStat2(filter);
        return ResultRespone.success(new CollectionData<>(list, list.size()));
    }

    @ApiOperationSupport(order = 11)
    @ApiOperation(value = "今日统计", tags = {"营业厅收费"})
    @GetMapping("/jrtj/{sfrybm}")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sfrybm", value = "收费人员编码", required = true, dataType = "String")
    })
    public ResultRespone<CollectionData<JrtjSelect>> jrtj(@PathVariable("sfrybm") String sfrybm) {
        List<JrtjSelect> list = yySspjxxService.jrtj(sfrybm);
        return ResultRespone.success(new CollectionData<>(list, list.size()));
    }

    @ApiOperationSupport(order = 12)
    @ApiOperation(value = "上笔金额", tags = {"营业厅收费"})
    @GetMapping("/sbje/{yhbh}")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "yhbh", value = "用户编号", required = true, dataType = "String")
    })
    public ResultRespone<BigDecimal> sbje(@PathVariable("yhbh") String yhbh) {
        return ResultRespone.success(yySspjxxService.sbje(yhbh));
    }

    @ApiOperationSupport(order = 13)
    @ApiOperation(value = "票据冲红", tags = {"收费票据管理"}, notes = "--***0（失败）;1（成功）\n" +
            "    --***2（收费金额不匹配）;\n" +
            "    --***3（不是最新缴费记录）;\n" +
            "    --***4（预付费余额已发生变化或剩余预付费不够退费）;\n" +
            "    --***5（收费转存的预付费不能单独退费）\n" +
            "    --***9（没有收费记录）\n" +
            "    --***-6生成电子发票（红票错误）")
    @PostMapping("/pjch")
    public ResultRespone<Integer> pjCh(@RequestBody PjCh pjch,
                                       @ApiIgnore @AuthenticationPrincipal CzyUserDetails czyUserDetails) {
        Integer result = yySspjxxService.pjCh(pjch, czyUserDetails);
        if (result == null) {
            return ResultRespone.failed("冲红失败");
        } else if (result == 1) {
            return ResultRespone.failed(result, "冲红成功");
        } else {
            return ResultRespone.failed(result, String.format("冲红失败，错误代码:%s", result));
        }
    }

    @ApiOperationSupport(order = 14)
    @GetMapping("/dailyStat")
    @ApiOperation(value = "收费日结报表", tags = "收费日结报表")
    public ResultRespone<YySspjxxDailyStat> dailyStat(@ModelAttribute YySspjxxDailyStatFilter filter) {
        return ResultRespone.success(yySspjxxService.dailyStat(filter));
    }


    @ApiOperationSupport(order = 15)
    @GetMapping("/monthlyStat")
    @ApiOperation(value = "实收月报表", tags = {"实收月报表"})
    public ResultRespone<CollectionData<YySspjxxMonthlyStat>> monthlyStat(@ModelAttribute YySspjxxStatFilter filter) {
        List<YySspjxxMonthlyStat> list = yySspjxxService.monthlyStat(filter);
        return ResultRespone.success(new CollectionData<>(list, list.size()));
    }

    @ApiOperationSupport(order = 20)
    @GetMapping("/zhSfStat")
    @ApiOperation(value = "综合收费报表", tags = {"综合收费报表"})
    public ResultRespone<CollectionData<YySspjxxZhSfStat>> zhSfStat(@ModelAttribute YySspjxxStatFilter filter) {
        List<YySspjxxZhSfStat> list = yySspjxxService.zhSfStat(filter);
        return ResultRespone.success(new CollectionData<>(list, list.size()));
    }

    @ApiOperationSupport(order = 22)
    @GetMapping("/zhSfStat/export")
    @ApiOperation(value = "导出综合收费报表", tags = {"综合收费报表"})
    public void zhSfStatExport(HttpServletResponse response, @ModelAttribute YySspjxxStatFilter filter) throws IOException {
        List<YySspjxxZhSfStat> list = yySspjxxService.zhSfStat(filter);
        if (!list.isEmpty()) {
            ExcleFileUtil.exportExcel(response, list, YySspjxxZhSfStat.class, "综合收费报表.xlsx");
        }
    }

    @ApiOperationSupport(order = 23)
    @GetMapping("/stat")
    @ApiOperation(value = "用户缴费信息统计", tags = {"综合查询"})
    public ResultRespone<YyStat> stat(@ModelAttribute YySspjxxStatFilter filter) {
        YyStat stat = yySspjxxService.stat(filter);
        return ResultRespone.success(stat);
    }

    @ApiOperationSupport(order = 24)
    @GetMapping("/yffyeseach")
    @ApiOperation(value = "预付费余额查询", tags = {"预付费余额"})
    public ResultRespone<PageResult<YffYeSeachSelect>> yffyeseach(PageDto pageDto, @ModelAttribute YffYeSeachFilter filter) {
        PageResult<YffYeSeachSelect> yffYeSeachSelectPageDto = yySspjxxService.yffyeseach(pageDto, filter);
        return ResultRespone.success(yffYeSeachSelectPageDto);
    }


    @ApiOperationSupport(order = 25)
    @GetMapping("/exportyffye")
    @ApiOperation(value = "导出预付费余额明细", tags = "预付费余额")
    public void export(HttpServletResponse response,
                       @ModelAttribute YffYeSeachFilter filter) throws IOException {
        List<YffYeSeachSelect> list = yySspjxxService.yffyeseachList(filter);
        if (!list.isEmpty()) {
            ExcleFileUtil.exportExcel(response, list, YffYeSeachSelect.class, "预付费余额列表.xlsx");
        }
    }

    @ApiOperationSupport(order = 25)
    @GetMapping("/printpj/{ssid}")
    @ApiOperation(value = "票据打印", tags = "预付费余额")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ssid", value = "实收ID", required = true, dataType = "String")
    })
    public ResultRespone PrintPj(@PathVariable("ssid") String ssid) {
        return ResultRespone.success(yySspjxxService.printPj(ssid));
    }
//    @ApiOperationSupport(order = 26)
//    @PostMapping("/ChargeCh")
//    @ApiOperation(value = "[Easy]冲红")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "V_YHBH", value = "用户编号", required = true, dataType = "String"),
//            @ApiImplicitParam(name = "V_SFLSBH", value = "收费流水编号", required = true, dataType = "NUMBER")
//    })
//    public ResultRespone ChargeCh(@RequestParam("V_YHBH") String V_YHBH,
//                                  @RequestParam("V_SFLSBH") String V_SFLSBH) throws Exception {
//        DbHelper dbHelper = DbHelper.getDbHelper();
//        ResultSet rs=dbHelper.executeQuery(String.format("select sum(sszje) as sszje from yy_sspjxx where sflsbh='%s' ",V_SFLSBH));
//
//        double v_ssje=0;
//        while (rs.next()){
//            v_ssje=rs.getDouble("sszje");
//        }
//        List<ProcParamModel> paramModels = new ArrayList<>();
//        paramModels.add(new ProcParamModel("V_YHBH", ParameterMode.IN, JDBCType.VARCHAR, V_YHBH));
//        paramModels.add(new ProcParamModel("V_SFLSBH", ParameterMode.IN, JDBCType.VARCHAR, V_SFLSBH));
//        paramModels.add(new ProcParamModel("V_SSJE", ParameterMode.IN, JDBCType.DOUBLE, v_ssje));
//        paramModels.add(new ProcParamModel("V_CHRYBM", ParameterMode.IN, JDBCType.DOUBLE, "admin"));
//        paramModels.add(new ProcParamModel("V_CHRYXM", ParameterMode.IN, JDBCType.DOUBLE, "admin"));
//        paramModels.add(new ProcParamModel("V_CHSJ", ParameterMode.IN, JDBCType.DOUBLE, new Date()));
//        paramModels.add(new ProcParamModel("V_SFCG", ParameterMode.OUT, JDBCType.VARCHAR, "-1"));
//
//        Map mp = dbHelper.executeProcedure("PROC_SF_CH", paramModels);
//        if(mp.get("V_SFCG").equals("1")){
//            return  ResultRespone.success("成功");
//        }
//        else if(mp.get("V_SFCG").equals("2"))
//        {
//            return  ResultRespone.failed("收费金额不匹配");
//        }
//        else if(mp.get("V_SFCG").equals("3"))
//        {
//            return  ResultRespone.failed("不是最新缴费记录");
//        }
//        else if(mp.get("V_SFCG").equals("4"))
//        {
//            return  ResultRespone.failed("预付费余额已发生变化或剩余预付费不够退费");
//        }
//        else if(mp.get("V_SFCG").equals("5"))
//        {
//            return  ResultRespone.failed("收费转存的预付费不能单独退费");
//        }
//        else if(mp.get("V_SFCG").equals("9"))
//        {
//            return  ResultRespone.failed("没有收费记录");
//        }else {
//            return  ResultRespone.failed("异常信息");
//        }
//    }

    /**
     * 修改收费明细
     * @param yySspjxx 收费明细
     * @return ResultRespone
     */
//    @ToFlyAppLog(title="修改收费明细" )
//    @ApiOperation(value = "修改收费明细")
//    @PutMapping
//    public ResultRespone updateById(@RequestBody YySspjxx yySspjxx) {
//      return ResultRespone.success(yySspjxxService.updateById(yySspjxx));
//    }

    /**
     * 通过id删除收费明细
     * @param fylx id
     * @return ResultRespone
     */
//    @ToFlyAppLog(title="通过id删除收费明细" )
//    @ApiOperation(value = "通过ID删除收费明细")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "fylx", value = "ID", required = true, dataType = "String")
//    })
//    @DeleteMapping("/{fylx}" )
//    public ResultRespone removeById(@PathVariable String fylx) {
//      return ResultRespone.success(yySspjxxService.removeById(fylx));
//    }

    /**
     * 通过id批量删除收费明细
     * @param ids id
     * @return ResultRespone
     */
//    @ToFlyAppLog(title="通过id批量删除收费明细" )
//    @ApiOperation(value = "通过ID批量删除收费明细")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
//    })
//    @DeleteMapping("/deleteByIds" )
//    public ResultRespone removeByIds(String ids) {
//        return ResultRespone.success(yySspjxxService.removeByIds(Arrays.asList(ids.split(","))));
//    }
}
