package com.tofly.feesapi.sfgl.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.NumberHelper;
import com.tofly.feesapi.bjgl.entity.YhCbkxx;
import com.tofly.feesapi.bjgl.service.YhCbkxxService;
import com.tofly.feesapi.bmgl.entity.BmCbb;
import com.tofly.feesapi.bmgl.entity.BmFylx;
import com.tofly.feesapi.bmgl.entity.BmSj;
import com.tofly.feesapi.bmgl.service.BmCbbService;
import com.tofly.feesapi.bmgl.service.BmSjService;
import com.tofly.feesapi.bmgl.service.IBmFylxService;
import com.tofly.feesapi.cbgl.entity.YyCblr;
import com.tofly.feesapi.cbgl.service.YyCblrService;
import com.tofly.feesapi.common.constant.*;
import com.tofly.feesapi.common.dbhelper.DbHelper;
import com.tofly.feesapi.common.entity.dto.Order;
import com.tofly.feesapi.common.entity.dto.PageDto;
import com.tofly.feesapi.common.entity.vo.PageResult;
import com.tofly.feesapi.common.enums.BcFylxEnum;
import com.tofly.feesapi.common.enums.PjlxEnum;
import com.tofly.feesapi.common.exception.FeesErrorCode;
import com.tofly.feesapi.common.exception.FeesException;
import com.tofly.feesapi.security.model.CzyUserDetails;
import com.tofly.feesapi.sfgl.entity.YyQfmx;
import com.tofly.feesapi.sfgl.entity.dto.*;
import com.tofly.feesapi.sfgl.entity.vo.SfJsJg;
import com.tofly.feesapi.sfgl.entity.vo.YyQfmxMonthlyStat;
import com.tofly.feesapi.sfgl.entity.vo.YyQfmxSelect;
import com.tofly.feesapi.sfgl.entity.vo.YyStat;
import com.tofly.feesapi.sfgl.mapper.YyQfmxMapper;
import com.tofly.feesapi.sfgl.service.YyQfmxService;
import com.tofly.feesapi.xtgl.service.ISysGdcsbService;
import com.tofly.feesapi.yhgl.entity.YhKhjbxx;
import com.tofly.feesapi.yhgl.service.YhKhjbxxService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.DefaultTransactionDefinition;
import org.springframework.transaction.support.TransactionTemplate;

import java.lang.reflect.InvocationTargetException;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * 应收明细：欠费明细
 */
@Slf4j
@Service("yyQfmxService")
public class YyQfmxServiceImpl extends ServiceImpl<YyQfmxMapper, YyQfmx> implements YyQfmxService {

    @Autowired
    private YyCblrService cblrService;
    @Autowired
    private BmCbbService cbbService;
    @Autowired
    private YhCbkxxService cbkxxService;

    @Autowired
    private YhKhjbxxService khjbxxService;

    @Autowired
    private BmSjService sjService;

    @Autowired
    private IBmFylxService fylxService;

    @Autowired
    private ISysGdcsbService gdcsbService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Override
    public void sf() {

        List<BmCbb> cbbs = cbbService.lambdaQuery().select(BmCbb::getSsgs, BmCbb::getSscbpq, BmCbb::getCbbbm).list();
        List<String> cbbbms = cbbs.stream().sorted(Comparator.comparing(BmCbb::getSsgs)
                        .thenComparing(BmCbb::getSscbpq))
                .map(e -> e.getCbbbm())
                .collect(Collectors.toList());
        CbbSf cbbSf = new CbbSf();
        cbbSf.setCbbs(cbbbms);
        cbbSf(cbbSf, null);
    }

    @Override
    public PageResult<YyQfmxSelect> getPage(PageDto pageDto, YyQfmxFilter filter, String sfbz, String chbz) {
        List<Order> orders = pageDto.getOrders();
        if (orders.isEmpty()) {
            orders.add(new Order("jsrq", false));
        }
        return this.baseMapper.getPage(pageDto, filter, sfbz, chbz).toVo();
    }

    @Override
    public boolean cbbSf(CbbSf cbbSf, CzyUserDetails czyUserDetails) {
        List<String> cbbs = cbbSf.getCbbs();
        if (cbbs == null || cbbs.isEmpty()) throw new FeesException("抄表本编码为空", FeesErrorCode.BUSINESS_ERROR);
        List<YyCblr> list = cblrService.lambdaQuery()
                .in(YyCblr::getSscbb, cbbs)
                .eq(YyCblr::getCbbz, CbbzEnum.已抄表)
                .eq(YyCblr::getJsbz, JsbzEnum.未算费)
                .ge(YyCblr::getSl, BigDecimal.ZERO).list();
        if (list.isEmpty()) throw new FeesException("可算费用户不存在", FeesErrorCode.BUSINESS_ERROR);
        return sf(list) == 1;
    }

    @Override
    public boolean yhSf(YhSf yhSf, CzyUserDetails czyUserDetails) {
        List<String> cbkids = yhSf.getCbkids();
        if (cbkids == null || cbkids.isEmpty()) throw new FeesException("抄表卡ID为空", FeesErrorCode.BUSINESS_ERROR);
        List<YyCblr> list = cblrService.lambdaQuery()
                .in(YyCblr::getCbkid, cbkids).list();
        if (list.isEmpty()) throw new FeesException("可算费用户不存在", FeesErrorCode.BUSINESS_ERROR);
        for (YyCblr e : list) {
            if (e.getCbbz() == null || e.getCbbz() == CbbzEnum.未抄表 || e.getSl() == null)
                throw new FeesException(String.format("未抄表:%s", e.getCbkid()), FeesErrorCode.BUSINESS_ERROR);
            if (e.getJsbz() == JsbzEnum.已算费)
                throw new FeesException(String.format("已算费:%s", e.getCbkid()), FeesErrorCode.BUSINESS_ERROR);
        }
        return sf(list) == 1;
    }

    @Override
    @Transactional
    public boolean sfBc(SfBc sfBc, CzyUserDetails czyUserDetails) {
        String id = sfBc.getId();
        String fylx = sfBc.getFylx();
        String bz = sfBc.getBz();
        if (StringUtils.isBlank(id)) throw new FeesException("欠费明细ID为空", FeesErrorCode.BUSINESS_ERROR);
        if (StringUtils.isBlank(bz)) throw new FeesException("补差备注ID为空", FeesErrorCode.BUSINESS_ERROR);
        BmFylx bmFylx = fylxService.getById(fylx);
        if (bmFylx == null) throw new FeesException("费用类型不存在", FeesErrorCode.BUSINESS_ERROR);
        if (StringUtils.isBlank(fylx)) throw new FeesException("费用类型为空", FeesErrorCode.BUSINESS_ERROR);
        YyQfmx yyQfmx = this.getById(id);
        if (yyQfmx == null) throw new FeesException("欠费明细不存在", FeesErrorCode.BUSINESS_ERROR);
        String qflsbh = yyQfmx.getQflsbh();
        String sfbc = yyQfmx.getSfbc();
        YyQfmx bcQfmx = new YyQfmx();
        bcQfmx.setQflsbh(qflsbh);
        bcQfmx.setYhbh(yyQfmx.getYhbh());
        bcQfmx.setYhxm(yyQfmx.getYhxm());
        bcQfmx.setCbkh(yyQfmx.getCbkh());
        bcQfmx.setCbkid(yyQfmx.getCbkid());
        bcQfmx.setSsgs(yyQfmx.getSsgs());
        bcQfmx.setSscbpq(yyQfmx.getSscbpq());
        bcQfmx.setSscbb(yyQfmx.getSscbb());
        bcQfmx.setCwrq(yyQfmx.getCwrq());
        bcQfmx.setYsyf(yyQfmx.getYsyf());
        bcQfmx.setFylx(fylx);
        bcQfmx.setFylxmc(bmFylx.getName());

        Date now = new Date();
        bcQfmx.setJsrq(now);
        bcQfmx.setLrrq(now);
        bcQfmx.setCbrq(now);
        if (czyUserDetails != null) {
            bcQfmx.setCbrybm(String.valueOf(czyUserDetails.getId()));
            bcQfmx.setCbryxm(czyUserDetails.getRealName());
        }
        BeanUtils.copyProperties(sfbc, bcQfmx, "id");
        BeanUtils.copyProperties(sfBc.getSfJsJg(), bcQfmx);
        bcQfmx.setSfbc("1");
//        this.lambdaUpdate()
//                .set(YyQfmx::getSfbc, "1")
//                .eq(YyQfmx::getId, id).update();
        return this.save(bcQfmx);
    }

    @Override
    public SfJsJg sfJs(SfJs sfJs, CzyUserDetails czyUserDetails) {
        String cbkid = sfJs.getCbkid();
        BigDecimal sjsl = sfJs.getSjsl();
        if (cbkid == null) throw new FeesException("抄表卡ID为空", FeesErrorCode.BUSINESS_ERROR);
        List<YhCbkxx> list = cbkxxService.lambdaQuery()
                .eq(YhCbkxx::getCbkid, cbkid).list();
        if (list.isEmpty()) throw new FeesException("抄表卡不存在", FeesErrorCode.BUSINESS_ERROR);
        sjCache();//缓存表价信息（计算前必须）
        return sfJs0(list.get(0), sjsl, sjsl);
    }

    @Override
    public SfJsJg sfCs(Sfcs sfcs, CzyUserDetails czyUserDetails) {
        SfJsJg jsJg = new SfJsJg();
        if (sfcs.getFylx().equals(BcFylxEnum.BcLjclf.getValue()) || sfcs.getFylx().equals(BcFylxEnum.BcWsclf.getValue())) {
            jsJg.setYsje(BigDecimal.valueOf(0.00));
            jsJg.setYssl(BigDecimal.valueOf(0));
            jsJg.setYssf(BigDecimal.valueOf(0.00));
            jsJg.setYswsf(BigDecimal.valueOf(0.00));
            jsJg.setYsljf(BigDecimal.valueOf(0.00));
            jsJg.setBcje(jsJg.getYsje().subtract(sfcs.getYje()));
            jsJg.setBcsl(jsJg.getYssl().subtract(sfcs.getYsl()));
            jsJg.setBcsf(jsJg.getYssf().subtract(sfcs.getYsf()));
            jsJg.setBcwsf(jsJg.getYswsf().subtract(sfcs.getYwsf()));
            jsJg.setBcljf(jsJg.getYsljf().subtract(sfcs.getYljf()));
        } else {
            TreeMap<String, String> slMap = new TreeMap<>();
            if (sfcs.getYbdjbz().equals("1")) {
                slMap.put("L1", String.format("%s_%s", sfcs.getSjbm(), sfcs.getSl()));
                if (StringUtils.isNotEmpty(sfcs.getSjbm1())) {
                    slMap.put("L2", String.format("%s_%s", sfcs.getSjbm1(), sfcs.getSl1()));
                }
                if (StringUtils.isNotEmpty(sfcs.getSjbm2())) {
                    slMap.put("L3", String.format("%s_%s", sfcs.getSjbm2(), sfcs.getSl2()));
                }
            } else {
                slMap.put("L0", String.format("%s_%s", sfcs.getSjbm(), sfcs.getSl()));
            }
            Date now = new Date();
            jsJg.setJsrq(now);
            String wsfbz = sfcs.getWsfbz();
            String ljfbz = sfcs.getLjfbz();

            sjCache();//缓存表价信息（计算前必须）

            Map<String, Object> values = new LinkedHashMap<>();
            slMap.forEach((key, value) -> {
                if (key.equals("L0") || key.equals("L1")) {
                    String sjTag = "";
                    String sjTag2 = "";
                    Map<String, Object> values1 = getValues(value, sfcs.getJtzqljl(), wsfbz, ljfbz, sjTag, sjTag2, sfcs.getYhbh());
                    values.putAll(values1);

                } else if (key.equals("L2")) {
                    String sjTag = "sj1";
                    String sjTag2 = "1";
                    Map<String, Object> values1 = getValues(value, sfcs.getJtzqljl(), wsfbz, ljfbz, sjTag, sjTag2, sfcs.getYhbh());
                    values.putAll(values1);

                } else if (key.equals("L3")) {
                    String sjTag = "sj2";
                    String sjTag2 = "2";
                    Map<String, Object> values1 = getValues(value, sfcs.getJtzqljl(), wsfbz, ljfbz, sjTag, sjTag2, sfcs.getYhbh());
                    values.putAll(values1);
                }
            });

            List<BigDecimal> sls = getMxValues(values, "sl");
            List<BigDecimal> sfs = getMxValues(values, "sf");
            List<BigDecimal> wsfs = getMxValues(values, "wsf");
            List<BigDecimal> ljfs = getMxValues(values, "ljf");

            BigDecimal yssl = sls.stream().reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
            BigDecimal yssf = sfs.stream().reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
            BigDecimal yswsf = wsfs.stream().reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
            BigDecimal ysljf = ljfs.stream().reduce(BigDecimal.ZERO, (x, y) -> x.add(y));

            try {
                BigDecimal ljsl = sfcs.getJtzqljl();
                org.apache.commons.beanutils.BeanUtils.populate(jsJg, values);
                CbjsfsEnum cbjsfs = getCbjsfs(slMap, values);//抄表计算方式（0正常单价，1一表多价，2阶梯,3 一表多价含阶梯；用于判断计价类型，处理显示水价）
                jsJg.setCbjsbz(cbjsfs.getValue());
                jsJg.setCbjsfs(cbjsfs.name());
                jsJg.setJtzqljlJsq(ljsl);
                jsJg.setJtzqljl(ljsl == null ? null : ljsl.add(yssl));
                jsJg.setYssl(yssl);
                jsJg.setYssf(yssf);
                jsJg.setYsljf(ysljf);
                jsJg.setYswsf(yswsf);
                jsJg.setYsje(yssf.add(ysljf).add(yswsf));

                jsJg.setBcsl(jsJg.getYssl().subtract(sfcs.getYsl()));
                jsJg.setBcsf(jsJg.getYssf().subtract(sfcs.getYsf()));
                jsJg.setBcljf(jsJg.getYsljf().subtract(sfcs.getYljf()));
                jsJg.setBcwsf(jsJg.getYswsf().subtract(sfcs.getYwsf()));
                jsJg.setBcje(jsJg.getYsje().subtract(sfcs.getYje()));
            } catch (IllegalAccessException e) {
                throw new RuntimeException(e);
            } catch (InvocationTargetException e) {
                throw new RuntimeException(e);
            }
        }
        return jsJg;
    }

    @Override
    public List<YyQfmxMonthlyStat> monthlyStat(YyQfmxStatFilter filter) {
        List<YySjStat> list0 = this.baseMapper.sjStat(filter, "");
        List<YySjStat> list1 = this.baseMapper.sjStat(filter, "1");
        List<YySjStat> list2 = this.baseMapper.sjStat(filter, "2");
        List<YySjStat> list3 = this.baseMapper.sjStat(filter, "3");
        BigDecimal bc = this.baseMapper.bcStat(filter);
        List<YySjStat> list = new ArrayList<>();
        list.addAll(list0);
        list.addAll(list1);
        list.addAll(list2);
        list.addAll(list3);

        Map<String, List<YySjStat>> sjmxMap = list.stream()
                .filter(e -> StringUtils.isNotBlank(e.getSjbm()))
                .collect(Collectors.groupingBy(YySjStat::getSjbm));
        Map<String, String> sjMap = getSjMap();
        List<YyQfmxMonthlyStat> results = new ArrayList<>();
        sjMap.forEach((sjbm, sjmc) -> {
            List<YySjStat> sjmxs = sjmxMap.getOrDefault(sjbm, new ArrayList<>());
            BigDecimal sl = sjmxs.stream().map(e -> e.getSl()).filter(Objects::nonNull).reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
            BigDecimal sf = sjmxs.stream().map(e -> e.getSf()).filter(Objects::nonNull).reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
            BigDecimal wsf = sjmxs.stream().map(e -> e.getWsf()).filter(Objects::nonNull).reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
            BigDecimal ljf = sjmxs.stream().map(e -> e.getLjf()).filter(Objects::nonNull).reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
            results.add(new YyQfmxMonthlyStat(sjmc, sl, sf, wsf, ljf, sf.add(wsf).add(ljf)));
        });
        if (!results.isEmpty()) {
            BigDecimal sl = results.stream().map(e -> e.getSl()).filter(Objects::nonNull).reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
            BigDecimal sf = results.stream().map(e -> e.getSf()).filter(Objects::nonNull).reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
            BigDecimal wsf = results.stream().map(e -> e.getWsf()).filter(Objects::nonNull).reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
            BigDecimal ljf = results.stream().map(e -> e.getLjf()).filter(Objects::nonNull).reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
            BigDecimal hj = results.stream().map(e -> e.getHj()).filter(Objects::nonNull).reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
            results.add(new YyQfmxMonthlyStat("小计", sl, sf, wsf, ljf, hj));
        }
        results.add(new YyQfmxMonthlyStat("水费补差", null, null, null, null, bc == null ? BigDecimal.ZERO : bc));
        BigDecimal hj = results.stream().map(e -> e.getHj()).filter(Objects::nonNull).reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
        results.add(new YyQfmxMonthlyStat("合计", null, null, null, null, hj));
        return results;
    }

    @Override
    public List<YyQfmxSelect> getList(YyQfmxFilter filter, String sfbz, String chbz) {
        return this.baseMapper.getList(filter, sfbz, chbz);
    }

    @Override
    public boolean cxSf(String id, CzyUserDetails czyUserDetails) {

        YyQfmx yyQfmx = this.getById(id);
        if (yyQfmx == null) throw new FeesException("欠费明细不存在", FeesErrorCode.BUSINESS_ERROR);
        String chbz = yyQfmx.getChbz();
        if (PjztEnum.已冲红.getValue().equals(chbz))
            throw new FeesException("只能撤销票据状态正常的欠费明细", FeesErrorCode.BUSINESS_ERROR);
        if (PjztEnum.被冲红.getValue().equals(chbz))
            throw new FeesException("只能撤销票据状态正常的欠费明细", FeesErrorCode.BUSINESS_ERROR);
        String cwrq = gdcsbService.selectCwrq();
        if (!cwrq.equals(yyQfmx.getCwrq()))
            throw new FeesException("只能撤销当前财务月份的欠费明细", FeesErrorCode.BUSINESS_ERROR);
        String sfbz = yyQfmx.getSfbz();
        if ("1".equals(sfbz))
            throw new FeesException("只能撤销未收费欠费明细", FeesErrorCode.BUSINESS_ERROR);
        String cbkid = yyQfmx.getCbkid();
        if (cbkid == null) throw new FeesException("只能撤销水费欠费明细", FeesErrorCode.BUSINESS_ERROR);
        boolean success = cblrService.lambdaUpdate()
                .set(YyCblr::getJsbz, JsbzEnum.未算费)
                .eq(YyCblr::getCbkid, cbkid)
                .update();
        Date chsj = new Date();
        if (success) {
            yyQfmx.setChbz(PjztEnum.已冲红.getValue());
            yyQfmx.setChsj(chsj);
            if (czyUserDetails != null) {
                yyQfmx.setCbrybm(String.valueOf(czyUserDetails.getId()));
                yyQfmx.setCbryxm(czyUserDetails.getRealName());
            }
            YyQfmx hp = createHp(yyQfmx, czyUserDetails, chsj);//红票
            this.save(hp);
            this.updateById(yyQfmx);
        }
        return success;
    }

    @Override
    public YyStat stat(YyQfmxStatFilter filter, String sfbz) {
        return this.baseMapper.stat(filter, sfbz);
    }

    @Override
    public String pjyl(pjYl pjyl, CzyUserDetails czyUserDetails) {
        try {
            YyQfmx yyQfmx = this.getById(pjyl.getQfid());
            if (yyQfmx == null)
                throw new FeesException("欠费明细不存在", FeesErrorCode.BUSINESS_ERROR);
            YhKhjbxx yhKhjbxx = khjbxxService.getById(yyQfmx.getYhbh());
            if (yhKhjbxx == null)
                throw new FeesException("客户信息不存在", FeesErrorCode.BUSINESS_ERROR);

            String qfmxsql = String.format("with sssf as" +
                            "     (" +
                            "     select  1 xh, '阶梯1' as mc, sjbm, b.name sjmc, jt1sj sj, jt1sl sl, jt1sf sf from yy_qfmx a left join bm_sj b on a.sjbm=b.code where  id='%s' and b.jtbz=1" +
                            "     union" +
                            "     select  2 xh, '阶梯2', sjbm, b.name sjmc, jt2sj, jt2sl, jt2sf from yy_qfmx a left join bm_sj b on a.sjbm=b.code where id='%s' and b.jtbz=1" +
                            "     union" +
                            "     select  3 xh, '阶梯3', sjbm, b.name sjmc, jt3sj, jt3sl, jt3sf from yy_qfmx a left join bm_sj b on a.sjbm=b.code where id='%s' and b.jtbz=1" +
                            "     union" +
                            "     select  4 xh, '' , a.sjbm, b.name sjmc, a.sj, a.sl, a.sf from yy_qfmx a left join bm_sj b on a.sjbm=b.code where id='%s' and nvl(b.jtbz,0)=0" +
                            "     )" +
                            "     select a.* from sssf a left  join bm_sj b on a.sjbm=b.code where nvl(a.sf,0)>0 "
                    , pjyl.getQfid()
                    , pjyl.getQfid()
                    , pjyl.getQfid()
                    , pjyl.getQfid());

            String qtfysql = String.format("select 5,'污水费' mc,wsfdj dj, yswsf sf,yssl sl from yy_qfmx" +
                            "     where id='%s' and  nvl(yswsf,0)>0  " +
                            "     union " +
                            "     select 6,'垃圾费',ljfdj dj,ysljf,yssl sl from yy_qfmx" +
                            "     where id='%s' and nvl(ysljf,0)>0  " +
                            "     union " +
                            "     select 7,'违约金',0 ,znj,0 sl from yy_qfmx" +
                            "     where id='%s' and nvl(ysljf,0)>0 "
                    , pjyl.getQfid()
                    , pjyl.getQfid()
                    , pjyl.getQfid());
            List<HashMap<String, Object>> qflist = DbHelper.getDbHelper().getList(qfmxsql);
            StringBuilder str = new StringBuilder();
            qflist.forEach(e -> {
                str.append("<tr>");
                str.append(String.format("<td>%s</td>", e.get("mc")));
                str.append(String.format("<td>%s</td>", e.get("sf")));
                str.append(String.format("<td>%s</td>", e.get("sj")));
                str.append(String.format("<td>%s</td>", e.get("sl")));
                str.append(String.format("<td>%s</td>", "\\"));
                str.append(String.format("<td>%s</td>", ""));
                str.append("</tr>");
            });
            List<HashMap<String, Object>> qtlist = DbHelper.getDbHelper().getList(qtfysql);
            qtlist.forEach(e -> {
                str.append("<tr>");
                str.append(String.format("<td>%s</td>", e.get("mc")));
                str.append(String.format("<td>%s</td>", e.get("sf")));
                str.append(String.format("<td>%s</td>", e.get("dj")));
                str.append(String.format("<td>%s</td>", e.get("sl")));
                str.append(String.format("<td>%s</td>", "\\"));
                str.append(String.format("<td>%s</td>", ""));
                str.append("</tr>");
            });
            String htmlstr = DbHelper.getDbHelper().getValue(String.format("select printtemplate from print_template where id=%s", PjlxEnum.实收票据信息.getValue()));
            htmlstr = htmlstr.replace("{{title}}", "水务");
            htmlstr = htmlstr.replace("{{yhbh}}", yyQfmx.getYhbh());
            htmlstr = htmlstr.replace("{{yhxm}}", yyQfmx.getYhxm());
            htmlstr = htmlstr.replace("{{cwyf}}", yyQfmx.getCwrq());
            htmlstr = htmlstr.replace("{{sfrq}}", LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
            htmlstr = htmlstr.replace("{{lxdh}}", yhKhjbxx.getLxdh());
            htmlstr = htmlstr.replace("{{addr}}", yhKhjbxx.getYhdz());
            htmlstr = htmlstr.replace("{{scds}}", yyQfmx.getScds().toString());
            htmlstr = htmlstr.replace("{{bcds}}", yyQfmx.getBcds().toString());
            htmlstr = htmlstr.replace("{{sl}}", yyQfmx.getSl().toString());

            htmlstr = htmlstr.replace("{{content}}", str.toString());
            htmlstr = htmlstr.replace("{{ysje}}", yyQfmx.getYsje().add(yyQfmx.getZnj()).toString());
            htmlstr = htmlstr.replace("{{ysjedx}}", NumberHelper.CmycurD(yyQfmx.getYsje().add(yyQfmx.getZnj())));
            htmlstr = htmlstr.replace("{{scjy}}", pjyl.getScjz().toString());
            htmlstr = htmlstr.replace("{{bcjy}}", pjyl.getBcjz().toString());
            htmlstr = htmlstr.replace("{{sszje}}", pjyl.getSsje().toString());
            if (!org.springframework.util.StringUtils.isEmpty(czyUserDetails)) {
                htmlstr = htmlstr.replace("{{sfry}}", czyUserDetails.getRealName());
            }
            htmlstr = htmlstr.replace("{{cbry}}", yyQfmx.getCbryxm());
            String cbrydh = DbHelper.getDbHelper().getValue(String.format("select olxdh from  sys_operators where ocode = (select cby from bm_cbb where cbbbm='%s')", yyQfmx.getSscbb()));
            if (StringUtils.isEmpty(cbrydh)) {
                cbrydh = "无";
            }
            htmlstr = htmlstr.replace("{{cbrydh}}", cbrydh);
            return htmlstr;
        } catch (Exception e) {
            throw new FeesException("查询数据错误！", FeesErrorCode.BUSINESS_ERROR);
        }
    }

    @Override
    public SfJsJg sfJs(YhCbkxx cbkxx, BigDecimal sjsl, CzyUserDetails czyUserDetails) {
        sjCache();//缓存表价信息（计算前必须）
        YhKhjbxx khjbxx = khjbxxService.getById(cbkxx.getYhbh());
        BigDecimal ljsl = khjbxx.getJtzqljl() == null ? BigDecimal.ONE.negate() : khjbxx.getJtzqljl();
        SfJsJg jsqJg;
        if (ljsl.compareTo(BigDecimal.ZERO) >= 0) {//年、月阶梯时
            jsqJg = sfJs0(cbkxx, ljsl, sjsl);
            cbkxxService.updateLjsl(cbkxx.getCbkid(), jsqJg.getJtzqljl());
            khjbxxService.lambdaUpdate()
                    .set(YhKhjbxx::getJtzqljl, jsqJg.getJtzqljl())
                    .eq(YhKhjbxx::getYhbh, khjbxx.getYhbh())
                    .update();
        } else {
            jsqJg = sfJs0(cbkxx, BigDecimal.ZERO, sjsl);
        }
        return jsqJg;
    }

    @Override
    public SfJsJg jtSfjs(Jtsfjs jtsfjs, CzyUserDetails czyUserDetails) {
        SfJsJg jsqJg = new SfJsJg();
        TreeMap<String, String> slMap = new TreeMap<>();
        slMap.put("L1", String.format("%s_%s", jtsfjs.getSjbm(), jtsfjs.getSl()));
        Date now = new Date();
        jsqJg.setJsrq(now);


        YhKhjbxx yhKhjbxx = khjbxxService.getById(jtsfjs.getYhbh());
        if (org.springframework.util.StringUtils.isEmpty(yhKhjbxx)) {
            throw new FeesException("用户档案不存在", FeesErrorCode.BUSINESS_ERROR);
        }
        YhCbkxx yhCbkxx = cbkxxService.getOne(new QueryWrapper<YhCbkxx>().lambda()
                .eq(YhCbkxx::getCbkh, jtsfjs.getCbkh())
                .eq(YhCbkxx::getYhbh, jtsfjs.getYhbh())
        );
        if (org.springframework.util.StringUtils.isEmpty(yhCbkxx)) {
            throw new FeesException("水表档案不存在", FeesErrorCode.BUSINESS_ERROR);
        }
        String wsfbz = yhCbkxx.getWsfbz();
        String ljfbz = yhCbkxx.getLjfbz();
        sjCache();//缓存表价信息（计算前必须）

        Map<String, Object> values = new LinkedHashMap<>();
        slMap.forEach((key, value) -> {
            if (key.equals("L1")) {
                String sjTag = "";
                String sjTag2 = "";
                Map<String, Object> values1 = getValues(value, jtsfjs.getJtljl(), wsfbz, ljfbz, sjTag, sjTag2, jtsfjs.getYhbh());
                values.putAll(values1);
            }
        });

        List<BigDecimal> sls = getMxValues(values, "sl");
        List<BigDecimal> sfs = getMxValues(values, "sf");
        List<BigDecimal> wsfs = getMxValues(values, "wsf");
        List<BigDecimal> ljfs = getMxValues(values, "ljf");

        BigDecimal yssl = sls.stream().reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
        BigDecimal yssf = sfs.stream().reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
        BigDecimal yswsf = wsfs.stream().reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
        BigDecimal ysljf = ljfs.stream().reduce(BigDecimal.ZERO, (x, y) -> x.add(y));

        try {
            BigDecimal ljsl = yhKhjbxx.getJtzqljl() == null ? BigDecimal.ONE.negate() : yhKhjbxx.getJtzqljl();
            org.apache.commons.beanutils.BeanUtils.populate(jsqJg, values);
            CbjsfsEnum cbjsfs = getCbjsfs(slMap, values);//抄表计算方式（0正常单价，1一表多价，2阶梯,3 一表多价含阶梯；用于判断计价类型，处理显示水价）
            jsqJg.setCbjsbz(cbjsfs.getValue());
            jsqJg.setCbjsfs(cbjsfs.name());
            jsqJg.setJtzqljlJsq(ljsl);
            jsqJg.setJtzqljl(ljsl == null ? null : ljsl.add(yssl));
            jsqJg.setYssl(yssl);
            jsqJg.setYssf(yssf);
            jsqJg.setYsljf(ysljf);
            jsqJg.setYswsf(yswsf);
            jsqJg.setYsje(yssf.add(ysljf).add(yswsf));

        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new RuntimeException(e);
        }
        return jsqJg;
    }

    private YyQfmx createHp(YyQfmx yyQfmx, CzyUserDetails czyUserDetails, Date chsj) {
        YyQfmx hp = new YyQfmx();
        Set<String> negateKeys = new LinkedHashSet<>();

        for (int i = 0; i < 4; i++) {
            negateKeys.add(i == 0 ? "sl" : "sl" + i);
            negateKeys.add(i == 0 ? "ljf" : "ljf" + i);
            negateKeys.add(i == 0 ? "wsf" : "wsf" + i);
            negateKeys.add(i == 0 ? "sf" : "sf" + i);
        }
        for (int i = 0; i < 3; i++) {
            for (int j = 1; j < 5; j++) {
                negateKeys.add(i == 0 ? String.format("jt%ssl", j) : String.format("sj%sJt%ssl", i, j));
                negateKeys.add(i == 0 ? String.format("jt%ssf", j) : String.format("sj%sJt%ssf", i, j));
            }
        }
        negateKeys.add("znj");
        negateKeys.add("yssl");
        negateKeys.add("yssf");
        negateKeys.add("yswsf");
        negateKeys.add("ysljf");
        negateKeys.add("ysqt");
        negateKeys.add("ysje");

        List<String> ignoreProperties = new ArrayList<>();
        ignoreProperties.addAll(Arrays.asList("id", "chbz", "sfbc", "kpbz", "kprq", "shry", "lockbz", "eiSgkpId", "taxrate", "bz"));
        ignoreProperties.addAll(negateKeys);

        BeanUtils.copyProperties(yyQfmx, hp, ignoreProperties.toArray(new String[ignoreProperties.size()]));

        hp.setChid(yyQfmx.getId());

        hp.setChsj(chsj);
        hp.setJsrq(chsj);
        hp.setChlx("0");
        hp.setChbz(PjztEnum.被冲红.getValue());

        negateKeys.forEach(name -> populateNegateValue(yyQfmx, name, hp));

        if (czyUserDetails != null) {
            hp.setChrybm(String.valueOf(czyUserDetails.getId()));
            hp.setChryxm(czyUserDetails.getRealName());
        }

        return hp;
    }


    private void populateNegateValue(YyQfmx from, String name, YyQfmx to) {
        Object value = null;
        try {
            value = PropertyUtils.getProperty(from, name);
            if (value != null) {
                if (value instanceof BigDecimal) {
                    PropertyUtils.setProperty(to, name, ((BigDecimal) value).negate());
                }
            }
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        }
    }

    private Map<String, String> getSjMap() {
        return sjService.list().stream().collect(Collectors.toMap(BmSj::getCode, BmSj::getName, (u1, u2) -> u1, LinkedHashMap::new));
    }

    /**
     * 水费及附加费计算
     *
     * @param cbkxx 抄表卡信息
     * @param ljsl  累积水量（非阶梯时与本次水量相同）
     * @param bcsl  本次水量
     * @return
     */
    private SfJsJg sfJs0(YhCbkxx cbkxx, BigDecimal ljsl, BigDecimal bcsl) {
        String cbkid = cbkxx.getCbkid();
        SfJsJg sfJsJg = new SfJsJg();
        Date now = new Date();
        sfJsJg.setJsrq(now);
        String wsfbz = cbkxx.getWsfbz();
        String ljfbz = cbkxx.getLjfbz();
        Map<String, BigDecimal> ljslMap = new HashMap<>();
        TreeMap<String, String> slMap = getSlMap(cbkxx, bcsl);

        Map<String, Object> values = new LinkedHashMap<>();
        slMap.forEach((key, value) -> {
            if (key.equals("L0") || key.equals("L1")) {
                String sjTag = "";
                String sjTag2 = "";
                Map<String, Object> values1 = getValues(value, ljsl, wsfbz, ljfbz, sjTag, sjTag2, cbkxx.getYhbh());
                if (values1.containsKey("ljsl")) {
                    ljslMap.put("ljsljsh", (BigDecimal) values1.get("ljsl"));
                }
                values.putAll(values1);

            } else if (key.equals("L2")) {
                String sjTag = "sj1";
                String sjTag2 = "1";
                Map<String, Object> values1 = getValues(value, ljsl, wsfbz, ljfbz, sjTag, sjTag2, cbkxx.getYhbh());
                if (values1.containsKey("ljsl")) {
                    ljslMap.put("ljsljsh", (BigDecimal) values1.get("ljsl"));
                }
                values.putAll(values1);

            } else if (key.equals("L3")) {
                String sjTag = "sj2";
                String sjTag2 = "2";
                Map<String, Object> values1 = getValues(value, ljsl, wsfbz, ljfbz, sjTag, sjTag2, cbkxx.getYhbh());
                if (values1.containsKey("ljsl")) {
                    ljslMap.put("ljsljsh", (BigDecimal) values1.get("ljsl"));
                }
                values.putAll(values1);
            }
        });

        List<BigDecimal> sls = getMxValues(values, "sl");
        List<BigDecimal> sfs = getMxValues(values, "sf");
        List<BigDecimal> wsfs = getMxValues(values, "wsf");
        List<BigDecimal> ljfs = getMxValues(values, "ljf");


        BigDecimal yssl = sls.stream().reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
        BigDecimal yssf = sfs.stream().reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
        BigDecimal yswsf = wsfs.stream().reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
        BigDecimal ysljf = ljfs.stream().reduce(BigDecimal.ZERO, (x, y) -> x.add(y));

        try {
            org.apache.commons.beanutils.BeanUtils.populate(sfJsJg, values);
            CbjsfsEnum cbjsfs = getCbjsfs(slMap, values);//抄表计算方式（0正常单价，1一表多价，2阶梯,3 一表多价含阶梯；用于判断计价类型，处理显示水价）
            sfJsJg.setCbjsbz(cbjsfs.getValue());
            sfJsJg.setCbjsfs(cbjsfs.name());
            sfJsJg.setJtzqljlJsq(ljsl);
            sfJsJg.setJtzqljl(ljslMap.containsKey("ljsljsh") ? ljslMap.get("ljsljsh") : BigDecimal.ZERO);
            sfJsJg.setYssl(yssl);
            sfJsJg.setYssf(yssf);
            sfJsJg.setYsljf(ysljf);
            sfJsJg.setYswsf(yswsf);
            sfJsJg.setYsje(yssf.add(ysljf).add(yswsf));
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new RuntimeException(e);
        }
        log.info("{}:{}", cbkid, values);
        return sfJsJg;
    }

    private CbjsfsEnum getCbjsfs(TreeMap<String, String> slMap, Map<String, Object> values) {
        CbjsfsEnum cbjsfs = CbjsfsEnum.正常单价计费;
        if (slMap.containsKey("L0")) {
            Set<String> keys = values.keySet();
            List<String> jt = keys.stream().filter(e -> e.contains("jt")).collect(Collectors.toList());
            if (jt.isEmpty()) {
                cbjsfs = CbjsfsEnum.正常单价计费;
            } else {
                cbjsfs = CbjsfsEnum.阶梯计费;
            }
        } else if (slMap.containsKey("L1")) {
            Set<String> keys = values.keySet();
            List<String> jt = keys.stream().filter(e -> e.contains("Jt")).collect(Collectors.toList());
            if (jt.isEmpty()) {
                cbjsfs = CbjsfsEnum.一表多价计费;
            } else {
                cbjsfs = CbjsfsEnum.一表多价含阶梯;
            }
        }
        return cbjsfs;
    }

    /**
     * 明细值
     *
     * @param values
     * @param prefix
     * @return
     */
    private List<BigDecimal> getMxValues(Map<String, Object> values, String prefix) {
        List<BigDecimal> mxValues = new ArrayList<>();

        for (int i = 0; i < 4; i++) {
            String key;
            if (i == 0) {
                key = String.format("%s", prefix);
            } else {
                key = String.format("%s%s", prefix, i);
            }
            if (values.containsKey(key)) {
                mxValues.add((BigDecimal) values.getOrDefault(key, BigDecimal.ZERO));
            }
        }
        return mxValues;
    }

    /**
     * @param sjsl   水价编码_水量
     * @param ljsl   累计水量
     * @param wsfbz  污水费标志
     * @param ljfbz  垃圾费标志
     * @param sjTag1 水价标签1
     * @param sjTag2 水价标签2
     * @return
     */
    private Map<String, Object> getValues(String sjsl, BigDecimal ljsl, String wsfbz, String ljfbz, String sjTag1, String sjTag2, String yhbh) {
        String[] values = sjsl.split("_");
        String sjbm = values[0];
        BigDecimal sl = new BigDecimal(values[1]);
        Sjmx sjmx = sfjs(sl, sjbm, ljsl, yhbh);
        Map<String, Object> map = new LinkedHashMap<>();
        String prefix = "";
        String suffix = sjTag2;
        map.put(String.format("%s%s%s", prefix, "sjbm", suffix), sjbm);
        Sjmx.Jtmx jtmx = sjmx.getJtmx();

        if (jtmx != null) {//非阶梯水价
            prefix = sjTag1;
            suffix = sjTag2;
            map.put(String.format("%s%s%s", prefix, "sl", suffix), jtmx.getSl());
            map.put(String.format("%s%s%s", prefix, "sj", suffix), jtmx.getSj());
            map.put(String.format("%s%s%s", prefix, "sf", suffix), jtmx.getSf());

            prefix = "";
            suffix = sjTag2;
            map.put(String.format("%s%s%s", prefix, "sl", suffix), jtmx.getSl());
            map.put(String.format("%s%s%s", prefix, "sj", suffix), jtmx.getSj());
            map.put(String.format("%s%s%s", prefix, "sf", suffix), jtmx.getSf());
            map.put(String.format("%s%s", "sjjtbz", suffix), "0");
        } else {//阶梯水价
            map.put(String.format("%s%s", "sjjtbz", suffix), "1");
            Sjmx.Jtmx jtmx1 = sjmx.getJtmx1();
            BigDecimal zsl = BigDecimal.ZERO;
            BigDecimal zsf = BigDecimal.ZERO;
            if (jtmx1 != null) {
                prefix = "jt1";
                if (StringUtils.isNotBlank(sjTag1)) prefix = sjTag1 + "Jt1";
                suffix = "";
                BigDecimal sl1 = jtmx1.getSl();
                zsl = zsl.add(sl1);
                map.put(String.format("%s%s%s", prefix, "sl", suffix), sl1);
                map.put(String.format("%s%s%s", prefix, "sj", suffix), jtmx1.getSj());
                BigDecimal sf = jtmx1.getSf();
                zsf = zsf.add(sf);
                map.put(String.format("%s%s%s", prefix, "sf", suffix), sf);
            }
            Sjmx.Jtmx jtmx2 = sjmx.getJtmx2();
            if (jtmx2 != null) {
                prefix = "jt2";
                if (StringUtils.isNotBlank(sjTag1)) prefix = sjTag1 + "Jt2";
                suffix = "";
                BigDecimal sl1 = jtmx2.getSl();
                zsl = zsl.add(sl1);
                map.put(String.format("%s%s%s", prefix, "sl", suffix), sl1);
                map.put(String.format("%s%s%s", prefix, "sj", suffix), jtmx2.getSj());
                BigDecimal sf = jtmx2.getSf();
                zsf = zsf.add(sf);
                map.put(String.format("%s%s%s", prefix, "sf", suffix), sf);
            }
            Sjmx.Jtmx jtmx3 = sjmx.getJtmx3();
            if (jtmx3 != null) {
                prefix = "jt3";
                if (StringUtils.isNotBlank(sjTag1)) prefix = sjTag1 + "Jt3";
                suffix = "";
                BigDecimal sl1 = jtmx3.getSl();
                zsl = zsl.add(sl1);
                map.put(String.format("%s%s%s", prefix, "sl", suffix), sl1);
                map.put(String.format("%s%s%s", prefix, "sj", suffix), jtmx3.getSj());
                BigDecimal sf = jtmx3.getSf();
                zsf = zsf.add(sf);
                map.put(String.format("%s%s%s", prefix, "sf", suffix), sf);
            }
            Sjmx.Jtmx jtmx4 = sjmx.getJtmx4();
            if (jtmx4 != null) {
                prefix = "jt4";
                if (StringUtils.isNotBlank(sjTag1)) prefix = sjTag1 + "Jt4";
                suffix = "";
                BigDecimal sl1 = jtmx4.getSl();
                zsl = zsl.add(sl1);
                map.put(String.format("%s%s%s", prefix, "sl", suffix), sl1);
                map.put(String.format("%s%s%s", prefix, "sj", suffix), jtmx4.getSj());
                BigDecimal sf = jtmx4.getSf();
                zsf = zsf.add(sf);
                map.put(String.format("%s%s%s", prefix, "sf", suffix), sf);
            }
            prefix = "";
            suffix = sjTag2;
            map.put(String.format("%s%s%s", prefix, "sl", suffix), zsl);
            map.put(String.format("%s%s%s", prefix, "sf", suffix), zsf);
            map.put("ljsl", ljsl.add(zsl));
        }
        BmSj bmSj = sjCache.get(sjbm);
        if (bmSj != null) {//其它费用计算，其他费用不参与累积计算
            prefix = "";
            suffix = sjTag2;
            BigDecimal wsfdj = bmSj.getWsfdj();
            BigDecimal ljfdj = bmSj.getLjfdj();
            map.put(String.format("%s%s%s", prefix, "wsfdj", suffix), wsfdj);
            map.put(String.format("%s%s%s", prefix, "ljfdj", suffix), ljfdj);
            if ("1".equals(wsfbz)) {
                BigDecimal wsf = fyjs(sl, wsfdj);
                map.put(String.format("%s%s%s", prefix, "wsf", suffix), wsf);
            }
            if ("1".equals(ljfbz)) {
                BigDecimal ljf = fyjs(sl, ljfdj);
                map.put(String.format("%s%s%s", prefix, "ljf", suffix), ljf);
            }
        }
        return map;
    }

    private final static Map<String, BmSj> sjCache = new HashMap<>();//水价缓存

    private void sjCache() {
        sjCache.clear();
        List<BmSj> sjs = sjService.lambdaQuery().list();
        sjs.forEach(sj -> sjCache.put(sj.getCode(), sj));
    }

    @Override
    public int Tasksf(List<YyCblr> cblrs) {
        List<Map<String, String>> listcblr = cblrService.lambdaQuery()
                .select(YyCblr::getCbkid)
                .eq(YyCblr::getCbbz, CbbzEnum.已抄表)
                .eq(YyCblr::getJsbz, JsbzEnum.未算费)
                //.eq(YyCblr::getShbz, ShbzEnum.审核通过)
                .inSql(YyCblr::getCbkid, "select cbkid_zb from yh_zfb")
                .orderByAsc(YyCblr::getYhbh, YyCblr::getCbkh)
                .list().stream().map(e -> {
                    Map<String, String> map = new HashMap<>();
                    map.put("cbkid", e.getCbkid());
                    return map;
                }).collect(Collectors.toList());
        if (listcblr.size() > 0) {
            // 查询当前总表下属分表
            listcblr.forEach(item -> {
                String cbkid = item.get("cbkid");
                if (StringUtils.isNotBlank(cbkid)) {
                    Map map = cblrService.getCblrMap(cbkid);
                    if (Integer.valueOf(map.get("fbs").toString()) > Integer.valueOf(map.get("fbycs").toString())) {
                        cblrService.lambdaUpdate()
                                .set(YyCblr::getSl, -1)
                                .eq(YyCblr::getCbkid, cbkid).update();
                    } else {
                        cblrService.lambdaUpdate()
                                .setSql("SL=BCDS-SCDS-" + map.get("fbhjsl"))
                                .setSql("JJL=0-" + map.get("fbhjsl"))
                                .eq(YyCblr::getCbkid, cbkid).update();
                    }
                }
            });
        }
        cblrs = cblrs.stream()
                .filter(e -> e.getCbbz() == CbbzEnum.已抄表)
                .filter(e -> e.getJsbz() == JsbzEnum.未算费)
                .filter(e -> e.getSl() != null && e.getSl().compareTo(BigDecimal.ZERO) >= 0)
                .collect(Collectors.toList());
        if (cblrs.isEmpty()) return 0;
        Set<String> yhbhs = cblrs.stream().map(e -> e.getYhbh()).collect(Collectors.toSet());
        Map<String, YyCblr> cblrMap = cblrs.stream()
                .collect(Collectors.toMap(e -> e.getCbkid(), e -> e,
                        (u1, u2) -> u1,
                        LinkedHashMap::new));//用户实际水量
        if (cblrMap.isEmpty()) return 0;
        List<YhCbkxx> cbkxxes = cbkxxService.lambdaQuery().select(
                        YhCbkxx::getYhbh, YhCbkxx::getCbkid,
                        YhCbkxx::getYbdjbz,
                        YhCbkxx::getSjbm, YhCbkxx::getSlbl,
                        YhCbkxx::getSjbm1, YhCbkxx::getSlbl1,
                        YhCbkxx::getSjbm2, YhCbkxx::getSlbl2,
                        YhCbkxx::getSjbm3, YhCbkxx::getSlbl3,
                        YhCbkxx::getWsfbz, YhCbkxx::getLjfbz)
                .in(YhCbkxx::getCbkid, cblrMap.keySet()).list();
        Map<String, YhCbkxx> cbkxxMap = cbkxxes.stream().collect(Collectors.toMap(e -> e.getCbkid(), e -> e));
        sjCache();//缓存表价信息（计算前必须）
        Map<String, YhKhjbxx> yhMap = getYhMap(yhbhs);
        AtomicInteger count = new AtomicInteger();
        cblrMap.forEach((cbkid, cblr) -> {
            int i = transactionTemplate.execute(status -> {
                try {
                    YhCbkxx cbkxx = cbkxxMap.get(cbkid);
                    if (cbkxx == null)
                        return 0;
                    String yhbh = cbkxx.getYhbh();
                    YhKhjbxx yhxx = yhMap.get(yhbh);
                    if (yhxx == null)
                        return 0;
                    BigDecimal sjsl = cblr.getSl();
                    if (sjsl.compareTo(BigDecimal.ZERO) < 0) {
                        return 0;
                    }
                    YyQfmx yyQfmx = new YyQfmx();
                    yyQfmx.setQflsbh(getQflsbh());
                    SfJsJg sfJsJg = sfJs(cbkxx, sjsl, null);
                    BeanUtils.copyProperties(yhxx, yyQfmx);
                    BeanUtils.copyProperties(cblr, yyQfmx, "sl");
                    BeanUtils.copyProperties(sfJsJg, yyQfmx);
                    yyQfmx.setSfbc("0");
                    yyQfmx.setFylx(BcFylxEnum.BcSf.getValue().toString());
                    yyQfmx.setFylxmc(BcFylxEnum.BcSf.getName());
                    yyQfmx.setYsyf(cblr.getCwrq());
                    yyQfmx.setChbz(PjztEnum.正常.getValue());
                    this.save(yyQfmx);
                    cblrService.updateJsbz(cbkid);
                    return 1;
                } catch (Exception e) {
                    status.setRollbackOnly();
                    return 0;
                }
            });
            count.addAndGet(i);
        });

        return count.get();
    }

    private int sf(List<YyCblr> cblrs) {
        //查询总表
        List<Map<String, String>> listcblr = cblrService.lambdaQuery()
                .select(YyCblr::getCbkid)
                .eq(YyCblr::getCbbz, CbbzEnum.已抄表)
                .eq(YyCblr::getJsbz, JsbzEnum.未算费)
                //.eq(YyCblr::getShbz, ShbzEnum.审核通过)
                .inSql(YyCblr::getCbkid, "select cbkid_zb from yh_zfb")
                .orderByAsc(YyCblr::getYhbh, YyCblr::getCbkh)
                .list().stream().map(e -> {
                    Map<String, String> map = new HashMap<>();
                    map.put("cbkid", e.getCbkid());
                    return map;
                }).collect(Collectors.toList());
        if (listcblr.size() > 0) {
            // 查询当前总表下属分表
            listcblr.forEach(item -> {
                String cbkid = item.get("cbkid");
                if (StringUtils.isNotBlank(cbkid)) {
                    Map map = cblrService.getCblrMap(cbkid);
                    if (Integer.valueOf(map.get("fbs").toString()) > Integer.valueOf(map.get("fbycs").toString())) {
                        cblrService.lambdaUpdate()
                                .set(YyCblr::getSl, -1)
                                .eq(YyCblr::getCbkid, cbkid).update();
                    } else {
                        cblrService.lambdaUpdate()
                                .setSql("SL=BCDS-SCDS-" + map.get("fbhjsl"))
                                .setSql("JJL=0-" + map.get("fbhjsl"))
                                .eq(YyCblr::getCbkid, cbkid).update();
                    }
                }
            });
        }
        cblrs = cblrs.stream()
                .filter(e -> e.getCbbz() == CbbzEnum.已抄表)
                .filter(e -> e.getJsbz() == JsbzEnum.未算费)
                .filter(e -> e.getSl() != null && e.getSl().compareTo(BigDecimal.ZERO) >= 0)
                .collect(Collectors.toList());
        if (cblrs.isEmpty()) return 0;
        Set<String> yhbhs = cblrs.stream().map(e -> e.getYhbh()).collect(Collectors.toSet());
        Map<String, YyCblr> cblrMap = cblrs.stream()
                .collect(Collectors.toMap(e -> e.getCbkid(), e -> e,
                        (u1, u2) -> u1,
                        LinkedHashMap::new));//用户实际水量
        if (cblrMap.isEmpty()) return 0;
        List<YhCbkxx> cbkxxes = cbkxxService.lambdaQuery().select(
                        YhCbkxx::getYhbh, YhCbkxx::getCbkid,
                        YhCbkxx::getYbdjbz,
                        YhCbkxx::getSjbm, YhCbkxx::getSlbl,
                        YhCbkxx::getSjbm1, YhCbkxx::getSlbl1,
                        YhCbkxx::getSjbm2, YhCbkxx::getSlbl2,
                        YhCbkxx::getSjbm3, YhCbkxx::getSlbl3,
                        YhCbkxx::getWsfbz, YhCbkxx::getLjfbz)
                .in(YhCbkxx::getCbkid, cblrMap.keySet()).list();
        Map<String, YhCbkxx> cbkxxMap = cbkxxes.stream().collect(Collectors.toMap(e -> e.getCbkid(), e -> e));
        sjCache();//缓存表价信息（计算前必须）
        Map<String, YhKhjbxx> yhMap = getYhMap(yhbhs);
        int ret = transactionTemplate.execute(status -> {
            try {
                cblrMap.forEach((cbkid, cblr) -> {
                    YhCbkxx cbkxx = cbkxxMap.get(cbkid);
                    if (cbkxx == null)
                        return;
                    String yhbh = cbkxx.getYhbh();
                    YhKhjbxx yhxx = yhMap.get(yhbh);
                    if (yhxx == null)
                        return;
                    BigDecimal sjsl = cblr.getSl();
                    if (sjsl.compareTo(BigDecimal.ZERO) < 0) {
                        return;
                    }
                    YyQfmx yyQfmx = new YyQfmx();
                    yyQfmx.setQflsbh(getQflsbh());
                    SfJsJg sfJsJg = sfJs(cbkxx, sjsl, null);
                    BeanUtils.copyProperties(yhxx, yyQfmx);
                    BeanUtils.copyProperties(cblr, yyQfmx, "sl");
                    BeanUtils.copyProperties(sfJsJg, yyQfmx);
                    yyQfmx.setSfbc("0");
                    yyQfmx.setFylx(BcFylxEnum.BcSf.getValue().toString());
                    yyQfmx.setFylxmc(BcFylxEnum.BcSf.getName());
                    yyQfmx.setYsyf(cblr.getCwrq());
                    yyQfmx.setChbz(PjztEnum.正常.getValue());
                    this.save(yyQfmx);
                    cblrService.updateJsbz(cbkid);
                });

                return 1;
            } catch (Exception e) {
                status.setRollbackOnly();
                throw e;
            }

        });

        return ret;


    }

    private Map<String, YhKhjbxx> getYhMap(Set<String> yhbhs) {
        if (yhbhs.size() == 0) return new HashMap<>();
        List<YhKhjbxx> yhs = khjbxxService.lambdaQuery().select(YhKhjbxx::getYhxm, YhKhjbxx::getYhbh)
                .in(YhKhjbxx::getYhbh, yhbhs)
                .list();
        if (yhs.size() == 0) return new HashMap<>();
        return yhs.stream().collect(Collectors.toMap(YhKhjbxx::getYhbh, e -> e));
    }

    private String getQflsbh() {
        String prefix = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss"));
        return String.format("Q%s%05d", prefix, (int) ((Math.random() * 9 + 1) * 10000));
    }

    /**
     * 水费计算(含阶梯运算)
     *
     * @param sl   水量
     * @param sjbm 水价编码
     * @return
     */
    private Sjmx sfjs(BigDecimal sl, String sjbm, BigDecimal ljsl, String yhbh) {
        BmSj bmSj = sjCache.get(sjbm);
        if (bmSj == null)
            new FeesException("水价编码不存在", FeesErrorCode.BUSINESS_ERROR);
        String jtbz = bmSj.getJtbz();
        if ("1".equals(jtbz)) {
            YhKhjbxx yhKhjbxx = khjbxxService.getById(yhbh);
            if (yhKhjbxx == null) {
                throw new FeesException(String.format("用户[%s]不存在,无法进行计算！", yhbh), FeesErrorCode.BUSINESS_ERROR);
            }
            Long rks = yhKhjbxx.getRks();
            BigDecimal rksqs = BigDecimal.ZERO;//人口数起数
            BigDecimal jtzl = BigDecimal.ZERO;//阶梯增量

            BigDecimal jtZszl = BigDecimal.ZERO;//阶梯止数增量
            try {
                rksqs = BigDecimal.valueOf(Long.parseLong(DbHelper.getDbHelper().getValue("select value from sys_config where pzid='jt_jcrks'")));
                jtzl = BigDecimal.valueOf(Long.parseLong(DbHelper.getDbHelper().getValue("select value from sys_config where pzid='jt_zl'")));
            } catch (Exception e) {
                throw new FeesException(e.getMessage(), FeesErrorCode.BUSINESS_ERROR);
            }
            if (rks != null && rks > 0) {
                if (BigDecimal.valueOf(rks).compareTo(rksqs) > 0) {
                    jtZszl = BigDecimal.valueOf(rks).subtract(rksqs).multiply(jtzl);
                }
            }
            Sjmx.Jtmx jtmx1 = null;
            Sjmx.Jtmx jtmx2 = null;
            Sjmx.Jtmx jtmx3 = null;
            Sjmx.Jtmx jtmx4 = null;
            BigDecimal yssl = BigDecimal.ZERO;
            BigDecimal yssf = BigDecimal.ZERO;

            BigDecimal jt1Zs = bmSj.getJt1Zs();
            BigDecimal jt1Sj = bmSj.getJt1Sj();
            BigDecimal sjJtZs = BigDecimal.ZERO;
            if (isEnabled(jt1Zs)) {
                jtmx1 = getJtSf(sl, jt1Zs.add(jtZszl), jt1Sj, sjJtZs, ljsl);
                sjJtZs = jt1Zs;
                yssl = yssl.add(jtmx1.getSl());
                yssf = yssf.add(jtmx1.getSf());
            }

            BigDecimal jt2Zs = bmSj.getJt2Zs();
            BigDecimal jt2Sj = bmSj.getJt2Sj();
            if (isEnabled(jt2Zs) && isEnabled(jt1Zs)) {
                jtmx2 = getJtSf(sl, jt2Zs.add(jtZszl), jt2Sj, sjJtZs.add(jtZszl), ljsl);
                sjJtZs = jt2Zs;
                yssl = yssl.add(jtmx2.getSl());
                yssf = yssf.add(jtmx2.getSf());
            }

            BigDecimal jt3Zs = bmSj.getJt3Zs();
            BigDecimal jt3Sj = bmSj.getJt3Sj();
            if (isEnabled(jt3Zs) && isEnabled(jt2Zs)) {
                jtmx3 = getJtSf(sl, jt3Zs.add(jtZszl), jt3Sj, sjJtZs.add(jtZszl), ljsl);
                sjJtZs = jt3Zs;
                yssl = yssl.add(jtmx3.getSl());
                yssf = yssf.add(jtmx3.getSf());
            }

            BigDecimal jt4Zs = bmSj.getJt4Zs();
            BigDecimal jt4Sj = bmSj.getJt4Sj();
            if (isEnabled(jt4Zs) && isEnabled(jt3Zs)) {
                jtmx4 = getJtSf(sl, jt4Zs.add(jtZszl), jt4Sj, sjJtZs.add(jtZszl), ljsl);
                //sjJtZs = jt4Zs;
                yssl = yssl.add(jtmx4.getSl());
                yssf = yssf.add(jtmx4.getSf());
            }
            if (ljsl.add(sl).compareTo(sjJtZs) > 0) {
                throw new FeesException(String.format("用户[%s]水量超过%s,无法进行计算！", yhbh, sjJtZs), FeesErrorCode.BUSINESS_ERROR);
            }
            return new Sjmx(jtmx1, jtmx2, jtmx3, jtmx4, yssl, yssf);
        } else {
            BigDecimal sj = bmSj.getSj();
            BigDecimal sf = fyjs(sl, sj);
            return new Sjmx(sj, sl, sf);
        }

    }

    /**
     * 获取不同水价的水量
     * 返回值map key: LO是水价0（非一表多价),L1是水价1,L2是水价2,L3是水价3
     * 返回值map value: 水价编码_水量
     *
     * @param cbkxx 抄表卡信息
     * @param ljsl  累积水量（非阶梯时与本次实际水量相同）
     * @return
     */
    private TreeMap<String, String> getSlMap(YhCbkxx cbkxx, BigDecimal ljsl) {
        TreeMap<String, String> slMap = new TreeMap<>();
        String cbkid = cbkxx.getCbkid();
        String ybdjbz = cbkxx.getYbdjbz();
        if (StringUtils.isBlank(ybdjbz) || "0".equals(ybdjbz)) {
            String sjbm = cbkxx.getSjbm();
            if (StringUtils.isBlank(sjbm))
                throw new FeesException(String.format("水价未配置:%s", cbkid), FeesErrorCode.BUSINESS_ERROR);
            slMap.put("L0", String.format("%s_%s", sjbm, ljsl));
            return slMap;
        } else {
            String sjbm1 = cbkxx.getSjbm();
            String sjbm2 = cbkxx.getSjbm1();
            String sjbm3 = cbkxx.getSjbm2();
            BigDecimal slbl1 = cbkxx.getSlbl();
            BigDecimal slbl2 = cbkxx.getSlbl1();
            BigDecimal slbl3 = cbkxx.getSlbl2();
            //定量
            BigDecimal sl1 = null;
            BigDecimal sl2 = null;
            BigDecimal sl3 = null;
            //BigDecimal sl4 = null;
            BigDecimal sysl = ljsl;//剩余水量
            if (slbl1 != null && slbl1.compareTo(BigDecimal.ZERO) > 0 && StringUtils.isNotEmpty(sjbm1)) {
                if (slbl1.compareTo(BigDecimal.ONE) > 0) {//按量
                    if (sysl.compareTo(slbl1) >= 0) {
                        sl1 = slbl1;
                        sysl = sysl.subtract(slbl1);
                    } else {
                        sl1 = sysl;
                        sysl = BigDecimal.ZERO;
                    }
                    slMap.put("L1", String.format("%s_%s", sjbm1, sl1));
                }

            }

            if (slbl2 != null && slbl2.compareTo(BigDecimal.ZERO) > 0 && StringUtils.isNotEmpty(sjbm2)) {
                if (slbl2.compareTo(BigDecimal.ONE) > 0) {//按量
                    if (sysl.compareTo(slbl2) >= 0) {
                        sl2 = slbl2;
                        sysl = sysl.subtract(slbl2);
                    } else {
                        sl2 = sysl;
                        sysl = BigDecimal.ZERO;
                    }
                    slMap.put("L2", String.format("%s_%s", sjbm2, sl2));
                }
            }

            if (slbl3 != null && slbl3.compareTo(BigDecimal.ZERO) > 0 && StringUtils.isNotEmpty(sjbm3)) {
                if (slbl3.compareTo(BigDecimal.ONE) > 0) {//按量
                    if (sysl.compareTo(slbl3) >= 0) {
                        sl3 = slbl3;
                        sysl = sysl.subtract(slbl3);
                    } else {
                        sl3 = sysl;
                        sysl = BigDecimal.ZERO;
                    }
                    slMap.put("L3", String.format("%s_%s", sjbm3, sl3));
                }
            }

            BigDecimal sysl2 = sysl;//按量以外的剩余水量
            if (slbl1 != null && slbl1.compareTo(BigDecimal.ZERO) > 0 && StringUtils.isNotEmpty(sjbm1)) {
                if (slbl1.compareTo(BigDecimal.ONE) <= 0) {//按比例
                    sl1 = sysl2.multiply(slbl1);
                    sysl = sysl.subtract(sl1);
                    slMap.put("L1", String.format("%s_%s", sjbm1, sl1));
                }
            }

            if (slbl2 != null && slbl2.compareTo(BigDecimal.ZERO) > 0 && StringUtils.isNotEmpty(sjbm2)) {
                if (slbl2.compareTo(BigDecimal.ONE) <= 0) {//按比例
                    sl2 = sysl2.multiply(slbl2);
                    sysl = sysl.subtract(sl2);
                    slMap.put("L2", String.format("%s_%s", sjbm2, sl2));
                }
            }

            if (slbl3 != null && slbl3.compareTo(BigDecimal.ZERO) > 0 && StringUtils.isNotEmpty(sjbm3)) {
                if (slbl3.compareTo(BigDecimal.ONE) <= 0) {//按比例
                    sl3 = sysl2.multiply(slbl3);
                    sysl = sysl.subtract(sl3);
                    slMap.put("L3", String.format("%s_%s", sjbm3, sl3));
                }
            }

            if (slbl1 != null && slbl1.compareTo(BigDecimal.ZERO) == 0 && StringUtils.isNotEmpty(sjbm1)) {
                sl1 = sysl;
                sysl = BigDecimal.ZERO;
                slMap.put("L1", String.format("%s_%s", sjbm1, sl1));

            }
            if (slbl2 != null && slbl2.compareTo(BigDecimal.ZERO) == 0 && StringUtils.isNotEmpty(sjbm2)) {
                sl2 = sysl;
                sysl = BigDecimal.ZERO;
                slMap.put("L2", String.format("%s_%s", sjbm2, sl2));
            }
            if (slbl3 != null && slbl3.compareTo(BigDecimal.ZERO) == 0 && StringUtils.isNotEmpty(sjbm3)) {
                sl3 = sysl;
                sysl = BigDecimal.ZERO;
                slMap.put("L3", String.format("%s_%s", sjbm3, sl3));
            }

            if (sysl.compareTo(BigDecimal.ZERO) > 0) {
                throw new FeesException(String.format("一表多价有水量未进入计算，剩余水量=%s:%s", sysl, cbkid), FeesErrorCode.BUSINESS_ERROR);
            }
            return slMap;
        }
    }


    private boolean isEnabled(BigDecimal jtZs) {
        return jtZs != null && jtZs.compareTo(BigDecimal.ZERO) > 0;
    }

    private Sjmx.Jtmx getJtSf(BigDecimal sl, BigDecimal jtZs, BigDecimal jtSj, BigDecimal sjJtZs, BigDecimal ljsl) {
        //累计水量小于阶梯止数 并且 累计水量+本次水量大于上一阶梯止数
        if (ljsl.compareTo(jtZs) < 0 && ljsl.add(sl).compareTo(sjJtZs) > 0) {
            BigDecimal jtSf;//阶梯水费
            BigDecimal ys = jtZs.subtract(ljsl.add(sl));//求阶梯余数
            BigDecimal jtsl;
            if (ys.compareTo(BigDecimal.ZERO) >= 0) {//阶梯余数大于0则没有超过阶梯止数
                if (ljsl.compareTo(sjJtZs) >= 0) {//累计水量大于上一阶梯止数则无需排除上一阶梯水量直接用sl
                    jtsl = sl;
                } else {//累计水量小于上一阶梯止数则需要排除上一阶梯的sl
                    jtsl = sl.subtract(sjJtZs).add(ljsl);
                }
            } else {//阶梯余数小于0则超过阶梯止数 需要求阶梯剩余水量
                if (ljsl.add(sl).compareTo(jtZs) > 0) {//累计水量加水量大于阶梯止数则直接求阶梯剩余水量
                    if (ljsl.compareTo(sjJtZs) > 0) {//累计水量大于上一阶梯止数则表示当前阶梯有累计水量
                        jtsl = jtZs.subtract(ljsl);
                    } else {
                        jtsl = jtZs.subtract(sjJtZs);
                    }
                } else {//累计水量小于上一阶梯止数则需要sl加上ljsl再减去上一阶梯止数求当前阶梯水量
                    jtsl = sl.subtract(sjJtZs).add(ljsl);
                }
            }
            jtSf = fyjs(jtsl, jtSj);
            return new Sjmx.Jtmx(jtSj, jtsl, jtSf);
        }
        return new Sjmx.Jtmx(jtSj, BigDecimal.ZERO, BigDecimal.ZERO);
    }

    /**
     * 费用算费
     *
     * @param sl
     * @param sj
     * @return
     */
    private BigDecimal fyjs(BigDecimal sl, BigDecimal sj) {
        if (sj != null) return sl.multiply(sj);
        return BigDecimal.ZERO;
    }


}
