/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.feesapi.wxgl.wechatbasicmgt.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.feesapi.wxgl.wechatbasicmgt.entity.WxMessage;
import com.tofly.feesapi.wxgl.wechatbasicmgt.service.WxMessageService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;

/**
 * 微信模板消息
 *
 * @author admin
 * @date Tue Jul 04 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/wechatbasicmgt/wxmessage")
@Api(tags = "微信管理—微信模板消息接口")
public class WxMessageController {

    private final WxMessageService wxMessageService;

    /**
     * 分页查询
     *
     * @param page      分页对象
     * @param wxMessage 微信模板消息
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone getWxMessagePage(Page page, @ModelAttribute WxMessage wxMessage) {
        return ResultRespone.success(wxMessageService.page(page, Wrappers.query(wxMessage)));
    }


    /**
     * 通过id查询微信模板消息
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("id") String id) {
        return ResultRespone.success(wxMessageService.getById(id));
    }

    /**
     * 新增微信模板消息
     *
     * @param wxMessage 微信模板消息
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增微信模板消息")
    @ApiOperation(value = "新增微信模板消息")
    @PostMapping
    public ResultRespone save(@RequestBody WxMessage wxMessage) {
        return ResultRespone.success(wxMessageService.save(wxMessage));
    }

    /**
     * 修改微信模板消息
     *
     * @param wxMessage 微信模板消息
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改微信模板消息")
    @ApiOperation(value = "修改微信模板消息")
    @PutMapping
    public ResultRespone updateById(@RequestBody WxMessage wxMessage) {
        return ResultRespone.success(wxMessageService.updateById(wxMessage));
    }

    /**
     * 通过id删除微信模板消息
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除微信模板消息")
    @ApiOperation(value = "通过ID删除微信模板消息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable String id) {
        return ResultRespone.success(wxMessageService.removeById(id));
    }

    /**
     * 通过id批量删除微信模板消息
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除微信模板消息")
    @ApiOperation(value = "通过ID批量删除微信模板消息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(wxMessageService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
