/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.feesapi.wxgl.wechatbasicmgt.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.feesapi.common.util.GuidGenerator;
import com.tofly.feesapi.common.wechat.UserListJsonResult;
import com.tofly.feesapi.common.wechat.WxPayConfig;
import com.tofly.feesapi.common.wechat.Wxapi;
import com.tofly.feesapi.wxgl.wechatbasicmgt.entity.WxMessage;
import com.tofly.feesapi.wxgl.wechatbasicmgt.entity.WxToken;
import com.tofly.feesapi.wxgl.wechatbasicmgt.entity.WxTstz;
import com.tofly.feesapi.wxgl.wechatbasicmgt.mapper.WxTstzMapper;
import com.tofly.feesapi.wxgl.wechatbasicmgt.service.WxMessageService;
import com.tofly.feesapi.wxgl.wechatbasicmgt.service.WxTokenService;
import com.tofly.feesapi.wxgl.wechatbasicmgt.service.WxTstzService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 微信推送停水通知
 *
 * @author sxb
 * @date Mon Jun 05 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/wechatbasicmgt/wxtsnotice")
@Api(tags = "微信管理-微信推送停水通知接口")
public class WxTstzController {

    private final WxTstzService wxTstzService;

    private final WxTstzMapper wxTsTzMapper;


    private final WxTokenService wxTokenService;

    private final WxMessageService wxMessageService;


    /**
     * 分页查询  微信推送停水通知接口
     *
     * @param page 分页对象
     * @param
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "current", value = "当前页码", required = true, dataType = "int"),
            @ApiImplicitParam(name = "size", value = "每页显示数据条数", required = true, dataType = "int"),
            @ApiImplicitParam(name = "pages", value = "共几页,输出参数", dataType = "int"),
            @ApiImplicitParam(name = "maxLimit", value = "每页最大显示数据", dataType = "int"),
            @ApiImplicitParam(name = "orders[0].asc", value = "排序,bool类型", dataType = "String"),
            @ApiImplicitParam(name = "orders[0].column", value = "排序,字段名", dataType = "String"),
            @ApiImplicitParam(name = "state", value = "是否发送，0,1", dataType = "String"),
            @ApiImplicitParam(name = "editDateStart", value = "开始日期", dataType = "String"),
            @ApiImplicitParam(name = "editDateEnd", value = "结束日期", dataType = "String"),
            @ApiImplicitParam(name = "searchText", value = "模糊查询(编辑人员、标题、停水区域)", dataType = "String")
    })
    public ResultRespone getWxTsNoticePage(Page page, String searchText, String state, String editDateStart, String editDateEnd) {
        IPage<WxTstz> WxTsPage = wxTsTzMapper.selectWxTsNoticePage(page, searchText, state, editDateStart, editDateEnd);
        return ResultRespone.success(WxTsPage);
    }


    /**
     * 通过id查询微信推送停水通知
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    public ResultRespone getById(@PathVariable("id") Integer id) {
        return ResultRespone.success(wxTstzService.getById(id));
    }

    /**
     * 新增微信推送停水通知
     *
     * @param wxTsTz 微信推送停水通知
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增微信推送停水通知")
    @ApiOperation(value = "新增微信推送停水通知")
    @PostMapping
    public ResultRespone save(@RequestBody WxTstz wxTsTz) {
        //查询当前ID
        int id = wxTsTzMapper.SelectChargeMaxCode();
        wxTsTz.setId(id + 1);
        wxTsTz.setState("0");//推送状态
        wxTsTz.setAuditFlag("0");//未审核
        return ResultRespone.success(wxTstzService.save(wxTsTz));
    }

    /**
     * 发送微信推送停水通知
     *
     * @param wxTsTz 微信推送停水通知
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "发送微信推送停水通知")
    @ApiOperation(value = "发送微信推送停水通知")
    @PostMapping("/send")
    public ResultRespone send(@RequestBody WxTstz wxTsTz) {
        WxPayConfig wxPayConfig = new WxPayConfig();

//region 获取accessToken和expiresdate，openid
        WxToken wxToken = wxTokenService.getById(wxPayConfig.getAPPID());
        String accessToken = wxToken.getAccessToken();
        Date expiresDate = wxToken.getExpiresdate();


        int resultcount = 10000;
        String nextopenid = "";
        List<String> listpenid = new ArrayList<>();
        //int j = 2;
        while (resultcount >= 10000) {
            Date date = new Date();
            // 判断access_token是否在有效期内,无效重新获取
            if (date.compareTo(expiresDate) >= 0 || accessToken == "") {

                Map<String, String> map = Wxapi.GetAccessToken();
                accessToken = map.get("accessToken");
                if (accessToken != "") {
                    long currentTime = System.currentTimeMillis();
                    currentTime += 120 * 60 * 1000;
                    expiresDate = new Date(currentTime);
                    wxToken.setAccessToken(accessToken);
                    wxToken.setExpiresdate(expiresDate);
                    wxTokenService.updateById(wxToken);
                } else {
                    break;
                }

            }
            UserListJsonResult userlist = Wxapi.GetUserList(accessToken, nextopenid);
            if (userlist != null && userlist.data != null) {
                listpenid.addAll(userlist.data.openid);
                resultcount = userlist.count;
                nextopenid = userlist.next_openid;

            } else {
                break;
            }

        }
//endregion

        List<WxMessage> ListMessage = new ArrayList<>();

        for (int i = 0; i < listpenid.size(); i++) {
//            for (int i = 0; i < 5; i++) {
            WxMessage wxMessage = new WxMessage();
            wxMessage.setId(GuidGenerator.getGuid());
            wxMessage.setMsgType("06");
            wxMessage.setOpenid(listpenid.get(i));
//                wxMessage.setOpenid("openid_test");
            wxMessage.setFirst(wxTsTz.getTitle());
            wxMessage.setKeyword1(wxTsTz.getStopType());
            wxMessage.setKeyword2(wxTsTz.getStopTime());
            wxMessage.setKeyword3(wxTsTz.getStopArea());
            wxMessage.setRemark(wxTsTz.getRemark());
            wxMessage.setAddDate(wxTsTz.getEditDate());
            wxMessage.setAddOcode(wxTsTz.getEditCode());
            wxMessage.setSendState("0");
            wxMessage.setIsLock("0");

            ListMessage.add(wxMessage);

        }
        return ResultRespone.success(wxMessageService.saveBatch(ListMessage));

    }

    /**
     * 修改微信推送停水通知
     *
     * @param wxTsTz 微信推送停水通知
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改微信推送停水通知")
    @ApiOperation(value = "修改微信推送停水通知")
    @PutMapping
    public ResultRespone updateById(@RequestBody WxTstz wxTsTz) {
        return ResultRespone.success(wxTstzService.updateById(wxTsTz));
    }

    /**
     * 通过id删除微信推送停水通知
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除微信推送停水通知")
    @ApiOperation(value = "通过ID删除微信推送停水通知")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable Integer id) {
        return ResultRespone.success(wxTstzService.removeById(id));
    }

    /**
     * 通过id批量删除微信推送停水通知
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除微信推送停水通知")
    @ApiOperation(value = "通过ID批量删除微信推送停水通知")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(wxTstzService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
