/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.feesapi.yhgl.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.feesapi.common.Excel.ExcleFileUtil;
import com.tofly.feesapi.common.dbhelper.PageSortHelper;
import com.tofly.feesapi.common.util.GuidGenerator;
import com.tofly.feesapi.sfgl.mapper.YyQfmxMapper;
import com.tofly.feesapi.udb.smpn.entity.Company;
import com.tofly.feesapi.udb.smpn.service.ICompanyService;
import com.tofly.feesapi.yhgl.entity.YhGhLog;
import com.tofly.feesapi.yhgl.entity.YhHfysLog;
import com.tofly.feesapi.yhgl.entity.YhKhjbxx;
import com.tofly.feesapi.yhgl.entity.Yhkhjbxx2;
import com.tofly.feesapi.yhgl.entity.vo.YhBtLogSelect;
import com.tofly.feesapi.yhgl.entity.vo.YhGhLogSelect;
import com.tofly.feesapi.yhgl.mapper.YhGhLogMapper;
import com.tofly.feesapi.yhgl.mapper.YhKhjbxxMapper;
import com.tofly.feesapi.yhgl.service.YhGhLogService;
import com.tofly.feesapi.yhgl.service.YhKhjbxxService;
import lombok.AllArgsConstructor;
import org.apache.commons.lang.StringUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 用户过户日志
 *
 * @author sxb
 * @date Thu Feb 29 00:00:00 CST 2024
 */
@RestController
@AllArgsConstructor
@RequestMapping("api/yhgl/yhghlog")
@Api(tags = "用户管理-用户过户日志接口")
public class YhGhLogController {

    private final YhKhjbxxService yhKhjbxxService;
    private final YhKhjbxxMapper yhKhjbxxMapper;
    private final YhGhLogService yhGhLogService;
    private final YhGhLogMapper yhGhLogMapper;
    private final YyQfmxMapper yyQfmxMapper;
    private final ICompanyService companyService;
    /**
     * 分页查询
     *
     * @param page 分页对象
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "current", value = "当前页码", required = true, dataType = "int"),
            @ApiImplicitParam(name = "size", value = "每页显示数据条数", required = true, dataType = "int"),
            @ApiImplicitParam(name = "pages", value = "共几页,输出参数", dataType = "int"),
            @ApiImplicitParam(name = "maxLimit", value = "每页最大显示数据", dataType = "int"),
            @ApiImplicitParam(name = "orders[0].asc", value = "排序,bool类型", dataType = "String"),
            @ApiImplicitParam(name = "orders[0].column", value = "排序,字段名", dataType = "String"),
            @ApiImplicitParam(name = "searchText", value = "模糊查询(用户编号、用户姓名、用户地址、联系电话)", dataType = "String"),
            @ApiImplicitParam(name = "ssgs", value = "公司", dataType = "String"),
            @ApiImplicitParam(name = "sscbpq", value = "片区", dataType = "String"),
            @ApiImplicitParam(name = "sscbb", value = "抄表本", dataType = "String"),
            @ApiImplicitParam(name = "ksrq", value = "开始时间", dataType = "String"),
            @ApiImplicitParam(name = "jsrq", value = "结束时间", dataType = "String")
    })
    public ResultRespone getDaGhLogPage(Page page, String ssgs, String sscbpq, String sscbb, String ksrq, String jsrq, String searchText) {
        PageSortHelper.setPageSort(YhGhLog.class, page);
        IPage<YhGhLogSelect> daGhLogIPage = yhGhLogMapper.selectDaCustomerGhPage(page, ssgs, sscbpq, sscbb, ksrq, jsrq, searchText);
        List<YhGhLogSelect> yhghList = daGhLogIPage.getRecords();
        for (YhGhLogSelect yhgh : yhghList) {
            if (yhgh.getSsgs() != null) {
                Company company = companyService.get(Long.valueOf(yhgh.getSsgs()));
                if (company != null) {
                    yhgh.setSsgsname(company.getCompanyName());
                }
            }
        }
        return ResultRespone.success(daGhLogIPage);
    }


    /**
     * 通过id查询用户过户日志
     *
     * @param bh id
     * @return ResultRespone
     */
    @GetMapping("/{bh}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "bh", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("bh") String bh) {
        return ResultRespone.success(yhGhLogService.getById(bh));
    }

    /**
     * 新增用户过户日志
     *
     * @param yhGhLog 用户过户日志
     * @return ResultRespone
     */
    @ApiOperation(value = "新增用户过户日志")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "yzqf", value = "是否验证欠费（1：需验证；0：不验证）", required = false, dataType = "String")
    })
    @PostMapping
    @Transactional(rollbackFor = Exception.class) //声明式事务管理
    public ResultRespone save(@RequestBody YhGhLog yhGhLog, @RequestParam(required = false) String yzqf) {
        ResultRespone<Object> rest = new ResultRespone<>();
        if (StringUtils.isNotBlank(yhGhLog.getYhbh())) {
            if (yzqf.equals("1")) {
                //判断用户是否存在欠费
                int count = yyQfmxMapper.selectExistQf(yhGhLog.getYhbh());
                if (count > 0) {
                    return ResultRespone.failed(false, "操作失败,用户存在欠费:" + count + "笔");
                }
            }

            yhGhLog.setBh(GuidGenerator.getGuid().substring(0, 8));//id
            yhGhLog.setGhsj(new Date());//操作时间
            if (yhGhLogService.save(yhGhLog)) {
                //同步用户档案状态
                YhKhjbxx yhKhjbxx = new YhKhjbxx();
                yhKhjbxx.setYhbh(yhGhLog.getYhbh());//用户编号
                yhKhjbxx.setYhxm(yhGhLog.getNewYhxm());//姓名
                yhKhjbxx.setLxdh(yhGhLog.getNewLxdh());//电话
                if (StringUtils.isNotBlank(yhGhLog.getNewYhdz())) {
                    yhKhjbxx.setYhdz(yhGhLog.getNewYhdz());//地址
                }
                if (StringUtils.isNotBlank(yhGhLog.getNewZjlb())) {
                    yhKhjbxx.setZjlb(yhGhLog.getNewZjlb());//证件类型
                }
                if (StringUtils.isNotBlank(yhGhLog.getYhbh())) {
                    yhKhjbxx.setZjhm(yhGhLog.getNewZjhm());//证件号码
                }

                if (yhKhjbxxService.updateById(yhKhjbxx)) {
                    return ResultRespone.success(true, "操作成功");
                } else {
                    return ResultRespone.failed(false, "操作失败");
                }
            } else {
                return ResultRespone.failed(false, "操作失败");
            }

        } else {
            return ResultRespone.failed(false, "操作失败,用户编号不能为空");
        }
    }


    /**
     * 修改用户过户日志
     *
     * @param yhGhLog 用户过户日志
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改用户过户日志")
    @ApiOperation(value = "修改用户过户日志")
    @PutMapping
    public ResultRespone updateById(@RequestBody YhGhLog yhGhLog) {
        return ResultRespone.success(yhGhLogService.updateById(yhGhLog));
    }

    /**
     * 通过id删除用户过户日志
     *
     * @param bh id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除用户过户日志")
    @ApiOperation(value = "通过ID删除用户过户日志")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "bh", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{bh}")
    public ResultRespone removeById(@PathVariable String bh) {
        return ResultRespone.success(yhGhLogService.removeById(bh));
    }

    /**
     * 通过id批量删除用户过户日志
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除用户过户日志")
    @ApiOperation(value = "通过ID批量删除用户过户日志")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(yhGhLogService.removeByIds(Arrays.asList(ids.split(","))));
    }

    @GetMapping("/export")
    @ApiOperation(value = "export导出")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ssgs", value = "公司", dataType = "String"),
            @ApiImplicitParam(name = "sscbpq", value = "片区", dataType = "String"),
            @ApiImplicitParam(name = "sscbb", value = "抄表本", dataType = "String"),
            @ApiImplicitParam(name = "ksrq", value = "开始时间", dataType = "String"),
            @ApiImplicitParam(name = "jsrq", value = "结束时间", dataType = "String"),
            @ApiImplicitParam(name = "searchText", value = "模糊查询(编号、姓名、电话、地址)", dataType = "String")
    })
    public void export(HttpServletResponse response, String ssgs, String sscbpq, String sscbb, String ksrq, String jsrq, String searchText) throws IOException {
        List<YhGhLogSelect> list = yhGhLogMapper.selectDaCustomerGhPage(ssgs, sscbpq, sscbb, ksrq, jsrq, searchText);
        for (YhGhLogSelect yhgh : list) {
            if (yhgh.getSsgs() != null) {
                Company company = companyService.get(Long.valueOf(yhgh.getSsgs()));
                if (company != null) {
                    yhgh.setSsgsname(company.getCompanyName());
                }
            }
        }
        //导出Excel
        ExcleFileUtil.exportExcel(response, list, YhGhLogSelect.class, "用户过户记录.xlsx");
    }
}
