/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.feesapi.yhgl.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.feesapi.common.Excel.ExcleFileUtil;
import com.tofly.feesapi.common.constant.YhztEnum;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.feesapi.common.dbhelper.PageSortHelper;
import com.tofly.feesapi.common.util.GuidGenerator;
import com.tofly.feesapi.sfgl.mapper.YyQfmxMapper;
import com.tofly.feesapi.udb.smpn.entity.Company;
import com.tofly.feesapi.udb.smpn.service.ICompanyService;
import com.tofly.feesapi.yhgl.entity.YhHfysLog;
import com.tofly.feesapi.yhgl.entity.YhKhjbxx;
import com.tofly.feesapi.yhgl.entity.YhhLog;
import com.tofly.feesapi.yhgl.entity.Yhkhjbxx2;
import com.tofly.feesapi.yhgl.entity.vo.YhBtLogSelect;
import com.tofly.feesapi.yhgl.entity.vo.YhhLogSelect;
import com.tofly.feesapi.yhgl.mapper.YhKhjbxxMapper;
import com.tofly.feesapi.yhgl.mapper.YhhLogMapper;
import com.tofly.feesapi.yhgl.service.YhKhjbxxService;
import com.tofly.feesapi.yhgl.service.YhhLogService;
import lombok.AllArgsConstructor;
import org.apache.commons.lang.StringUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

/**
 * 用户销户记录
 *
 * @author sxb
 * @date Tue Aug 08 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("api/yhgl/yhhlog")
@Api(tags="用户管理-用户销户记录接口")
public class YhhLogController {

    private final  YhhLogService yhhLogService;
    private final YhhLogMapper yhhLogMapper;
    private final YhKhjbxxMapper yhKhjbxxMapper;
    private final YhKhjbxxService yhKhjbxxService;
    private  final YyQfmxMapper yyQfmxMapper;
    private final ICompanyService companyService;
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "current", value = "当前页码", required = true, dataType = "int"),
            @ApiImplicitParam(name = "size", value = "每页显示数据条数", required = true, dataType = "int"),
            @ApiImplicitParam(name = "pages", value = "共几页,输出参数", dataType = "int"),
            @ApiImplicitParam(name = "maxLimit", value = "每页最大显示数据", dataType = "int"),
            @ApiImplicitParam(name = "orders[0].asc", value = "排序,bool类型", dataType = "String"),
            @ApiImplicitParam(name = "orders[0].column", value = "排序,字段名", dataType = "String"),
            @ApiImplicitParam(name = "ssgs", value = "公司", dataType = "String"),
            @ApiImplicitParam(name = "sscbpq", value = "片区", dataType = "String"),
            @ApiImplicitParam(name = "sscbb", value = "抄表本", dataType = "String"),
            @ApiImplicitParam(name = "ksrq", value = "开始时间", dataType = "String"),
            @ApiImplicitParam(name = "jsrq", value = "结束时间", dataType = "String"),
            @ApiImplicitParam(name = "searchText", value = "模糊查询(编号、姓名、电话、地址)", dataType = "String")
    })
    public ResultRespone getDahLogPage(Page page, String ssgs, String sscbpq, String sscbb, String ksrq, String jsrq, String searchText) {
        PageSortHelper.setPageSort(YhhLog.class, page);
        IPage<YhhLogSelect> dahLogIPage = yhhLogMapper.SelectDaxhPage(page, ssgs, sscbpq, sscbb, ksrq, jsrq, searchText);
        List<YhhLogSelect> yhxhList = dahLogIPage.getRecords();
        for (YhhLogSelect yhxh : yhxhList) {
            if (yhxh.getSsgs() != null) {
                Company company = companyService.get(Long.valueOf(yhxh.getSsgs()));
                if (company != null) {
                    yhxh.setSsgsname(company.getCompanyName());
                }
            }
        }
        return ResultRespone.success(dahLogIPage);
    }


    /**
     * 通过id查询用户销户记录
     * @param bh id
     * @return ResultRespone
     */
    @GetMapping("/{bh}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "bh", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("bh" ) String bh) {
      return ResultRespone.success(yhhLogService.getById(bh));
    }

    /**
     * 新增用户销户记录
     *
     * @param yhhLog 用户销户记录
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增用户销户记录")
    @ApiOperation(value = "新增用户销户记录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "yzqf", value = "是否验证欠费（1：需验证；0：不验证）", required = false, dataType = "String")
    })
    @PostMapping
    @Transactional(rollbackFor = Exception.class) //声明式事务管理
    public ResultRespone save(@RequestBody YhhLog yhhLog, @RequestParam(required = false) String yzqf) {
        ResultRespone<Object> rest = new ResultRespone<>();
        yhhLog.setBh(GuidGenerator.getGuid().substring(0, 8));

        if (StringUtils.isNotBlank(yhhLog.getYhbh())) {
            if (yzqf.equals("1")) {
                //判断用户是否存在欠费
                int count= yyQfmxMapper.selectExistQf(yhhLog.getYhbh());
                if (count > 0) {
                    rest.setCode(-1);
                    rest.setMessage("操作失败,用户存在欠费:" + count + "笔");
                    return rest;
                }
            }

            if (yhhLogService.save(yhhLog)) {
                //同步用户档案状态
                YhKhjbxx yhKhjbxx = new YhKhjbxx();
                yhKhjbxx.setYhbh(yhhLog.getYhbh());//用户编号
                yhKhjbxx.setYhzt(YhztEnum.销户用户);
                if (yhKhjbxxService.updateById(yhKhjbxx)) {
                    rest.setCode(1);
                    rest.setMessage("操作成功");
                } else {
                    rest.setCode(-1);
                    rest.setMessage("操作失败");
                }
            } else {
                rest.setCode(-1);
                rest.setMessage("操作失败");
            }
        } else {
            rest.setCode(-1);
            rest.setMessage("操作失败，用户编号不能为空");
        }
        return rest;
    }

    /**
     * 修改用户销户记录
     * @param yhhLog 用户销户记录
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改用户销户记录" )
    @ApiOperation(value = "修改用户销户记录")
    @PutMapping
    public ResultRespone updateById(@RequestBody YhhLog yhhLog) {
      return ResultRespone.success(yhhLogService.updateById(yhhLog));
    }

    /**
     * 通过id删除用户销户记录
     * @param bh id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除用户销户记录" )
    @ApiOperation(value = "通过ID删除用户销户记录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "bh", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{bh}" )
    public ResultRespone removeById(@PathVariable String bh) {
      return ResultRespone.success(yhhLogService.removeById(bh));
    }

    /**
     * 通过id批量删除用户销户记录
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除用户销户记录" )
    @ApiOperation(value = "通过ID批量删除用户销户记录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(yhhLogService.removeByIds(Arrays.asList(ids.split(","))));
    }

    @GetMapping("/export")
    @ApiOperation(value = "export导出")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ssgs", value = "公司", dataType = "String"),
            @ApiImplicitParam(name = "sscbpq", value = "片区", dataType = "String"),
            @ApiImplicitParam(name = "sscbb", value = "抄表本", dataType = "String"),
            @ApiImplicitParam(name = "ksrq", value = "开始时间", dataType = "String"),
            @ApiImplicitParam(name = "jsrq", value = "结束时间", dataType = "String"),
            @ApiImplicitParam(name = "searchText", value = "模糊查询(编号、姓名、电话、地址)", dataType = "String")
    })
    public void export(HttpServletResponse response, String ssgs, String sscbpq, String sscbb, String ksrq, String jsrq, String searchText) throws IOException {
        // 遍历查询结果进行赋值
        List<YhhLogSelect> list = yhhLogMapper.SelectDaxhPage(ssgs, sscbpq, sscbb, ksrq, jsrq, searchText);
        for (YhhLogSelect yhxh : list) {
            if (yhxh.getSsgs() != null) {
                Company company = companyService.get(Long.valueOf(yhxh.getSsgs()));
                if (company != null) {
                    yhxh.setSsgsname(company.getCompanyName());
                }
            }
        }
        //导出excle
        ExcleFileUtil.exportExcel(response, list, YhhLogSelect.class, "用户销户记录.xlsx");
    }
}
