package com.tofly.feesapi.znbgl.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.feesapi.znbgl.entity.ZnbDaLog;
import com.tofly.feesapi.znbgl.service.ZnbDaLogService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;

/**
 * <p>
 * 智能表档案明细 前端控制器
 * </p>
 *
 * @author decai
 * @since 2024-04-23
 */
@RestController
@AllArgsConstructor
@RequestMapping("/znbDaLog")
@Api(tags="智能表用户档案记录日志接口")
public class ZnbDaLogController {

    private final ZnbDaLogService znbDaLogService;

    /**
     * 分页查询
     * @param page 分页对象
     * @param znbDaLog 智能表用户档案
     * @return
     */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getZnbDaLogPage(Page page, @ModelAttribute ZnbDaLog znbDaLog) {
        return ResultRespone.success(znbDaLogService.page(page, Wrappers.query(znbDaLog)));
    }


    /**
     * 通过id查询智能表用户档案记录
     * @param yhbh id
     * @return ResultRespone
     */
    @GetMapping("/{yhbh}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "yhbh", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("yhbh" ) String yhbh) {
        return ResultRespone.success(znbDaLogService.getById(yhbh));
    }

    /**
     * 新增智能表用户档案记录
     * @param znbDaLog 智能表用户档案记录
     * @return ResultRespone
     */
    @ApiOperation(value = "新增智能表用户档案记录")
    @PostMapping
    public ResultRespone save(@RequestBody ZnbDaLog znbDaLog) {
        return ResultRespone.success(znbDaLogService.save(znbDaLog));
    }

    /**
     * 修改智能表用户档案记录
     * @param znbDaLog 智能表用户档案记录
     * @return ResultRespone
     */
    @ApiOperation(value = "修改智能表用户档案记录")
    @PutMapping
    public ResultRespone updateById(@RequestBody ZnbDaLog znbDaLog) {
        return ResultRespone.success(znbDaLogService.updateById(znbDaLog));
    }

    /**
     * 通过id删除智能表用户档案记录
     * @param yhbh id
     * @return ResultRespone
     */
    @ApiOperation(value = "通过ID删除智能表用户档案记录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "yhbh", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{yhbh}" )
    public ResultRespone removeById(@PathVariable String yhbh) {
        return ResultRespone.success(znbDaLogService.removeById(yhbh));
    }

    /**
     * 通过id批量删除智能表用户档案记录
     * @param ids id
     * @return ResultRespone
     */
    @ApiOperation(value = "通过ID批量删除智能表用户档案记录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(znbDaLogService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
