package com.tofly.common.core.entity;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.constant.ResponeStatus;
import com.tofly.fees.common.model.CollectionData;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;

public class ResultRespone<T> implements Serializable {
    private static final long serialVersionUID = -7275852267551207696L;
    private int code;
    private String message;
    private T result;

    public ResultRespone() {
        this.code = ResponeStatus.SUCCESS.getCode();
    }

    public static <T> ResultRespone<Object> success() {
        return restResult((Object) null, ResponeStatus.SUCCESS.getCode(), (String) null);
    }

    public static <T> ResultRespone<T> success(T result) {
        return restResult(result, ResponeStatus.SUCCESS.getCode(), (String) null);
    }

    public static <T> ResultRespone<T> success(T result, String message) {
        return restResult(result, ResponeStatus.SUCCESS.getCode(), message);
    }

    public static <T> ResultRespone<Object> failed() {
        return restResult((Object) null, ResponeStatus.FAIL.getCode(), (String) null);
    }

    public static <T> ResultRespone<Object> failed(String message) {
        return restResult((Object) null, ResponeStatus.FAIL.getCode(), message);
    }

    public static <T> ResultRespone<T> failed(T result) {
        return restResult(result, ResponeStatus.FAIL.getCode(), (String) null);
    }

    public static <T> ResultRespone<T> failed(T result, String message) {
        return restResult(result, ResponeStatus.FAIL.getCode(), message);
    }

    private static <T> ResultRespone<T> restResult(T result, int code, String message) {
        ResultRespone<T> apiResult = new ResultRespone();
        apiResult.setCode(code);
        if (result instanceof ArrayList) {
            //自定义 List<HashMap<String, Object>>
            result = (T) getConvertList((ArrayList) result);
            CollectionData data=new CollectionData((Collection) result,((ArrayList<?>) result).size());
            result= (T) data;
        } else if (result instanceof Page) {
            //自定义 IPage<Map<String, Object>>
            if (((Page<?>) result).getRecords().size() > 0) {
                ArrayList list = (ArrayList) getConvertList((ArrayList) ((Page<?>) result).getRecords());
                ((Page<?>) result).setRecords(list);
            }
        } else if (result instanceof CollectionData) {
            //DbHelper方式
            CollectionData<?> collectionData = ((CollectionData<?>) result);
            collectionData.setRecords(getConvertList((ArrayList) collectionData.getRecords()));
            collectionData.setTotal(collectionData.getTotal());
            result = (T) collectionData;
        }
        apiResult.setResult(result);
        apiResult.setMessage(message);
        return apiResult;
    }

    public String toString() {
        return "ResultRespone(code=" + this.getCode() + ", message=" + this.getMessage() + ", result=" + this.getResult() + ")";
    }

    public int getCode() {
        return this.code;
    }

    public void setCode(final int code) {
        this.code = code;
    }

    public String getMessage() {
        return this.message;
    }

    public void setMessage(final String message) {
        this.message = message;
    }

    public T getResult() {
        return this.result;
    }

    public void setResult(final T result) {
        this.result = result;
    }


    /**
     * 获取转换后的驼峰命名ArrayList集合
     *
     * @param listData
     * @return
     */
    public static ArrayList getConvertList(ArrayList listData) {
        if (listData != null && listData.size() > 0) {
            Object obj = (Object) listData.get(0);
            if (!(obj instanceof HashMap)) {
                return listData;
            }
            List<HashMap> mapList = new ArrayList<>();
            for (int i = 0, len = listData.size(); i < len; i++) {
                HashMap<String, Object> resultHashMap = new HashMap<>();
                HashMap<String, Object> hasMap = (HashMap) listData.get(i);
                for (Object key : hasMap.keySet()) {
                    if (key instanceof ArrayList) {
                        mapList = getConvertList((ArrayList) key);
                    } else if (key instanceof String) {
                        String[] arrSrc = key.toString().split("_");
                        String strNewKey = "";
                        String strTemp = "";
                        for (int k = 0; k < arrSrc.length; k++) {
                            if (arrSrc[k].equals(arrSrc[k].toUpperCase())) {
                                strTemp = arrSrc[k].toLowerCase();
                            } else {
                                strTemp = arrSrc[k];
                            }
                            if (k > 0) {
                                strTemp = strTemp.substring(0, 1).toUpperCase() + strTemp.substring(1);
                            }
                            strNewKey += strTemp;
                        }
                        resultHashMap.put(strNewKey, hasMap.get(key));
                    }
                }
                mapList.add(resultHashMap);
            }

            return (ArrayList) mapList;
        }
        return listData;

    }
}
