/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.codemgt.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.codemgt.entity.BmChargeChannel;
import com.tofly.fees.codemgt.mapper.BmChargeChannelMapper;
import com.tofly.fees.codemgt.service.BmChargeChannelService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 收费点
 *
 * @author sxb
 * @date Thu May 18 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/codemgt/bmchargechannel")
@Api(tags="编码管理-收费点接口")
public class BmChargeChannelController {

    private final BmChargeChannelService bmChargeChannelService;
    private final BmChargeChannelMapper bmChargeChannelMapper;

    /**
     * page分页查询 收费点接口
     * @param page
     * @param searchText
     * @return
     */
    @GetMapping("/page" )
    @ApiOperation(value = "page分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "current", value = "当前页码", required = true, dataType = "int"),
            @ApiImplicitParam(name = "size", value = "每页显示数据条数", required = true, dataType = "int"),
            @ApiImplicitParam(name = "pages", value = "共几页,输出参数",  dataType = "int"),
            @ApiImplicitParam(name = "maxLimit", value = "每页最大显示数据",  dataType = "int"),
            @ApiImplicitParam(name = "orders[0].asc", value = "排序,bool类型",  dataType = "String"),
            @ApiImplicitParam(name = "orders[0].column", value = "排序,字段名",  dataType = "String"),
            @ApiImplicitParam(name = "searchText", value = "模糊查询(营业所名称)",  dataType = "String")
    })
    public ResultRespone getBmChargeChannelPage(Page page,String searchText) {
        QueryWrapper<BmChargeChannel> queryWrapper = new QueryWrapper<BmChargeChannel>();
        if(StringUtils.isNotBlank(searchText))
        {  //like ‘%营业点名称%’
            queryWrapper.like("NAME",searchText);//营业点名称
        }
        // 执行分页查询
        IPage<BmChargeChannel> yydPage = bmChargeChannelService.page(page,queryWrapper);
        // 遍历查询结果进行赋值
        List<BmChargeChannel> yydList = yydPage.getRecords();
//        for (BmChargeChannel yyd : yydList) {
//            if(yyd.getIsextunit().equals("0"))
//            {
//                yyd.setIsextunit("否");
//            }else
//            {
//                yyd.setIsextunit("是");
//            }
//        }


      return ResultRespone.success(yydPage);
    }


    /**
     * 通过id查询收费点
     * @param code id
     * @return ResultRespone
     */
    @GetMapping("/{code}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("code" ) String code) {
      return ResultRespone.success(bmChargeChannelService.getById(code));
    }

    /**
     * 新增收费点
     * @param bmChargeChannel 收费点
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增收费点" )
    @ApiOperation(value = "新增收费点")
    @PostMapping
    public ResultRespone save(@RequestBody BmChargeChannel bmChargeChannel) {

        //查询当前ID
        Map<String ,Object> map=  bmChargeChannelMapper.SelectChargeMaxCode();
         int id=Integer.parseInt(map.get("ID").toString());
         bmChargeChannel.setCode(id+1);

      return ResultRespone.success(bmChargeChannelService.save(bmChargeChannel));
    }

    /**
     * 修改收费点
     * @param bmChargeChannel 收费点
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改收费点" )
    @ApiOperation(value = "修改收费点")
    @PutMapping
    public ResultRespone updateById(@RequestBody BmChargeChannel bmChargeChannel) {
      return ResultRespone.success(bmChargeChannelService.updateById(bmChargeChannel));
    }

    /**
     * 通过id删除收费点
     * @param code id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除收费点" )
    @ApiOperation(value = "通过ID删除收费点")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{code}" )
    public ResultRespone removeById(@PathVariable String code) {
      return ResultRespone.success(bmChargeChannelService.removeById(code));
    }

    /**
     * 通过id批量删除收费点
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除收费点" )
    @ApiOperation(value = "通过ID批量删除收费点")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(bmChargeChannelService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
