/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.codemgt.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.codemgt.entity.BmMeterFactory;
import com.tofly.fees.codemgt.entity.BmMeterType;
import com.tofly.fees.codemgt.mapper.BmMeterFactoryMapper;
import com.tofly.fees.codemgt.service.BmMeterFactoryService;
import com.tofly.fees.codemgt.service.BmMeterTypeService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 水表生产厂家
 *
 * @author sxb
 * @date Thu May 18 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/codemgt/bmmeterfactory")
@Api(tags="编码管理-水表生产厂家接口")
public class BmMeterFactoryController {

    private final BmMeterFactoryService bmMeterFactoryService;

    private final BmMeterTypeService bmMeterTypeService; //水表类型

     private  final  BmMeterFactoryMapper bmMeterFactoryMapper;
    /**
     * page分页查询
     * @param page
     * @param searchText
     * @return
     */
    @GetMapping("/page" )
    @ApiOperation(value = "page分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "current", value = "当前页码", required = true, dataType = "int"),
            @ApiImplicitParam(name = "size", value = "每页显示数据条数", required = true, dataType = "int"),
            @ApiImplicitParam(name = "pages", value = "共几页,输出参数",  dataType = "int"),
            @ApiImplicitParam(name = "maxLimit", value = "每页最大显示数据",  dataType = "int"),
            @ApiImplicitParam(name = "orders[0].asc", value = "排序,bool类型",  dataType = "String"),
            @ApiImplicitParam(name = "orders[0].column", value = "排序,字段名",  dataType = "String"),
            @ApiImplicitParam(name = "searchText", value = "模糊查询(编码，水表厂家名称、厂家地址)",  dataType = "String"),
            @ApiImplicitParam(name = "isdisabled", value = "是否启用,0,1",  dataType = "String")
    })
    public ResultRespone getBmMeterFactoryPage(Page page,String searchText,String isdisabled) {
        //条件构造
        QueryWrapper<BmMeterFactory> queryWrapper = new QueryWrapper<BmMeterFactory>();
        if(StringUtils.isNotBlank(searchText))
        {
             if(StringUtils.isNotBlank(isdisabled))
            {//like ‘%编码\水表厂家名称\水表类别%’
                queryWrapper.in("ISDISABLED",isdisabled)   //是否启用
                           .and(i->i.like("code",searchText).or().like("name",searchText).or().like("meter_type",searchText));
            }
             else
             {
                 queryWrapper.like("code",searchText)  //编码
                         .or()
                         .like("name",searchText) //水表厂家名称
                         .or()
                         .like("meter_type",searchText);//水表类别
             }
        }
        else
        {
            if(StringUtils.isNotBlank(isdisabled)) {
                queryWrapper.in("ISDISABLED", isdisabled);  //编码
            }
        }
        // 执行分页查询
        IPage<BmMeterFactory>  sbcjPage = bmMeterFactoryService.page(page,queryWrapper);
        // 遍历查询结果进行赋值
         List<BmMeterFactory> sbcjList = sbcjPage.getRecords();
         //水表类型查询
        List<BmMeterType> sblxList=bmMeterTypeService.list();
        Map<String,String> map = new HashMap<>();
        sblxList.forEach(e->map.put(e.getCode().toString(), e.getName()));

        for (BmMeterFactory sbcj : sbcjList)
         {
             //水表类型
//             sbcj.setMeterType(map.get(sbcj.getMeterType()));
//             if(sbcj.getIsdisabled().equals("1"))
//             {
//                 sbcj.setIsdisabled("启用");
//             }
//             else {
//                 sbcj.setIsdisabled("禁用");
//             }
         }
      return ResultRespone.success(sbcjPage);
    }


    /**
     * 通过id查询水表生产厂家
     * @param code id
     * @return ResultRespone
     */
    @GetMapping("/{code}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "ID", required = true, dataType = "BigDecimal")
    })
    public ResultRespone getById(@PathVariable("code" ) BigDecimal code) {
      return ResultRespone.success(bmMeterFactoryService.getById(code));
    }

    /**
     * 新增水表生产厂家
     * @param bmMeterFactory 水表生产厂家
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增水表生产厂家" )
    @ApiOperation(value = "新增水表生产厂家")
    @PostMapping
    public ResultRespone save(@RequestBody BmMeterFactory bmMeterFactory) {
        //查询当前ID
        Map<String ,Object> map=  bmMeterFactoryMapper.SelectChargeMaxCode();
        int id=Integer.parseInt(map.get("ID").toString());
        bmMeterFactory.setCode(id+1);
      return ResultRespone.success(bmMeterFactoryService.save(bmMeterFactory));
    }

    /**
     * 修改水表生产厂家
     * @param bmMeterFactory 水表生产厂家
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改水表生产厂家" )
    @ApiOperation(value = "修改水表生产厂家")
    @PutMapping
    public ResultRespone updateById(@RequestBody BmMeterFactory bmMeterFactory) {
      return ResultRespone.success(bmMeterFactoryService.updateById(bmMeterFactory));
    }

    /**
     * 通过id删除水表生产厂家
     * @param code id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除水表生产厂家" )
    @ApiOperation(value = "通过ID删除水表生产厂家")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "ID", required = true, dataType = "BigDecimal")
    })
    @DeleteMapping("/{code}" )
    public ResultRespone removeById(@PathVariable BigDecimal code) {
      return ResultRespone.success(bmMeterFactoryService.removeById(code));
    }

    /**
     * 通过id批量删除水表生产厂家
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除水表生产厂家" )
    @ApiOperation(value = "通过ID批量删除水表生产厂家")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(bmMeterFactoryService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
