/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.codemgt.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.codemgt.entity.BmMeterFactory;
import com.tofly.fees.codemgt.entity.BmMeterModel;
import com.tofly.fees.codemgt.mapper.BmMeterModelMapper;
import com.tofly.fees.codemgt.service.BmMeterFactoryService;
import com.tofly.fees.codemgt.service.BmMeterModelService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 水表型号
 *
 * @author sxb
 * @date Thu May 18 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/codemgt/bmmetermodel")
@Api(tags="编码管理-水表型号接口")
public class BmMeterModelController {

    private final BmMeterModelService bmMeterModelService;
    private final BmMeterFactoryService bmMeterFactoryService;

   private final BmMeterModelMapper bmMeterModelMapper;
    /**
     * 分页查询
     * @param page
     * @param searchText
     * @return
     */
    @GetMapping("/page" )
    @ApiOperation(value = "page分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "current", value = "当前页码", required = true, dataType = "int"),
            @ApiImplicitParam(name = "size", value = "每页显示数据条数", required = true, dataType = "int"),
            @ApiImplicitParam(name = "pages", value = "共几页,输出参数",  dataType = "int"),
            @ApiImplicitParam(name = "maxLimit", value = "每页最大显示数据",  dataType = "int"),
            @ApiImplicitParam(name = "orders[0].asc", value = "排序,bool类型",  dataType = "String"),
            @ApiImplicitParam(name = "orders[0].column", value = "排序,字段名",  dataType = "String"),
            @ApiImplicitParam(name = "searchText", value = "模糊查询(类型名称)", dataType = "String"),
            @ApiImplicitParam(name = "isdisabled", value = "是否启用,0,1",  dataType = "String")
    })
    public ResultRespone getBmMeterModelPage(Page page,String searchText,String isdisabled) {
        //条件构造
        QueryWrapper<BmMeterModel> queryWrapper = new QueryWrapper<BmMeterModel>();
        if(StringUtils.isNotBlank(searchText)) {

            if(StringUtils.isNotBlank(isdisabled))
            {
                //like ‘%类型名称%’
                queryWrapper.in("ISDISABLED",isdisabled)   //是否启用
                        .and(i->i.like("name",searchText));
            }
            else
            {
                queryWrapper.like("name",searchText);   //是否启用
            }
        }
        else
        {
            if(StringUtils.isNotBlank(isdisabled))
            {
                queryWrapper.in("ISDISABLED",isdisabled);  //是否启用
            }
        }
        // 执行分页查询
        IPage<BmMeterModel> sbxhPage = bmMeterModelService.page(page,queryWrapper);
        // 遍历查询结果进行赋值
        List<BmMeterModel> sbxhList = sbxhPage.getRecords();
        //水表厂家查询
        List<BmMeterFactory> sbcjList=bmMeterFactoryService.list();
        Map<String,String> map = new HashMap<>();
        sbcjList.forEach(e->map.put(e.getCode().toString(), e.getName()));

        for (BmMeterModel sbxh : sbxhList)
        {
            //水表类型
//            sbxh.setMeterFactory(map.get(sbxh.getMeterFactory()));
//            if(sbxh.getIsdisabled().equals("1"))
//            {
//                sbxh.setIsdisabled("启用");
//            }
//            else {
//                sbxh.setIsdisabled("禁用");
//            }
        }
      return ResultRespone.success(sbxhPage);
    }


    /**
     * 通过id查询水表型号
     * @param code id
     * @return ResultRespone
     */
    @GetMapping("/{code}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("code" ) String code) {
      return ResultRespone.success(bmMeterModelService.getById(code));
    }

    /**
     * 新增水表型号
     * @param bmMeterModel 水表型号
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增水表型号" )
    @ApiOperation(value = "新增水表型号")
    @PostMapping
    public ResultRespone save(@RequestBody BmMeterModel bmMeterModel) {
        //查询当前ID
        Map<String ,Object> map=  bmMeterModelMapper.SelectChargeMaxCode();
        int id=Integer.parseInt(map.get("ID").toString());
        bmMeterModel.setCode(id+1);
      return ResultRespone.success(bmMeterModelService.save(bmMeterModel));
    }

    /**
     * 修改水表型号
     * @param bmMeterModel 水表型号
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改水表型号" )
    @ApiOperation(value = "修改水表型号")
    @PutMapping
    public ResultRespone updateById(@RequestBody BmMeterModel bmMeterModel) {
      return ResultRespone.success(bmMeterModelService.updateById(bmMeterModel));
    }

    /**
     * 通过id删除水表型号
     * @param code id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除水表型号" )
    @ApiOperation(value = "通过ID删除水表型号")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{code}" )
    public ResultRespone removeById(@PathVariable String code) {
      return ResultRespone.success(bmMeterModelService.removeById(code));
    }

    /**
     * 通过id批量删除水表型号
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除水表型号" )
    @ApiOperation(value = "通过ID批量删除水表型号")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(bmMeterModelService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
