/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.codemgt.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.codemgt.entity.BmMeterType;
import com.tofly.fees.codemgt.mapper.BmMeterTypeMapper;
import com.tofly.fees.codemgt.service.BmMeterTypeService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 水表类别
 *
 * @author sxb
 * @date Thu May 18 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/codemgt/bmmetertype")
@Api(tags="编码管理-水表类别接口")
public class BmMeterTypeController {

    private final BmMeterTypeService bmMeterTypeService;

   private final BmMeterTypeMapper bmMeterTypeMapper;

    /**
     *
     * @param page
     * @param searchText
     * @param isdisabled
     * @return
     */
    @GetMapping("/page" )
    @ApiOperation(value = "page分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "current", value = "当前页码", required = true, dataType = "int"),
            @ApiImplicitParam(name = "size", value = "每页显示数据条数", required = true, dataType = "int"),
            @ApiImplicitParam(name = "pages", value = "共几页,输出参数", dataType = "int"),
            @ApiImplicitParam(name = "maxLimit", value = "每页最大显示数据",  dataType = "int"),
            @ApiImplicitParam(name = "orders[0].asc", value = "排序,bool类型", dataType = "String"),
            @ApiImplicitParam(name = "orders[0].column", value = "排序,字段名",  dataType = "String"),
            @ApiImplicitParam(name = "searchText", value = "模糊查询(水表类别名称)", dataType = "String"),
            @ApiImplicitParam(name = "isdisabled", value = "是否启用,0,1",  dataType = "String")
    })
    public ResultRespone getBmMeterTypePage(Page page, String searchText,String isdisabled) {
        QueryWrapper<BmMeterType> queryWrapper = new QueryWrapper<BmMeterType>();
        if(StringUtils.isNotBlank(searchText)) {

            if(StringUtils.isNotBlank(isdisabled))
            {
                //like ‘%类型名称%’
                queryWrapper.in("ISDISABLED",isdisabled)   //是否启用
                        .and(i->i.like("name",searchText));
            }
            else
            {
                queryWrapper.like("name",searchText);   //是否启用
            }
        }
        else
        {
            if(StringUtils.isNotBlank(isdisabled))
            {
                queryWrapper.in("ISDISABLED",isdisabled);  //是否启用
            }
        }
        // 执行分页查询
        IPage<BmMeterType> sblbPage = bmMeterTypeService.page(page,queryWrapper);
        // 遍历查询结果进行赋值
        List<BmMeterType> sblbList = sblbPage.getRecords();
//        for (BmMeterType sblb : sblbList) {
//            if(sblb.getIsdisabled().equals("0"))
//            {
//                sblb.setIsdisabled("禁用");
//            }else
//            {
//                sblb.setIsdisabled("启用");
//            }
//        }

        return ResultRespone.success(sblbPage);
    }


    /**
     * 通过id查询水表类别
     * @param code id
     * @return ResultRespone
     */
    @GetMapping("/{code}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("code" ) String code) {
      return ResultRespone.success(bmMeterTypeService.getById(code));
    }

    /**
     * 新增水表类别
     * @param bmMeterType 水表类别
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增水表类别" )
    @ApiOperation(value = "新增水表类别")
    @PostMapping
    public ResultRespone save(@RequestBody BmMeterType bmMeterType) {
        //查询当前ID
        Map<String ,Object> map=  bmMeterTypeMapper.SelectChargeMaxCode();
        int id=Integer.parseInt(map.get("ID").toString());
        bmMeterType.setCode(id+1);
      return ResultRespone.success(bmMeterTypeService.save(bmMeterType));
    }

    /**
     * 修改水表类别
     * @param bmMeterType 水表类别
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改水表类别" )
    @ApiOperation(value = "修改水表类别")
    @PutMapping
    public ResultRespone updateById(@RequestBody BmMeterType bmMeterType) {
      return ResultRespone.success(bmMeterTypeService.updateById(bmMeterType));
    }

    /**
     * 通过id删除水表类别
     * @param code id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除水表类别" )
    @ApiOperation(value = "通过ID删除水表类别")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{code}" )
    public ResultRespone removeById(@PathVariable String code) {
      return ResultRespone.success(bmMeterTypeService.removeById(code));
    }

    /**
     * 通过id批量删除水表类别
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除水表类别" )
    @ApiOperation(value = "通过ID批量删除水表类别")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(bmMeterTypeService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
