package com.tofly.fees.codemgt.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.codemgt.entity.BmMrArea;
import com.tofly.fees.codemgt.mapper.BmMrAreaMapper;
import com.tofly.fees.codemgt.service.BmMrAreaService;
import com.tofly.fees.common.BaseController;
import com.tofly.fees.common.dbhelper.DbHelper;
import com.tofly.fees.common.dbhelper.PageSortHelper;
import com.tofly.fees.common.dbhelper.ProduceSql;
import com.tofly.fees.common.enums.TableIdEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;

/**
 * 抄表片区
 *
 * @author admin
 * @date Fri Jun 30 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/codemgt/bmmrarea")
@Api(tags = "编码管理-抄表片区接口")
public class BmMrAreaController extends BaseController {

    private final BmMrAreaService bmMrAreaService;
    private final BmMrAreaMapper bmMrAreaMapper;

    @GetMapping("/page")
    @ApiOperation(value = "自定义page分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "searchText", value = "模糊查询(编码、名称、备注)", required = false, dataType = "String"),
            @ApiImplicitParam(name = "companyBranch", value = "公司", required = false, dataType = "String")
    })
    public ResultRespone getPageList(Page page, String searchText, String companyBranch) {
        PageSortHelper.setPageSort(BmMrArea.class, page);
        IPage<BmMrArea> pageList = bmMrAreaMapper.getPageList(page, searchText, companyBranch);
        return ResultRespone.success(pageList);
    }

    /**
     * 通过id查询抄表片区
     *
     * @param code id
     * @return ResultRespone
     */
    @GetMapping("/{code}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "ID", required = true, dataType = "BigDecimal")
    })
    public ResultRespone getById(@PathVariable("code") BigDecimal code) {
        return ResultRespone.success(bmMrAreaService.getById(code));
    }

    /**
     * 新增抄表片区
     *
     * @param bmMrArea 抄表片区
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增抄表片区")
    @ApiOperation(value = "新增抄表片区")
    @PostMapping
    public ResultRespone save(@RequestBody BmMrArea bmMrArea) throws Exception {
        DbHelper db = DbHelper.getDbHelper();
        db.addTransactionSql(ProduceSql.getInsertSql(bmMrArea, TableIdEnum.MaxID));
        boolean isSuccess = db.executeTransaction();
        String strMsg = isSuccess ? "保存成功！" : "保存失败！";
        return ResultRespone.success(isSuccess, strMsg);
    }

    /**
     * 修改抄表片区
     *
     * @param bmMrArea 抄表片区
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改抄表片区")
    @ApiOperation(value = "修改抄表片区")
    @PutMapping
    public ResultRespone updateById(@RequestBody BmMrArea bmMrArea) {
        return ResultRespone.success(bmMrAreaService.updateById(bmMrArea));
    }

    /**
     * 通过id删除表本(小区)编码
     *
     * @param code id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除抄表片区")
    @ApiOperation(value = "通过ID删除抄表片区")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "ID", required = true, dataType = "BigDecimal")
    })
    @DeleteMapping("/{code}")
    public ResultRespone removeById(@PathVariable BigDecimal code) {
        //判断code是否已被使用
        if (bmMrAreaMapper.checkMrCodeIsUse(code) > 0) {
            return ResultRespone.failed("编码【" + code + "】已在用户档案中正常使用，不可删除！");
        } else {
            return ResultRespone.success(bmMrAreaService.removeById(code));
        }
    }
}
