package com.tofly.fees.codemgt.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.codemgt.entity.BmMrBook;
import com.tofly.fees.codemgt.mapper.BmMrBookMapper;
import com.tofly.fees.codemgt.service.BmMrBookService;
import com.tofly.fees.common.dbhelper.DbHelper;
import com.tofly.fees.common.dbhelper.PageSortHelper;
import com.tofly.fees.common.dbhelper.ProduceSql;
import com.tofly.fees.common.model.SystemUserInfo;
import com.tofly.fees.common.util.SysConfigUtils;
import com.tofly.fees.common.util.SystemUtils;
import com.tofly.fees.feesmgt.entity.SfCharge;
import com.tofly.fees.financialmgt.entity.SfRec;
import com.tofly.fees.meterreadingmgt.entity.BwMeterReadPlan;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.text.MessageFormat;
import java.util.Arrays;
import java.util.Map;

/**
 * 表本(小区)编码
 *
 * @author admin
 * @date Fri Jun 30 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/codemgt/bmmrbook")
@Api(tags = "编码管理-抄表本接口")
public class BmMrBookController {

    private final BmMrBookService bmMrBookService;

    private final BmMrBookMapper bmMrBookMapper;

    /**
     * 自定义分页查询
     *
     * @param page          分页对象
     * @param companyBranch 公司编码
     * @param mrArea        片区编码
     * @param searchText    模糊查询（编码、名称、备注）
     * @return 分页数据
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "companyBranch", value = "公司", required = false, dataType = "String"),
            @ApiImplicitParam(name = "mrArea", value = "片区", required = false, dataType = "String"),
            @ApiImplicitParam(name = "searchText", value = "模糊查询(编码、名称、备注)", required = false, dataType = "String"),
    })
    public ResultRespone getBmMrBookPage(Page page, String companyBranch, String mrArea, String searchText) {
        PageSortHelper.setPageSort(page, new String[]{"mrOName"});
        IPage<Map<String, Object>> pageList = bmMrBookMapper.getPageList(page, companyBranch, mrArea, searchText);
        return ResultRespone.success(pageList);
    }

    /**
     * 通过id查询表本(小区)编码
     *
     * @param code id
     * @return ResultRespone
     */
    @GetMapping("/{code}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "ID", required = true, dataType = "BigDecimal")
    })
    public ResultRespone getById(@PathVariable("code") BigDecimal code) {
        return ResultRespone.success(bmMrBookService.getById(code));
    }

    /**
     * 新增表本(小区)编码
     *
     * @param bmMrBook 表本(小区)编码
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增表本(小区)编码")
    @ApiOperation(value = "新增表本(小区)编码")
    @PostMapping
    public ResultRespone save(@RequestBody BmMrBook bmMrBook) {

        return ResultRespone.success(bmMrBookService.save(bmMrBook));
    }

    /**
     * 修改表本(小区)编码
     *
     * @param bmMrBook 表本(小区)编码
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改表本(小区)编码")
    @ApiOperation(value = "修改表本(小区)编码")
    @PutMapping
    public ResultRespone updateById(@RequestBody BmMrBook bmMrBook) throws Exception {
        boolean isFlag = false;
        if (bmMrBook.getSxFlag().equals("0")) {
            //本月生效
            //本月生效则更新 TF_YWYS_BW_METER_READ_PLAN、TF_YWYS_SF_REC、TF_YWYS_SF_CHARGE中本月的MrOcode、MrOname
            bmMrBook.setXyMrOcode("");
            SystemUserInfo userInfo = SystemUtils.getUserInfo(bmMrBook.getOcode());
            DbHelper dbHelper = DbHelper.getDbHelper();
            //公用where条件
            String strWhere = MessageFormat.format("(mr_ocode<>''{0}'' or (mr_ocode=''{0}'' and mr_oname<>''{1}'')) and mr_book=''{2}''"
                    , userInfo.getUsername(), userInfo.getRealName(), bmMrBook.getCode().toString());

            //抄表录入计划表
            BwMeterReadPlan bwMeterReadPlan = new BwMeterReadPlan();
            bwMeterReadPlan.setMrOcode(userInfo.getUsername());
            bwMeterReadPlan.setMrOname(userInfo.getRealName());
            dbHelper.addTransactionSql(ProduceSql.getUpdateSql(bwMeterReadPlan, strWhere));

            //重新赋值where
            strWhere = MessageFormat.format("{0} and year_month=''{1}''"
                    , strWhere, SysConfigUtils.getBillYearMonth(bmMrBook.getCompanyBranch()));

            //应收明细表
            SfRec sfRec = new SfRec();
            sfRec.setMrOcode(userInfo.getUsername());
            sfRec.setMrOname(userInfo.getRealName());
            dbHelper.addTransactionSql(ProduceSql.getUpdateSql(sfRec, strWhere));

            //实收票据明细表
            SfCharge sfCharge = new SfCharge();
            sfCharge.setMrOcode(userInfo.getUsername());
            sfCharge.setMrOname(userInfo.getRealName());
            dbHelper.addTransactionSql(ProduceSql.getUpdateSql(sfCharge, strWhere));

            //抄表本数据更新
            dbHelper.addTransactionSql(ProduceSql.getUpdateSql(bmMrBook, "code='" + bmMrBook.getCode() + "'"));

            isFlag = dbHelper.executeTransaction();
        } else {
            //下月生效
            isFlag = bmMrBookService.updateById(bmMrBook);
        }
        if (isFlag) {
            return ResultRespone.success(true, "更新成功！");
        } else {
            return ResultRespone.failed(false, "更新失败！");
        }
    }

    /**
     * 通过id删除表本(小区)编码
     *
     * @param code id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除表本(小区)编码")
    @ApiOperation(value = "通过ID删除表本(小区)编码")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "ID", required = true, dataType = "BigDecimal")
    })
    @DeleteMapping("/{code}")
    public ResultRespone removeById(@PathVariable BigDecimal code) {
        //判断code是否已被使用
        if (bmMrBookMapper.checkBookCodeIsUse(code) > 0) {
            return ResultRespone.failed("编码【" + code + "】已在用户档案中正常使用，不可删除！");
        } else {
            return ResultRespone.success(bmMrBookService.removeById(code));
        }
    }

    /**
     * 通过id批量删除表本(小区)编码
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除表本(小区)编码")
    @ApiOperation(value = "通过ID批量删除表本(小区)编码")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(bmMrBookService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
