/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.codemgt.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.codemgt.entity.BmPayMode;
import com.tofly.fees.codemgt.mapper.BmPayModeMapper;
import com.tofly.fees.codemgt.service.BmPayModeService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 付费方式
 *
 * @author sxb
 * @date Thu May 18 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/codemgt/bmpaymode")
@Api(tags="编码管理-付费方式接口")
public class BmPayModeController {

    private final BmPayModeService bmPayModeService;
   private final BmPayModeMapper bmPayModeMapper;

    @GetMapping("/page" )
    @ApiOperation(value = "page分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "current", value = "当前页码", required = true, dataType = "int"),
            @ApiImplicitParam(name = "size", value = "每页显示数据条数", required = true, dataType = "int"),
            @ApiImplicitParam(name = "pages", value = "共几页,输出参数", dataType = "int"),
            @ApiImplicitParam(name = "maxLimit", value = "每页最大显示数据",  dataType = "int"),
            @ApiImplicitParam(name = "orders[0].asc", value = "排序,bool类型", dataType = "String"),
            @ApiImplicitParam(name = "orders[0].column", value = "排序,字段名",  dataType = "String"),
            @ApiImplicitParam(name = "searchText", value = "模糊查询(付费方式名称)",  dataType = "String"),
            @ApiImplicitParam(name = "isdisabled", value = "是否启用,0,1",  dataType = "String")
    })
    public ResultRespone getBmPayModePage(Page page, String searchText,String isdisabled) {
        QueryWrapper<BmPayMode> queryWrapper = new QueryWrapper<BmPayMode>();
        if(StringUtils.isNotBlank(searchText)) {

            if(StringUtils.isNotBlank(isdisabled))
            {
                //like ‘%付款名称%’
                queryWrapper.in("ISDISABLED",isdisabled)   //是否启用
                        .and(i->i.like("name",searchText));
            }
            else
            {
                queryWrapper.like("name",searchText);   //是否启用
            }
        }
        else
        {
            if(StringUtils.isNotBlank(isdisabled))
            {
                queryWrapper.in("ISDISABLED",isdisabled);  //是否启用
            }
        }
        // 执行分页查询
        IPage<BmPayMode> sffsPage = bmPayModeService.page(page,queryWrapper);
        // 遍历查询结果进行赋值
        List<BmPayMode> sffsList = sffsPage.getRecords();
        for (BmPayMode sffs : sffsList) {
//            if(sffs.getIsdisabled().equals("0"))
//            {
//                sffs.setIsdisabled("禁用");
//            }else
        }

        return ResultRespone.success(sffsPage);
    }


    /**
     * 通过id查询付费方式
     * @param code id
     * @return ResultRespone
     */
    @GetMapping("/{code}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("code" ) String code) {
      return ResultRespone.success(bmPayModeService.getById(code));
    }

    /**
     * 新增付费方式
     * @param bmPayMode 付费方式
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增付费方式" )
    @ApiOperation(value = "新增付费方式")
    @PostMapping
    public ResultRespone save(@RequestBody BmPayMode bmPayMode) {
        //查询当前ID
        Map<String ,Object> map=  bmPayModeMapper.SelectChargeMaxCode();
        int id=Integer.parseInt(map.get("ID").toString());
        bmPayMode.setCode(id+1);
      return ResultRespone.success(bmPayModeService.save(bmPayMode));
    }

    /**
     * 修改付费方式
     * @param bmPayMode 付费方式
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改付费方式" )
    @ApiOperation(value = "修改付费方式")
    @PutMapping
    public ResultRespone updateById(@RequestBody BmPayMode bmPayMode) {
      return ResultRespone.success(bmPayModeService.updateById(bmPayMode));
    }

    /**
     * 通过id删除付费方式
     * @param code id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除付费方式" )
    @ApiOperation(value = "通过ID删除付费方式")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{code}" )
    public ResultRespone removeById(@PathVariable String code) {
      return ResultRespone.success(bmPayModeService.removeById(code));
    }

    /**
     * 通过id批量删除付费方式
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除付费方式" )
    @ApiOperation(value = "通过ID批量删除付费方式")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(bmPayModeService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
