/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.codemgt.controller;


import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.codemgt.entity.SysChildCode;
import com.tofly.fees.codemgt.mapper.SysChildCodeMapper;
import com.tofly.fees.codemgt.service.SysChildCodeService;
import com.tofly.fees.common.model.CollectionData;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.*;

/**
 * 子集编码表
 *
 * @author sxb
 * @date Thu May 18 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/codemgt/syschildcode")
@Api(tags = "编码管理-子集编码表接口")
public class SysChildCodeController {

    private final SysChildCodeService sysChildCodeService;

   private final   SysChildCodeMapper sysChildCodeMapper;

    /**
     * list父级码表列表
     *
     * @return list
     */
    @GetMapping("/list")
    @ApiOperation(value = "list父级码表列表")
    public ResultRespone getSysCodeList() {
        //父级编码表列表
        List<Map<String, Object>> list = sysChildCodeMapper.selectListMapParentCode();
        return ResultRespone.success(new CollectionData(list, list.size()));
    }

    /**
     * list子级码表列表
     *
     * @param pcId
     * @return
     */
    @GetMapping("/list/{pcId}")
    @ApiOperation(value = "list子级码表列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pcId", value = "所属父级节点", required = true, dataType = "String"),

    })
    public ResultRespone getSysChildCodeList(@PathVariable("pcId") String pcId) {
        //子级码表列表
        List<Map<String, Object>> list = sysChildCodeMapper.selectListMapChildCode(pcId);
        return ResultRespone.success(new CollectionData(list, list.size()));
    }

    /**
     * 分页查询
     *
     * @param page         分页对象
     * @param sysChildCode 子集编码表
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")

    public ResultRespone getSysChildCodePage(Page page, @ModelAttribute SysChildCode sysChildCode) {
        return ResultRespone.success(sysChildCodeService.page(page, Wrappers.query(sysChildCode)));
    }


    /**
     * 通过id查询子集编码表
     *
     * @param ccid id
     * @return ResultRespone
     */
    @GetMapping("/{ccid}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ccid", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("ccid") Integer ccid) {
        return ResultRespone.success(sysChildCodeService.getById(ccid));
    }

    /**
     * 新增子集编码表
     *
     * @param sysChildCode 子集编码表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增子集编码表")
    @ApiOperation(value = "新增子集编码表")
    @PostMapping
    public ResultRespone save(@RequestBody SysChildCode sysChildCode) {
        //查询当前ID
        Map<String, Object> map = sysChildCodeMapper.SelectChargeMaxCode();
        Integer id = Integer.parseInt(map.get("ID").toString());
        sysChildCode.setCcid(Integer.toUnsignedLong(id + 1));
        return ResultRespone.success(sysChildCodeService.save(sysChildCode));
    }

    /**
     * 修改子集编码表
     *
     * @param sysChildCode 子集编码表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改子集编码表")
    @ApiOperation(value = "修改子集编码表")
    @PutMapping
    public ResultRespone updateById(@RequestBody SysChildCode sysChildCode) {
        return ResultRespone.success(sysChildCodeService.updateById(sysChildCode));
    }

    /**
     * 通过id删除子集编码表
     *
     * @param ccid id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除子集编码表")
    @ApiOperation(value = "通过ID删除子集编码表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ccid", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{ccid}")
    public ResultRespone removeById(@PathVariable Integer ccid) {
        return ResultRespone.success(sysChildCodeService.removeById(ccid));
    }

    /**
     * 通过id批量删除子集编码表
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除子集编码表")
    @ApiOperation(value = "通过ID批量删除子集编码表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(sysChildCodeService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
