/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.codemgt.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.codemgt.entity.SysParentCode;
import com.tofly.fees.codemgt.mapper.SysParentCodeMapper;
import com.tofly.fees.codemgt.service.SysParentCodeService;
import lombok.AllArgsConstructor;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Map;

/**
 * 通用码表父级节点名称
 *
 * @author sxb
 * @date Tue May 30 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/codemgt/sysparentcode")
@Api(tags="编码管理-通用码表父级节点名称接口")
public class SysParentCodeController {

    private final  SysParentCodeService sysParentCodeService;

    private final SysParentCodeMapper sysParentCodeMapper;
    /**
    * 分页查询
    * @param page 分页对象
    * @param sysParentCode 通用码表父级节点名称
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getSysParentCodePage(Page page, @ModelAttribute SysParentCode sysParentCode) {
      return ResultRespone.success(sysParentCodeService.page(page, Wrappers.query(sysParentCode)));
    }


    /**
     * 通过id查询通用码表父级节点名称
     * @param pcid id
     * @return ResultRespone
     */
    @GetMapping("/{pcid}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pcid", value = "ID", required = true, dataType = "BigDecimal")
    })
    public ResultRespone getById(@PathVariable("pcid" ) Integer pcid) {
      return ResultRespone.success(sysParentCodeService.getById(pcid));
    }

    /**
     * 新增通用码表父级节点名称
     * @param sysParentCode 通用码表父级节点名称
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增通用码表父级节点名称" )
    @ApiOperation(value = "新增通用码表父级节点名称")
    @PostMapping
    public ResultRespone save(@RequestBody SysParentCode sysParentCode) {
        //查询当前ID
        Map<String ,Object> map=  sysParentCodeMapper.SelectChargeMaxCode();
        int id=Integer.parseInt(map.get("ID").toString());
        sysParentCode.setPcid(id+1);
      return ResultRespone.success(sysParentCodeService.save(sysParentCode));
    }

    /**
     * 修改通用码表父级节点名称
     * @param sysParentCode 通用码表父级节点名称
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改通用码表父级节点名称" )
    @ApiOperation(value = "修改通用码表父级节点名称")
    @PutMapping
    public ResultRespone updateById(@RequestBody SysParentCode sysParentCode) {
      return ResultRespone.success(sysParentCodeService.updateById(sysParentCode));
    }

    /**
     * 通过id删除通用码表父级节点名称
     * @param pcid id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除通用码表父级节点名称" )
    @ApiOperation(value = "通过ID删除通用码表父级节点名称")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pcid", value = "ID", required = true, dataType = "BigDecimal")
    })
    @DeleteMapping("/{pcid}" )
    public ResultRespone<Object>  removeById(@PathVariable Integer pcid) {
        Map<String ,Object> map= sysParentCodeMapper.SelectChildCount(pcid);
        int count=Integer.parseInt(map.get("SL").toString());

        ResultRespone<Object> rest = new ResultRespone<>();
        if(count>0)
        {
            rest.setCode(-1);
            rest.setMessage("子级已存在");
        }
        else
        {
            rest.setCode(1);
            rest.setMessage("添加成功");
            rest.setResult(sysParentCodeService.removeById(pcid));
        }

        return rest;
    }

    /**
     * 通过id批量删除通用码表父级节点名称
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除通用码表父级节点名称" )
    @ApiOperation(value = "通过ID批量删除通用码表父级节点名称")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(sysParentCodeService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
