package com.tofly.fees.common;

import com.tofly.common.core.entity.ResultRespone;
import com.tofly.fees.common.enums.UserRoleEnum;
import com.tofly.fees.common.mapper.CommonMapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;

/**
 * 用户控制处理类
 *
 * @author shudong
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/common/user")
@Api(tags = "公共-操作员信息")
public class UserController extends BaseController {

    private CommonMapper commonMapper;

    /**
     * 获取所有抄表员数据
     *
     * @return
     */
    @PostMapping("/getAllMeterReader")
    @ApiOperation(value = "获取所有抄表员数据")
    public ResultRespone getAllMeterReaderList() {
        return getListResult(UserRoleEnum.MeterReader);
    }

    /**
     * 获取所有收费人员数据
     *
     * @return
     */
    @PostMapping("/getAllTollCollector")
    @ApiOperation(value = "获取所有收费人员数据")
    public ResultRespone getAllTollCollectorList() {
        return getListResult(UserRoleEnum.TollCollector);
    }

    /**
     * 获取所有第三方收费人员数据
     *
     * @return
     */
    @PostMapping("/getAllThirdPartyTollCollector")
    @ApiOperation(value = "获取第三方收费人员数据")
    public ResultRespone getAllThirdPartyTollCollectorList() {
        return getListResult(UserRoleEnum.ThirdPartyTollCollector);
    }

    /**
     * 公用提取枚举方法
     *
     * @param userRoleEnum 用户角色枚举
     * @return 结果相应集
     */
    private ResultRespone<List<HashMap<String, Object>>> getListResult(UserRoleEnum userRoleEnum) {
        List<HashMap<String, Object>> listInfo = commonMapper.getUserListByRoleId(userRoleEnum);
        return ResultRespone.success(listInfo);
    }
}
