package com.tofly.fees.config;

import com.tofly.fees.common.exception.FeesWebResponseExceptionTranslator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.web.client.RestTemplateBuilder;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.ResponseEntity;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.oauth2.config.annotation.web.configuration.EnableResourceServer;
import org.springframework.security.oauth2.config.annotation.web.configuration.ResourceServerConfigurerAdapter;
import org.springframework.security.oauth2.config.annotation.web.configurers.ResourceServerSecurityConfigurer;
import org.springframework.security.oauth2.provider.error.OAuth2AuthenticationEntryPoint;

@EnableResourceServer
@EnableGlobalMethodSecurity(prePostEnabled = true)
@EnableWebSecurity
@Configuration
public class ResourceServerConfig extends ResourceServerConfigurerAdapter {

    @Autowired
    private RestTemplateBuilder restTemplateBuilder;

    protected static final String[] NON_TOKEN_BASED_AUTH_ENTRY_POINTS = new String[]{
            "/index.html", "/doc.html", "/robots.txt",
            "/manifest.json", "/assets/**", "/static/**",
            "/api/noauth/**", "/webjars/**", "/api/license/**",
            "/swagger-resources/**", "/*/api-docs", "/*.js"};
    protected static final String[] TOKEN_BASED_AUTH_ENTRY_POINTS = new String[]{"/api/financialmgt/bmPriceEntity/**"};

    @Override
    public void configure(ResourceServerSecurityConfigurer resources) throws Exception {
        OAuth2AuthenticationEntryPoint authenticationEntryPoint = new OAuth2AuthenticationEntryPoint() {
            @Override
            protected ResponseEntity<?> enhanceResponse(ResponseEntity<?> response, Exception exception) {
                return super.enhanceResponse(response, exception);
            }
        };
        authenticationEntryPoint.setExceptionTranslator(new FeesWebResponseExceptionTranslator());
        resources.authenticationEntryPoint(authenticationEntryPoint);
    }

    @Override
    public void configure(HttpSecurity http) throws Exception {
        http.authorizeRequests()
                .antMatchers(NON_TOKEN_BASED_AUTH_ENTRY_POINTS).permitAll()
                .antMatchers(TOKEN_BASED_AUTH_ENTRY_POINTS).authenticated()
                .anyRequest().permitAll();
    }
}