package com.tofly.fees.feesmgt.controller;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.fees.feesmgt.entity.SfCharge;
import com.tofly.fees.feesmgt.mapper.SfChargeMapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@AllArgsConstructor
@RequestMapping("/api/feesmgt/invoiceManagement")
@Api(tags="收费管理-票据管理接口")
public class InvoiceManagementController {
    private final SfChargeMapper sfChargeMapper;
    /**
     * 实收明细
     *
     */
    @GetMapping("/page" )
    @ApiOperation(value = "票据-分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "current", value = "当前页码", required = true, dataType = "int"),
            @ApiImplicitParam(name = "size", value = "每页显示数据条数", required = true, dataType = "int"),
            @ApiImplicitParam(name = "pages", value = "共几页,输出参数",  dataType = "int"),
            @ApiImplicitParam(name = "maxLimit", value = "每页最大显示数据",  dataType = "int"),
            @ApiImplicitParam(name = "orders[0].asc", value = "排序,bool类型",  dataType = "String"),
            @ApiImplicitParam(name = "orders[0].column", value = "排序,字段名", dataType = "String"),
            @ApiImplicitParam(name = "companyBranch", value = "公司",  dataType = "String"),
            @ApiImplicitParam(name = "mrArea", value = "片区",  dataType = "String"),
            @ApiImplicitParam(name = "mrBook", value = "抄表本",  dataType = "String"),
            @ApiImplicitParam(name = "YearMonth", value = "财务日期",  dataType = "String"),
            @ApiImplicitParam(name = "customerNo", value = "用户编号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "chargeOcode", value = "收费人员", dataType = "String"),
            @ApiImplicitParam(name = "realPayMode", value = "付费方式",  dataType = "String"),
            @ApiImplicitParam(name = "chargeType", value = "收费类型",  dataType = "String"),
            @ApiImplicitParam(name = "cancelFlag", value = "票据状态",  dataType = "String"),
            @ApiImplicitParam(name = "chargeDate_Start", value = "收费开始日期",  dataType = "String"),
            @ApiImplicitParam(name = "chargeDate_End", value = "收费结束日期",  dataType = "String"),
            @ApiImplicitParam(name = "searchText", value = "模糊查询(用户编号、用户姓名、用户地址、联系电话)",  dataType = "String")
    })
    public ResultRespone getSfChargePage(Page page, String mrArea, String mrBook, String companyBranch, String YearMonth, String customerNo, String chargeOcode, String realPayMode, String chargeType, String chargeDate_Start, String chargeDate_End, String searchText,String cancelFlag) {
        ResultRespone<Object> rest = new ResultRespone<>();
        if (StringUtils.isEmpty(customerNo)) {
            rest.setCode(-1);
            rest.setMessage("查询失败，用户编号不能为空");
        }
        Page<SfCharge> sfCharge = null;
        if (rest.getCode() != -1) {
            sfCharge = sfChargeMapper.selectSfmxSearch(page, mrArea, mrBook, companyBranch, YearMonth, customerNo, chargeOcode, realPayMode, chargeType, chargeDate_Start, chargeDate_End, searchText,cancelFlag);
            rest.setCode(1);
            rest.setMessage("查询成功");
            rest.setResult(sfCharge);
        }
        return rest;
    }


    /**
     * 通过id查询实收明细：冲红票据
     * @param chargeSerialNo chargeSerialNo
     * @return ResultRespone
     */
    @PostMapping("/{chargeSerialNo}" )
    @ApiOperation(value = "通过收费chargeSerialNo查询,冲红票据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "chargeSerialNo", value = "收费流水号", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("chargeSerialNo" ) String chargeSerialNo) {
        ResultRespone<Object> rest=new ResultRespone<>();
        //判断chargeSerialNo不能为空，并且判断是否存在该条业务数据
        if(StringUtils.isEmpty(chargeSerialNo))
        {
            rest.setCode(-1);
            rest.setMessage("操作失败，收费流水编号不能为空");
        }
        //收费记录信息查询
      List<SfCharge> sfChargeList= sfChargeMapper.selectSfmxChargeSerialNo(chargeSerialNo);
        if (sfChargeList != null && !sfChargeList.isEmpty()) {
            // 查询值不为空的处理逻辑

            //预留调用存储过程冲红

        } else {
            // 查询值为空的处理逻辑
            rest.setCode(-1);
            rest.setMessage("操作失败，未查询出用户对应收费记录信息");
        }
        return rest;
    }


}
