/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.feesmgt.controller;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.feesmgt.entity.SfCharge;
import com.tofly.fees.feesmgt.mapper.SfChargeMapper;
import com.tofly.fees.feesmgt.service.SfChargeService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;


import java.util.Arrays;
/**
 * 实收明细：实收明细
 *
 * @author xb
 * @date Wed Jun 07 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/feesmgt/sfcharge")
@Api(tags="实收明细：实收明细接口")
public class SfChargeController {

    private final  SfChargeService sfChargeService;
    private final  SfChargeMapper sfChargeMapper;

    /**
     * 实收明细
     *
     */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "current", value = "当前页码", required = true, dataType = "int"),
            @ApiImplicitParam(name = "size", value = "每页显示数据条数", required = true, dataType = "int"),
            @ApiImplicitParam(name = "pages", value = "共几页,输出参数",  dataType = "int"),
            @ApiImplicitParam(name = "maxLimit", value = "每页最大显示数据",  dataType = "int"),
            @ApiImplicitParam(name = "orders[0].asc", value = "排序,bool类型",  dataType = "String"),
            @ApiImplicitParam(name = "orders[0].column", value = "排序,字段名", dataType = "String"),
            @ApiImplicitParam(name = "companyBranch", value = "公司",  dataType = "String"),
            @ApiImplicitParam(name = "mrArea", value = "片区",  dataType = "String"),
            @ApiImplicitParam(name = "mrBook", value = "抄表本",  dataType = "String"),
            @ApiImplicitParam(name = "YearMonth", value = "财务日期",  dataType = "String"),
            @ApiImplicitParam(name = "customerNo", value = "用户编号", dataType = "String"),
            @ApiImplicitParam(name = "chargeOcode", value = "收费人员", dataType = "String"),
            @ApiImplicitParam(name = "realPayMode", value = "付费方式",  dataType = "String"),
            @ApiImplicitParam(name = "chargeType", value = "收费类型",  dataType = "String"),
            @ApiImplicitParam(name = "chargeDate_Start", value = "收费开始日期",  dataType = "String"),
            @ApiImplicitParam(name = "chargeDate_End", value = "收费结束日期",  dataType = "String"),
            @ApiImplicitParam(name = "searchText", value = "模糊查询(用户编号、用户姓名、用户地址、联系电话)", required = false, dataType = "String")
    })
    public ResultRespone getSfChargePage(Page page,String mrArea, String mrBook, String companyBranch, String YearMonth, String customerNo,String chargeOcode,String realPayMode,String chargeType,String chargeDate_Start,String chargeDate_End,String searchText) {
        Page<SfCharge> sfCharge=sfChargeMapper.selectSfmxSearch(page, mrArea,  mrBook,  companyBranch,  YearMonth,  customerNo, chargeOcode, realPayMode, chargeType, chargeDate_Start, chargeDate_End,searchText,"");
      return ResultRespone.success(sfCharge);
    }


    /**
     * 通过id查询实收明细：实收明细
     * @param chargeType id
     * @return ResultRespone
     */
    @GetMapping("/{chargeType}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "chargeType", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("chargeType" ) String chargeType) {
      return ResultRespone.success(sfChargeService.getById(chargeType));
    }

    /**
     * 新增实收明细：实收明细
     * @param sfCharge 实收明细：实收明细
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增实收明细：实收明细" )
    @ApiOperation(value = "新增实收明细：实收明细")
    @PostMapping
    public ResultRespone save(@RequestBody SfCharge sfCharge) {
      return ResultRespone.success(sfChargeService.save(sfCharge));
    }

    /**
     * 修改实收明细：实收明细
     * @param sfCharge 实收明细：实收明细
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改实收明细：实收明细" )
    @ApiOperation(value = "修改实收明细：实收明细")
    @PutMapping
    public ResultRespone updateById(@RequestBody SfCharge sfCharge) {
      return ResultRespone.success(sfChargeService.updateById(sfCharge));
    }

    /**
     * 通过id删除实收明细：实收明细
     * @param chargeType id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除实收明细：实收明细" )
    @ApiOperation(value = "通过ID删除实收明细：实收明细")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "chargeType", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{chargeType}" )
    public ResultRespone removeById(@PathVariable String chargeType) {
      return ResultRespone.success(sfChargeService.removeById(chargeType));
    }

    /**
     * 通过id批量删除实收明细：实收明细
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除实收明细：实收明细" )
    @ApiOperation(value = "通过ID批量删除实收明细：实收明细")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(sfChargeService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
