/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.feesmgt.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.feesmgt.entity.SfCusAcc;
import com.tofly.fees.feesmgt.service.SfCusAccService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 客户资金账号
 *
 * @author xb
 * @date Wed Jun 07 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/feesmgt/sfcusacc")
@Api(tags="客户资金账号接口")
public class SfCusAccController {

    private final  SfCusAccService sfCusAccService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param sfCusAcc 客户资金账号
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getSfCusAccPage(Page page, @ModelAttribute SfCusAcc sfCusAcc) {
      return ResultRespone.success(sfCusAccService.page(page, Wrappers.query(sfCusAcc)));
    }


    /**
     * 通过id查询客户资金账号
     * @param customerNo id
     * @return ResultRespone
     */
    @GetMapping("/{customerNo}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "customerNo", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("customerNo" ) String customerNo) {
      return ResultRespone.success(sfCusAccService.getById(customerNo));
    }

    /**
     * 新增客户资金账号
     * @param sfCusAcc 客户资金账号
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增客户资金账号" )
    @ApiOperation(value = "新增客户资金账号")
    @PostMapping
    public ResultRespone save(@RequestBody SfCusAcc sfCusAcc) {
      return ResultRespone.success(sfCusAccService.save(sfCusAcc));
    }

    /**
     * 修改客户资金账号
     * @param sfCusAcc 客户资金账号
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改客户资金账号" )
    @ApiOperation(value = "修改客户资金账号")
    @PutMapping
    public ResultRespone updateById(@RequestBody SfCusAcc sfCusAcc) {
      return ResultRespone.success(sfCusAccService.updateById(sfCusAcc));
    }

    /**
     * 通过id删除客户资金账号
     * @param customerNo id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除客户资金账号" )
    @ApiOperation(value = "通过ID删除客户资金账号")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "customerNo", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{customerNo}" )
    public ResultRespone removeById(@PathVariable String customerNo) {
      return ResultRespone.success(sfCusAccService.removeById(customerNo));
    }

    /**
     * 通过id批量删除客户资金账号
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除客户资金账号" )
    @ApiOperation(value = "通过ID批量删除客户资金账号")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(sfCusAccService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
