package com.tofly.fees.feesmgt.controller_easy;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.fees.common.BaseController;
import com.tofly.fees.common.annotate.requestPostSingle.RequestPostSingleParam;
import com.tofly.fees.common.dbhelper.DbHelper;
import com.tofly.fees.common.enums.SysEnum;
import com.tofly.fees.common.model.PageLink;
import com.tofly.fees.common.model.ProcParamModel;
import com.tofly.fees.common.util.DateUtils;
import com.tofly.fees.common.util.ToolsUtils;
import com.tofly.fees.feesmgt.entity.SfCharge;
import com.tofly.fees.feesmgt.service.SfChargeService;
import io.swagger.annotations.*;
import lombok.AllArgsConstructor;
import org.apache.ibatis.mapping.ParameterMode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.sql.JDBCType;
import java.sql.ResultSet;
import java.util.*;

import static com.tofly.fees.common.ControllerConstants.*;

@RestController
@AllArgsConstructor
@RequestMapping("/api/feesmgt/sfcharge")
@Api(tags = "实收明细：实收明细接口")
public class SfChargeEasyController extends BaseController {

    @Autowired
    private final SfChargeService sfChargeService;

    @GetMapping("/pageInfo")
    @ApiOperation(value = "[Easy]自定义分页查询")
    public ResultRespone getWxBzsqPage(@ApiParam(value = PAGE_SIZE_DESCRIPTION, required = true)
                                       @RequestParam(value = PAGE_SIZE_PARA_NAME) int pageSize,
                                       @ApiParam(value = PAGE_NUMBER_DESCRIPTION, required = true)
                                       @RequestParam(value = PAGE_NUMBER_PARA_NAME) int page,
                                       @ApiParam(value = "用户编号", required = true)
                                       @RequestParam(value = "customerNo") String customerNo
    ) throws Exception {
        DbHelper db = DbHelper.getDbHelper();
        PageLink pg = createPageLink(pageSize, page);

        String strSql = String.format("select (case cancel_flag when '0' then '正常' when '1' then '已冲红' when '2' then '被冲红' else '未知'end ) ZT_NAME,b.name payMode,c.name chargeType,t.* from TF_YWYS_SF_CHARGE t");
        strSql+=String.format("  left join tf_ywys_bm_pay_mode b  on t.real_pay_mode=b.code ");
        strSql+=String.format("  left join tf_ywys_bm_charge_type c  on t.charge_type=c.code ");
        strSql += String.format("  where 1=1  and customer_no='%s' and charge_flag='1' order by charge_date,fee_serial_no desc", customerNo);
        Object obj = db.getPage(pg, strSql, SysEnum.MyBatis.getName());

        return ResultRespone.success(obj);
    }

    @PostMapping("/ChargePayment")
    @ApiOperation(value = "[Easy]营业厅欠费收取")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "inYhbh", value = "用户编号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "inSSJE", value = "实收金额", required = true, dataType = "NUMBER"),
            @ApiImplicitParam(name = "inFFFS", value = "付费方式", required = true, dataType = "String"),
            @ApiImplicitParam(name = "inSFRYXM", value = "收费人员姓名", required = true, dataType = "String"),
            @ApiImplicitParam(name = "inSFRYBM", value = "收费人员编码", required = true, dataType = "String"),

    })
    public ResultRespone ChargePayment(@RequestPostSingleParam HashMap<String, Object> data) throws Exception {

        DbHelper db = DbHelper.getDbHelper();
        List<HashMap<String, Object>> lst = new ArrayList<>();
        //参数
        Map<String, Object> params = new HashMap<>();


        String inyhbh = data.get("V_YHBH").toString();
        String fffs = data.get("V_FFFS").toString();
        String ids = data.get("V_IDS").toString();

        if (StringUtils.isBlank(inyhbh)) {
            return ResultRespone.failed("未找到用户编号信息.");
        }
        Double inSSJE = Double.parseDouble(data.get("V_SSJE").toString());
        if (inSSJE <= 0) {
            return ResultRespone.failed("金额必须大于0元.");
        }
        if (StringUtils.isBlank(ids)) {
            return ResultRespone.failed("请选择欠费.");
        }


        String sfrybm = "administrator";
        String sfryxm = "系统管理员";

        params.put("V_YHBH", inyhbh);
        params.put("V_SFRYBM", sfrybm);
        params.put("V_SFRYXM", sfryxm);
        params.put("V_SFRQ", new Date());
        params.put("V_SSJE", inSSJE);
        params.put("V_SFD", "1");
        params.put("V_FFFS", fffs);
        params.put("V_YESFGD", "1");
        params.put("V_IDS", ids);
        params.put("V_SFLSBH", ToolsUtils.getGuid());
        params.put("V_SFCG", "-1");

        //注意：jdbctype值为大写, 日期类型为DATE （yyyy-MM-dd）
        //注意：jdbctype值为大写, 日期类型为TIMESTAMP （yyyy-MM-dd HH:mi:ss）
        Map mp = db.procurdeCmd("{call PROC_SF_ZS(" +
                "#{V_YHBH,mode=IN,jdbcType=VARCHAR}," +
                "#{V_SFRYBM,mode=IN,jdbcType=VARCHAR}," +
                "#{V_SFRYXM,mode=IN,jdbcType=VARCHAR}," +
                "#{V_SFRQ,mode=IN,jdbcType=TIMESTAMP}," +
                "#{V_SSJE,mode=IN,jdbcType=DOUBLE}," +
                "#{V_SFD,mode=IN,jdbcType=VARCHAR}," +
                "#{V_FFFS,mode=IN,jdbcType=VARCHAR}," +
                "#{V_YESFGD,mode=IN,jdbcType=VARCHAR}," +
                "#{V_IDS,mode=IN,jdbcType=VARCHAR}," +
                "#{V_SFLSBH,mode=IN,jdbcType=VARCHAR}," +
                "#{V_SFCG,mode=OUT,jdbcType=INTEGER}" +
                ")}", params, SysEnum.MyBatis.getName());

        //存储过程返回的参数
        String jym = mp.get("V_SFCG").toString();
        if (jym.equals("0")) {
            return ResultRespone.success("缴费成功.");
        } else if (jym.equals("2")) {
            return ResultRespone.failed("没有欠费.");
        } else if (jym.equals("3")) {
            return ResultRespone.failed("剩余预付费大于等于最早一笔欠费.");
        } else if (jym.equals("4")) {
            return ResultRespone.failed("缴费金额低于欠费金额.");
        } else {
            return ResultRespone.failed("异常缴费信息.");
        }
    }


    @PostMapping("/ChargeAdvPayment")
    @ApiOperation(value = "[Easy]预存")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "inYhbh", value = "用户编号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "inSSJE", value = "实收金额", required = true, dataType = "NUMBER"),
            @ApiImplicitParam(name = "inFFFS", value = "付费方式", required = true, dataType = "String"),
            @ApiImplicitParam(name = "inSFRYXM", value = "收费人员姓名", required = true, dataType = "String"),
            @ApiImplicitParam(name = "inSFRYBM", value = "收费人员编码", required = true, dataType = "String"),

    })
    public ResultRespone ChargeAdvPayment(@RequestPostSingleParam HashMap<String, Object> data) throws Exception {
        DbHelper db = DbHelper.getDbHelper();
        List<HashMap<String, Object>> lst = new ArrayList<>();
        //参数
        Map<String, Object> params = new HashMap<>();
        String inyhbh = data.get("inYhbh").toString();

        String fffs = data.get("V_FFFS").toString();

        if (StringUtils.isBlank(inyhbh)) {
            return ResultRespone.failed("未找到用户编号信息.");
        }
        Double inSSJE = Double.parseDouble(data.get("inSSJE").toString());
        if (inSSJE <= 0) {
            return ResultRespone.failed("金额必须大于0元.");
        }


        String sfrybm = "administrator";
        String sfryxm = "系统管理员";

        params.put("inYhbh", inyhbh);
        params.put("inSfLSH", ToolsUtils.getGuid());
        params.put("inSfRQ", new Date());
        params.put("inSSJE", inSSJE);
        params.put("inYYD", "0");
        params.put("inFFFS", fffs);
        params.put("inSFRYXM", sfryxm);
        params.put("inSFRYBM", sfrybm);
        params.put("out_JYM", "-1");  //默认失败

        //注意：jdbctype值为大写, 日期类型为DATE （yyyy-MM-dd）
        //注意：jdbctype值为大写, 日期类型为TIMESTAMP （yyyy-MM-dd HH:mi:ss）
        Map mp = db.procurdeCmd("{call PROC_GT_YFF(" +
                "#{inYhbh,mode=IN,jdbcType=VARCHAR}," +
                "#{inSfLSH,mode=IN,jdbcType=VARCHAR}," +
                "#{inSfRQ,mode=IN,jdbcType=TIMESTAMP}," +
                "#{inSSJE,mode=IN,jdbcType=DOUBLE}," +
                "#{inYYD,mode=IN,jdbcType=VARCHAR}," +
                "#{inFFFS,mode=IN,jdbcType=VARCHAR}," +
                "#{inSFRYXM,mode=IN,jdbcType=VARCHAR}," +
                "#{inSFRYBM,mode=IN,jdbcType=VARCHAR}," +
                "#{out_JYM,mode=OUT,jdbcType=INTEGER}" +
                ")}", params, SysEnum.MyBatis.getName());

        //存储过程返回的参数
        String jym = mp.get("out_JYM").toString();
        if (jym.equals("0")) {
            return ResultRespone.success("预存成功.");
        } else {
            return ResultRespone.failed("预存失败.");
        }

        // return ResultRespone.success(true);
    }

    @GetMapping("/GetToDayChargeInfo")
    @ApiOperation(value = "[Easy]获取当日实收笔数信息")
    public ResultRespone GetToDayChargeInfo() throws Exception {
        QueryWrapper<SfCharge> wrapper = new QueryWrapper<>();
        wrapper.select(" (case  CHARGE_TYPE when '3' then '预存' else '实收' end ) CHARGE_TYPE", "count(distinct(CHARGE_SERIAL_NO)) as count", "sum(pay_total_amount) as total_price") // 选择需要统计的
                .eq("charge_ocode", "administrator")
                .and(i -> i.eq("CHARGE_TYPE", "3").or().eq("CHARGE_TYPE", "2"))
                .apply("charge_date>=TO_DATE({0}, 'yyyy-MM-dd HH24:mi:ss') and charge_date<TO_DATE({1}, 'yyyy-MM-dd HH24:mi:ss')", DateUtils.formatDate(new Date(), "yyyy-MM-dd 00:00:00"), DateUtils.formatDate(new Date(), "yyyy-MM-dd 23:59:59"))
                //.and(i->i.apply("charege_date<TO_DATE({0}, 'yyyy-MM-dd HH24:mi:ss')",DateUtils.formatDate(new Date(),"yyyy-MM-dd 23:59:59"))
                //.le("charege_date",DateUtils.formatDate(new Date(),"yyyy-MM-dd 00:00:00"))
                //.ge("charege_date",DateUtils.formatDate(new Date(),"yyyy-MM-dd 23:59:59"))
                .groupBy("CHARGE_TYPE"); // 按照状态分组
        List<Map<String, Object>> orderStats = sfChargeService.listMaps(wrapper);


        int count = 0;

//        for(Map<String, Object> mapList : orderStats ) {
//            for( String key : mapList.keySet() ) {
//                System.out.println( key + "-->" + mapList.get(key) );
//                if(key.equals("COUNT"))
//                    count+=(Integer) mapList.get(key);
//                if(key.equals("TOTAL_PRICE"))
//                    count+=(Integer) mapList.get(key);
//            }
//        }

        if (orderStats.size() > 0) {

            BigDecimal total_price_sum = orderStats.stream().map(x -> new BigDecimal(String.valueOf(x.get("TOTAL_PRICE")))).reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal count_sum = orderStats.stream().map(x -> new BigDecimal(String.valueOf(x.get("COUNT")))).reduce(BigDecimal.ZERO, BigDecimal::add);

            HashMap<String, Object> map = new HashMap<>();
            map.put("CHARGE_TYPE", "总计");
            map.put("TOTAL_PRICE", total_price_sum);
            map.put("COUNT", count_sum);

            orderStats.add(map);
        }
        return ResultRespone.success(orderStats);
    }


    @PostMapping("/ChargeCh")
    @ApiOperation(value = "[Easy]冲红")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "V_YHBH", value = "用户编号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "V_SFLSBH", value = "收费流水编号", required = true, dataType = "NUMBER")
    })
    public ResultRespone ChargeCh(@RequestParam("V_YHBH") String V_YHBH,
                                  @RequestParam("V_SFLSBH") String V_SFLSBH) throws Exception {
        DbHelper dbHelper = DbHelper.getDbHelper();
        ResultSet rs=dbHelper.executeQuery(String.format("select sum(PAY_TOTAL_AMOUNT) as PAY_TOTAL_AMOUNT from tf_ywys_sf_charge where CHARGE_SERIAL_NO='%s' ",V_SFLSBH));

        double v_ssje=0;
        while (rs.next()){
            v_ssje=rs.getDouble("PAY_TOTAL_AMOUNT");
        }
        List<ProcParamModel> paramModels = new ArrayList<>();
        paramModels.add(new ProcParamModel("V_YHBH", ParameterMode.IN, JDBCType.VARCHAR, V_YHBH));
        paramModels.add(new ProcParamModel("V_SFLSBH", ParameterMode.IN, JDBCType.VARCHAR, V_SFLSBH));
        paramModels.add(new ProcParamModel("V_SSJE", ParameterMode.IN, JDBCType.DOUBLE, v_ssje));
        paramModels.add(new ProcParamModel("V_CHRYBM", ParameterMode.IN, JDBCType.DOUBLE, "admin"));
        paramModels.add(new ProcParamModel("V_CHRYXM", ParameterMode.IN, JDBCType.DOUBLE, "admin"));
        paramModels.add(new ProcParamModel("V_CHSJ", ParameterMode.IN, JDBCType.DOUBLE, new Date()));
        paramModels.add(new ProcParamModel("V_SFCG", ParameterMode.OUT, JDBCType.VARCHAR, "-1"));

        Map mp = dbHelper.executeProcedure("PROC_SF_CH", paramModels);
        if(mp.get("V_SFCG").equals("1")){
            return  ResultRespone.success("成功");
        }
        else if(mp.get("V_SFCG").equals("2"))
        {
            return  ResultRespone.failed("收费金额不匹配");
        }
        else if(mp.get("V_SFCG").equals("3"))
        {
            return  ResultRespone.failed("不是最新缴费记录");
        }
        else if(mp.get("V_SFCG").equals("4"))
        {
            return  ResultRespone.failed("预付费余额已发生变化或剩余预付费不够退费");
        }
        else if(mp.get("V_SFCG").equals("5"))
        {
            return  ResultRespone.failed("收费转存的预付费不能单独退费");
        }
        else if(mp.get("V_SFCG").equals("9"))
        {
            return  ResultRespone.failed("没有收费记录");
        }else {
            return  ResultRespone.failed("异常信息");
        }
    }


}
