package com.tofly.fees.feesmgt.controller_easy;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.fees.common.BaseController;
import com.tofly.fees.common.annotate.requestPostSingle.RequestPostSingleParam;
import com.tofly.fees.common.model.CollectionData;
import com.tofly.fees.common.util.DateUtils;
import com.tofly.fees.common.util.ToolsUtils;
import com.tofly.fees.feesmgt.entity.SfCharge;
import com.tofly.fees.feesmgt.entity.SfCusAcc;
import com.tofly.fees.feesmgt.service.SfChargeService;
import com.tofly.fees.feesmgt.service.SfCusAccService;
import com.tofly.fees.meterprofilemgt.entity.DaMeter;
import com.tofly.fees.meterprofilemgt.service.DaMeterService;
import com.tofly.fees.userprofilemgt.entity.DaCustomer;
import com.tofly.fees.userprofilemgt.service.DaCustomerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.apache.commons.lang.StringUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;

import static com.tofly.fees.common.ControllerConstants.*;

@RestController
@AllArgsConstructor
@RequestMapping("/api/feesmgt/sfcusacc")
@Api(tags = "客户资金账号接口")
public class SfCusAccEasyController extends BaseController {


    private final SfCusAccService sfCusAccService;

    private final DaCustomerService daCustomerService;

    private final DaMeterService daMeterService;

    private final SfChargeService sfChargeService;

    //  private final  SfCusAccService sfCusAccService;

    /**
     * 分页查询
     *
     * @param page     分页对象
     * @param  客户资金账号
     * @return
     */
    @GetMapping("/pageInfo")
    @ApiOperation(value = "分页查询")
    public ResultRespone getSfCusAccPage(@ApiParam(value = PAGE_SIZE_DESCRIPTION, required = true)
                                         @RequestParam(value = PAGE_SIZE_PARA_NAME) int pageSize,
                                         @ApiParam(value = PAGE_NUMBER_DESCRIPTION, required = true)
                                         @RequestParam(value = PAGE_NUMBER_PARA_NAME) int page,
                                         @ApiParam(value = SORT_PROPERTY_DESCRIPTION, allowableValues = "customerName,customerNo")
                                         @RequestParam(required = false) String sortProperty,
                                         @ApiParam(value = SORT_ORDER_DESCRIPTION, allowableValues = SORT_ORDER_ALLOWABLE_VALUES)
                                         @RequestParam(required = false) String sortOrder) {


//        QueryWrapper<SfCusAcc> queryWrapper = new QueryWrapper<SfCusAcc>();
//        queryWrapper.like("CUSTOMER_NO","71012");   //是否启用
//        Page pgs=new Page<SfCusAcc>(1,15);

        Page<HashMap<String, Object>> pg = new Page<HashMap<String, Object>>(page, pageSize);

        //自定义SQL语句
        Page<List<HashMap<String, Object>>> newPage = sfCusAccService.selCusAcc(pg, "customerName", sortOrder);

        //组装返回对象
        CollectionData<List<HashMap<String, Object>>> m = new CollectionData<>(newPage.getRecords().stream().collect(Collectors.toList())
                , newPage.getTotal());
        //参数返回
        return ResultRespone.success(m);
    }


    @PostMapping("/updateInfo")
    @ApiOperation(value = "用户余额转存")
    @Transactional
    public ResultRespone updateCusAcc(@ApiParam(value = "用户编号") @RequestPostSingleParam(required = false) String customerNo,
                                      @ApiParam(value = "操作类型", allowableValues = "+,-") @RequestPostSingleParam(required = false) String opt,
                                      @ApiParam(value = "操作金额") @RequestPostSingleParam(required = false) Double money,
                                      @ApiParam(value = "备注") @RequestPostSingleParam(required = false) String remark) {
        //查询用户
//        LambdaQueryWrapper<DaCustomer> wrapper = new LambdaQueryWrapper();
//

        String ucode="1";
        String uname="admin";



        QueryWrapper<DaCustomer> wrapper = new QueryWrapper<>();
        if (!StringUtils.isBlank(customerNo)) {
            wrapper.eq("customer_No", customerNo);
            wrapper.eq("CUSTOMER_STATE", "1");
        }else {
            return ResultRespone.failed("customerNo为null");
        }

        DaCustomer customer = daCustomerService.getOne(wrapper);

        if (customer == null) {
            return ResultRespone.failed(false, "当前用户状态异常.");
        }

        LambdaQueryWrapper<DaMeter> meterwrapper = new LambdaQueryWrapper();
        if (!StringUtils.isBlank(customerNo)) {
            meterwrapper.eq(DaMeter::getCustomerNo, customerNo);
        }

        DaMeter daMeter = daMeterService.getOne(meterwrapper);
        if (customer == null) {
            return ResultRespone.failed(false, "当前用户水表类型异常.");
        }

        Double balance = 0.0;
        Double nbBalance = 0.0;
        Double prestoreAmount = 0.0;


        SfCusAcc accmodel = sfCusAccService.getById(customerNo);
         boolean isAcc=false;
        if (accmodel != null) {
            isAcc=true;
            balance = accmodel.getBalanceAmount();
            nbBalance = accmodel.getNbBalanceAmount();
            prestoreAmount = accmodel.getPrestoreAmount();
        } else {
            accmodel = new SfCusAcc();
            accmodel.setCustomerNo(customerNo);
            accmodel.setDeductAmount(0.00);
        }

        SfCharge chargeModel = new SfCharge();
        String id = ToolsUtils.getGuid();
        chargeModel.setId(id);
        chargeModel.setRecId(id);
        chargeModel.setFeeSerialNo(id);
        chargeModel.setMeterCardId(BigDecimal.valueOf(daMeter.getMeterCardId()));
        chargeModel.setCustomerNo(customer.getCustomerNo());
        chargeModel.setMeterCardNo(daMeter.getMeterCardNo());
        chargeModel.setCompanyBranch(customer.getCompanyBranch());
        chargeModel.setMrArea(customer.getMrArea());
        chargeModel.setMrBook(customer.getMrBook());
        String yearmonth = DateUtils.formatDate(new Date(), "yyyyMM");
        BigDecimal bigYearMonth = new BigDecimal(yearmonth);
        chargeModel.setYearMonth(bigYearMonth);
        chargeModel.setBillYearMonth(bigYearMonth);
        chargeModel.setChargeType("1");
        chargeModel.setInputOcode(ucode);
        chargeModel.setInputOname(uname);
        chargeModel.setInputDate(new Date());
        chargeModel.setCalDate(new Date());
        chargeModel.setChargeFlag("1");
        chargeModel.setChargeDate(new Date());
        chargeModel.setChargeSerialNo("2");
        chargeModel.setIsBc("1");
        chargeModel.setLastBalance(balance);//上次余额
        chargeModel.setThisPrestore(0.0);//本次预存
        chargeModel.setThisDeduct(0.0); //本次扣减
        chargeModel.setCustomerName(customer.getCustomerName());
//        if (opt.equals("+")) {
            chargeModel.setChangeBalance(money);
            chargeModel.setThisBalance(balance + money);
//        }
//        else {
//            chargeModel.setChangeBalance(-money);
//            chargeModel.setThisBalance(balance - money);
//        }
            balance += money;
            nbBalance += money;
            prestoreAmount += money;
//        if (opt.equals("+")) {
//            balance += money;
//            nbBalance += money;
//            prestoreAmount += money;
//        } else {
//            balance -= money;
//            nbBalance -= money;
//            prestoreAmount -= money;
//        } 
        accmodel.setBalanceAmount(balance);
        accmodel.setNbBalanceAmount(nbBalance);
        accmodel.setPrestoreAmount(prestoreAmount);

        boolean isNext = true;
        if (isAcc) {
            boolean isUpdate = sfCusAccService.updateById(accmodel);
            String msg = "保存成功.";
            if (!isUpdate) {
                isNext = false;
                msg = "保存失败.";
            }
            //  return ResultRespone.success(isUpdate, msg);
        } else {
            boolean isAdd = sfCusAccService.saveOrUpdate(accmodel);
            String msg = "保存成功.";
            if (!isAdd) {
                isNext = false;
                msg = "保存失败.";
            }
        }
        if (isNext)
            isNext = sfChargeService.save(chargeModel);
       if(isNext) {
           return ResultRespone.success(isNext);
       }
       else{
           return ResultRespone.failed(isNext);
       }

    }
}
