package com.tofly.fees.financialmgt.controller;

import com.tofly.common.core.entity.ResultRespone;
import com.tofly.fees.common.BaseController;
import com.tofly.fees.common.exception.FeesException;
import com.tofly.fees.common.model.PageLink;
import com.tofly.fees.financialmgt.model.BmPriceAddForm;
import com.tofly.fees.financialmgt.model.BmPriceUpdateForm;
import com.tofly.fees.financialmgt.service.IBmPriceHisService;
import com.tofly.fees.financialmgt.service.IBmPriceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.security.Principal;
import java.util.Set;

import static com.tofly.fees.common.ControllerConstants.*;


/**
 * <p>
 * 水价编码 前端控制器
 * </p>
 *
 * @author decai
 * @since 2023-05-18
 */
@RestController
@RequestMapping("/api/financialmgt/bmPriceEntity")
@Api(tags = "财务管理-水价管理")
@AllArgsConstructor
public class BmPriceController extends BaseController {

    private static final String CODE_DESCRIPTION = "水价编码";
    private final IBmPriceService bmPriceService;
    private final IBmPriceHisService bmPriceHisService;


    @GetMapping("/page")
    @ApiOperation(value = "水价配置分页查询")
    public ResultRespone pageList(@ApiParam(value = PAGE_SIZE_DESCRIPTION, required = true)
                                  @RequestParam(value = PAGE_SIZE_PARA_NAME) int pageSize,
                                  @ApiParam(value = PAGE_NUMBER_DESCRIPTION, required = true)
                                  @RequestParam(value = PAGE_NUMBER_PARA_NAME) int page,
                                  @ApiParam(value = "所属公司编码", required = true)
                                  @RequestParam Long companyBranch,
                                  @ApiParam(value = "水价名称(模糊查询)")
                                  @RequestParam String name,
                                  @ApiParam(value = SORT_PROPERTY_DESCRIPTION, allowableValues = BMPRICE_SORT_PROPERTY_ALLOWABLE_VALUES)
                                  @RequestParam(required = false) String sortProperty,
                                  @ApiParam(value = SORT_ORDER_DESCRIPTION, allowableValues = SORT_ORDER_ALLOWABLE_VALUES)
                                  @RequestParam(required = false) String sortOrder) throws FeesException {
        PageLink pageLink = createPageLink(pageSize, page, sortProperty, sortOrder);
        return ResultRespone.success(bmPriceService.pageList(companyBranch, name, pageLink));
    }
    @GetMapping("")
    @ApiOperation(value = "水价配置列表查询")
    public ResultRespone getList() throws FeesException {
        return ResultRespone.success(bmPriceService.getList());
    }

    @PostMapping("")
    @ApiOperation(value = "新增水价配置")
    public ResultRespone add(@Valid @RequestBody BmPriceAddForm form,
                             @ApiIgnore Principal principal) throws FeesException {
        return ResultRespone.success(bmPriceService.add(form, principal.getName()));
    }

    @PutMapping("")
    @ApiOperation(value = "修改水价配置")
    public ResultRespone update(@Valid @RequestBody BmPriceUpdateForm form,
                                @ApiIgnore Principal principal) throws FeesException {
        return ResultRespone.success(bmPriceService.update(form, principal.getName()));
    }

    @DeleteMapping("")
    @ApiOperation(value = "删除水价配置")
    public ResultRespone delete(@ApiParam(value = CODE_DESCRIPTION, required = true) @RequestParam("code") Set<String> codes,
                                @ApiIgnore Principal principal) throws FeesException {
        return ResultRespone.success(bmPriceService.delete(codes, principal.getName()));
    }

    @GetMapping("/{code}")
    @ApiOperation(value = "水价配置详情")
    public ResultRespone get(@ApiParam(value = CODE_DESCRIPTION, required = true)
                             @PathVariable("code") String code) throws FeesException {
        return ResultRespone.success(bmPriceService.get(code));
    }
    @GetMapping("/{code}/his")
    @ApiOperation(value = "水价配置历史记录")
    public ResultRespone getHisList(@ApiParam(value = CODE_DESCRIPTION, required = true)
                             @PathVariable("code") String code) throws FeesException {
        return ResultRespone.success(bmPriceHisService.getHis(code));
    }
    @GetMapping("/his/{id}")
    @ApiOperation(value = "水价配置历史记录详情")
    public ResultRespone getHisById(@ApiParam(value = "历史记录ID", required = true)
                             @PathVariable("id") String id) throws FeesException {
        return ResultRespone.success(bmPriceHisService.getById(id));
    }

}
