package com.tofly.fees.financialmgt.controller;

import com.tofly.common.core.entity.ResultRespone;
import com.tofly.fees.common.BaseController;
import com.tofly.fees.common.exception.FeesException;
import com.tofly.fees.common.model.PageLink;
import com.tofly.fees.financialmgt.model.BmUwTypeAddForm;
import com.tofly.fees.financialmgt.model.BmUwTypeUpdateForm;
import com.tofly.fees.financialmgt.service.IBmUwTypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.security.Principal;
import java.util.Set;

import static com.tofly.fees.common.ControllerConstants.*;
import static com.tofly.fees.common.ControllerConstants.SORT_ORDER_ALLOWABLE_VALUES;

/**
 * <p>
 * 用水类型 前端控制器
 * </p>
 *
 * @author decai
 * @since 2023-05-30
 */
@RestController
@RequestMapping("/api/financialmgt/bmUwTypeEntity")
@Api(tags = "财务管理-用水类型")
@AllArgsConstructor
public class BmUwTypeController extends BaseController {
    private static final String CODE_DESCRIPTION = "用水类型编码";
    public static final String BMUWTYPE_SORT_PROPERTY_ALLOWABLE_VALUES = "xxx";
    private final IBmUwTypeService BmUwTypeService;


    @GetMapping("/page")
    @ApiOperation(value = "用水类型分页查询")
    public ResultRespone pageList(@ApiParam(value = PAGE_SIZE_DESCRIPTION, required = true)
                                  @RequestParam(value = PAGE_SIZE_PARA_NAME) int pageSize,
                                  @ApiParam(value = PAGE_NUMBER_DESCRIPTION, required = true)
                                  @RequestParam(value = PAGE_NUMBER_PARA_NAME) int page,
                                  @ApiParam(value = "所属公司编码", required = true)
                                  @RequestParam Long companyBranch,
                                  @ApiParam(value = "用水类型名称(模糊查询)")
                                  @RequestParam String name,
                                  @ApiParam(value = SORT_PROPERTY_DESCRIPTION, allowableValues = BMUWTYPE_SORT_PROPERTY_ALLOWABLE_VALUES)
                                  @RequestParam(required = false) String sortProperty,
                                  @ApiParam(value = SORT_ORDER_DESCRIPTION, allowableValues = SORT_ORDER_ALLOWABLE_VALUES)
                                  @RequestParam(required = false) String sortOrder) throws FeesException {
        PageLink pageLink = createPageLink(pageSize, page, sortProperty, sortOrder);
        return ResultRespone.success(BmUwTypeService.pageList(companyBranch, name, pageLink));
    }

    @PostMapping("")
    @ApiOperation(value = "新增用水类型")
    public ResultRespone add(@Valid @RequestBody BmUwTypeAddForm form,
                                 @ApiIgnore  Principal principal) throws FeesException {

        return ResultRespone.success(BmUwTypeService.add(form, principal.getName()));
    }

    @PutMapping("")
    @ApiOperation(value = "修改用水类型")
    public ResultRespone update(@Valid @RequestBody BmUwTypeUpdateForm form,
                                @ApiIgnore Principal principal) throws FeesException {
        return ResultRespone.success(BmUwTypeService.update(form, principal.getName()));
    }

    @DeleteMapping("")
    @ApiOperation(value = "删除用水类型")
    public ResultRespone delete(@ApiParam(value = CODE_DESCRIPTION, required = true) @RequestParam("code") Set<String> codes,
                                @ApiIgnore Principal principal) throws FeesException {
        return ResultRespone.success(BmUwTypeService.delete(codes, principal.getName()));
    }

    @GetMapping("/{code}")
    @ApiOperation(value = "用水类型详情")
    public ResultRespone get(@ApiParam(value = CODE_DESCRIPTION, required = true)
                             @PathVariable("code") String code) throws FeesException {
        return ResultRespone.success(BmUwTypeService.get(code));
    }
}
