/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.financialmgt.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.financialmgt.entity.SfPenalty;
import com.tofly.fees.financialmgt.service.SfPenaltyService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 违约金调整记录表
 *
 * @author sxb
 * @date Tue Jun 27 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/financialmgt/sfpenalty")
@Api(tags="财务管理-违约金调整记录表接口")
public class SfPenaltyController {

    private final  SfPenaltyService sfPenaltyService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param sfPenalty 违约金调整记录表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getSfPenaltyPage(Page page, @ModelAttribute SfPenalty sfPenalty) {
      return ResultRespone.success(sfPenaltyService.page(page, Wrappers.query(sfPenalty)));
    }


    /**
     * 通过id查询违约金调整记录表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("id" ) String id) {
      return ResultRespone.success(sfPenaltyService.getById(id));
    }

    /**
     * 新增违约金调整记录表
     * @param sfPenalty 违约金调整记录表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增违约金调整记录表" )
    @ApiOperation(value = "新增违约金调整记录表")
    @PostMapping
    public ResultRespone save(@RequestBody SfPenalty sfPenalty) {
      return ResultRespone.success(sfPenaltyService.save(sfPenalty));
    }

    /**
     * 修改违约金调整记录表
     * @param sfPenalty 违约金调整记录表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改违约金调整记录表" )
    @ApiOperation(value = "修改违约金调整记录表")
    @PutMapping
    public ResultRespone updateById(@RequestBody SfPenalty sfPenalty) {
      return ResultRespone.success(sfPenaltyService.updateById(sfPenalty));
    }

    /**
     * 通过id删除违约金调整记录表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除违约金调整记录表" )
    @ApiOperation(value = "通过ID删除违约金调整记录表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable String id) {
      return ResultRespone.success(sfPenaltyService.removeById(id));
    }

    /**
     * 通过id批量删除违约金调整记录表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除违约金调整记录表" )
    @ApiOperation(value = "通过ID批量删除违约金调整记录表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(sfPenaltyService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
