/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.financialmgt.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.financialmgt.entity.SfRec;
import com.tofly.fees.financialmgt.mapper.SfRecMapper;
import com.tofly.fees.financialmgt.service.SfRecService;
import com.tofly.fees.meterreadingmgt.entity.BwMeterReadPlan;
import com.tofly.fees.meterreadingmgt.service.IBwMeterReadPlanService;
import com.tofly.fees.userprofilemgt.mapper.DaCustomerMapper;
import lombok.AllArgsConstructor;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import com.tofly.fees.common.util.GuidGenerator;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;



/**
 * 应收明细：欠费明细
 *
 * @author sxb
 * @Date Fri Jun 16 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/financialmgt/sfrec")
@Api(tags="核算管理-欠费明细(含欠费变更)接口")
public class SfRecController {

    private final  SfRecService sfRecService;

    private final IBwMeterReadPlanService iBwMeterReadPlanService;

    private final SfRecMapper sfRecMapper;

    private final DaCustomerMapper daCustomerMapper;

    /**
     * 应收明细
     */
    @GetMapping("/page" )
    @ApiOperation(value = "欠费明细-分页查询(欠费、补差)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "current", value = "当前页码", required = true, dataType = "int"),
            @ApiImplicitParam(name = "size", value = "每页显示数据条数", required = true, dataType = "int"),
            @ApiImplicitParam(name = "pages", value = "共几页,输出参数",  dataType = "int"),
            @ApiImplicitParam(name = "maxLimit", value = "每页最大显示数据", dataType = "int"),
            @ApiImplicitParam(name = "orders[0].asc", value = "排序,bool类型",  dataType = "String"),
            @ApiImplicitParam(name = "orders[0].column", value = "排序,字段名",  dataType = "String"),
            @ApiImplicitParam(name = "searchText", value = "模糊查询(用户编号、用户姓名、用户地址、联系电话)",  dataType = "String"),
            @ApiImplicitParam(name = "companyBranch", value = "公司",  dataType = "String"),
            @ApiImplicitParam(name = "mrArea", value = "片区",  dataType = "String"),
            @ApiImplicitParam(name = "mrBook", value = "抄表本", dataType = "String"),
            @ApiImplicitParam(name = "lYearMonth", value = "财务日期",  dataType = "String"),
            @ApiImplicitParam(name = "id", value = "id",  dataType = "String"),
            @ApiImplicitParam(name = "customerNo", value = "用户编号",  dataType = "String"),
            @ApiImplicitParam(name = "isBc", value = "是否水费补差",  dataType = "String")
    })
    public ResultRespone getSfRecPage(Page page,String searchText, String mrArea, String mrBook, String companyBranch,String YearMonth,String id,String customerNo,String isBc) {
        IPage<SfRec> sfRecIpage=sfRecMapper.SelectQfmxPage(page, searchText, mrArea, mrBook,companyBranch ,YearMonth,id,customerNo,isBc,"0");
         return ResultRespone.success(sfRecIpage);
    }

    /**
     * 通过id查询应收明细：欠费明细
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "欠费明细ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("id" ) String id) {
      return ResultRespone.success(sfRecService.getById(id));
    }

    /**
     * 新增应收明细：欠费明细
     * @param sfRec 应收明细：欠费明细
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增应收明细：欠费明细（水费补差）" )
    @ApiOperation(value = "新增应收明细：欠费明细（水费补差）")
    @PostMapping
    public ResultRespone save(@RequestBody SfRec sfRec) {
        return ResultRespone.success(sfRecService.save(sfRec));
    }

    /**
     * 修改应收明细：欠费明细
     * @param sfRec 应收明细：欠费明细
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改应收明细：欠费明细变更" )
    @ApiOperation(value = "修改应收明细：欠费明细变更")
    @PutMapping
    public ResultRespone updateById(@RequestBody SfRec sfRec) {
        //修改往月数据
      return ResultRespone.success(sfRecService.updateById(sfRec));
    }

    @ApiOperation(value = "撤销算费：欠费明细变更")
    @PostMapping("/CancelQfmx")
    @Transactional(rollbackFor = Exception.class) //声明式事务管理
    public ResultRespone CancelQfmx(@RequestBody SfRec sfRec) {
        //1.只能撤销当前财务月份的欠费明细
        ResultRespone<Object> rest = new ResultRespone<>();
        Map<String ,Object> mapcwrq= daCustomerMapper.selectYearMonth();//获取财务日期
        BigDecimal bigcwrq=new BigDecimal(mapcwrq.get("CWRQ").toString()) ;//202305

        if(sfRec.getYearMonth()!=null)
        {
            rest.setCode(-1);
            rest.setMessage("用水月份不能为空");
        }
        if(!bigcwrq.equals(sfRec.getYearMonth()))
        {
            rest.setCode(-1);
            rest.setMessage("只能撤销当前财务月份的欠费明细");
        }
        if(sfRec.getId().isEmpty())
        {
            rest.setCode(-1);
            rest.setMessage("修改欠费数据不存在");
        }
        if(rest.getCode()!=-1)
        {
            //2.判断是否存在欠费
            List<SfRec> sfRecList=sfRecMapper.SelectQfmxPage("", "", "","" ,sfRec.getBillYearMonth().toString(),sfRec.getId(),sfRec.getCustomerNo().toString(),"","0");
            if(sfRecList.isEmpty())
            {
                rest.setCode(-1);
                rest.setMessage("该用户不存在欠费");
            }
        }
        return getObjectResultRespone(sfRec, rest);
    }

    /**
     * 撤销欠费明细-变更
     */
    private ResultRespone<Object> getObjectResultRespone(SfRec sfRec, ResultRespone<Object> rest) {
        boolean result=false;
        if(rest.getCode()!=-1)
        {
            //3.抄表录入计算标识调为未算费状态
            BwMeterReadPlan bwMeterReadPlan=new BwMeterReadPlan();
            bwMeterReadPlan.setCalFlag("0");//算费标识
            bwMeterReadPlan.setMeterCardId(sfRec.getMeterCardId().toString());//表ID
            bwMeterReadPlan.setCustomerNo(sfRec.getCustomerNo().toString());//用户编号
            result=iBwMeterReadPlanService.updateById(bwMeterReadPlan);
            //3.修改原数据
            sfRec.setCancelFlag("1"); //冲红标识
            sfRec.setId(sfRec.getId());//欠费ID
            result= sfRecService.updateById(sfRec);
            //4.新增冲红数据
            sfRec.setId(new GuidGenerator().toString()); //冲红id
            sfRec.setCancelFlag("2"); //冲红标识
            sfRec.setCancelDate(new Date());//冲红时间
            sfRec.setInputDate(new Date());//录入时间

            double jjl = sfRec.getJjl();//加减量
            sfRec.setJjl(jjl != 0 ? -jjl : 0);
            double sl= sfRec.getQuantity();//水量
            sfRec.setQuantity(sl!=0 ? -sl :0);

            double sl1= sfRec.getQuantity1();//水量1
            sfRec.setQuantity1(sl1!=0 ? -sl1 :0);
            double sl2= sfRec.getQuantity2();//水量2
            sfRec.setQuantity1(sl2!=0 ? -sl2 :0);
            double sf= sfRec.getAmount();//水费
            sfRec.setAmount(sf!=0 ? -sf : 0);
            double sf1= sfRec.getAmount1();//水费1
            sfRec.setAmount1(sf1!=0 ? -sf1 : 0);
            double sf2= sfRec.getAmount2();//水费1
            sfRec.setAmount2(sf2!=0 ? -sf2 : 0);
            double wsf= sfRec.getSewageFee();//污水费
            sfRec.setSewageFee(wsf!=0 ? -wsf :0);
            double wsf1= sfRec.getSewageFee1();//污水费1
            sfRec.setSewageFee1(wsf1!=0 ? -wsf1 :0);
            double wsf2= sfRec.getSewageFee2();//污水费2
            sfRec.setSewageFee2(wsf2 !=0 ? -wsf2 :0);
            double ljf= sfRec.getGarbageFee();//垃圾费
            sfRec.setGarbageFee(ljf!=0 ? -ljf : 0);
            double ljf1= sfRec.getGarbageFee1();//垃圾费1
            sfRec.setGarbageFee1(ljf1!=0 ? -ljf1 : 0);
            double ljf2= sfRec.getGarbageFee2();//垃圾费1
            sfRec.setGarbageFee2(ljf2!=0 ? -ljf2 : 0);
            double yssf= sfRec.getRecAmount();//应收金额
            sfRec.setRecAmount(yssf!=0 ? -yssf : 0);
            double yssl= sfRec.getRecQuantity();//应收水量
            sfRec.setRecQuantity(yssl!=0 ?-yssl : 0);
            double ysljf= sfRec.getRecGarbageFee();//应收垃圾费
            sfRec.setRecGarbageFee(ysljf!=0 ?-ysljf : 0);
            double yswsf= sfRec.getRecSewageFee();//应收污水费
            sfRec.setRecSewageFee(yswsf!=0 ?-yswsf : 0);
            double ysqtf= sfRec.getRecOther();//应收其它费
            sfRec.setRecOther(ysqtf!=0 ?-ysqtf : 0);
            double jt1sl= sfRec.getPriceTiered1Quantity();// 阶梯1水量
            sfRec.setPriceTiered1Quantity(jt1sl!=0 ? -jt1sl :0);
            double jt1sf= sfRec.getPriceTiered1Amount();// 阶梯1水费
            sfRec.setPriceTiered1Amount(jt1sf!=0 ? -jt1sf :0);
            double jt2sl= sfRec.getPriceTiered2Quantity();// 阶梯2水量
            sfRec.setPriceTiered2Quantity(jt2sl!=0 ? -jt2sl :0);

            double jt2sf= sfRec.getPriceTiered2Amount();// 阶梯2水费
            sfRec.setPriceTiered2Amount(jt2sf!=0 ? -jt2sf :0);
            double jt3sl= sfRec.getPriceTiered3Quantity();// 阶梯3水量
            sfRec.setPriceTiered3Quantity(jt3sl!=0 ? -jt3sl :0);
            double jt3sf= sfRec.getPriceTiered3Amount();// 阶梯3水费
            sfRec.setPriceTiered3Amount(jt3sf!=0 ? -jt3sf :0);
            double jt4sl= sfRec.getPriceTiered4Quantity();// 阶梯4水量
            sfRec.setPriceTiered4Quantity(jt4sl!=0 ? -jt4sl :0);
            double jt4sf= sfRec.getPriceTiered4Amount();// 阶梯4水费
            sfRec.setPriceTiered4Amount(jt4sf!=0 ? -jt4sf :0);
            result= sfRecService.save(sfRec); ///执行撤销算费
        }
        if(result)
        {
            rest.setCode(1);
            rest.setMessage("撤销算费成功");
        }
        else
        {
            rest.setCode(-1);
            rest.setMessage("撤销算费失败,"+rest.getMessage());
        }
        return rest;
    }

    /**
     * 通过id删除应收明细：欠费明细
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除应收明细：欠费明细" )
    @ApiOperation(value = "通过ID删除应收明细：欠费明细")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable String id) {
        //限当月数据
      return ResultRespone.success(sfRecService.removeById(id));
    }

    /**
     * 通过id批量删除应收明细：欠费明细
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除应收明细：欠费明细" )
    @ApiOperation(value = "通过ID批量删除应收明细：欠费明细")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(sfRecService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
