package com.tofly.fees.financialmgt.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.tofly.fees.common.exception.FeesErrorCode;
import com.tofly.fees.common.exception.FeesException;
import com.tofly.fees.common.model.CollectionData;
import com.tofly.fees.financialmgt.entity.BmPriceEntity;
import com.tofly.fees.financialmgt.entity.BmPriceHisEntity;
import com.tofly.fees.financialmgt.entity.SysGdcsbEntity;
import com.tofly.fees.financialmgt.mapper.BmPriceHisMapper;
import com.tofly.fees.financialmgt.model.BmPriceHisModel;
import com.tofly.fees.financialmgt.service.IBmPriceHisService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.fees.financialmgt.service.ISysGdcsbService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 水价编码 服务实现类
 * </p>
 *
 * @author decai
 * @since 2023-05-30
 */
@Transactional
@Service
public class BmPriceHisServiceImpl extends ServiceImpl<BmPriceHisMapper, BmPriceHisEntity> implements IBmPriceHisService {

    @Autowired
    private ISysGdcsbService sysGdcsbService;

    @Override
    public boolean store(BmPriceHisEntity hisEntity, String username) throws FeesException {
        List<SysGdcsbEntity> list = sysGdcsbService
                .list(Wrappers.lambdaQuery(SysGdcsbEntity.class)
                        .eq(SysGdcsbEntity::getGdFlag, 0)
                        .orderBy(true, false, SysGdcsbEntity::getBillYearMonth)
                );
        if (list.size() == 0) throw new FeesException("财务日期为空", FeesErrorCode.GENERAL);
        long billYearMonth = list.get(0).getBillYearMonth();
        List<BmPriceHisEntity> bmPriceHisEntities = this
                .list(Wrappers.lambdaQuery(BmPriceHisEntity.class)
                        .eq(BmPriceHisEntity::getBillYearMonth, billYearMonth));
        if (bmPriceHisEntities.size() == 0) {//新增
            hisEntity.setBillYearMonth(billYearMonth);
            hisEntity.setOperatorCode(username);
            hisEntity.setOperatorDate(new Date());
            hisEntity.setIsdisabled("1");
            return this.save(hisEntity);
        } else {//更新
            BmPriceHisEntity hisEntity1 = bmPriceHisEntities.get(0);
            String id = hisEntity1.getId();
            hisEntity.setId(id);
            hisEntity.setIsdisabled(hisEntity1.getIsdisabled());
            hisEntity.setBillYearMonth(billYearMonth);
            hisEntity.setOperatorDate(new Date());
            hisEntity.setOperatorCode(username);
            return this.updateById(hisEntity);
        }
    }

    @Override
    public CollectionData<BmPriceHisModel> getHis(String code) throws FeesException {
        List<BmPriceHisEntity> bmPriceHisEntities = this
                .list(Wrappers.lambdaQuery(BmPriceHisEntity.class)
                        .eq(BmPriceHisEntity::getCode, code)
                        .orderBy(true, false, BmPriceHisEntity::getOperatorDate));
        List<BmPriceHisModel> models = bmPriceHisEntities
                .stream()
                .map(e -> e.toModel())
                .collect(Collectors.toList());
        return new CollectionData<>(models, models.size());
    }
}
