package com.tofly.fees.financialmgt.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.base.baseapi.UnitMgtApiService;
import com.tofly.base.baseapi.model.CompanyModel;
import com.tofly.fees.common.exception.FeesErrorCode;
import com.tofly.fees.common.exception.FeesException;
import com.tofly.fees.common.model.CollectionData;
import com.tofly.fees.common.model.PageLink;
import com.tofly.fees.financialmgt.entity.BmPriceEntity;
import com.tofly.fees.financialmgt.entity.BmPriceHisEntity;
import com.tofly.fees.financialmgt.mapper.BmPriceMapper;
import com.tofly.fees.financialmgt.model.*;
import com.tofly.fees.financialmgt.service.IBmPriceHisService;
import com.tofly.fees.financialmgt.service.IBmPriceService;
import com.tofly.fees.financialmgt.service.IBmUwTypeService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 水价编码 服务实现类
 * </p>
 *
 * @author decai
 * @since 2023-05-18
 */
@Service
@Transactional
public class BmPriceServiceImpl extends ServiceImpl<BmPriceMapper, BmPriceEntity> implements IBmPriceService {
    @Autowired
    private UnitMgtApiService unitMgtApiService;

    @Autowired
    private IBmUwTypeService bmUwTypeService;

    @Autowired
    private IBmPriceHisService bmPriceHisService;

    @Override
    public CollectionData<BmPriceModel> pageList(Long companyBranch, String name, PageLink pageLink) throws FeesException {
        Page<BmPriceEntity> page = new Page<>(pageLink.getPage(), pageLink.getPageSize());


//        IPage<BmPriceEntity> newPage = this.page(page, Wrappers.lambdaQuery(BmPriceEntity.class)
//                .eq(companyBranch != null, BmPriceEntity::getCompanyBranch, companyBranch)
//                .like(name != null, BmPriceEntity::getName, "%" + name + "%")
//                .orderBy(true, false, BmPriceEntity::getCdate));

        IPage<BmPriceEntity> newPage = this.page(page);

        Set<Long> companyIds = newPage.getRecords().stream().map(e -> e.getCompanyBranch()).collect(Collectors.toSet());
        Set<String> uwTypeCodes = newPage.getRecords().stream().map(e -> e.getUseWaterType()).collect(Collectors.toSet());

        Map<Long, String> companyMap = getCompanyMap(companyIds);
        Map<String, String> uwTypeMap = bmUwTypeService.getEntityMap(uwTypeCodes);

        return new CollectionData<>(newPage.getRecords()
                .stream()
                .map(e -> e.toModel())
                .map(e -> {
                    e.setCompanyName(companyMap.get(e.getCompanyBranch()));
                    e.setUseWaterTypeName(uwTypeMap.get(e.getUseWaterType()));
                    return e;
                })
                .collect(Collectors.toList()), newPage.getTotal());
    }

    private Map<Long, String> getCompanyMap(Set<Long> companyIds) {
        List<CompanyModel> companyModelList;
        if (companyIds.size() > 0) {
            companyModelList = unitMgtApiService.getCompanyById(companyIds);
        } else {
            return new HashMap<>();
        }
        return companyModelList.stream()
                .collect(Collectors.toMap(CompanyModel::getId, CompanyModel::getCompanyName));
    }

    @Override
    public boolean add(BmPriceAddForm form, String username) throws FeesException {
        BmPriceEntity entity = form.toEntity();
        entity.setCdate(new Date());
        entity.setOcode(username);
        QueryWrapper<BmPriceEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.select("max(code-0) + 1 as code");
        BmPriceEntity bpe = this.getOne(queryWrapper);
        if (bpe != null) {
            entity.setCode(bpe.getCode());
        } else {
            entity.setCode("1");
        }
        BmPriceHisEntity hisEntity = entity.toHisEntity();
        bmPriceHisService.store(hisEntity, username);
        return this.save(entity);
    }

    @Override
    public boolean update(BmPriceUpdateForm form, String username) throws FeesException {
        String code = form.getCode();
        BmPriceEntity entity = this.getById(code);
        if (entity == null) {
            throw new FeesException("编码不存在", FeesErrorCode.BAD_REQUEST_PARAMS);
        }
        BeanUtils.copyProperties(form, entity);
        BmPriceHisEntity hisEntity = entity.toHisEntity();
        bmPriceHisService.store(hisEntity, username);
        return this.updateById(entity);
    }

    @Override
    public boolean delete(Set<String> codes, String username) throws FeesException {
        if (codes == null || codes.size() == 0) {
            throw new FeesException("编码为空", FeesErrorCode.BAD_REQUEST_PARAMS);
        }
        for (String code : codes) {
            BmPriceEntity entity = this.getById(code);
            if (entity == null) {
                throw new FeesException("编码不存在", FeesErrorCode.BAD_REQUEST_PARAMS);
            }
        }
        return this.removeBatchByIds(codes);
    }

    @Override
    public BmPriceDetailModel get(String code) throws FeesException {
        if (code == null) {
            throw new FeesException("编码为空", FeesErrorCode.BAD_REQUEST_PARAMS);
        }
        BmPriceEntity entity = this.getById(code);
        if (entity == null) {
            throw new FeesException("编码不存在", FeesErrorCode.BAD_REQUEST_PARAMS);
        }
        return entity.toDetailModel();
    }

    @Override
    public CollectionData<BmPriceModel> getList() {
        List<BmPriceEntity> list = this.list(Wrappers
                .lambdaQuery(BmPriceEntity.class)
                .orderBy(true, false, BmPriceEntity::getCdate));
        Set<String> uwTypeCodes = list.stream().map(e -> e.getUseWaterType()).collect(Collectors.toSet());
        Map<String, String> uwTypeMap = bmUwTypeService.getEntityMap(uwTypeCodes);
        return new CollectionData<>(list.stream()
                .map(e -> {
                    String tieredFlag = e.getTieredFlag();
                    BmPriceModel model = new BmPriceModel();
                    BeanUtils.copyProperties(e, model);
                    StringBuilder sbPrice = new StringBuilder();
                    if ("1".equals(tieredFlag)) {
                        sbPrice.append(uwTypeMap.get(e.getUseWaterType())).append("<br/>");
                        if (e.getTiered1Price() != null) {
                            sbPrice.append(String.format("阶梯一:单价(%s)<br/>", e.getTiered1Price()));
                        }
                        if (e.getTiered2Price() != null) {
                            sbPrice.append(String.format("阶梯二:单价(%s)<br/>", e.getTiered2Price()));
                        }
                        if (e.getTiered3Price() != null) {
                            sbPrice.append(String.format("阶梯三:单价(%s)<br/>", e.getTiered3Price()));
                        }
                        if (e.getTiered4Price() != null) {
                            sbPrice.append(String.format("阶梯四:单价(%s)<br/>", e.getTiered4Price()));
                        }
                    } else {
                        sbPrice.append(uwTypeMap.get(e.getUseWaterType())).append(" ").append(e.getPrice());
                    }
                    model.setPrice(sbPrice.toString());
                    return model;
                }).collect(Collectors.toList()), list.size());
    }
}
