package com.tofly.fees.financialmgt.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.base.baseapi.UnitMgtApiService;
import com.tofly.base.baseapi.model.CompanyModel;
import com.tofly.fees.common.exception.FeesErrorCode;
import com.tofly.fees.common.exception.FeesException;
import com.tofly.fees.common.model.CollectionData;
import com.tofly.fees.common.model.PageLink;
import com.tofly.fees.financialmgt.entity.BmUwTypeEntity;
import com.tofly.fees.financialmgt.mapper.BmUwTypeMapper;
import com.tofly.fees.financialmgt.model.BmUwTypeAddForm;
import com.tofly.fees.financialmgt.model.BmUwTypeDetailModel;
import com.tofly.fees.financialmgt.model.BmUwTypeModel;
import com.tofly.fees.financialmgt.model.BmUwTypeUpdateForm;
import com.tofly.fees.financialmgt.service.IBmUwTypeService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 用水类型 服务实现类
 * </p>
 *
 * @author decai
 * @since 2023-05-30
 */
@Service
public class BmUwTypeServiceImpl extends ServiceImpl<BmUwTypeMapper, BmUwTypeEntity> implements IBmUwTypeService {
    @Autowired
    private UnitMgtApiService unitMgtApiService;

    @Override
    public CollectionData<BmUwTypeModel> pageList(Long companyBranch, String name, PageLink pageLink) throws FeesException {
        Page<BmUwTypeEntity> page = new Page<>(pageLink.getPage(), pageLink.getPageSize());
        IPage<BmUwTypeEntity> newPage = this.page(page, Wrappers.lambdaQuery(BmUwTypeEntity.class)
                .eq(companyBranch != null, BmUwTypeEntity::getCompanyBranch, companyBranch)
                .like(name != null, BmUwTypeEntity::getName, "%" + name + "%")
                .orderBy(true, false, BmUwTypeEntity::getCdate));

        Set<Long> companyIds = newPage.getRecords().stream()
                .map(e -> e.getCompanyBranch()).collect(Collectors.toSet());

        Map<Long, String> companyMap = getCompanyMap(companyIds);

        return new CollectionData<>(newPage.getRecords()
                .stream()
                .map(e -> e.toModel())
                .map(e -> {
                    e.setCompanyName(companyMap.get(e.getCompanyBranch()));
                    return e;
                })
                .collect(Collectors.toList()), newPage.getTotal());
    }

    private Map<Long, String> getCompanyMap(Set<Long> companyIds) {
        List<CompanyModel> companyModelList = new ArrayList<>();
        if (companyIds.size() > 0) {
            companyModelList = unitMgtApiService.getCompanyById(companyIds);
        } else {
            return new HashMap<>();
        }
        return companyModelList.stream()
                .collect(Collectors.toMap(CompanyModel::getId, CompanyModel::getCompanyName));
    }

    @Override
    public boolean add(BmUwTypeAddForm form, String username) throws FeesException {
        BmUwTypeEntity entity = form.toEntity();
        entity.setCdate(new Date());
        entity.setOcode(username);
        QueryWrapper<BmUwTypeEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.select("max(code-0) + 1 as code");
        BmUwTypeEntity bme = this.getOne(queryWrapper);
        if (bme != null) {
            entity.setCode(bme.getCode());
        } else {
            entity.setCode("1");
        }
        return this.save(entity);
    }

    @Override
    public boolean update(BmUwTypeUpdateForm form, String username) throws FeesException {
        String code = form.getCode();
        BmUwTypeEntity entity = this.getById(code);
        if (entity == null) {
            throw new FeesException("编码不存在", FeesErrorCode.BAD_REQUEST_PARAMS);
        }
        BeanUtils.copyProperties(form, entity);
        return this.updateById(entity);
    }

    @Override
    public boolean delete(Set<String> codes, String username) throws FeesException {
        if (codes == null || codes.size() == 0) {
            throw new FeesException("编码为空", FeesErrorCode.BAD_REQUEST_PARAMS);
        }
        for (String code : codes) {
            BmUwTypeEntity entity = this.getById(code);
            if (entity == null) {
                throw new FeesException("编码不存在", FeesErrorCode.BAD_REQUEST_PARAMS);
            }
        }
        return this.removeBatchByIds(codes);
    }

    @Override
    public BmUwTypeDetailModel get(String code) throws FeesException {
        if (code == null) {
            throw new FeesException("编码为空", FeesErrorCode.BAD_REQUEST_PARAMS);
        }
        BmUwTypeEntity entity = this.getById(code);
        if (entity == null) {
            throw new FeesException("编码不存在", FeesErrorCode.BAD_REQUEST_PARAMS);
        }
        return entity.toDetailModel();
    }

    @Override
    public Map<String, String> getEntityMap(Set<String> codes) {
        List<BmUwTypeEntity> list = this.list(new QueryWrapper<BmUwTypeEntity>().lambda()
                .in(BmUwTypeEntity::getCode, codes));
        //List<Company> list = companyService.listByIds();
        return list.stream().collect(Collectors.toMap(BmUwTypeEntity::getCode, BmUwTypeEntity::getName));
    }
}
