/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.meterprofilemgt.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.common.util.SysConfigUtils;
import com.tofly.fees.meterprofilemgt.entity.DaMeter;
import com.tofly.fees.meterprofilemgt.mapper.DaMeterMapper;
import com.tofly.fees.meterprofilemgt.service.DaMeterService;
import com.tofly.fees.meterreadingmgt.entity.BwMeterReadPlan;
import com.tofly.fees.meterreadingmgt.service.IBwMeterReadPlanService;
import com.tofly.fees.userprofilemgt.entity.DaCustomer;
import com.tofly.fees.userprofilemgt.service.DaCustomerService;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Map;

/**
 * 用户表计信息
 *
 * @author admin
 * @date Wed Jun 07 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/meterprofilemgt/dameter")
@Api(tags = "水表档案-用户表计信息接口")
public class DaMeterController {

    private final DaMeterService daMeterService;
    private final DaCustomerService daCustomerService;
    private final IBwMeterReadPlanService bwMeterReadPlanService;
    DaMeterMapper daMeterMapper;

    /**
     * 分页查询
     *
     * @param page    分页对象
     * @param daMeter 用户表计信息
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone getDaMeterPage(Page page, @ModelAttribute DaMeter daMeter) {
        return ResultRespone.success(daMeterService.page(page, Wrappers.query(daMeter)));
    }


    @GetMapping("/{customerNo}/{meterCardId}")
    @ApiOperation(value = "通过用户编号查询，ID号")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "customerNo", value = "用户编号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "meterCardId", value = "ID号meterCardId", required = true, dataType = "String")
    })
    public ResultRespone getDaMeter(@PathVariable("customerNo") String customerNo, @PathVariable("meterCardId") String meterCardId) {

        DaMeter daMeter = daMeterMapper.SelectDaMeter(customerNo, meterCardId);
        return ResultRespone.success(daMeter);
    }


    /**
     * 通过id查询用户表计信息
     *
     * @param customerNo id
     * @return ResultRespone
     */
    @GetMapping("/{customerNo}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "customerNo", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("customerNo") String customerNo) {
        return ResultRespone.success(daMeterService.getById(customerNo));
    }

    /**
     * 新增用户表计信息
     *
     * @param daMeter 用户表计信息
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增用户表计信息")
    @ApiOperation(value = "新增用户表计信息")
    @Transactional(rollbackFor = Exception.class)
    @PostMapping
    public ResultRespone save(@RequestBody DaMeter daMeter) {
        //水表类别不等于卡表时才添加抄表计划
        if (daMeter.getMeterType() != "1") {
            //查询当前用户的用户档案实体
            DaCustomer daCustomer = daCustomerService.getById(daMeter.getCustomerNo());

            //region 抄表计划实体赋值
            //抄表计划添加
            BwMeterReadPlan bwMeterReadPlan = new BwMeterReadPlan();
            bwMeterReadPlan.setCustomerNo(daCustomer.getCustomerNo());
            bwMeterReadPlan.setMeterCardNo(daMeter.getMeterCardNo());
            bwMeterReadPlan.setMeterCardId(daMeter.getMeterCardId().toString());

            //财务日期
            String strYearMonth = SysConfigUtils.getBillYearMonth("1");
            bwMeterReadPlan.setYearMonth(new BigDecimal(strYearMonth));

            bwMeterReadPlan.setCalFlag("0");
            bwMeterReadPlan.setCalState("1");
            bwMeterReadPlan.setMrFlag("0");
            bwMeterReadPlan.setAuditFlag("0");
            bwMeterReadPlan.setCompanyBranch(daCustomer.getCompanyBranch());
            bwMeterReadPlan.setMrArea(daCustomer.getMrArea());
            bwMeterReadPlan.setMrBook(daCustomer.getMrBook());
            //endregion

            if (bwMeterReadPlanService.save(bwMeterReadPlan)
                    && daMeterService.save(daMeter)) {
                return ResultRespone.success(true);
            } else {
                return ResultRespone.failed("添加失败！");
            }
        } else {
            return ResultRespone.success(daMeterService.save(daMeter));
        }
    }

    /**
     * 修改用户表计信息
     *
     * @param daMeter 用户表计信息
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改用户表计信息")
    @ApiOperation(value = "修改用户表计信息")
    @PutMapping
    public ResultRespone updateById(@RequestBody DaMeter daMeter) {
        return ResultRespone.success(daMeterService.updateById(daMeter));
    }

    /**
     * 通过id删除用户表计信息
     *
     * @param customerNo id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除用户表计信息")
    @ApiOperation(value = "通过ID删除用户表计信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "customerNo", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{customerNo}")
    public ResultRespone removeById(@PathVariable String customerNo) {
        return ResultRespone.success(daMeterService.removeById(customerNo));
    }

    /**
     * 通过id批量删除用户表计信息
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除用户表计信息")
    @ApiOperation(value = "通过ID批量删除用户表计信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(daMeterService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
