/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.meterprofilemgt.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.meterprofilemgt.entity.DaMeterLog;
import com.tofly.fees.meterprofilemgt.service.DaMeterLogService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;

import java.util.Arrays;
/**
 * 水表档案日志记录表
 *
 * @author sxb
 * @date Tue Jun 27 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/meterprofilemgt/dameterlog")
@Api(tags="水表档案-水表档案日志记录表接口")
public class DaMeterLogController {

    private final  DaMeterLogService daMeterLogService;

    /**
     * 分页查询 水表档案
     * @param page
     * @param logCdate_Start
     * @param logCdate_End
     * @param customerNo
     * @return 0
     */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "logCdate_Start", value = "操作开始日期", required = false, dataType = "String"),
            @ApiImplicitParam(name = "logCdate_End", value = "操作结束日期", required = false, dataType = "String"),
            @ApiImplicitParam(name = "customerNo", value = "用户编号", required = false, dataType = "String"),
    })
    public ResultRespone getDaMeterLogPage(Page page, String logCdate_Start,String logCdate_End , String customerNo) {
        QueryWrapper<DaMeterLog> queryWrapper=new QueryWrapper<>();
        queryWrapper.eq("1","1");
        if(StringUtils.isNotBlank(customerNo)) {
            queryWrapper.and(i->i.eq("CUSTOMER_NO",customerNo));
        }
        if(StringUtils.isNotBlank(logCdate_Start) && StringUtils.isNotBlank(logCdate_End)) {
            queryWrapper.and(i->i.apply("LOG_CDATE BETWEEN to_date({0},'yyyy-mm-dd hh24:mi:ss') AND to_date({1},'yyyy-mm-dd hh24:mi:ss')", logCdate_Start, logCdate_End));
        }
        IPage<DaMeterLog> daMeterLogIPage=daMeterLogService.page(page,queryWrapper);

      return ResultRespone.success(daMeterLogIPage);
    }


    /**
     * 通过id查询水表档案日志记录表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("id" ) String id) {
      return ResultRespone.success(daMeterLogService.getById(id));
    }

    /**
     * 新增水表档案日志记录表
     * @param daMeterLog 水表档案日志记录表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增水表档案日志记录表" )
    @ApiOperation(value = "新增水表档案日志记录表")
    @PostMapping
    public ResultRespone save(@RequestBody DaMeterLog daMeterLog) {
      return ResultRespone.success(daMeterLogService.save(daMeterLog));
    }

    /**
     * 修改水表档案日志记录表
     * @param daMeterLog 水表档案日志记录表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改水表档案日志记录表" )
    @ApiOperation(value = "修改水表档案日志记录表")
    @PutMapping
    public ResultRespone updateById(@RequestBody DaMeterLog daMeterLog) {
      return ResultRespone.success(daMeterLogService.updateById(daMeterLog));
    }

    /**
     * 通过id删除水表档案日志记录表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除水表档案日志记录表" )
    @ApiOperation(value = "通过ID删除水表档案日志记录表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable String id) {
      return ResultRespone.success(daMeterLogService.removeById(id));
    }

    /**
     * 通过id批量删除水表档案日志记录表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除水表档案日志记录表" )
    @ApiOperation(value = "通过ID批量删除水表档案日志记录表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(daMeterLogService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
