package com.tofly.fees.meterprofilemgt.controller_easy;

import com.tofly.common.core.entity.ResultRespone;
import com.tofly.fees.common.BaseController;
import com.tofly.fees.common.dbhelper.DbHelper;
import com.tofly.fees.common.dbhelper.PageSortHelper;
import com.tofly.fees.common.dbhelper.ProduceSql;
import com.tofly.fees.common.enums.SysEnum;
import com.tofly.fees.common.enums.TableIdEnum;
import com.tofly.fees.common.model.PageLink;
import com.tofly.fees.common.util.DateUtils;
import com.tofly.fees.meterprofilemgt.entity.BwReplaceMeter;
import com.tofly.fees.meterprofilemgt.entity.DaMeter;
import com.tofly.fees.meterreadingmgt.entity.BwMeterReadPlan;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.AllArgsConstructor;
import org.apache.commons.lang.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.text.MessageFormat;
import java.util.Date;

import static com.tofly.fees.common.ControllerConstants.*;
import static com.tofly.fees.common.ControllerConstants.SORT_ORDER_ALLOWABLE_VALUES;

/**
 * 水表更换简易处理类
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/meterprofilemgt/BwReplaceMeter")
@Api(tags = "水表档案-用户水表更换接口")
public class BwReplaceMeterEasyController extends BaseController {

    @ApiOperation(value = "水表更换数据列表接口")
    @PostMapping("/getMeterListForChange")
    public ResultRespone getMeterListForChange(
            @ApiParam(value = "所属公司") @RequestParam(required = false) String company,
            @ApiParam(value = "所属片区") @RequestParam(required = false) String mrArea,
            @ApiParam(value = "所属表册") @RequestParam(required = false) String mrBook,
            @ApiParam(value = "关键字（用户编号、姓名、地址、电话）") @RequestParam(required = false) String searchText,
            @ApiParam(value = PAGE_SIZE_DESCRIPTION, required = true, defaultValue = "10") @RequestParam(value = PAGE_SIZE_PARA_NAME, required = true) int pageSize,
            @ApiParam(value = PAGE_NUMBER_DESCRIPTION, required = true, defaultValue = "1") @RequestParam(value = PAGE_NUMBER_PARA_NAME, required = true) int page,
            @ApiParam(value = SORT_PROPERTY_DESCRIPTION) @RequestParam(required = false) String sortProperty,
            @ApiParam(value = SORT_ORDER_DESCRIPTION, allowableValues = SORT_ORDER_ALLOWABLE_VALUES) @RequestParam(required = false) String sortOrder)
            throws Exception {

        //region 分页、排序
        DbHelper db = DbHelper.getDbHelper();
        PageLink pg = createPageLink(pageSize, page);


        //String strOrder = StringUtils.isBlank(sortProperty) ? "a.customer_no asc" : sortProperty + " " + sortOrder;

        String[] arrExcludeColumns = {
                "oldMeterType", "oldMeterFactory", "oldMeterModel", "oldMeterFactoryDatetime", "oldMeterCaliber"
                , "oldMeterNo", "oldMeterRange", "oldMeterNowReading", "oldMeterLastReading"
        };
        String strOrder = PageSortHelper.setPageSort(sortProperty, sortOrder, arrExcludeColumns);

        //endregion

        //region 查询条件
        String strWhere = "";
        //所属公司
        if (!StringUtils.isBlank(company)) {
            strWhere += String.format(" and a.company_branch='%s'", company);
        }
        //所属片区
        if (!StringUtils.isBlank(mrArea)) {
            strWhere += String.format(" and a.mr_area='%s'", mrArea);
        }
        //所属表册
        if (!StringUtils.isBlank(mrBook)) {
            strWhere += String.format(" and a.mr_book='%s'", mrBook);
        }
        //关键字
        if (!StringUtils.isBlank(searchText)) {
            strWhere += MessageFormat.format("and (a.customer_no like ''%{0}%'' or a.customer_name like ''%{0}%'' "
                    + "or a.customer_address like ''%{0}%'' or a.customer_phone like ''%{0}%'')", searchText);
        }
        //endregion

        //region 查询数据
        String strSql = MessageFormat.format("select a.customer_no,b.meter_card_no,b.meter_card_id,customer_name,customer_address,customer_phone" +
                ",meter_type as \"oldMeterType\",meter_factory as \"oldMeterFactory\",meter_model as \"oldMeterModel\"" +
                ",meter_factory_datetime as \"oldMeterFactoryDatetime\",meter_caliber as \"oldMeterCaliber\",meter_no as \"oldMeterNo\"" +
                ",meter_range as \"oldMeterRange\",this_reading as \"oldMeterNowReading\",last_reading as \"oldMeterLastReading\" " +
                "from tf_ywys_da_customer a,tf_ywys_da_meter b,tf_ywys_bw_meter_read_plan c " +
                "where a.customer_no=b.customer_no and a.customer_no=c.customer_no " +
                "and a.customer_state=1 and b.meter_state=1 {0} order by {1}", strWhere, strOrder);

        Object obj = db.getPage(pg, strSql, SysEnum.MyBatis.getName());
        return ResultRespone.success(obj);
        //endregion
    }

    @ApiOperation(value = "水表更换数据存储")
    @PostMapping("/saveMeterChange")
    public ResultRespone saveMeterChange(@RequestBody BwReplaceMeter bwReplaceMeter)
            throws Exception {

        //region 必填验证
        String strNewMeterType = bwReplaceMeter.getNewMeterType();
        if (StringUtils.isBlank(strNewMeterType)) {
            return ResultRespone.failed("「新表类别」不能为空！");
        }

        String strNewMeterFactory = bwReplaceMeter.getNewMeterFactory();
        if (StringUtils.isBlank(strNewMeterFactory)) {
            return ResultRespone.failed("「新表厂家」不能为空！");
        }

        String strNewMeterModel = bwReplaceMeter.getNewMeterModel();
        if (StringUtils.isBlank(strNewMeterModel)) {
            return ResultRespone.failed("「新表型号」不能为空！");
        }

        String strNewMeterCaliber = bwReplaceMeter.getNewMeterCaliber();
        if (StringUtils.isBlank(strNewMeterCaliber)) {
            return ResultRespone.failed("「新表口径」不能为空！");
        }

        BigDecimal meterCardId = bwReplaceMeter.getMeterCardId();
        if (meterCardId.doubleValue() == 0) {
            return ResultRespone.failed("「抄表卡ID」不能为空！");
        }
        //endregion

        //region 数据存储
        Date dtNow = new Date();

        //换表记录，依次为：录入时间
        bwReplaceMeter.setId(null);
        bwReplaceMeter.setInputDate(dtNow);

        //水表档案更新，依次为：水表编号、水表类别、水表厂家、水表型号、水表口径、水表量程、出厂日期
        DaMeter daMeter = new DaMeter();
        daMeter.setMeterNo(bwReplaceMeter.getNewMeterNo());
        daMeter.setMeterType(bwReplaceMeter.getNewMeterType());
        daMeter.setMeterFactory(bwReplaceMeter.getNewMeterFactory());
        daMeter.setMeterModel(bwReplaceMeter.getNewMeterModel());
        daMeter.setMeterCaliber(bwReplaceMeter.getNewMeterCaliber());
        daMeter.setMeterRange(bwReplaceMeter.getNewMeterRange());
        daMeter.setMeterFactoryDatetime(bwReplaceMeter.getNewMeterFactoryDatetime());

        //更新抄表计划
        BwMeterReadPlan bwMeterReadPlan = new BwMeterReadPlan();
        bwMeterReadPlan.setLastReading(bwReplaceMeter.getOldMeterLastReading());
        bwMeterReadPlan.setThisReading(bwReplaceMeter.getNewMeterNowReading());
        bwMeterReadPlan.setOldMeterEreading(bwReplaceMeter.getOldMeterNowReading());
        bwMeterReadPlan.setNewMeterBreading(bwReplaceMeter.getNewMeterLastReading());
        bwMeterReadPlan.setMrFlag("1");
        bwMeterReadPlan.setCalFlag("0");
        bwMeterReadPlan.setMrDate(dtNow);
        bwMeterReadPlan.setInputDate(dtNow);
        bwMeterReadPlan.setInputOcode(bwReplaceMeter.getInputOcode());
        bwMeterReadPlan.setInputOname(bwReplaceMeter.getInputName());

        bwMeterReadPlan.setMrType(new BigDecimal(-3));
        bwMeterReadPlan.setQuantity(bwMeterReadPlan.getQuantity());

        //数据库存储处理
        DbHelper db = DbHelper.getDbHelper();
        //换表记录insert sql
        db.addTransactionSql(ProduceSql.getInsertSql(bwReplaceMeter, TableIdEnum.GUID));
        //更新水表档案表sql
        db.addTransactionSql(ProduceSql.getUpdateSql(daMeter, "METER_CARD_ID='" + bwReplaceMeter.getMeterCardId() + "'"));
        //更新抄表录入表sql
        db.addTransactionSql(ProduceSql.getUpdateSql(bwMeterReadPlan, "METER_CARD_ID='" + bwReplaceMeter.getMeterCardId() + "'"));

        boolean isSuccess = db.executeTransaction();
        String strMsg = isSuccess ? "换表信息保存成功！" : "错误：换表信息保存失败！";
        return ResultRespone.success(isSuccess, strMsg);
        //endregion
    }
}

