/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.meterreadingmgt.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.meterreadingmgt.entity.BmMrFrequency;
import com.tofly.fees.meterreadingmgt.service.BmMrFrequencyService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 抄表频率
 *
 * @author hdc
 * @date Mon May 29 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/meterreadingmgt/bmmrfrequency")
@Api(tags="抄表频率接口")
public class BmMrFrequencyController {

    private final  BmMrFrequencyService bmMrFrequencyService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param bmMrFrequency 抄表频率
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getBmMrFrequencyPage(Page page, @ModelAttribute BmMrFrequency bmMrFrequency) {
      return ResultRespone.success(bmMrFrequencyService.page(page, Wrappers.query(bmMrFrequency)));
    }


    /**
     * 通过id查询抄表频率
     * @param code id
     * @return ResultRespone
     */
    @GetMapping("/{code}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("code" ) String code) {
      return ResultRespone.success(bmMrFrequencyService.getById(code));
    }

    /**
     * 新增抄表频率
     * @param bmMrFrequency 抄表频率
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增抄表频率" )
    @ApiOperation(value = "新增抄表频率")
    @PostMapping
    public ResultRespone save(@RequestBody BmMrFrequency bmMrFrequency) {
      return ResultRespone.success(bmMrFrequencyService.save(bmMrFrequency));
    }

    /**
     * 修改抄表频率
     * @param bmMrFrequency 抄表频率
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改抄表频率" )
    @ApiOperation(value = "修改抄表频率")
    @PutMapping
    public ResultRespone updateById(@RequestBody BmMrFrequency bmMrFrequency) {
      return ResultRespone.success(bmMrFrequencyService.updateById(bmMrFrequency));
    }

    /**
     * 通过id删除抄表频率
     * @param code id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除抄表频率" )
    @ApiOperation(value = "通过ID删除抄表频率")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "code", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{code}" )
    public ResultRespone removeById(@PathVariable String code) {
      return ResultRespone.success(bmMrFrequencyService.removeById(code));
    }

    /**
     * 通过id批量删除抄表频率
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除抄表频率" )
    @ApiOperation(value = "通过ID批量删除抄表频率")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById2(String ids) {
        return ResultRespone.success(bmMrFrequencyService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
