/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.meterreadingmgt.controller;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.common.BaseController;
import com.tofly.fees.common.annotate.requestPostSingle.RequestPostSingleParam;
import com.tofly.fees.common.dbhelper.DbHelper;
import com.tofly.fees.common.dbhelper.LogHelper;
import com.tofly.fees.common.dbhelper.PageSortHelper;
import com.tofly.fees.common.dbhelper.ProduceSql;
import com.tofly.fees.common.model.ProcParamModel;
import com.tofly.fees.common.util.DateUtils;
import com.tofly.fees.common.util.NumberUtils;
import com.tofly.fees.common.util.SysConfigUtils;
import com.tofly.fees.meterprofilemgt.entity.DaMeter;
import com.tofly.fees.meterprofilemgt.service.DaMeterService;
import com.tofly.fees.meterreadingmgt.entity.BwMeterReadPlan;
import com.tofly.fees.meterreadingmgt.mapper.BwMeterReadPlanMapper;
import com.tofly.fees.meterreadingmgt.service.IBwMeterReadPlanService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang.StringUtils;
import org.apache.ibatis.mapping.ParameterMode;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.sql.JDBCType;
import java.text.MessageFormat;
import java.util.*;
import java.util.concurrent.ConcurrentLinkedQueue;

/**
 * 抄表计划-抄表录入算费接口
 *
 * @author shudong
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/bwMeterReadPlan")
@Api(tags = "抄表管理-抄表算费接口")
public class BwMeterReadPlanController extends BaseController {

    private final IBwMeterReadPlanService bwMeterReadPlanService;
    private final BwMeterReadPlanMapper bwMeterReadPlanMapper;

    private final DaMeterService daMeterService;


    /**
     * 抄表录入查询列表数据
     */
    @PostMapping("/page")
    @ApiOperation(value = "抄表录入查询列表数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "companyBranch", value = "公司", dataType = "String"),
            @ApiImplicitParam(name = "mrArea", value = "片区", dataType = "String"),
            @ApiImplicitParam(name = "mrBook", value = "抄表本", dataType = "String"),
            @ApiImplicitParam(name = "mrOCode", value = "抄表员", dataType = "String"),
            @ApiImplicitParam(name = "mrMode", value = "抄表方式", dataType = "String"),
            @ApiImplicitParam(name = "mrFlag", value = "抄表状态", dataType = "String"),
            @ApiImplicitParam(name = "calFlag", value = "算费状态", dataType = "String"),
            @ApiImplicitParam(name = "searchText", value = "模糊查询(用户编号、姓名)", dataType = "String")
    })
    public ResultRespone getMeterReadList(Page page, String companyBranch, String mrArea
            , String mrBook, String mrOCode, String mrMode, String mrFlag, String calFlag, String searchText) {
        PageSortHelper.setPageSort(page, new String[]{"mrModeName", "mrTypeName"});
        IPage<Map<String, Object>> pageList = bwMeterReadPlanMapper.getMeterReadList(page, companyBranch, mrArea
                , mrBook, mrOCode, mrMode, mrFlag, calFlag, searchText);
        return ResultRespone.success(pageList);
    }

    /**
     * 修改抄表计划_抄表录入
     *
     * @param data 抄表录入对象
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "保存抄表录入")
    @ApiOperation(value = "保存抄表录入")
    @PostMapping("/save")
    public ResultRespone svaeMeterReadInfo(@RequestPostSingleParam("data") JSONObject data) throws Exception {
        //region 初始赋值

        //抄表类型
        int v_cblx = Convert.toInt(data.get("cblx"));
        //水表量程
        double v_sblc = Convert.toDouble(data.get("sblc"));
        String czrybm = Convert.toStr(data.get("czrybm"));
        String czryxm = Convert.toStr(data.get("czryxm"));

        BwMeterReadPlan bwMeterReadPlan = new BwMeterReadPlan();
        bwMeterReadPlan.setMrType(Convert.toBigDecimal(v_cblx));
        bwMeterReadPlan.setThisReading(Convert.toBigDecimal(data.get("bcds")));
        bwMeterReadPlan.setLastReading(Convert.toBigDecimal(data.get("scds")));
        bwMeterReadPlan.setJjl(Convert.toBigDecimal(data.get("jjl")));
        bwMeterReadPlan.setOldMeterEreading(Convert.toBigDecimal(data.get("jbzd")));
        bwMeterReadPlan.setNewMeterBreading(Convert.toBigDecimal(data.get("xbqd")));
        bwMeterReadPlan.setRemark(Convert.toStr(data.get("bz")));
        bwMeterReadPlan.setMeterCardId(data.get("cbkid").toString());
        bwMeterReadPlan.setQuantity(Convert.toBigDecimal(data.get("sjsl")));
        bwMeterReadPlan.setMrMode(data.get("cbfs").toString());
        //endregion

        //region 验证
        int sjsl = 0;
        if (v_cblx == 0) {
            //正常
            sjsl = Convert.toInt(bwMeterReadPlan.getThisReading()
                    .subtract(bwMeterReadPlan.getLastReading())
                    .add(bwMeterReadPlan.getJjl()));
        } else if (v_cblx == -1) {
            //表倒走
            sjsl = Math.abs(Convert.toInt(
                    bwMeterReadPlan.getLastReading().subtract(bwMeterReadPlan.getThisReading())))
                    + Convert.toInt(bwMeterReadPlan.getJjl());
        } else if (v_cblx == -2) {
            //表满度
            double sbmdValue = Math.pow(10, v_sblc) - 1;
            sjsl = Math.abs(Convert.toInt(sbmdValue)
                    - Convert.toInt(bwMeterReadPlan.getLastReading())
                    + Convert.toInt(bwMeterReadPlan.getThisReading()))
                    + Convert.toInt(bwMeterReadPlan.getJjl());
        } else if (v_cblx == -3) {
            //洗换表
            sjsl = Convert.toInt(bwMeterReadPlan.getOldMeterEreading()
                    .subtract(bwMeterReadPlan.getLastReading()))
                    + Convert.toInt(bwMeterReadPlan.getThisReading()
                    .subtract(bwMeterReadPlan.getNewMeterBreading()));
        }

        if (bwMeterReadPlan.getMeterCardId().equals("")) {
            return ResultRespone.failed(false, "用户编号不能为空！");
        }
        if (Convert.toInt(bwMeterReadPlan.getQuantity()) != sjsl) {
            return ResultRespone.failed(false, "实际水量与计算不符！");
        }
        //endregion

        //region 保存
        DbHelper db = DbHelper.getDbHelper();

        Date nowDate = new Date();
        bwMeterReadPlan.setMrDate(nowDate);
        bwMeterReadPlan.setInputOcode(czrybm);
        bwMeterReadPlan.setInputOname(czryxm);
        bwMeterReadPlan.setInputDate(nowDate);
        bwMeterReadPlan.setMrFlag("1");
        bwMeterReadPlan.setMrDataSource("0");
        bwMeterReadPlan.setMrTimes(new BigDecimal(1));

        //审核信息，默认审核通过，暂无审核抄表
        bwMeterReadPlan.setAuditFlag("1");
        bwMeterReadPlan.setAuditOcode(czrybm);
        bwMeterReadPlan.setAuditOname(czryxm);
        bwMeterReadPlan.setAuditDate(nowDate);
        bwMeterReadPlan.setAuditSuggestion("自动审核");

        //更新抄表计划表
        db.addTransactionSql(ProduceSql.getUpdateSql(bwMeterReadPlan
                , "meter_card_id='" + bwMeterReadPlan.getMeterCardId() + "'"));

        String meterCardId = bwMeterReadPlan.getMeterCardId();
        if (v_cblx == -2) {
            //region 表满度
            DaMeter daMeter = daMeterService.getById(meterCardId);
            if (Convert.toDouble(daMeter.getMeterRange()) != v_sblc) {
                //更新水表量程
                DaMeter daMeterNew = new DaMeter();
                daMeterNew.setMeterRange(NumberUtils.toString(v_sblc));
                daMeterNew.setOperatorCode(czrybm);
                daMeterNew.setOperatorName(czryxm);
                daMeterNew.setOperatorDatetime(nowDate);
                db.addTransactionSql(ProduceSql.getUpdateSql(daMeterNew, "METER_CARD_ID='" + meterCardId + "'"));

                //添加修改日志记录（水表满度，修改量程）
                String strLog = LogHelper.getModifySQL(daMeterNew, "METER_CARD_ID='" + meterCardId + "'", "抄表录入[水表满度，修改量程]");
                if (!StringUtils.isBlank(strLog)) {
                    db.addTransactionSql(strLog);
                }
            }
            //endregion
        } else if (v_cblx == -3) {
            //添加修改日志记录（洗换表）
            String strLog = LogHelper.getModifySQL(bwMeterReadPlan, "METER_CARD_ID='" + meterCardId + "'", "抄表录入-洗换表");
            if (!StringUtils.isBlank(strLog)) {
                db.addTransactionSql(strLog);
            }
        }

        if (Convert.toBool(data.get("xgds"))) {
            //上次读数修改前的值，数据库查询
            BwMeterReadPlan bwMeterReadPlanOld = bwMeterReadPlanService.getById(meterCardId);
            BwMeterReadPlan bwMeterReadPlanNew = new BwMeterReadPlan();

            //克隆新对象
            BeanUtils.copyProperties(bwMeterReadPlanOld, bwMeterReadPlanNew);
            bwMeterReadPlanNew.setLastReading(bwMeterReadPlan.getLastReading());

            //判断是否真实被修改
            if (!bwMeterReadPlanOld.getLastReading().equals(bwMeterReadPlanNew.getLastReading())) {
                //添加修改日志记录
                String strLog = LogHelper.getModifySQL(bwMeterReadPlanOld, bwMeterReadPlanNew, "抄表录入-修改底数");
                if (!StringUtils.isBlank(strLog)) {
                    db.addTransactionSql(strLog);
                }
            }
        }
        //事物提交数据
        boolean isFlag = db.executeTransaction();
        if (isFlag) {
            return ResultRespone.success(true, "保存成功！");
        } else {
            return ResultRespone.failed(false, "更新失败！");
        }
        //endregion
    }


    /**
     * 抄表算费：查询抄表册选择列表数据
     *
     * @param page          分页对象
     * @param companyBranch 公司编码值
     * @param mrArea        抄表片区值
     * @return
     */
    @PostMapping("/getMrBookList")
    @ApiOperation(value = "抄表算费-查询抄表册列表数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "companyBranch", value = "公司", dataType = "String"),
            @ApiImplicitParam(name = "mrArea", value = "片区", dataType = "String")
    })
    public ResultRespone getMrBookList(Page page, String companyBranch, String mrArea) {
        PageSortHelper.setPageSort(page, null);
        IPage<Map<String, Object>> pageList = bwMeterReadPlanMapper.getMrBookList(page, companyBranch, mrArea);
        return ResultRespone.success(pageList);
    }

    /**
     * 抄表算费：获取可算费的用户明细数据
     *
     * @param page          分页对象
     * @param companyBranch 公司编码值
     * @param mrBooks       抄表本集合
     * @param searchText    关键字（用户姓名、地址、用户编号）
     * @return
     */
    @PostMapping("/getUserList")
    @ApiOperation(value = "抄表算费-获取可算费的用户明细数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "companyBranch", value = "公司", dataType = "String"),
            @ApiImplicitParam(name = "mrBooks", value = "抄表本集合", dataType = "String"),
            @ApiImplicitParam(name = "searchText", value = "关键字（用户姓名、地址、用户编号）", dataType = "String")
    })
    public ResultRespone getUserList(Page page, String companyBranch, String mrBooks, String searchText) {
        PageSortHelper.setPageSort(page, null);
        if (!StringUtils.isBlank(mrBooks)) {
            // 12345,12346,12344  >>  '12345','12346','12344'
            mrBooks = "'" + mrBooks.replace(",", "','") + "'";
        }
        IPage<Map<String, Object>> pageList = bwMeterReadPlanMapper.getUserList(page, companyBranch, mrBooks, searchText);
        return ResultRespone.success(pageList);
    }

    /**
     * 抄表算费-按选中用户算费
     *
     * @param meterCardIds 抄表卡ID集（多个英文逗号隔开）
     * @return
     * @throws Exception
     */
    @PostMapping("/doCalcFeeForUser")
    @ApiOperation(value = "抄表算费-按选中用户算费")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "meterCardIds", value = "抄表卡ID集（多个英文逗号隔开）", dataType = "String")
    })
    public ResultRespone doCalcFeeForUser(@RequestPostSingleParam("meterCardIds") String meterCardIds) throws Exception {
        if (StringUtils.isBlank(meterCardIds)) {
            return ResultRespone.failed("算费失败：请选择需要算费的用户！");
        }
        String[] arrCardIds = meterCardIds.split(",");
        return getResultResponse(arrCardIds);
    }

    /**
     * 抄表算费-按选中抄表册
     *
     * @param mrBooks 抄表册编码集（多个英文逗号隔开）
     * @return
     * @throws Exception
     */
    @PostMapping("/doCalcFeeForMrBook")
    @ApiOperation(value = "抄表算费-按选中抄表册")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "mrBooks", value = "抄表册编码集（多个英文逗号隔开）", dataType = "String")
    })
    public ResultRespone doCalcFeeForMrBook(@RequestPostSingleParam("mrBooks") String mrBooks) throws Exception {
        if (StringUtils.isBlank(mrBooks)) {
            return ResultRespone.failed("算费失败：抄表册参数为空！");
        }
        mrBooks = "'" + mrBooks.replace(",", "','") + "'";
        List<HashMap<String, Object>> hashMapList = bwMeterReadPlanMapper.getMeterCardIds(mrBooks);
        if (hashMapList.size() == 0) {
            return ResultRespone.failed("算费失败：当前所选的抄表册不存在需算费的用户！");
        }
        int len = hashMapList.size();
        String[] arrList = new String[len];
        for (int i = 0; i < len; i++) {
            arrList[i] = (String) hashMapList.get(i).get("METER_CARD_ID");
        }
        return getResultResponse(arrList);
    }

    /**
     * 算费相关处理
     *
     * @param arrCardIds 抄表卡ID集数组
     * @return
     * @throws Exception
     */
    private ResultRespone<Object> getResultResponse(String[] arrCardIds) throws Exception {
        int maxCalcNum = Integer.parseInt(SysConfigUtils.getConfigValue("CBJSMaxCount"));
        if (arrCardIds.length > maxCalcNum) {
            return ResultRespone.failed(MessageFormat.format("算费失败：选择的【{0}】条抄表记录过多,每次算费最多不超过【{1}】条记录,请分批计算费用！", arrCardIds.length, maxCalcNum));
        }
        DbHelper db = DbHelper.getDbHelper();
        //先进行总表水量计算
        List<ProcParamModel> listZB = new ArrayList<>();
        listZB.add(new ProcParamModel("P_CBKID", ParameterMode.IN, JDBCType.INTEGER, null));
        listZB.add(new ProcParamModel("P_SFCG", ParameterMode.OUT, JDBCType.INTEGER, 0));
        listZB.add(new ProcParamModel("P_MSG", ParameterMode.OUT, JDBCType.VARCHAR, ""));
        Map mapZB = db.executeProcedure("PROC_SLJS_ZB", listZB);

        //存储过程返回的参数
        if (!(mapZB != null && mapZB.get("P_SFCG").equals(1))) {
            return ResultRespone.failed(MessageFormat.format("算费失败：总表水量计算异常【异常消息：{0}】，已终止！", mapZB.get("P_MSG")));
        }
        //声明队列
        ConcurrentLinkedQueue<String> queue = new ConcurrentLinkedQueue<>();
        for (String meterCardId : arrCardIds) {
            queue.add(meterCardId);
        }
        //开始时间
        Date dtBegin = new Date();
        //运行队列
        int intSuccessNum = doRunDataQueue(db, queue);
        //结束时间
        Date dtEnd = new Date();
        //获取时间运行的秒数
        double secNum = DateUtils.getSecond(dtBegin, dtEnd);
        //保留2位小数
        String strSecNum = String.format("%.2f", secNum);

        String strMsg = "";
        if (intSuccessNum == 0) {
            strMsg = MessageFormat.format("算费失败：已算费0条抄表记录，共耗时{0}秒！", strSecNum);
        } else {
            strMsg = MessageFormat.format("算费成功：成功算费{0}个用户，共耗时{1}秒！", intSuccessNum, strSecNum);
        }
        LogHelper.excuteCustom(BwMeterReadPlan.class, "抄表算费：按选中抄表册", strMsg);
        return (intSuccessNum == 0) ? ResultRespone.failed(false, strMsg) : ResultRespone.success(true, strMsg);
    }

    /**
     * 运行数据队列
     *
     * @param db    数据处理对象
     * @param queue 队列对象
     * @return 运行的成功数
     */
    private int doRunDataQueue(DbHelper db, ConcurrentLinkedQueue<String> queue) {
        //成功数量
        int intReturnNum = 0;
        while (!queue.isEmpty()) {
            String meterCId = queue.poll();
            //调用水量算费存储过程
            List<ProcParamModel> listSLSF = new ArrayList<>();
            listSLSF.add(new ProcParamModel("P_CBKID", ParameterMode.IN, JDBCType.INTEGER, meterCId));
            listSLSF.add(new ProcParamModel("P_SFCG", ParameterMode.OUT, JDBCType.INTEGER, 0));
            listSLSF.add(new ProcParamModel("P_MSG", ParameterMode.OUT, JDBCType.VARCHAR, ""));
            Map mapSLSF = db.executeProcedure("PROC_SLSFJS", listSLSF);
            //判断返回结果
            if (mapSLSF != null && mapSLSF.get("P_SFCG").equals(1)) {
                intReturnNum++;
            }
            //移除已处理的子项
            queue.remove(meterCId);
        }
        return intReturnNum;
    }




}
