/*
 * Author: wangjian
 * 生成mapper
 */
package com.tofly.fees.meterreadingmgt.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.fees.meterreadingmgt.entity.BwMeterReadPlan;
import org.apache.ibatis.annotations.MapKey;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 抄表计划_抄表录入
 *
 * @author xb
 * @date Thu Jun 01 00:00:00 CST 2023
 */
@Repository
public interface BwMeterReadPlanMapper extends BaseMapper<BwMeterReadPlan> {

    /**
     * 抄表录入列表数据
     */
    @MapKey("meter_card_id")
    Page<Map<String, Object>> getMeterReadList(Page page, String companyBranch, String mrArea
            , String mrBook, String mrOCode, String mrMode, String mrFlag, String calFlag, String searchText);

    /**
     * 抄表算费：查询抄表册选择列表数据
     *
     * @param page          分页对象
     * @param companyBranch 公司编码值
     * @param mrArea        抄表片区值
     * @return
     */
    @MapKey("cbbbm")
    Page<Map<String, Object>> getMrBookList(Page page, String companyBranch, String mrArea);

    /**
     * 抄表算费：获取可算费的用户明细数据
     *
     * @param page          分页对象
     * @param companyBranch 公司编码值
     * @param mrBooks       抄表本集合
     * @param searchText    关键字（用户编号、用户姓名、地址）
     * @return
     */
    @MapKey("meter_card_id")
    Page<Map<String, Object>> getUserList(Page page, String companyBranch, String mrBooks, String searchText);

    /**
     * 通过抄表本集合查询已抄表待算费的抄表卡ID集
     *
     * @param mrBooks 抄表本集合
     * @return
     */
    @MapKey("meter_card_id")
    List<HashMap<String, Object>> getMeterCardIds(String mrBooks);

    /**
     * 抄表册批量设置：公用数据的用户列表
     *
     * @param page          分页对象
     * @param companyBranch 公司编码值
     * @param mrArea        抄表片区值
     * @param mrBook       抄表本集合
     * @param searchText    关键字（用户姓名、地址、用户编号）
     * @param strType       操作类型
     * @return
     */
    @MapKey("customer_no")
    Page<Map<String, Object>> getMrBookUserList(Page page, String companyBranch, String mrArea, String mrBook, String searchText, String strType);

    int delByNo(String no);

    /**
     * @MethodName cancelMrBook
     * @Description 撤销分配
     * @Author xb
     * @Date 13:46 2023-06-12
     */
    int cancelMeterReadPlan(String customerNos);


    /**
     * @MethodName cancelMrBook
     * @Description 撤销分配
     * @Author xb
     * @Date 13:46 2023-06-12
     */
    int cancelDaCustomerReadPlan(String customerNos);


    /**
     * @MethodName toMeterReadPlan
     * @Description 分配
     * @Author xb
     * @Date 16:36 2023-06-12
     */
    int toMeterReadPlan(String customerNos, Integer mrBook);

    /**
     * @MethodName toDaCustomerReadPlan
     * @Description 分配
     * @Author xb
     * @Date 16:37 2023-06-12
     */
    int toDaCustomerReadPlan(String customerNos, Integer mrBook);


}
