package com.tofly.fees.reportmgt.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.fees.common.dbhelper.PageSortHelper;
import com.tofly.fees.common.model.CollectionData;
import com.tofly.fees.feesmgt.entity.SfCharge;
import com.tofly.fees.feesmgt.mapper.SfChargeMapper;
import com.tofly.fees.financialmgt.entity.SfRec;
import com.tofly.fees.financialmgt.mapper.SfRecMapper;
import com.tofly.fees.meterprofilemgt.entity.DaMeter;
import com.tofly.fees.meterprofilemgt.mapper.DaMeterMapper;
import com.tofly.fees.userprofilemgt.entity.DaCustomer;
import com.tofly.fees.userprofilemgt.mapper.DaCustomerMapper;
import com.tofly.fees.userprofilemgt.service.DaCustomerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RestController
@AllArgsConstructor
@RequestMapping("/api/reportmgt/generalenquiries")
@Api(tags="查询统计-用户综合查询接口")
public class ConsumerGeneralSearchController {
    private final DaCustomerService daCustomerService;
    private final  DaMeterMapper daMeterMapper;
    private final   SfRecMapper sfRecMapper;
    private final   SfChargeMapper sfChargeMapper;
    private final DaCustomerMapper daCustomerMapper;
    /**
     * 通过用户编号查询用户基本信息
     * @param customerNo id
     * @return ResultRespone
     */
    @GetMapping("/dacustomer/{customerNo}" )
    @ApiOperation(value = "综合查询-用户编号查询(基础档案)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "customerNo", value = "用户编号", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("customerNo" ) String customerNo) {
        if(StringUtils.isEmpty(customerNo))
        {
            return   ResultRespone.failed("查询失败，编号不能为空");
        }
        int count=daCustomerMapper.selectExistCustomerNo(customerNo);
        if(count==0)
        {
            return   ResultRespone.failed("查询失败，该用户不存在");
        }
        DaCustomer daCustomer=daCustomerService.getById(customerNo);
        if(StringUtils.isEmpty(daCustomer.getCustomerNo()))
        {
            return   ResultRespone.failed("查询失败");
        }
        return ResultRespone.success(daCustomer);
    }

    @GetMapping("/dameter/{customerNo}/{meterCardId}" )
    @ApiOperation(value = "综合查询-用户编号、ID(水表基础)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "customerNo", value = "用户编号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "meterCardId", value = "ID号meterCardId", required = true, dataType = "String")
    })
    public ResultRespone getDaMeter(@PathVariable("customerNo" ) String customerNo,@PathVariable("meterCardId" ) String meterCardId) {
        if(StringUtils.isEmpty(customerNo)||StringUtils.isEmpty(meterCardId))
        {
          return   ResultRespone.failed("查询失败，用户编号或表卡ID不能为空");
        }
        DaMeter daMeter=daMeterMapper.SelectDaMeter(customerNo,meterCardId);
        if(StringUtils.isEmpty(daMeter.getCustomerNo()))
        {
            return   ResultRespone.failed("查询失败，无相关水表信息");
        }
        return ResultRespone.success(daMeter);
    }


    /**
     * 分页查询 sfRec 应收明细：欠费明细
     * @param page 分页对象
     * @param
     * @return 11
     */
    @GetMapping("/qfrec/page" )
    @ApiOperation(value = "用户综合-分页查询(用户欠费)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "current", value = "当前页码", required = true, dataType = "int"),
            @ApiImplicitParam(name = "size", value = "每页显示数据条数", required = true, dataType = "int"),
            @ApiImplicitParam(name = "pages", value = "共几页,输出参数",  dataType = "int"),
            @ApiImplicitParam(name = "maxLimit", value = "每页最大显示数据",  dataType = "int"),
            @ApiImplicitParam(name = "orders[0].asc", value = "排序,bool类型",  dataType = "String"),
            @ApiImplicitParam(name = "orders[0].column", value = "排序,字段名", dataType = "String"),
            @ApiImplicitParam(name = "customerNo", value = "用户编号", required =true, dataType = "String")
    })
    public ResultRespone getSfRecPage(Page page,  String customerNo) {
        IPage<SfRec> sfRecIpage=sfRecMapper.SelectQfmxPage(page, "", "", "","" ,"","",customerNo,"","0");
        return ResultRespone.success(sfRecIpage);
    }

    @GetMapping("/cbrecord/page" )
    @ApiOperation(value = "用户综合-分页查询(抄表记录)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "current", value = "当前页码", required = true, dataType = "int"),
            @ApiImplicitParam(name = "size", value = "每页显示数据条数", required = true, dataType = "int"),
            @ApiImplicitParam(name = "pages", value = "共几页,输出参数",  dataType = "int"),
            @ApiImplicitParam(name = "maxLimit", value = "每页最大显示数据",  dataType = "int"),
            @ApiImplicitParam(name = "orders[0].asc", value = "排序,bool类型",  dataType = "String"),
            @ApiImplicitParam(name = "orders[0].column", value = "排序,字段名", dataType = "String"),
            @ApiImplicitParam(name = "customerNo", value = "用户编号", required =true, dataType = "String")
    })
    public ResultRespone getcbRecordPage(Page page,  String customerNo) {
        if(StringUtils.isEmpty(customerNo))
        {
            return  ResultRespone.failed("查询失败，编号不能为空");
        }
        int count=daCustomerMapper.selectExistCustomerNo(customerNo);
        if(count==0)
        {
            return  ResultRespone.failed("查询失败，该用户不存在");
        }
        IPage<Map<String, Object>> listPage=sfRecMapper.selectYhbhQfmx(page,customerNo);
        return ResultRespone.success(listPage);
    }

    /**
     * 分页查询 实收明细：实收明细
     * @param page 分页对象
     * @param
     * @return 11
     */
    @GetMapping("/sfrec/page" )
    @ApiOperation(value = "用户综合-分页查询(已缴费)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "current", value = "当前页码", required = true, dataType = "int"),
            @ApiImplicitParam(name = "size", value = "每页显示数据条数", required = true, dataType = "int"),
            @ApiImplicitParam(name = "pages", value = "共几页,输出参数",  dataType = "int"),
            @ApiImplicitParam(name = "maxLimit", value = "每页最大显示数据",  dataType = "int"),
            @ApiImplicitParam(name = "orders[0].asc", value = "排序,bool类型", dataType = "String"),
            @ApiImplicitParam(name = "orders[0].column", value = "排序,字段名",  dataType = "String"),
            @ApiImplicitParam(name = "customerNo", value = "用户编号", required = true, dataType = "String")
    })
    public ResultRespone getSfChargePage(Page page,String customerNo) {
        if(StringUtils.isEmpty(customerNo))
        {
            return  ResultRespone.failed("查询失败，用户编号不能为空");
        }
        PageSortHelper.setPageSort(page,null);//解决排序问题
        IPage<Map<String,Object>>  listpage=sfChargeMapper.selectYhbhSfData(page,customerNo);

        return ResultRespone.success(listpage);
    }

    /**
     * 分页查询  实收明细：收费日报表明细-预付
     * @param page 分页对象
     * @param
     * @return 0
     */
    @GetMapping("/yffrec/page" )
    @ApiOperation(value = "用户综合-分页查询(预付记录)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "current", value = "当前页码", required = true, dataType = "int"),
            @ApiImplicitParam(name = "size", value = "每页显示数据条数", required = true, dataType = "int"),
            @ApiImplicitParam(name = "pages", value = "共几页,输出参数",  dataType = "int"),
            @ApiImplicitParam(name = "maxLimit", value = "每页最大显示数据", dataType = "int"),
            @ApiImplicitParam(name = "orders[0].asc", value = "排序,bool类型",  dataType = "String"),
            @ApiImplicitParam(name = "orders[0].column", value = "排序,字段名",  dataType = "String"),
            @ApiImplicitParam(name = "customerNo", value = "用户编号",required = true, dataType = "String")
    })
    public ResultRespone getSfDaiyReportPage(Page page, String customerNo,String chargeType) {

        IPage<SfCharge>  sfpage= sfChargeMapper.selectSfmxSearch(page,"","","","",customerNo,"","","3","","","","");
        return ResultRespone.success(sfpage);
    }

}



