/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.userprofilemgt.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.common.dbhelper.PageSortHelper;
import com.tofly.fees.common.util.ToolsUtils;
import com.tofly.fees.userprofilemgt.entity.DaCustomerFfgx;
import com.tofly.fees.userprofilemgt.service.DaCustomerFfgxService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;

/**
 * 付费关系
 *
 * @author admin
 * @date Thu Jul 06 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/userprofilemgt/dacustomerffgx")
@Api(tags = "客户档案-付费关系接口")
public class DaCustomerFfgxController {

    private final DaCustomerFfgxService daCustomerFfgxService;

    /**
     * 分页查询
     *
     * @param page           分页对象
     * @param daCustomerFfgx 付费关系
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone getDaCustomerFfgxPage(Page page, @ModelAttribute DaCustomerFfgx daCustomerFfgx) {
        PageSortHelper.setPageSort(DaCustomerFfgx.class, page);
        return ResultRespone.success(daCustomerFfgxService.page(page, Wrappers.query(daCustomerFfgx)));
    }


    /**
     * 通过id查询付费关系
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("id") String id) {
        return ResultRespone.success(daCustomerFfgxService.getById(id));
    }

    /**
     * 新增付费关系
     *
     * @param daCustomerFfgx 付费关系
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增付费关系")
    @ApiOperation(value = "新增付费关系")
    @PostMapping
    public ResultRespone save(@RequestBody DaCustomerFfgx daCustomerFfgx) {
        daCustomerFfgx.setId(ToolsUtils.getGuid());
        return ResultRespone.success(daCustomerFfgxService.save(daCustomerFfgx));
    }

    /**
     * 修改付费关系
     *
     * @param daCustomerFfgx 付费关系
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改付费关系")
    @ApiOperation(value = "修改付费关系")
    @PutMapping
    public ResultRespone updateById(@RequestBody DaCustomerFfgx daCustomerFfgx) {
        return ResultRespone.success(daCustomerFfgxService.updateById(daCustomerFfgx));
    }

    /**
     * 通过id删除付费关系
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除付费关系")
    @ApiOperation(value = "通过ID删除付费关系")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable String id) {
        return ResultRespone.success(daCustomerFfgxService.removeById(id));
    }

    /**
     * 通过id批量删除付费关系
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除付费关系")
    @ApiOperation(value = "通过ID批量删除付费关系")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(daCustomerFfgxService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
