/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.userprofilemgt.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.userprofilemgt.entity.DaCustomerLog;
import com.tofly.fees.userprofilemgt.service.DaCustomerLogService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;

import java.util.Arrays;
/**
 * 用户基本信息日志
 *
 * @author sxb
 * @date Tue May 23 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/userprofilemgt/dacustomerlog")
@Api(tags="客户档案-用户基本信息日志接口")
public class DaCustomerLogController {

    private final  DaCustomerLogService daCustomerLogService;

    /**
    * 分页查询 用户基本信息日志
    * @param page 分页对象
    * @param
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "logCdate_Start", value = "操作开始日期",  dataType = "String"),
            @ApiImplicitParam(name = "logCdate_End", value = "操作结束日期",  dataType = "String"),
            @ApiImplicitParam(name = "customerNo", value = "用户编号", dataType = "String"),
    })
    public ResultRespone getDaCustomerLogPage(Page page, String logCdate_Start, String logCdate_End ,String customerNo) {

        QueryWrapper<DaCustomerLog> queryWrapper=new QueryWrapper<>();
        queryWrapper.eq("1","1");
        if(StringUtils.isNotBlank(customerNo)) {
            queryWrapper.and(i->i.eq("CUSTOMER_NO",customerNo));
        }
        if(StringUtils.isNotBlank(logCdate_Start) && StringUtils.isNotBlank(logCdate_End)) {
            queryWrapper.and(i->i.apply("LOG_CDATE BETWEEN to_date({0},'yyyy-mm-dd hh24:mi:ss') AND to_date({1},'yyyy-mm-dd hh24:mi:ss')", logCdate_Start, logCdate_End));
        }
        IPage<DaCustomerLog> daCustomerLogIPage=daCustomerLogService.page(page,queryWrapper);
      return ResultRespone.success(daCustomerLogIPage);
    }


    /**
     * 通过id查询用户基本信息日志
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("id" ) String id) {
      return ResultRespone.success(daCustomerLogService.getById(id));
    }

    /**
     * 新增用户基本信息日志
     * @param daCustomerLog 用户基本信息日志
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增用户基本信息日志" )
    @ApiOperation(value = "新增用户基本信息日志")
    @PostMapping
    public ResultRespone save(@RequestBody DaCustomerLog daCustomerLog) {
      return ResultRespone.success(daCustomerLogService.save(daCustomerLog));
    }

    /**
     * 修改用户基本信息日志
     * @param daCustomerLog 用户基本信息日志
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改用户基本信息日志" )
    @ApiOperation(value = "修改用户基本信息日志")
    @PutMapping
    public ResultRespone updateById(@RequestBody DaCustomerLog daCustomerLog) {
      return ResultRespone.success(daCustomerLogService.updateById(daCustomerLog));
    }

    /**
     * 通过id删除用户基本信息日志
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除用户基本信息日志" )
    @ApiOperation(value = "通过ID删除用户基本信息日志")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable String id) {
      return ResultRespone.success(daCustomerLogService.removeById(id));
    }

    /**
     * 通过id批量删除用户基本信息日志
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除用户基本信息日志" )
    @ApiOperation(value = "通过ID批量删除用户基本信息日志")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(daCustomerLogService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
