/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.userprofilemgt.controller_easy;

import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.common.dbhelper.DbHelper;
import com.tofly.fees.common.util.DateUtils;
import com.tofly.fees.common.util.SqlConvertor;
import com.tofly.fees.userprofilemgt.service.BwMeterZfbService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.text.MessageFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.Map;

/**
 * 总分表关系
 *
 * @author admin
 * @date Tue Jul 04 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/userprofilemgt/bwmeterzfb")
@Api(tags = "客户档案-总分表关系接口")
public class BwMeterZfbEasyController {

    /**
     * 新增总分表关系
     *
     * @param data 请求参数
     * @return
     */
    @ToFlyAppLog(title = "新增总分表关系(批量)")
    @ApiOperation(value = "新增总分表关系(批量)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "customerNoZb", value = "用户编号-总表", required = true, dataType = "String"),
            @ApiImplicitParam(name = "customerNoFbList", value = "用户编号-分表（逗号分割列表）", required = true, dataType = "String"),
            @ApiImplicitParam(name = "operatorCode", value = "操作人员编码", required = true, dataType = "String"),
            @ApiImplicitParam(name = "operatorName", value = "操作人员姓名", required = true, dataType = "String"),
            @ApiImplicitParam(name = "operatorDatetime", value = "操作时间", required = true, dataType = "String(date-time)")
    })
    @PostMapping("/saveBatch")
    public ResultRespone saveBatch(@RequestBody Map<String, Object> data) throws Exception {
        //region 参数解析
        String customerNoZb = "";
        String customerNoFbList = "";
        String operatorCode = "";
        String operatorName = "";
        String operatorDatetime = "";
        try {
            customerNoZb = data.get("customerNoZb").toString();
            customerNoFbList = data.get("customerNoFbList").toString();
            if (data.get("operatorCode") != null) operatorCode = data.get("operatorCode").toString();
            if (data.get("operatorName") != null) operatorName = data.get("operatorName").toString();
            if (data.get("operatorDatetime") != null) {
                operatorDatetime = data.get("operatorDatetime").toString();
            } else {
                operatorDatetime = DateUtils.formatDate_pattern(new Date(), "yyyy-MM-dd HH:mm:ss");
            }
        } catch (Exception e) {
            e.printStackTrace();
            return ResultRespone.failed(e.getMessage());
        }
        //endregion

        //region 业务处理
        //验证
        if (StringUtils.isBlank(customerNoZb)) {
            return ResultRespone.failed("总表用户编号不能为空！");
        }
        if (StringUtils.isBlank(customerNoFbList)) {
            return ResultRespone.failed("分表用户编号数量不能为0！");
        }
        if (Arrays.asList(customerNoFbList.split(",")).contains(customerNoZb)) {
            return ResultRespone.failed("用户编号错误：选择的分表用户中包含了总表用户！");
        }
        //处理
        DbHelper db = DbHelper.getDbHelper();
        String strSql = MessageFormat.format("delete from TF_YWYS_BW_METER_ZFB where customer_no_fb in(''{0}'')", customerNoFbList.replace(",", "','"));
        db.addTransactionSql(strSql);
        strSql = "insert into tf_ywys_bw_meter_zfb select sys_guid(),customer_no_zb,customer_no_fb,meter_card_id_zb,meter_card_id_fb,'{2}','{3}',{4} from (\n" +
                "  select '1' k, customer_no as customer_no_zb,meter_card_id as meter_card_id_zb from tf_ywys_da_meter where customer_no='{0}' and meter_card_no=1\n" +
                ") zb inner join (\n" +
                "  select '1' k, customer_no as customer_no_fb,meter_card_id as meter_card_id_fb from tf_ywys_da_meter where customer_no in('{1}')\n" +
                ") fb on(zb.k=fb.k)";
        strSql = MessageFormat.format(strSql.replace("'", "''"), customerNoZb, customerNoFbList.replace(",", "','"), operatorCode, operatorName, SqlConvertor.toDate(operatorDatetime));
        db.addTransactionSql(strSql);

        boolean result = db.executeTransaction();
        if (!result) {
            return ResultRespone.failed("数据处理失败！");
        }
        //endregion

        //region 成功返回
        return ResultRespone.success(result);
        //endregion
    }
}
