/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.wechatbasicmgt.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.common.util.GuidGenerator;
import com.tofly.fees.wechatbasicmgt.entity.WxBusiness;
import com.tofly.fees.wechatbasicmgt.entity.WxBusinessCode;
import com.tofly.fees.wechatbasicmgt.entity.WxDtbm;
import com.tofly.fees.wechatbasicmgt.service.WxBusinessCodeService;
import com.tofly.fees.wechatbasicmgt.service.WxBusinessService;
import com.tofly.fees.wechatofficalacctmgt.entity.WxGsdt;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 业务须知
 *
 * @author admin
 * @date Wed Jul 05 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/wechatbasicmgt/wxbusiness")
@Api(tags="微信管理—业务须知接口")
public class WxBusinessController {

    private final  WxBusinessService wxBusinessService;

    private final  WxBusinessCodeService wxBusinessCodeService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "current", value = "当前页码", required = true, dataType = "int"),
            @ApiImplicitParam(name = "size", value = "每页显示数据条数", required = true, dataType = "int"),
            @ApiImplicitParam(name = "pages", value = "共几页,输出参数",  dataType = "int"),
            @ApiImplicitParam(name = "maxLimit", value = "每页最大显示数据",  dataType = "int"),
            @ApiImplicitParam(name = "orders[0].asc", value = "排序,bool类型",  dataType = "String"),
            @ApiImplicitParam(name = "orders[0].column", value = "排序,字段名",  dataType = "String"),
            @ApiImplicitParam(name = "type", value = "类型",  dataType = "String"),
            @ApiImplicitParam(name = "searchText", value = "模糊查询(标题)",  dataType = "String")
    })
    public ResultRespone getWxBusinessPage(Page page , String searchText,String type) {
        QueryWrapper<WxBusiness> queryWrapper = new QueryWrapper<WxBusiness>();
        queryWrapper.eq("isdisabled","1");
        if(StringUtils.isNotBlank(searchText))
        {
            queryWrapper.and(i->i.like("title",searchText));
        }
        if(StringUtils.isNotBlank(type) && type!="all")
        {
            queryWrapper.in("type",type);
        }
        Page<WxBusiness> wxPage=wxBusinessService.page(page, queryWrapper);

        List<WxBusiness> wxList = wxPage.getRecords();

        for (WxBusiness wxBusiness : wxList) {
            //动态编码
            List<WxBusinessCode> wxBusinessCodeList= wxBusinessCodeService.list();
            Map<String,String> map = new HashMap<>();
            wxBusinessCodeList.forEach(e->map.put(e.getId().toString(), e.getName()));
            //类型
            wxBusiness.setTypeName(map.get(wxBusiness.getType().toString()));
        }
        return ResultRespone.success(wxPage);
    }


    /**
     * 通过id查询业务须知
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("id" ) String id) {
      return ResultRespone.success(wxBusinessService.getById(id));
    }

    /**
     * 新增业务须知
     * @param wxBusiness 业务须知
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增业务须知" )
    @ApiOperation(value = "新增业务须知")
    @PostMapping
    public ResultRespone save(@RequestBody WxBusiness wxBusiness) {
        wxBusiness.setId(GuidGenerator.getGuid());
      return ResultRespone.success(wxBusinessService.save(wxBusiness));
    }

    /**
     * 修改业务须知
     * @param wxBusiness 业务须知
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改业务须知" )
    @ApiOperation(value = "修改业务须知")
    @PutMapping
    public ResultRespone updateById(@RequestBody WxBusiness wxBusiness) {
      return ResultRespone.success(wxBusinessService.updateById(wxBusiness));
    }

    /**
     * 通过id删除业务须知
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除业务须知" )
    @ApiOperation(value = "通过ID删除业务须知")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable String id) {
      return ResultRespone.success(wxBusinessService.removeById(id));
    }

    /**
     * 通过id批量删除业务须知
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除业务须知" )
    @ApiOperation(value = "通过ID批量删除业务须知")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(wxBusinessService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
