/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.wechatbasicmgt.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.wechatbasicmgt.entity.WxCustomer;
import com.tofly.fees.wechatbasicmgt.service.WxCustomerService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;

import java.util.Arrays;
/**
 * 微信解绑
 *
 * @author sxb
 * @date Mon Jun 05 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/wechatbasicmgt/wxcustomer")
@Api(tags="微信管理-微信解绑接口")
public class WxCustomerController {

    private final  WxCustomerService wxCustomerService;

    /**
    * 分页查询 wxCustomer 微信解绑
    * @param page 分页对象
    * @param
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "current", value = "当前页码", required = true, dataType = "int"),
            @ApiImplicitParam(name = "size", value = "每页显示数据条数", required = true, dataType = "int"),
            @ApiImplicitParam(name = "pages", value = "共几页,输出参数",  dataType = "int"),
            @ApiImplicitParam(name = "maxLimit", value = "每页最大显示数据",  dataType = "int"),
            @ApiImplicitParam(name = "orders[0].asc", value = "排序,bool类型", dataType = "String"),
            @ApiImplicitParam(name = "orders[0].column", value = "排序,字段名",  dataType = "String"),
            @ApiImplicitParam(name = "bindFlag", value = "是否绑定，0,1",  dataType = "String"),
            @ApiImplicitParam(name = "searchText", value = "模糊查询(用户编号、用户姓名、用户地址)", required = false, dataType = "String")
    })
    public ResultRespone getWxCustomerPage(Page page,String searchText,String bindFlag) {
        QueryWrapper<WxCustomer> queryWrapper = new QueryWrapper<WxCustomer>();
        queryWrapper.eq("1","1");
        if(StringUtils.isNotBlank(searchText))
        {
            queryWrapper.and(i->i.like("customer_no",searchText).or().like("name",searchText).or().like("address",searchText));
        }
        if(StringUtils.isNotBlank(bindFlag) && bindFlag!="all")
        {
            queryWrapper.in("bind_flag",bindFlag);
        }
        return ResultRespone.success(wxCustomerService.page(page, queryWrapper));
    }


    /**
     * 通过id查询微信解绑
     * @param customerNo id
     * @return ResultRespone
     */
    @GetMapping("/{customerNo}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "customerNo", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("customerNo" ) String customerNo) {
      return ResultRespone.success(wxCustomerService.getById(customerNo));
    }

    /**
     * 新增微信解绑
     * @param wxCustomer 微信解绑
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增微信解绑" )
    @ApiOperation(value = "新增微信解绑")
    @PostMapping
    public ResultRespone save(@RequestBody WxCustomer wxCustomer) {
      return ResultRespone.success(wxCustomerService.save(wxCustomer));
    }

    /**
     * 修改微信解绑
     * @param wxCustomer 微信解绑
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改微信解绑" )
    @ApiOperation(value = "修改微信解绑")
    @PutMapping
    public ResultRespone updateById(@RequestBody WxCustomer wxCustomer) {
      return ResultRespone.success(wxCustomerService.updateById(wxCustomer));
    }

    /**
     * 通过id删除微信解绑
     * @param customerNo id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除微信解绑" )
    @ApiOperation(value = "通过ID删除微信解绑")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "customerNo", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{customerNo}" )
    public ResultRespone removeById(@PathVariable String customerNo) {
      return ResultRespone.success(wxCustomerService.removeById(customerNo));
    }

    /**
     * 通过id批量删除微信解绑
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除微信解绑" )
    @ApiOperation(value = "通过ID批量删除微信解绑")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(wxCustomerService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
