/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.wechatbasicmgt.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.wechatbasicmgt.entity.WxCustomerHis;
import com.tofly.fees.wechatbasicmgt.mapper.WxCustomerHisMapper;
import com.tofly.fees.wechatbasicmgt.service.WxCustomerHisService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;

import java.util.Arrays;
import java.util.Map;

/**
 * 微信解绑记录
 *
 * @author sxb
 * @date Mon Jun 05 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/wechatbasicmgt/wxcustomerhis")
@Api(tags="微信管理-微信解绑记录接口")
public class WxCustomerHisController {

    private final  WxCustomerHisService wxCustomerHisService;
    private final WxCustomerHisMapper wxCustomerHisMapper;

    /**
    * 分页查询 wxCustomerHis 微信解绑记录
    * @param page 分页对象
    * @param
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "current", value = "当前页码", required = true, dataType = "int"),
            @ApiImplicitParam(name = "size", value = "每页显示数据条数", required = true, dataType = "int"),
            @ApiImplicitParam(name = "pages", value = "共几页,输出参数",  dataType = "int"),
            @ApiImplicitParam(name = "maxLimit", value = "每页最大显示数据",  dataType = "int"),
            @ApiImplicitParam(name = "orders[0].asc", value = "排序,bool类型",  dataType = "String"),
            @ApiImplicitParam(name = "orders[0].column", value = "排序,字段名",  dataType = "String"),
           @ApiImplicitParam(name = "bindFlag", value = "是否绑定，0,1",  dataType = "String"),
           @ApiImplicitParam(name = "searchText", value = "模糊查询(用户编号、用户姓名、用户地址)", required = false, dataType = "String")
    })
    public ResultRespone getWxCustomerHisPage(Page page, String searchText,String bindFlag) {
        QueryWrapper<WxCustomerHis> queryWrapper = new QueryWrapper<WxCustomerHis>();
        queryWrapper.eq("1","1");
        if(StringUtils.isNotBlank(searchText))
        {
            queryWrapper.and(i->i.like("customer_no",searchText).or().like("name",searchText).or().like("address",searchText));
        }
        if(StringUtils.isNotBlank(bindFlag) && bindFlag!="all")
        {
            queryWrapper.in("bind_flag",bindFlag);
        }
        return ResultRespone.success(wxCustomerHisService.page(page, queryWrapper));
    }


    /**
     * 通过id查询微信解绑记录
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    public ResultRespone getById(@PathVariable("id" ) Integer id) {
      return ResultRespone.success(wxCustomerHisService.getById(id));
    }

    /**
     * 新增微信解绑记录
     * @param wxCustomerHis 微信解绑记录
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增微信解绑记录" )
    @ApiOperation(value = "新增微信解绑记录")
    @PostMapping
    public ResultRespone save(@RequestBody WxCustomerHis wxCustomerHis) {
        int id=  wxCustomerHisMapper.SelectChargeMaxCode();
        wxCustomerHis.setId(id+1);
      return ResultRespone.success(wxCustomerHisService.save(wxCustomerHis));
    }

    /**
     * 修改微信解绑记录
     * @param wxCustomerHis 微信解绑记录
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改微信解绑记录" )
    @ApiOperation(value = "修改微信解绑记录")
    @PutMapping
    public ResultRespone updateById(@RequestBody WxCustomerHis wxCustomerHis) {
      return ResultRespone.success(wxCustomerHisService.updateById(wxCustomerHis));
    }

    /**
     * 通过id删除微信解绑记录
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除微信解绑记录" )
    @ApiOperation(value = "通过ID删除微信解绑记录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Integer id) {
      return ResultRespone.success(wxCustomerHisService.removeById(id));
    }

    /**
     * 通过id批量删除微信解绑记录
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除微信解绑记录" )
    @ApiOperation(value = "通过ID批量删除微信解绑记录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(wxCustomerHisService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
