/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.wechatbasicmgt.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.wechatbasicmgt.entity.WxFwNotice;
import com.tofly.fees.wechatbasicmgt.mapper.WxFwNoticeMapper;
import com.tofly.fees.wechatbasicmgt.service.WxFwNoticeService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;

import java.util.Arrays;
import java.util.Map;

/**
 * 微信推送服务通知
 *
 * @author sxb
 * @date Mon Jun 05 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/wechatbasicmgt/wxfwnotice")
@Api(tags="微信管理-微信推送服务通知接口")
public class WxFwNoticeController {

    private final  WxFwNoticeService wxFwNoticeService;
    private final   WxFwNoticeMapper wxFwNoticeMapper;

    /**
    * 分页查询 微信推送服务通知
    * @param page 分页对象
    * @param
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "current", value = "当前页码", required = true, dataType = "int"),
            @ApiImplicitParam(name = "size", value = "每页显示数据条数", required = true, dataType = "int"),
            @ApiImplicitParam(name = "pages", value = "共几页,输出参数",  dataType = "int"),
            @ApiImplicitParam(name = "maxLimit", value = "每页最大显示数据",  dataType = "int"),
            @ApiImplicitParam(name = "orders[0].asc", value = "排序,bool类型", dataType = "String"),
            @ApiImplicitParam(name = "orders[0].column", value = "排序,字段名",  dataType = "String"),
            @ApiImplicitParam(name = "state", value = "是否发送，0,1",  dataType = "String"),
            @ApiImplicitParam(name = "editDateStart", value = "开始日期",  dataType = "String"),
            @ApiImplicitParam(name = "editDateEnd", value = "结束日期",  dataType = "String"),
            @ApiImplicitParam(name = "searchText", value = "模糊查询(编辑人员、标题、停水区域)",  dataType = "String")
    })
    public ResultRespone getWxFwNoticePage(Page page, String searchText,String state,String editDateStart,String editDateEnd) {
        IPage<WxFwNotice> WxTsPage=wxFwNoticeMapper.selectWxFwNoticePage(page,searchText,state,editDateStart,editDateEnd);
        return ResultRespone.success(WxTsPage);
    }


    /**
     * 通过id查询微信推送服务通知
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    public ResultRespone getById(@PathVariable("id" ) Integer id) {
      return ResultRespone.success(wxFwNoticeService.getById(id));
    }

    /**
     * 新增微信推送服务通知
     * @param wxFwNotice 微信推送服务通知
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增微信推送服务通知" )
    @ApiOperation(value = "新增微信推送服务通知")
    @PostMapping
    public ResultRespone save(@RequestBody WxFwNotice wxFwNotice) {
        int id=  wxFwNoticeMapper.SelectChargeMaxCode();
        wxFwNotice.setId(id+1);
        wxFwNotice.setState("0");//发送状态
        wxFwNotice.setAuditFlag("0");//审核标志
      return ResultRespone.success(wxFwNoticeService.save(wxFwNotice));
    }

    /**
     * 修改微信推送服务通知
     * @param wxFwNotice 微信推送服务通知
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改微信推送服务通知" )
    @ApiOperation(value = "修改微信推送服务通知")
    @PutMapping
    public ResultRespone updateById(@RequestBody WxFwNotice wxFwNotice) {
      return ResultRespone.success(wxFwNoticeService.updateById(wxFwNotice));
    }

    /**
     * 通过id删除微信推送服务通知
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除微信推送服务通知" )
    @ApiOperation(value = "通过ID删除微信推送服务通知")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Integer id) {
      return ResultRespone.success(wxFwNoticeService.removeById(id));
    }

    /**
     * 通过id批量删除微信推送服务通知
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除微信推送服务通知" )
    @ApiOperation(value = "通过ID批量删除微信推送服务通知")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(wxFwNoticeService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
