/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.wechatofficalacctmgt.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.wechatofficalacctmgt.entity.WxBzsq;
import com.tofly.fees.wechatofficalacctmgt.service.WxBzsqService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;

/**
 * 微信-报装申请
 *
 * @author admin
 * @date Mon Jun 12 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/wechatofficalacctmgt/wxbzsq")
@Api(tags = "微信公众号-报装申请接口")
public class WxBzsqController {

    private final WxBzsqService wxBzsqService;

    /**
     * 分页查询
     *
     * @param page   分页对象
     * @param wxBzsq 微信-报装申请
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone getWxBzsqPage(Page page, @ModelAttribute WxBzsq wxBzsq) {
        return ResultRespone.success(wxBzsqService.page(page, Wrappers.query(wxBzsq)));
    }


    /**
     * 通过id查询微信-报装申请
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("id") String id) {
        return ResultRespone.success(wxBzsqService.getById(id));
    }

    /**
     * 新增微信-报装申请
     *
     * @param wxBzsq 微信-报装申请
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增微信-报装申请")
    @ApiOperation(value = "新增微信-报装申请")
    @PostMapping
    public ResultRespone save(@RequestBody WxBzsq wxBzsq) {
        return ResultRespone.success(wxBzsqService.save(wxBzsq));
    }

    /**
     * 修改微信-报装申请
     *
     * @param wxBzsq 微信-报装申请
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改微信-报装申请")
    @ApiOperation(value = "修改微信-报装申请")
    @PutMapping
    public ResultRespone updateById(@RequestBody WxBzsq wxBzsq) {
        return ResultRespone.success(wxBzsqService.updateById(wxBzsq));
    }

    /**
     * 通过id删除微信-报装申请
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除微信-报装申请")
    @ApiOperation(value = "通过ID删除微信-报装申请")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable String id) {
        return ResultRespone.success(wxBzsqService.removeById(id));
    }

    /**
     * 通过id批量删除微信-报装申请
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除微信-报装申请")
    @ApiOperation(value = "通过ID批量删除微信-报装申请")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(wxBzsqService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
