/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.fees.wechatofficalacctmgt.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.fees.wechatbasicmgt.entity.WxDtbm;
import com.tofly.fees.wechatbasicmgt.service.WxDtbmService;
import com.tofly.fees.wechatofficalacctmgt.entity.WxGsdt;
import com.tofly.fees.wechatofficalacctmgt.mapper.WxGsdtMapper;
import com.tofly.fees.wechatofficalacctmgt.service.WxGsdtService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 公示公告
 *
 * @author sxb
 * @date Mon Jun 05 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/wechatofficalacctmgt/wxgsdt")
@Api(tags="微信公众号-公示公告接口")
public class WxGsdtController {

    private final  WxGsdtService wxGsdtService;

    private final WxDtbmService wxDtbmService;
    private final WxGsdtMapper wxGsdtMapper;

    /**
    * 分页查询wxGsdt 公示公告
    * @param page 分页对象
    * @param
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "current", value = "当前页码", required = true, dataType = "int"),
            @ApiImplicitParam(name = "size", value = "每页显示数据条数", required = true, dataType = "int"),
            @ApiImplicitParam(name = "pages", value = "共几页,输出参数",  dataType = "int"),
            @ApiImplicitParam(name = "maxLimit", value = "每页最大显示数据",  dataType = "int"),
            @ApiImplicitParam(name = "orders[0].asc", value = "排序,bool类型",  dataType = "String"),
            @ApiImplicitParam(name = "orders[0].column", value = "排序,字段名",  dataType = "String"),
            @ApiImplicitParam(name = "type", value = "类型",  dataType = "String"),
            @ApiImplicitParam(name = "searchText", value = "模糊查询(标题)",  dataType = "String")
    })
    public ResultRespone getWxGsdtPage(Page page, String searchText,String type) {
        QueryWrapper<WxGsdt> queryWrapper = new QueryWrapper<WxGsdt>();
        queryWrapper.eq("1","1");
        if(StringUtils.isNotBlank(searchText))
        {
            queryWrapper.and(i->i.like("title",searchText));
        }
        if(StringUtils.isNotBlank(type) && type!="all")
        {
            queryWrapper.in("dt_type",type);
        }
        Page<WxGsdt> wxpage=wxGsdtService.page(page, queryWrapper);

        List<WxGsdt> wxList = wxpage.getRecords();

        for (WxGsdt wxGsdt : wxList) {
            //动态编码
            List<WxDtbm> wxDtbmsList=wxDtbmService.list();
            Map<String,String> map = new HashMap<>();
            wxDtbmsList.forEach(e->map.put(e.getId(), e.getName()));
            //类型
            wxGsdt.setDtTypeName(map.get(wxGsdt.getDtType()));
        }
        return ResultRespone.success(wxpage);
    }


    /**
     * 通过id查询公示公告
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    public ResultRespone getById(@PathVariable("id" ) Integer id) {
      return ResultRespone.success(wxGsdtService.getById(id));
    }

    /**
     * 新增公示公告
     * @param wxGsdt 公示公告
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增公示公告" )
    @ApiOperation(value = "新增公示公告")
    @PostMapping
    public ResultRespone save(@RequestBody WxGsdt wxGsdt) {
        //查询当前ID
        int id=  wxGsdtMapper.SelectChargeMaxCode();
        wxGsdt.setId(id+1);
        return ResultRespone.success(wxGsdtService.save(wxGsdt));
    }

    /**
     * 修改公示公告
     * @param wxGsdt 公示公告
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改公示公告" )
    @ApiOperation(value = "修改公示公告")
    @PutMapping
    public ResultRespone updateById(@RequestBody WxGsdt wxGsdt) {
      return ResultRespone.success(wxGsdtService.updateById(wxGsdt));
    }

    /**
     * 通过id删除公示公告
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除公示公告" )
    @ApiOperation(value = "通过ID删除公示公告")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "BigDecimal")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Integer id) {
      return ResultRespone.success(wxGsdtService.removeById(id));
    }

    /**
     * 通过id批量删除公示公告
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除公示公告" )
    @ApiOperation(value = "通过ID批量删除公示公告")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(wxGsdtService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
