package com.tofly.fees.wechatofficalacctmgt.controller_easy;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.fees.common.BaseController;
import com.tofly.fees.common.annotate.multiRequest.MultiRequestBody;
import com.tofly.fees.common.dbhelper.DbHelper;
import com.tofly.fees.common.dbhelper.PageSortHelper;
import com.tofly.fees.common.enums.SysEnum;
import com.tofly.fees.common.model.PageLink;
import com.tofly.fees.common.util.DateUtils;
import com.tofly.fees.common.util.ToolsUtils;
import com.tofly.fees.common.util.XmlUtils;
import com.tofly.fees.wechatofficalacctmgt.entity.WxBzsq;
import com.tofly.fees.wechatofficalacctmgt.entity.vo.WxBzsqVo;
import com.tofly.fees.wechatofficalacctmgt.service.WxBzsqService;
import io.swagger.annotations.*;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.web.bind.annotation.*;

import java.io.*;
import java.text.MessageFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;

import static com.tofly.fees.common.ControllerConstants.*;
import static com.tofly.fees.common.ControllerConstants.PAGE_NUMBER_PARA_NAME;

/**
 * @description:
 * @Title: WxBzsqEsayController
 * @Package com.tofly.fees.wechatofficalacctmgt.controller_esay
 * @Author 微信报装申请扩展接口
 * @Date 2023-06-12 17:42
 */
@RestController
@AllArgsConstructor
@RequestMapping("/api/wechatofficalacctmgt/wxbzsq")
@Api(tags = "微信公众号-报装申请接口")
public class WxBzsqEasyController extends BaseController {

    private final WxBzsqService wxBzsqService;

    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 分页查询
     *
     * @param page 分页对象
     * @return
     */
    @GetMapping("/pageInfo")
    @ApiOperation(value = "[Easy]自定义分页查询")
    public ResultRespone getWxBzsqPage(@ApiParam(value = PAGE_SIZE_DESCRIPTION, required = true)
                                       @RequestParam(value = PAGE_SIZE_PARA_NAME) int pageSize,
                                       @ApiParam(value = PAGE_NUMBER_DESCRIPTION, required = true)
                                       @RequestParam(value = PAGE_NUMBER_PARA_NAME) int page,
                                       @ApiParam(value = "状态")
                                       @RequestParam(value = "state", defaultValue = "") String state,
                                       @ApiParam(value = "起始时间") @RequestParam("startTime") @DateTimeFormat(pattern = "yyyy-MM-dd")
                                       Date startTime,
                                       @ApiParam(value = "截至时间") @RequestParam("endTime") @DateTimeFormat(pattern = "yyyy-MM-dd")
                                       Date endTime,
                                       @ApiParam(value = "类型")
                                       @RequestParam(value = "type", defaultValue = "") String type,
                                       @ApiParam(value = "模糊查询")
                                       @RequestParam(value = "searchText", defaultValue = "") String searchText,
                                       @ApiParam(value = SORT_PROPERTY_DESCRIPTION)
                                       @RequestParam(required = false) String sortProperty,
                                       @ApiParam(value = SORT_ORDER_DESCRIPTION, allowableValues = SORT_ORDER_ALLOWABLE_VALUES)
                                       @RequestParam(required = false) String sortOrder
    ) throws Exception {
        DbHelper db = DbHelper.getDbHelper();
        PageLink pg = createPageLink(pageSize, page);

        String strSql = String.format("select * from TF_YWYS_WX_BZSQ t where 1=1");
        if (!StringUtils.isBlank(state)) {
            strSql += String.format(" and state='%s'", state);
        }

        if (!StringUtils.isBlank(type)) {
            strSql += String.format(" and type='%s'", type);
        }

        if (!StringUtils.isBlank(searchText)) {
            strSql += String.format(" and (REGISTER_NAME like '%1$s%%' or REGISTER_address like '%1$s%%'  or REGISTER_phone like '%1$s%%' or bh like '%1$s%%')", searchText);
        }

        if (startTime != null && endTime != null) {
            strSql += String.format(" and REGISTER_Date>=to_date('%s','yyyy-MM-dd HH24:mi:ss') and REGISTER_Date<=to_date('%s','yyyy-MM-dd HH24:mi:ss')", DateUtils.formatDate(startTime, "yyyy-MM-dd hh:mm:ss"), DateUtils.formatDate(endTime, "yyyy-MM-dd hh:mm:ss"));
        }

//        if(!StringUtils.isBlank(sortProperty)) {
//            HashMap<String, String> es = (HashMap<String, String>) redisTemplate.opsForValue().get(Mapper_Redis_Prix + "wxBzsqMap");
//            if (es == null) {
//                es = XmlUtils.getHashMapperColumnByElement("wechatofficalacctmgt/wxBzsqmapper.xml", "wxBzsqMap");
//                redisTemplate.opsForValue().set(Mapper_Redis_Prix + "wxBzsqMap", es);
//            }
//            String sortDataProperty = (String) es.get(sortProperty);
//            strSql += String.format(" order by %s %s", sortDataProperty,sortOrder);
//        }
        strSql += MessageFormat.format(" order by {0}"
                , PageSortHelper.setPageSort(WxBzsq.class, sortProperty, sortOrder));

        System.out.println(strSql);
        Object obj = db.getPage(pg, strSql, SysEnum.MyBatis.getName());
        return ResultRespone.success(obj);
    }


//    /**
//     * @MethodName setState
//     * @Description 业务流程处理接口
//     * @Author xb
//     * @Date 10:54 2023-06-13
//     */
//    @ApiOperation(value = "业务流程处理")
//    @PutMapping("/setState")
//    public ResultRespone setState(@ApiParam(value = "ID")
//                                  @RequestParam(value = "id", defaultValue = "", required = true) String id,
//                                  @ApiParam(value = "状态")
//                                  @RequestParam(value = "state", defaultValue = "", required = true) String state,
//                                  @ApiParam(value = "处理结果")
//                                  @RequestParam(value = "result", defaultValue = "", required = true) String result,
//                                  @ApiParam(value = "处理人员编码")
//                                  @RequestParam(value = "code", defaultValue = "", required = true) String code
//    ) throws Exception {
//        DbHelper db = DbHelper.getDbHelper();
//        String name = "administrator";
//        if (!StringUtils.isBlank(code)) {
//            name = code;
//        }
//
//        String date = DateUtils.formatDate(new Date(), "yyyy-MM-dd hh:mm:ss");
//        String strSql = String.format("update TF_YWYS_WX_BZSQ set state='%s',handle_name='%s',handle_date=to_date('%s','yyyy-MM-dd HH24:mi:ss'),handle_result='%s' " +
//                "where id='%s'", state, name, date, result, id);
//        System.out.println(strSql);
//        int rows = db.executeCmd(strSql);
//        boolean isTrue = false;
//        if (rows > 0) {
//            isTrue = true;
//        }
//        return ResultRespone.success(isTrue, strSql);
//    }

    @ApiOperation(value = "[Easy]新增")
    @PostMapping("/saveBzsq")
    public ResultRespone saveBzsq(@RequestBody WxBzsq wxBzsq) throws Exception {
        wxBzsq.setId(ToolsUtils.getGuid());
        wxBzsq.setBh(ToolsUtils.getLsbh());
        boolean isAdd = wxBzsqService.save(wxBzsq);
        if (isAdd)
            return ResultRespone.success(wxBzsq.getBh());
        else
            return ResultRespone.failed();
    }

    @ApiOperation(value = "[Easy]业务流程处理")
    @PutMapping("/updateBzsq")
    public ResultRespone updateBzsq(@RequestBody WxBzsqVo wxBzsqVo) throws Exception {
        System.out.println(wxBzsqVo);
        boolean save = wxBzsqService.updateById(wxBzsqVo);
        if (save)
            return ResultRespone.success(true);
        else
            return ResultRespone.failed(false);
    }

    /**
     * 根据文件路径读取文件内容
     *
     * @param fileInPath
     * @throws IOException
     */
    public void getFileContent(Object fileInPath, String id, String property) throws IOException {
        BufferedReader br = null;
        if (fileInPath == null) {
            return;
        }
        if (fileInPath instanceof String) {
            br = new BufferedReader(new FileReader(new File((String) fileInPath)));
        } else if (fileInPath instanceof InputStream) {
            br = new BufferedReader(new InputStreamReader((InputStream) fileInPath));
        }
        String line;
        while ((line = br.readLine()) != null) {
            boolean isChecked = line.startsWith("he");
            if (isChecked) {

            }
        }
        br.close();
    }


}
