package com.tofly.fees.wechatwebsite.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.fees.meterprofilemgt.entity.DaMeter;
import com.tofly.fees.meterprofilemgt.service.DaMeterService;
import com.tofly.fees.userprofilemgt.entity.DaCustomer;
import com.tofly.fees.userprofilemgt.service.DaCustomerService;
import com.tofly.fees.wechatbasicmgt.entity.WxCustomer;
import com.tofly.fees.wechatbasicmgt.entity.WxCustomerHis;
import com.tofly.fees.wechatbasicmgt.service.WxCustomerHisService;
import com.tofly.fees.wechatbasicmgt.service.WxCustomerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Date;
import java.util.List;

@RestController
@AllArgsConstructor
@RequestMapping("/api/wechatwebsite/BindCustomer")
@Api(tags="微网厅-用户绑定接口bak")
public class BindCustomerWController {

    private final DaCustomerService daCustomerService;
    private final WxCustomerService wxCustomerService;
    private final WxCustomerHisService wxCustomerHisService;
    private final DaMeterService daMeterService;



    @GetMapping("/selectCustomer" )
    @ApiOperation(value = "查询用户")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "openid", value = "微信openid", required = true, dataType = "String"),
            @ApiImplicitParam(name = "customerNo", value = "客户编号", required = true, dataType = "String"),
            @ApiImplicitParam(name = "customerName", value = "客户姓名", required = true, dataType = "String")
    })
    public ResultRespone  selectCustomer(HttpServletResponse response, @RequestParam("openid") String openid, @RequestParam("customerNo") String customerNo, @RequestParam("customerName") String customerName) throws IOException {

        ResultRespone<Object> resultModel = new ResultRespone<>();
        resultModel.setCode(0);

        //region 参数验证
        //验证
        if (StringUtils.isBlank(customerNo)) {
            resultModel.setCode(-1);
            resultModel.setMessage("客户编号不能为空！");
            return resultModel;
        }
        if (StringUtils.isBlank(customerName)) {
            resultModel.setCode(-1);
            resultModel.setMessage("客户名称不能为空！");
            return resultModel;
        }
        //endregion

        QueryWrapper<DaCustomer> queryWrapper = new QueryWrapper<DaCustomer>();

        queryWrapper.eq("customer_no",customerNo).notIn("customer_state", '3','4');
        List<DaCustomer> daCustomerList = daCustomerService.list(queryWrapper);
        if (daCustomerList.size()==0) {
            resultModel.setCode(1);
            resultModel.setMessage("客户名称不存在！");
            return resultModel;
        }
        queryWrapper.clear();
        queryWrapper.eq("customer_no",customerNo).notIn("customer_state",'3','4').like("customer_name",customerName);
        daCustomerList = daCustomerService.list(queryWrapper);
        if (daCustomerList.size()==0 || daCustomerList.size()>1) {
            resultModel.setCode(1);
            resultModel.setMessage("客户名称不匹配，请重新输入！");
            return resultModel;
        }
        QueryWrapper<WxCustomer> queryWrapperWx = new QueryWrapper<WxCustomer>();
        queryWrapperWx.eq("customer_no",customerNo).eq("openid",openid);
        List<WxCustomer> wxCustomerList = wxCustomerService.list(queryWrapperWx);
        if ( wxCustomerList.size()>0)
        {

            if ( !StringUtils.isBlank(wxCustomerList.get(0).getBindFlag()) && wxCustomerList.get(0).getBindFlag()=="1")
            {
                resultModel.setCode(3);
                resultModel.setMessage("该收费号已绑定微信！");
                return resultModel;
            }
        }
        queryWrapperWx.clear();
        queryWrapperWx.eq("openid",openid).eq("bind_flag","1");
        wxCustomerList = wxCustomerService.list(queryWrapperWx);

        if ( wxCustomerList.size()>10)
        {
            resultModel.setCode(5);
            resultModel.setMessage("此微信已绑定满10个收费号！");
            return resultModel;
        }
        queryWrapperWx.clear();
        queryWrapperWx.eq("customer_no",customerNo).eq("bind_flag","1");
        wxCustomerList = wxCustomerService.list(queryWrapperWx);

        if ( wxCustomerList.size()>10)
        {
            resultModel.setCode(5);
            resultModel.setMessage("此收费号已绑定满10个微信号!");
            return resultModel;
        }

        QueryWrapper<DaMeter> queryWrapperDaMeter = new QueryWrapper<DaMeter>();
        queryWrapperDaMeter.eq("customer_no",customerNo).in("meter_type",'1');
        List<DaMeter> daMeterList = daMeterService.list(queryWrapperDaMeter);

        if ( daMeterList.size()>0)
        {
            resultModel.setCode(6);
            resultModel.setMessage("IC卡用户不能绑定,请持卡到营业点缴费!！");
            return resultModel;
        }

        //region 业务处理
        queryWrapper.clear();
        queryWrapper.eq("customer_no",customerNo);
        daCustomerList = daCustomerService.list(queryWrapper);
        resultModel.setResult(daCustomerList.get(0));
        resultModel.setMessage("成功！");
        //endregion
        return resultModel;
    }

    @PostMapping ("/addCustomer" )
    @ApiOperation(value = "添加用户")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "openid", value = "微信openid", required = true, dataType = "String"),
            @ApiImplicitParam(name = "customerNo", value = "客户编号", required = true, dataType = "String")
    })
    public ResultRespone  addCustomer(@RequestParam("openid") String openid, @RequestParam("customerNo") String customerNo) throws IOException {

        ResultRespone<Object> resultModel = new ResultRespone<>();
        resultModel.setCode(0);
        //region 参数验证
        //验证
        if (StringUtils.isBlank(customerNo)) {
            resultModel.setCode(-1);
            resultModel.setMessage("客户编号不能为空！");
            return resultModel;
        }
        if (StringUtils.isBlank(openid)) {
            resultModel.setCode(-1);
            resultModel.setMessage("微信openid不能为空！");
            return resultModel;
        }
        //endregion

        QueryWrapper<DaCustomer> queryWrapper = new QueryWrapper<DaCustomer>();

        queryWrapper.eq("customer_no",customerNo).notIn("customer_state",'3','4');
        List<DaCustomer> daCustomerList = daCustomerService.list(queryWrapper);
        if (daCustomerList.size()==0) {
            resultModel.setCode(1);
            resultModel.setMessage("客户名称不存在！");
            return resultModel;
        }

        QueryWrapper<WxCustomer> queryWrapperWx = new QueryWrapper<WxCustomer>();
        queryWrapperWx.eq("customer_no",customerNo).eq("openid",openid);
        List<WxCustomer> wxCustomerList = wxCustomerService.list(queryWrapperWx);
        if(wxCustomerList.size()>0) {
            if (!StringUtils.isBlank(wxCustomerList.get(0).getBindFlag()) && wxCustomerList.get(0).getBindFlag() == "1") {
                resultModel.setCode(3);
                resultModel.setMessage("该收费号已绑定微信！");
                return resultModel;
            }
       }

        //region 业务处理
        queryWrapper.clear();
        queryWrapper.eq("customer_no",customerNo);
        daCustomerList = daCustomerService.list(queryWrapper);
        WxCustomer wxCustomer=new WxCustomer();
        if ( wxCustomerList.size()>0 )
        {

            wxCustomer=wxCustomerList.get(0);
            wxCustomer.setBindFlag("1");
            wxCustomer.setName(daCustomerList.get(0).getCustomerName());
            wxCustomer.setAddress(daCustomerList.get(0).getCustomerAddress());
            wxCustomer.setPhone(daCustomerList.get(0).getCustomerPhone());
            wxCustomer.setBindDate(new Date());
             if (wxCustomerService.update(wxCustomer,queryWrapperWx)) {
                resultModel.setCode(0);
                resultModel.setMessage("绑定成功！");
            }
            else
            {
                resultModel.setCode(-1);
                resultModel.setMessage("绑定失败！");
            }
        }
        else
        {
            wxCustomer.setCustomerNo(customerNo);
            wxCustomer.setOpenid(openid);
            wxCustomer.setBindFlag("1");
            wxCustomer.setName(daCustomerList.get(0).getCustomerName());
            wxCustomer.setAddress(daCustomerList.get(0).getCustomerAddress());
            wxCustomer.setPhone(daCustomerList.get(0).getCustomerPhone());
            wxCustomer.setBindDate(new Date());
            if (wxCustomerService.save(wxCustomer)) {
                resultModel.setCode(0);
                resultModel.setMessage("绑定成功！");
            }
            else
            {
                resultModel.setCode(-1);
                resultModel.setMessage("绑定失败！");
            }
        }

        //endregion
        return resultModel;
    }

    @PostMapping ("/removeCustomer" )
    @ApiOperation(value = "刪除用户")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "openid", value = "微信openid", required = true, dataType = "String"),
            @ApiImplicitParam(name = "customerNo", value = "客户编号", required = true, dataType = "String")
    })
    public ResultRespone  deleteCustomer(@RequestParam("openid") String openid, @RequestParam("customerNo") String customerNo) throws IOException {

        ResultRespone<Object> resultModel = new ResultRespone<>();
        resultModel.setCode(0);
        //region 参数验证
        //验证
        if (StringUtils.isBlank(customerNo)) {
            resultModel.setCode(-1);
            resultModel.setMessage("客户编号不能为空！");
            return resultModel;
        }
        if (StringUtils.isBlank(openid)) {
            resultModel.setCode(-1);
            resultModel.setMessage("微信openid不能为空！");
            return resultModel;
        }
        //endregion


        QueryWrapper<WxCustomer> queryWrapperWx = new QueryWrapper<WxCustomer>();
        queryWrapperWx.eq("customer_no",customerNo).eq("openid",openid);
        List<WxCustomer> wxCustomerList = wxCustomerService.list(queryWrapperWx);
        WxCustomer wxCustomer =new WxCustomer();
        WxCustomerHis wxCustomerHis=new WxCustomerHis();
        if(wxCustomerList.size()>0) {
            wxCustomer=wxCustomerList.get(0);
            int id=  wxCustomerHisService.SelectChargeMaxCode();
            wxCustomerHis.setId(id+1);
            wxCustomerHis.setCustomerNo(wxCustomer.getCustomerNo());
            wxCustomerHis.setName(wxCustomer.getName());
            wxCustomerHis.setAddress(wxCustomer.getAddress());
            wxCustomerHis.setPhone(wxCustomer.getPhone());
            wxCustomerHis.setBindFlag(wxCustomer.getBindFlag());
            wxCustomerHis.setOpenid(wxCustomer.getOpenid());
            wxCustomerHis.setBindDate(wxCustomer.getBindDate());
            wxCustomerHis.setBindRemark(wxCustomer.getBindRemark());
            wxCustomerHis.setBindWxname(wxCustomer.getBindWxname());
            wxCustomerHis.setBindSex(wxCustomer.getBindSex());
            wxCustomerHis.setUnbindDate(new Date());
            wxCustomerHis.setUnbindName("本人");
            wxCustomerHis.setUnbindRemark("自愿");
            boolean saveWxCustomerHis = wxCustomerHisService.save(wxCustomerHis);
            wxCustomer.setBindFlag("0");
            wxCustomer.setUnbindDate(new Date());
            wxCustomer.setUnbindName("本人");
            wxCustomer.setUnbindRemark("自愿");
            boolean updatewxCustomer = wxCustomerService.update(wxCustomer, queryWrapperWx);
            if (saveWxCustomerHis && updatewxCustomer)
            {
                resultModel.setCode(0);
                resultModel.setMessage("解绑成功！");
            }
            else
            {resultModel.setCode(-1);
                resultModel.setMessage("解绑失败！");}

        }
        else
        {resultModel.setCode(-1);
            resultModel.setMessage("解绑失败！");}

        //region 业务处理


        //endregion
        return resultModel;
    }

    @GetMapping("/getBindCustomer" )
    @ApiOperation(value = "获取绑定用户")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "openid", value = "微信openid", required = true, dataType = "String")
    })
    public ResultRespone  getBindCustomer( @RequestParam("openid") String openid) {

        ResultRespone<Object> resultModel = new ResultRespone<>();
        resultModel.setCode(0);

        //region 参数验证
        //验证
        if (StringUtils.isBlank(openid)) {
            resultModel.setCode(-1);
            resultModel.setMessage("传入微信编号不能为空！");
            return resultModel;
        }

        //endregion

        QueryWrapper<WxCustomer> queryWrapperWx = new QueryWrapper<WxCustomer>();
        queryWrapperWx.eq("openid",openid).eq("bind_flag","1").orderByAsc("customer_no");
        List<WxCustomer> wxCustomerList = wxCustomerService.list(queryWrapperWx);
        if ( wxCustomerList.size()==0)
        {
                resultModel.setCode(1);
                resultModel.setMessage("请绑定用户！");
                return resultModel;
        }


        //region 业务处理

        resultModel.setResult(wxCustomerList);
        resultModel.setMessage("成功！");
        //endregion
        return resultModel;
    }


}
